/*
Copyright (C) 2018-2019 de4dot@gmail.com

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Linq;
using Generator.Enums;

namespace Generator.Decoder {
	static class MnemonicsTable {
		public static readonly (EnumValue codeEnum, EnumValue mnemonicEnum)[] Table = CreateTable();

		static (EnumValue codeEnum, EnumValue mnemonicEnum)[] CreateTable() {
			var code = CodeEnum.Instance;
			var mnemonic = MnemonicEnum.Instance;

			var result = new (EnumValue codeEnum, EnumValue mnemonicEnum)[] {
				(code[nameof(Code.INVALID)], mnemonic[nameof(Mnemonic.INVALID)]),
				(code[nameof(Code.DeclareByte)], mnemonic[nameof(Mnemonic.Db)]),
				(code[nameof(Code.DeclareWord)], mnemonic[nameof(Mnemonic.Dw)]),
				(code[nameof(Code.DeclareDword)], mnemonic[nameof(Mnemonic.Dd)]),
				(code[nameof(Code.DeclareQword)], mnemonic[nameof(Mnemonic.Dq)]),
				(code[nameof(Code.Add_rm8_r8)], mnemonic[nameof(Mnemonic.Add)]),
				(code[nameof(Code.Add_rm16_r16)], mnemonic[nameof(Mnemonic.Add)]),
				(code[nameof(Code.Add_rm32_r32)], mnemonic[nameof(Mnemonic.Add)]),
				(code[nameof(Code.Add_rm64_r64)], mnemonic[nameof(Mnemonic.Add)]),
				(code[nameof(Code.Add_r8_rm8)], mnemonic[nameof(Mnemonic.Add)]),
				(code[nameof(Code.Add_r16_rm16)], mnemonic[nameof(Mnemonic.Add)]),
				(code[nameof(Code.Add_r32_rm32)], mnemonic[nameof(Mnemonic.Add)]),
				(code[nameof(Code.Add_r64_rm64)], mnemonic[nameof(Mnemonic.Add)]),
				(code[nameof(Code.Add_AL_imm8)], mnemonic[nameof(Mnemonic.Add)]),
				(code[nameof(Code.Add_AX_imm16)], mnemonic[nameof(Mnemonic.Add)]),
				(code[nameof(Code.Add_EAX_imm32)], mnemonic[nameof(Mnemonic.Add)]),
				(code[nameof(Code.Add_RAX_imm32)], mnemonic[nameof(Mnemonic.Add)]),
				(code[nameof(Code.Pushw_ES)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Pushd_ES)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Popw_ES)], mnemonic[nameof(Mnemonic.Pop)]),
				(code[nameof(Code.Popd_ES)], mnemonic[nameof(Mnemonic.Pop)]),
				(code[nameof(Code.Or_rm8_r8)], mnemonic[nameof(Mnemonic.Or)]),
				(code[nameof(Code.Or_rm16_r16)], mnemonic[nameof(Mnemonic.Or)]),
				(code[nameof(Code.Or_rm32_r32)], mnemonic[nameof(Mnemonic.Or)]),
				(code[nameof(Code.Or_rm64_r64)], mnemonic[nameof(Mnemonic.Or)]),
				(code[nameof(Code.Or_r8_rm8)], mnemonic[nameof(Mnemonic.Or)]),
				(code[nameof(Code.Or_r16_rm16)], mnemonic[nameof(Mnemonic.Or)]),
				(code[nameof(Code.Or_r32_rm32)], mnemonic[nameof(Mnemonic.Or)]),
				(code[nameof(Code.Or_r64_rm64)], mnemonic[nameof(Mnemonic.Or)]),
				(code[nameof(Code.Or_AL_imm8)], mnemonic[nameof(Mnemonic.Or)]),
				(code[nameof(Code.Or_AX_imm16)], mnemonic[nameof(Mnemonic.Or)]),
				(code[nameof(Code.Or_EAX_imm32)], mnemonic[nameof(Mnemonic.Or)]),
				(code[nameof(Code.Or_RAX_imm32)], mnemonic[nameof(Mnemonic.Or)]),
				(code[nameof(Code.Pushw_CS)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Pushd_CS)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Popw_CS)], mnemonic[nameof(Mnemonic.Pop)]),
				(code[nameof(Code.Adc_rm8_r8)], mnemonic[nameof(Mnemonic.Adc)]),
				(code[nameof(Code.Adc_rm16_r16)], mnemonic[nameof(Mnemonic.Adc)]),
				(code[nameof(Code.Adc_rm32_r32)], mnemonic[nameof(Mnemonic.Adc)]),
				(code[nameof(Code.Adc_rm64_r64)], mnemonic[nameof(Mnemonic.Adc)]),
				(code[nameof(Code.Adc_r8_rm8)], mnemonic[nameof(Mnemonic.Adc)]),
				(code[nameof(Code.Adc_r16_rm16)], mnemonic[nameof(Mnemonic.Adc)]),
				(code[nameof(Code.Adc_r32_rm32)], mnemonic[nameof(Mnemonic.Adc)]),
				(code[nameof(Code.Adc_r64_rm64)], mnemonic[nameof(Mnemonic.Adc)]),
				(code[nameof(Code.Adc_AL_imm8)], mnemonic[nameof(Mnemonic.Adc)]),
				(code[nameof(Code.Adc_AX_imm16)], mnemonic[nameof(Mnemonic.Adc)]),
				(code[nameof(Code.Adc_EAX_imm32)], mnemonic[nameof(Mnemonic.Adc)]),
				(code[nameof(Code.Adc_RAX_imm32)], mnemonic[nameof(Mnemonic.Adc)]),
				(code[nameof(Code.Pushw_SS)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Pushd_SS)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Popw_SS)], mnemonic[nameof(Mnemonic.Pop)]),
				(code[nameof(Code.Popd_SS)], mnemonic[nameof(Mnemonic.Pop)]),
				(code[nameof(Code.Sbb_rm8_r8)], mnemonic[nameof(Mnemonic.Sbb)]),
				(code[nameof(Code.Sbb_rm16_r16)], mnemonic[nameof(Mnemonic.Sbb)]),
				(code[nameof(Code.Sbb_rm32_r32)], mnemonic[nameof(Mnemonic.Sbb)]),
				(code[nameof(Code.Sbb_rm64_r64)], mnemonic[nameof(Mnemonic.Sbb)]),
				(code[nameof(Code.Sbb_r8_rm8)], mnemonic[nameof(Mnemonic.Sbb)]),
				(code[nameof(Code.Sbb_r16_rm16)], mnemonic[nameof(Mnemonic.Sbb)]),
				(code[nameof(Code.Sbb_r32_rm32)], mnemonic[nameof(Mnemonic.Sbb)]),
				(code[nameof(Code.Sbb_r64_rm64)], mnemonic[nameof(Mnemonic.Sbb)]),
				(code[nameof(Code.Sbb_AL_imm8)], mnemonic[nameof(Mnemonic.Sbb)]),
				(code[nameof(Code.Sbb_AX_imm16)], mnemonic[nameof(Mnemonic.Sbb)]),
				(code[nameof(Code.Sbb_EAX_imm32)], mnemonic[nameof(Mnemonic.Sbb)]),
				(code[nameof(Code.Sbb_RAX_imm32)], mnemonic[nameof(Mnemonic.Sbb)]),
				(code[nameof(Code.Pushw_DS)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Pushd_DS)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Popw_DS)], mnemonic[nameof(Mnemonic.Pop)]),
				(code[nameof(Code.Popd_DS)], mnemonic[nameof(Mnemonic.Pop)]),
				(code[nameof(Code.And_rm8_r8)], mnemonic[nameof(Mnemonic.And)]),
				(code[nameof(Code.And_rm16_r16)], mnemonic[nameof(Mnemonic.And)]),
				(code[nameof(Code.And_rm32_r32)], mnemonic[nameof(Mnemonic.And)]),
				(code[nameof(Code.And_rm64_r64)], mnemonic[nameof(Mnemonic.And)]),
				(code[nameof(Code.And_r8_rm8)], mnemonic[nameof(Mnemonic.And)]),
				(code[nameof(Code.And_r16_rm16)], mnemonic[nameof(Mnemonic.And)]),
				(code[nameof(Code.And_r32_rm32)], mnemonic[nameof(Mnemonic.And)]),
				(code[nameof(Code.And_r64_rm64)], mnemonic[nameof(Mnemonic.And)]),
				(code[nameof(Code.And_AL_imm8)], mnemonic[nameof(Mnemonic.And)]),
				(code[nameof(Code.And_AX_imm16)], mnemonic[nameof(Mnemonic.And)]),
				(code[nameof(Code.And_EAX_imm32)], mnemonic[nameof(Mnemonic.And)]),
				(code[nameof(Code.And_RAX_imm32)], mnemonic[nameof(Mnemonic.And)]),
				(code[nameof(Code.Daa)], mnemonic[nameof(Mnemonic.Daa)]),
				(code[nameof(Code.Sub_rm8_r8)], mnemonic[nameof(Mnemonic.Sub)]),
				(code[nameof(Code.Sub_rm16_r16)], mnemonic[nameof(Mnemonic.Sub)]),
				(code[nameof(Code.Sub_rm32_r32)], mnemonic[nameof(Mnemonic.Sub)]),
				(code[nameof(Code.Sub_rm64_r64)], mnemonic[nameof(Mnemonic.Sub)]),
				(code[nameof(Code.Sub_r8_rm8)], mnemonic[nameof(Mnemonic.Sub)]),
				(code[nameof(Code.Sub_r16_rm16)], mnemonic[nameof(Mnemonic.Sub)]),
				(code[nameof(Code.Sub_r32_rm32)], mnemonic[nameof(Mnemonic.Sub)]),
				(code[nameof(Code.Sub_r64_rm64)], mnemonic[nameof(Mnemonic.Sub)]),
				(code[nameof(Code.Sub_AL_imm8)], mnemonic[nameof(Mnemonic.Sub)]),
				(code[nameof(Code.Sub_AX_imm16)], mnemonic[nameof(Mnemonic.Sub)]),
				(code[nameof(Code.Sub_EAX_imm32)], mnemonic[nameof(Mnemonic.Sub)]),
				(code[nameof(Code.Sub_RAX_imm32)], mnemonic[nameof(Mnemonic.Sub)]),
				(code[nameof(Code.Das)], mnemonic[nameof(Mnemonic.Das)]),
				(code[nameof(Code.Xor_rm8_r8)], mnemonic[nameof(Mnemonic.Xor)]),
				(code[nameof(Code.Xor_rm16_r16)], mnemonic[nameof(Mnemonic.Xor)]),
				(code[nameof(Code.Xor_rm32_r32)], mnemonic[nameof(Mnemonic.Xor)]),
				(code[nameof(Code.Xor_rm64_r64)], mnemonic[nameof(Mnemonic.Xor)]),
				(code[nameof(Code.Xor_r8_rm8)], mnemonic[nameof(Mnemonic.Xor)]),
				(code[nameof(Code.Xor_r16_rm16)], mnemonic[nameof(Mnemonic.Xor)]),
				(code[nameof(Code.Xor_r32_rm32)], mnemonic[nameof(Mnemonic.Xor)]),
				(code[nameof(Code.Xor_r64_rm64)], mnemonic[nameof(Mnemonic.Xor)]),
				(code[nameof(Code.Xor_AL_imm8)], mnemonic[nameof(Mnemonic.Xor)]),
				(code[nameof(Code.Xor_AX_imm16)], mnemonic[nameof(Mnemonic.Xor)]),
				(code[nameof(Code.Xor_EAX_imm32)], mnemonic[nameof(Mnemonic.Xor)]),
				(code[nameof(Code.Xor_RAX_imm32)], mnemonic[nameof(Mnemonic.Xor)]),
				(code[nameof(Code.Aaa)], mnemonic[nameof(Mnemonic.Aaa)]),
				(code[nameof(Code.Cmp_rm8_r8)], mnemonic[nameof(Mnemonic.Cmp)]),
				(code[nameof(Code.Cmp_rm16_r16)], mnemonic[nameof(Mnemonic.Cmp)]),
				(code[nameof(Code.Cmp_rm32_r32)], mnemonic[nameof(Mnemonic.Cmp)]),
				(code[nameof(Code.Cmp_rm64_r64)], mnemonic[nameof(Mnemonic.Cmp)]),
				(code[nameof(Code.Cmp_r8_rm8)], mnemonic[nameof(Mnemonic.Cmp)]),
				(code[nameof(Code.Cmp_r16_rm16)], mnemonic[nameof(Mnemonic.Cmp)]),
				(code[nameof(Code.Cmp_r32_rm32)], mnemonic[nameof(Mnemonic.Cmp)]),
				(code[nameof(Code.Cmp_r64_rm64)], mnemonic[nameof(Mnemonic.Cmp)]),
				(code[nameof(Code.Cmp_AL_imm8)], mnemonic[nameof(Mnemonic.Cmp)]),
				(code[nameof(Code.Cmp_AX_imm16)], mnemonic[nameof(Mnemonic.Cmp)]),
				(code[nameof(Code.Cmp_EAX_imm32)], mnemonic[nameof(Mnemonic.Cmp)]),
				(code[nameof(Code.Cmp_RAX_imm32)], mnemonic[nameof(Mnemonic.Cmp)]),
				(code[nameof(Code.Aas)], mnemonic[nameof(Mnemonic.Aas)]),
				(code[nameof(Code.Inc_r16)], mnemonic[nameof(Mnemonic.Inc)]),
				(code[nameof(Code.Inc_r32)], mnemonic[nameof(Mnemonic.Inc)]),
				(code[nameof(Code.Dec_r16)], mnemonic[nameof(Mnemonic.Dec)]),
				(code[nameof(Code.Dec_r32)], mnemonic[nameof(Mnemonic.Dec)]),
				(code[nameof(Code.Push_r16)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Push_r32)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Push_r64)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Pop_r16)], mnemonic[nameof(Mnemonic.Pop)]),
				(code[nameof(Code.Pop_r32)], mnemonic[nameof(Mnemonic.Pop)]),
				(code[nameof(Code.Pop_r64)], mnemonic[nameof(Mnemonic.Pop)]),
				(code[nameof(Code.Pushaw)], mnemonic[nameof(Mnemonic.Pusha)]),
				(code[nameof(Code.Pushad)], mnemonic[nameof(Mnemonic.Pusha)]),
				(code[nameof(Code.Popaw)], mnemonic[nameof(Mnemonic.Popa)]),
				(code[nameof(Code.Popad)], mnemonic[nameof(Mnemonic.Popa)]),
				(code[nameof(Code.Bound_r16_m1616)], mnemonic[nameof(Mnemonic.Bound)]),
				(code[nameof(Code.Bound_r32_m3232)], mnemonic[nameof(Mnemonic.Bound)]),
				(code[nameof(Code.Arpl_rm16_r16)], mnemonic[nameof(Mnemonic.Arpl)]),
				(code[nameof(Code.Arpl_r32m16_r32)], mnemonic[nameof(Mnemonic.Arpl)]),
				(code[nameof(Code.Movsxd_r16_rm16)], mnemonic[nameof(Mnemonic.Movsxd)]),
				(code[nameof(Code.Movsxd_r32_rm32)], mnemonic[nameof(Mnemonic.Movsxd)]),
				(code[nameof(Code.Movsxd_r64_rm32)], mnemonic[nameof(Mnemonic.Movsxd)]),
				(code[nameof(Code.Push_imm16)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Pushd_imm32)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Pushq_imm32)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Imul_r16_rm16_imm16)], mnemonic[nameof(Mnemonic.Imul)]),
				(code[nameof(Code.Imul_r32_rm32_imm32)], mnemonic[nameof(Mnemonic.Imul)]),
				(code[nameof(Code.Imul_r64_rm64_imm32)], mnemonic[nameof(Mnemonic.Imul)]),
				(code[nameof(Code.Pushw_imm8)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Pushd_imm8)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Pushq_imm8)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Imul_r16_rm16_imm8)], mnemonic[nameof(Mnemonic.Imul)]),
				(code[nameof(Code.Imul_r32_rm32_imm8)], mnemonic[nameof(Mnemonic.Imul)]),
				(code[nameof(Code.Imul_r64_rm64_imm8)], mnemonic[nameof(Mnemonic.Imul)]),
				(code[nameof(Code.Insb_m8_DX)], mnemonic[nameof(Mnemonic.Insb)]),
				(code[nameof(Code.Insw_m16_DX)], mnemonic[nameof(Mnemonic.Insw)]),
				(code[nameof(Code.Insd_m32_DX)], mnemonic[nameof(Mnemonic.Insd)]),
				(code[nameof(Code.Outsb_DX_m8)], mnemonic[nameof(Mnemonic.Outsb)]),
				(code[nameof(Code.Outsw_DX_m16)], mnemonic[nameof(Mnemonic.Outsw)]),
				(code[nameof(Code.Outsd_DX_m32)], mnemonic[nameof(Mnemonic.Outsd)]),
				(code[nameof(Code.Jo_rel8_16)], mnemonic[nameof(Mnemonic.Jo)]),
				(code[nameof(Code.Jo_rel8_32)], mnemonic[nameof(Mnemonic.Jo)]),
				(code[nameof(Code.Jo_rel8_64)], mnemonic[nameof(Mnemonic.Jo)]),
				(code[nameof(Code.Jno_rel8_16)], mnemonic[nameof(Mnemonic.Jno)]),
				(code[nameof(Code.Jno_rel8_32)], mnemonic[nameof(Mnemonic.Jno)]),
				(code[nameof(Code.Jno_rel8_64)], mnemonic[nameof(Mnemonic.Jno)]),
				(code[nameof(Code.Jb_rel8_16)], mnemonic[nameof(Mnemonic.Jb)]),
				(code[nameof(Code.Jb_rel8_32)], mnemonic[nameof(Mnemonic.Jb)]),
				(code[nameof(Code.Jb_rel8_64)], mnemonic[nameof(Mnemonic.Jb)]),
				(code[nameof(Code.Jae_rel8_16)], mnemonic[nameof(Mnemonic.Jae)]),
				(code[nameof(Code.Jae_rel8_32)], mnemonic[nameof(Mnemonic.Jae)]),
				(code[nameof(Code.Jae_rel8_64)], mnemonic[nameof(Mnemonic.Jae)]),
				(code[nameof(Code.Je_rel8_16)], mnemonic[nameof(Mnemonic.Je)]),
				(code[nameof(Code.Je_rel8_32)], mnemonic[nameof(Mnemonic.Je)]),
				(code[nameof(Code.Je_rel8_64)], mnemonic[nameof(Mnemonic.Je)]),
				(code[nameof(Code.Jne_rel8_16)], mnemonic[nameof(Mnemonic.Jne)]),
				(code[nameof(Code.Jne_rel8_32)], mnemonic[nameof(Mnemonic.Jne)]),
				(code[nameof(Code.Jne_rel8_64)], mnemonic[nameof(Mnemonic.Jne)]),
				(code[nameof(Code.Jbe_rel8_16)], mnemonic[nameof(Mnemonic.Jbe)]),
				(code[nameof(Code.Jbe_rel8_32)], mnemonic[nameof(Mnemonic.Jbe)]),
				(code[nameof(Code.Jbe_rel8_64)], mnemonic[nameof(Mnemonic.Jbe)]),
				(code[nameof(Code.Ja_rel8_16)], mnemonic[nameof(Mnemonic.Ja)]),
				(code[nameof(Code.Ja_rel8_32)], mnemonic[nameof(Mnemonic.Ja)]),
				(code[nameof(Code.Ja_rel8_64)], mnemonic[nameof(Mnemonic.Ja)]),
				(code[nameof(Code.Js_rel8_16)], mnemonic[nameof(Mnemonic.Js)]),
				(code[nameof(Code.Js_rel8_32)], mnemonic[nameof(Mnemonic.Js)]),
				(code[nameof(Code.Js_rel8_64)], mnemonic[nameof(Mnemonic.Js)]),
				(code[nameof(Code.Jns_rel8_16)], mnemonic[nameof(Mnemonic.Jns)]),
				(code[nameof(Code.Jns_rel8_32)], mnemonic[nameof(Mnemonic.Jns)]),
				(code[nameof(Code.Jns_rel8_64)], mnemonic[nameof(Mnemonic.Jns)]),
				(code[nameof(Code.Jp_rel8_16)], mnemonic[nameof(Mnemonic.Jp)]),
				(code[nameof(Code.Jp_rel8_32)], mnemonic[nameof(Mnemonic.Jp)]),
				(code[nameof(Code.Jp_rel8_64)], mnemonic[nameof(Mnemonic.Jp)]),
				(code[nameof(Code.Jnp_rel8_16)], mnemonic[nameof(Mnemonic.Jnp)]),
				(code[nameof(Code.Jnp_rel8_32)], mnemonic[nameof(Mnemonic.Jnp)]),
				(code[nameof(Code.Jnp_rel8_64)], mnemonic[nameof(Mnemonic.Jnp)]),
				(code[nameof(Code.Jl_rel8_16)], mnemonic[nameof(Mnemonic.Jl)]),
				(code[nameof(Code.Jl_rel8_32)], mnemonic[nameof(Mnemonic.Jl)]),
				(code[nameof(Code.Jl_rel8_64)], mnemonic[nameof(Mnemonic.Jl)]),
				(code[nameof(Code.Jge_rel8_16)], mnemonic[nameof(Mnemonic.Jge)]),
				(code[nameof(Code.Jge_rel8_32)], mnemonic[nameof(Mnemonic.Jge)]),
				(code[nameof(Code.Jge_rel8_64)], mnemonic[nameof(Mnemonic.Jge)]),
				(code[nameof(Code.Jle_rel8_16)], mnemonic[nameof(Mnemonic.Jle)]),
				(code[nameof(Code.Jle_rel8_32)], mnemonic[nameof(Mnemonic.Jle)]),
				(code[nameof(Code.Jle_rel8_64)], mnemonic[nameof(Mnemonic.Jle)]),
				(code[nameof(Code.Jg_rel8_16)], mnemonic[nameof(Mnemonic.Jg)]),
				(code[nameof(Code.Jg_rel8_32)], mnemonic[nameof(Mnemonic.Jg)]),
				(code[nameof(Code.Jg_rel8_64)], mnemonic[nameof(Mnemonic.Jg)]),
				(code[nameof(Code.Add_rm8_imm8)], mnemonic[nameof(Mnemonic.Add)]),
				(code[nameof(Code.Or_rm8_imm8)], mnemonic[nameof(Mnemonic.Or)]),
				(code[nameof(Code.Adc_rm8_imm8)], mnemonic[nameof(Mnemonic.Adc)]),
				(code[nameof(Code.Sbb_rm8_imm8)], mnemonic[nameof(Mnemonic.Sbb)]),
				(code[nameof(Code.And_rm8_imm8)], mnemonic[nameof(Mnemonic.And)]),
				(code[nameof(Code.Sub_rm8_imm8)], mnemonic[nameof(Mnemonic.Sub)]),
				(code[nameof(Code.Xor_rm8_imm8)], mnemonic[nameof(Mnemonic.Xor)]),
				(code[nameof(Code.Cmp_rm8_imm8)], mnemonic[nameof(Mnemonic.Cmp)]),
				(code[nameof(Code.Add_rm16_imm16)], mnemonic[nameof(Mnemonic.Add)]),
				(code[nameof(Code.Add_rm32_imm32)], mnemonic[nameof(Mnemonic.Add)]),
				(code[nameof(Code.Add_rm64_imm32)], mnemonic[nameof(Mnemonic.Add)]),
				(code[nameof(Code.Or_rm16_imm16)], mnemonic[nameof(Mnemonic.Or)]),
				(code[nameof(Code.Or_rm32_imm32)], mnemonic[nameof(Mnemonic.Or)]),
				(code[nameof(Code.Or_rm64_imm32)], mnemonic[nameof(Mnemonic.Or)]),
				(code[nameof(Code.Adc_rm16_imm16)], mnemonic[nameof(Mnemonic.Adc)]),
				(code[nameof(Code.Adc_rm32_imm32)], mnemonic[nameof(Mnemonic.Adc)]),
				(code[nameof(Code.Adc_rm64_imm32)], mnemonic[nameof(Mnemonic.Adc)]),
				(code[nameof(Code.Sbb_rm16_imm16)], mnemonic[nameof(Mnemonic.Sbb)]),
				(code[nameof(Code.Sbb_rm32_imm32)], mnemonic[nameof(Mnemonic.Sbb)]),
				(code[nameof(Code.Sbb_rm64_imm32)], mnemonic[nameof(Mnemonic.Sbb)]),
				(code[nameof(Code.And_rm16_imm16)], mnemonic[nameof(Mnemonic.And)]),
				(code[nameof(Code.And_rm32_imm32)], mnemonic[nameof(Mnemonic.And)]),
				(code[nameof(Code.And_rm64_imm32)], mnemonic[nameof(Mnemonic.And)]),
				(code[nameof(Code.Sub_rm16_imm16)], mnemonic[nameof(Mnemonic.Sub)]),
				(code[nameof(Code.Sub_rm32_imm32)], mnemonic[nameof(Mnemonic.Sub)]),
				(code[nameof(Code.Sub_rm64_imm32)], mnemonic[nameof(Mnemonic.Sub)]),
				(code[nameof(Code.Xor_rm16_imm16)], mnemonic[nameof(Mnemonic.Xor)]),
				(code[nameof(Code.Xor_rm32_imm32)], mnemonic[nameof(Mnemonic.Xor)]),
				(code[nameof(Code.Xor_rm64_imm32)], mnemonic[nameof(Mnemonic.Xor)]),
				(code[nameof(Code.Cmp_rm16_imm16)], mnemonic[nameof(Mnemonic.Cmp)]),
				(code[nameof(Code.Cmp_rm32_imm32)], mnemonic[nameof(Mnemonic.Cmp)]),
				(code[nameof(Code.Cmp_rm64_imm32)], mnemonic[nameof(Mnemonic.Cmp)]),
				(code[nameof(Code.Add_rm8_imm8_82)], mnemonic[nameof(Mnemonic.Add)]),
				(code[nameof(Code.Or_rm8_imm8_82)], mnemonic[nameof(Mnemonic.Or)]),
				(code[nameof(Code.Adc_rm8_imm8_82)], mnemonic[nameof(Mnemonic.Adc)]),
				(code[nameof(Code.Sbb_rm8_imm8_82)], mnemonic[nameof(Mnemonic.Sbb)]),
				(code[nameof(Code.And_rm8_imm8_82)], mnemonic[nameof(Mnemonic.And)]),
				(code[nameof(Code.Sub_rm8_imm8_82)], mnemonic[nameof(Mnemonic.Sub)]),
				(code[nameof(Code.Xor_rm8_imm8_82)], mnemonic[nameof(Mnemonic.Xor)]),
				(code[nameof(Code.Cmp_rm8_imm8_82)], mnemonic[nameof(Mnemonic.Cmp)]),
				(code[nameof(Code.Add_rm16_imm8)], mnemonic[nameof(Mnemonic.Add)]),
				(code[nameof(Code.Add_rm32_imm8)], mnemonic[nameof(Mnemonic.Add)]),
				(code[nameof(Code.Add_rm64_imm8)], mnemonic[nameof(Mnemonic.Add)]),
				(code[nameof(Code.Or_rm16_imm8)], mnemonic[nameof(Mnemonic.Or)]),
				(code[nameof(Code.Or_rm32_imm8)], mnemonic[nameof(Mnemonic.Or)]),
				(code[nameof(Code.Or_rm64_imm8)], mnemonic[nameof(Mnemonic.Or)]),
				(code[nameof(Code.Adc_rm16_imm8)], mnemonic[nameof(Mnemonic.Adc)]),
				(code[nameof(Code.Adc_rm32_imm8)], mnemonic[nameof(Mnemonic.Adc)]),
				(code[nameof(Code.Adc_rm64_imm8)], mnemonic[nameof(Mnemonic.Adc)]),
				(code[nameof(Code.Sbb_rm16_imm8)], mnemonic[nameof(Mnemonic.Sbb)]),
				(code[nameof(Code.Sbb_rm32_imm8)], mnemonic[nameof(Mnemonic.Sbb)]),
				(code[nameof(Code.Sbb_rm64_imm8)], mnemonic[nameof(Mnemonic.Sbb)]),
				(code[nameof(Code.And_rm16_imm8)], mnemonic[nameof(Mnemonic.And)]),
				(code[nameof(Code.And_rm32_imm8)], mnemonic[nameof(Mnemonic.And)]),
				(code[nameof(Code.And_rm64_imm8)], mnemonic[nameof(Mnemonic.And)]),
				(code[nameof(Code.Sub_rm16_imm8)], mnemonic[nameof(Mnemonic.Sub)]),
				(code[nameof(Code.Sub_rm32_imm8)], mnemonic[nameof(Mnemonic.Sub)]),
				(code[nameof(Code.Sub_rm64_imm8)], mnemonic[nameof(Mnemonic.Sub)]),
				(code[nameof(Code.Xor_rm16_imm8)], mnemonic[nameof(Mnemonic.Xor)]),
				(code[nameof(Code.Xor_rm32_imm8)], mnemonic[nameof(Mnemonic.Xor)]),
				(code[nameof(Code.Xor_rm64_imm8)], mnemonic[nameof(Mnemonic.Xor)]),
				(code[nameof(Code.Cmp_rm16_imm8)], mnemonic[nameof(Mnemonic.Cmp)]),
				(code[nameof(Code.Cmp_rm32_imm8)], mnemonic[nameof(Mnemonic.Cmp)]),
				(code[nameof(Code.Cmp_rm64_imm8)], mnemonic[nameof(Mnemonic.Cmp)]),
				(code[nameof(Code.Test_rm8_r8)], mnemonic[nameof(Mnemonic.Test)]),
				(code[nameof(Code.Test_rm16_r16)], mnemonic[nameof(Mnemonic.Test)]),
				(code[nameof(Code.Test_rm32_r32)], mnemonic[nameof(Mnemonic.Test)]),
				(code[nameof(Code.Test_rm64_r64)], mnemonic[nameof(Mnemonic.Test)]),
				(code[nameof(Code.Xchg_rm8_r8)], mnemonic[nameof(Mnemonic.Xchg)]),
				(code[nameof(Code.Xchg_rm16_r16)], mnemonic[nameof(Mnemonic.Xchg)]),
				(code[nameof(Code.Xchg_rm32_r32)], mnemonic[nameof(Mnemonic.Xchg)]),
				(code[nameof(Code.Xchg_rm64_r64)], mnemonic[nameof(Mnemonic.Xchg)]),
				(code[nameof(Code.Mov_rm8_r8)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_rm16_r16)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_rm32_r32)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_rm64_r64)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_r8_rm8)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_r16_rm16)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_r32_rm32)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_r64_rm64)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_rm16_Sreg)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_r32m16_Sreg)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_r64m16_Sreg)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Lea_r16_m)], mnemonic[nameof(Mnemonic.Lea)]),
				(code[nameof(Code.Lea_r32_m)], mnemonic[nameof(Mnemonic.Lea)]),
				(code[nameof(Code.Lea_r64_m)], mnemonic[nameof(Mnemonic.Lea)]),
				(code[nameof(Code.Mov_Sreg_rm16)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_Sreg_r32m16)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_Sreg_r64m16)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Pop_rm16)], mnemonic[nameof(Mnemonic.Pop)]),
				(code[nameof(Code.Pop_rm32)], mnemonic[nameof(Mnemonic.Pop)]),
				(code[nameof(Code.Pop_rm64)], mnemonic[nameof(Mnemonic.Pop)]),
				(code[nameof(Code.Nopw)], mnemonic[nameof(Mnemonic.Nop)]),
				(code[nameof(Code.Nopd)], mnemonic[nameof(Mnemonic.Nop)]),
				(code[nameof(Code.Nopq)], mnemonic[nameof(Mnemonic.Nop)]),
				(code[nameof(Code.Xchg_r16_AX)], mnemonic[nameof(Mnemonic.Xchg)]),
				(code[nameof(Code.Xchg_r32_EAX)], mnemonic[nameof(Mnemonic.Xchg)]),
				(code[nameof(Code.Xchg_r64_RAX)], mnemonic[nameof(Mnemonic.Xchg)]),
				(code[nameof(Code.Pause)], mnemonic[nameof(Mnemonic.Pause)]),
				(code[nameof(Code.Cbw)], mnemonic[nameof(Mnemonic.Cbw)]),
				(code[nameof(Code.Cwde)], mnemonic[nameof(Mnemonic.Cwde)]),
				(code[nameof(Code.Cdqe)], mnemonic[nameof(Mnemonic.Cdqe)]),
				(code[nameof(Code.Cwd)], mnemonic[nameof(Mnemonic.Cwd)]),
				(code[nameof(Code.Cdq)], mnemonic[nameof(Mnemonic.Cdq)]),
				(code[nameof(Code.Cqo)], mnemonic[nameof(Mnemonic.Cqo)]),
				(code[nameof(Code.Call_ptr1616)], mnemonic[nameof(Mnemonic.Call)]),
				(code[nameof(Code.Call_ptr1632)], mnemonic[nameof(Mnemonic.Call)]),
				(code[nameof(Code.Wait)], mnemonic[nameof(Mnemonic.Wait)]),
				(code[nameof(Code.Pushfw)], mnemonic[nameof(Mnemonic.Pushf)]),
				(code[nameof(Code.Pushfd)], mnemonic[nameof(Mnemonic.Pushf)]),
				(code[nameof(Code.Pushfq)], mnemonic[nameof(Mnemonic.Pushf)]),
				(code[nameof(Code.Popfw)], mnemonic[nameof(Mnemonic.Popf)]),
				(code[nameof(Code.Popfd)], mnemonic[nameof(Mnemonic.Popf)]),
				(code[nameof(Code.Popfq)], mnemonic[nameof(Mnemonic.Popf)]),
				(code[nameof(Code.Sahf)], mnemonic[nameof(Mnemonic.Sahf)]),
				(code[nameof(Code.Lahf)], mnemonic[nameof(Mnemonic.Lahf)]),
				(code[nameof(Code.Mov_AL_moffs8)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_AX_moffs16)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_EAX_moffs32)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_RAX_moffs64)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_moffs8_AL)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_moffs16_AX)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_moffs32_EAX)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_moffs64_RAX)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Movsb_m8_m8)], mnemonic[nameof(Mnemonic.Movsb)]),
				(code[nameof(Code.Movsw_m16_m16)], mnemonic[nameof(Mnemonic.Movsw)]),
				(code[nameof(Code.Movsd_m32_m32)], mnemonic[nameof(Mnemonic.Movsd)]),
				(code[nameof(Code.Movsq_m64_m64)], mnemonic[nameof(Mnemonic.Movsq)]),
				(code[nameof(Code.Cmpsb_m8_m8)], mnemonic[nameof(Mnemonic.Cmpsb)]),
				(code[nameof(Code.Cmpsw_m16_m16)], mnemonic[nameof(Mnemonic.Cmpsw)]),
				(code[nameof(Code.Cmpsd_m32_m32)], mnemonic[nameof(Mnemonic.Cmpsd)]),
				(code[nameof(Code.Cmpsq_m64_m64)], mnemonic[nameof(Mnemonic.Cmpsq)]),
				(code[nameof(Code.Test_AL_imm8)], mnemonic[nameof(Mnemonic.Test)]),
				(code[nameof(Code.Test_AX_imm16)], mnemonic[nameof(Mnemonic.Test)]),
				(code[nameof(Code.Test_EAX_imm32)], mnemonic[nameof(Mnemonic.Test)]),
				(code[nameof(Code.Test_RAX_imm32)], mnemonic[nameof(Mnemonic.Test)]),
				(code[nameof(Code.Stosb_m8_AL)], mnemonic[nameof(Mnemonic.Stosb)]),
				(code[nameof(Code.Stosw_m16_AX)], mnemonic[nameof(Mnemonic.Stosw)]),
				(code[nameof(Code.Stosd_m32_EAX)], mnemonic[nameof(Mnemonic.Stosd)]),
				(code[nameof(Code.Stosq_m64_RAX)], mnemonic[nameof(Mnemonic.Stosq)]),
				(code[nameof(Code.Lodsb_AL_m8)], mnemonic[nameof(Mnemonic.Lodsb)]),
				(code[nameof(Code.Lodsw_AX_m16)], mnemonic[nameof(Mnemonic.Lodsw)]),
				(code[nameof(Code.Lodsd_EAX_m32)], mnemonic[nameof(Mnemonic.Lodsd)]),
				(code[nameof(Code.Lodsq_RAX_m64)], mnemonic[nameof(Mnemonic.Lodsq)]),
				(code[nameof(Code.Scasb_AL_m8)], mnemonic[nameof(Mnemonic.Scasb)]),
				(code[nameof(Code.Scasw_AX_m16)], mnemonic[nameof(Mnemonic.Scasw)]),
				(code[nameof(Code.Scasd_EAX_m32)], mnemonic[nameof(Mnemonic.Scasd)]),
				(code[nameof(Code.Scasq_RAX_m64)], mnemonic[nameof(Mnemonic.Scasq)]),
				(code[nameof(Code.Mov_r8_imm8)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_r16_imm16)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_r32_imm32)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_r64_imm64)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Rol_rm8_imm8)], mnemonic[nameof(Mnemonic.Rol)]),
				(code[nameof(Code.Ror_rm8_imm8)], mnemonic[nameof(Mnemonic.Ror)]),
				(code[nameof(Code.Rcl_rm8_imm8)], mnemonic[nameof(Mnemonic.Rcl)]),
				(code[nameof(Code.Rcr_rm8_imm8)], mnemonic[nameof(Mnemonic.Rcr)]),
				(code[nameof(Code.Shl_rm8_imm8)], mnemonic[nameof(Mnemonic.Shl)]),
				(code[nameof(Code.Shr_rm8_imm8)], mnemonic[nameof(Mnemonic.Shr)]),
				(code[nameof(Code.Sal_rm8_imm8)], mnemonic[nameof(Mnemonic.Sal)]),
				(code[nameof(Code.Sar_rm8_imm8)], mnemonic[nameof(Mnemonic.Sar)]),
				(code[nameof(Code.Rol_rm16_imm8)], mnemonic[nameof(Mnemonic.Rol)]),
				(code[nameof(Code.Rol_rm32_imm8)], mnemonic[nameof(Mnemonic.Rol)]),
				(code[nameof(Code.Rol_rm64_imm8)], mnemonic[nameof(Mnemonic.Rol)]),
				(code[nameof(Code.Ror_rm16_imm8)], mnemonic[nameof(Mnemonic.Ror)]),
				(code[nameof(Code.Ror_rm32_imm8)], mnemonic[nameof(Mnemonic.Ror)]),
				(code[nameof(Code.Ror_rm64_imm8)], mnemonic[nameof(Mnemonic.Ror)]),
				(code[nameof(Code.Rcl_rm16_imm8)], mnemonic[nameof(Mnemonic.Rcl)]),
				(code[nameof(Code.Rcl_rm32_imm8)], mnemonic[nameof(Mnemonic.Rcl)]),
				(code[nameof(Code.Rcl_rm64_imm8)], mnemonic[nameof(Mnemonic.Rcl)]),
				(code[nameof(Code.Rcr_rm16_imm8)], mnemonic[nameof(Mnemonic.Rcr)]),
				(code[nameof(Code.Rcr_rm32_imm8)], mnemonic[nameof(Mnemonic.Rcr)]),
				(code[nameof(Code.Rcr_rm64_imm8)], mnemonic[nameof(Mnemonic.Rcr)]),
				(code[nameof(Code.Shl_rm16_imm8)], mnemonic[nameof(Mnemonic.Shl)]),
				(code[nameof(Code.Shl_rm32_imm8)], mnemonic[nameof(Mnemonic.Shl)]),
				(code[nameof(Code.Shl_rm64_imm8)], mnemonic[nameof(Mnemonic.Shl)]),
				(code[nameof(Code.Shr_rm16_imm8)], mnemonic[nameof(Mnemonic.Shr)]),
				(code[nameof(Code.Shr_rm32_imm8)], mnemonic[nameof(Mnemonic.Shr)]),
				(code[nameof(Code.Shr_rm64_imm8)], mnemonic[nameof(Mnemonic.Shr)]),
				(code[nameof(Code.Sal_rm16_imm8)], mnemonic[nameof(Mnemonic.Sal)]),
				(code[nameof(Code.Sal_rm32_imm8)], mnemonic[nameof(Mnemonic.Sal)]),
				(code[nameof(Code.Sal_rm64_imm8)], mnemonic[nameof(Mnemonic.Sal)]),
				(code[nameof(Code.Sar_rm16_imm8)], mnemonic[nameof(Mnemonic.Sar)]),
				(code[nameof(Code.Sar_rm32_imm8)], mnemonic[nameof(Mnemonic.Sar)]),
				(code[nameof(Code.Sar_rm64_imm8)], mnemonic[nameof(Mnemonic.Sar)]),
				(code[nameof(Code.Retnw_imm16)], mnemonic[nameof(Mnemonic.Ret)]),
				(code[nameof(Code.Retnd_imm16)], mnemonic[nameof(Mnemonic.Ret)]),
				(code[nameof(Code.Retnq_imm16)], mnemonic[nameof(Mnemonic.Ret)]),
				(code[nameof(Code.Retnw)], mnemonic[nameof(Mnemonic.Ret)]),
				(code[nameof(Code.Retnd)], mnemonic[nameof(Mnemonic.Ret)]),
				(code[nameof(Code.Retnq)], mnemonic[nameof(Mnemonic.Ret)]),
				(code[nameof(Code.Les_r16_m1616)], mnemonic[nameof(Mnemonic.Les)]),
				(code[nameof(Code.Les_r32_m1632)], mnemonic[nameof(Mnemonic.Les)]),
				(code[nameof(Code.Lds_r16_m1616)], mnemonic[nameof(Mnemonic.Lds)]),
				(code[nameof(Code.Lds_r32_m1632)], mnemonic[nameof(Mnemonic.Lds)]),
				(code[nameof(Code.Mov_rm8_imm8)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Xabort_imm8)], mnemonic[nameof(Mnemonic.Xabort)]),
				(code[nameof(Code.Mov_rm16_imm16)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_rm32_imm32)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_rm64_imm32)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Xbegin_rel16)], mnemonic[nameof(Mnemonic.Xbegin)]),
				(code[nameof(Code.Xbegin_rel32)], mnemonic[nameof(Mnemonic.Xbegin)]),
				(code[nameof(Code.Enterw_imm16_imm8)], mnemonic[nameof(Mnemonic.Enter)]),
				(code[nameof(Code.Enterd_imm16_imm8)], mnemonic[nameof(Mnemonic.Enter)]),
				(code[nameof(Code.Enterq_imm16_imm8)], mnemonic[nameof(Mnemonic.Enter)]),
				(code[nameof(Code.Leavew)], mnemonic[nameof(Mnemonic.Leave)]),
				(code[nameof(Code.Leaved)], mnemonic[nameof(Mnemonic.Leave)]),
				(code[nameof(Code.Leaveq)], mnemonic[nameof(Mnemonic.Leave)]),
				(code[nameof(Code.Retfw_imm16)], mnemonic[nameof(Mnemonic.Retf)]),
				(code[nameof(Code.Retfd_imm16)], mnemonic[nameof(Mnemonic.Retf)]),
				(code[nameof(Code.Retfq_imm16)], mnemonic[nameof(Mnemonic.Retf)]),
				(code[nameof(Code.Retfw)], mnemonic[nameof(Mnemonic.Retf)]),
				(code[nameof(Code.Retfd)], mnemonic[nameof(Mnemonic.Retf)]),
				(code[nameof(Code.Retfq)], mnemonic[nameof(Mnemonic.Retf)]),
				(code[nameof(Code.Int3)], mnemonic[nameof(Mnemonic.Int)]),
				(code[nameof(Code.Int_imm8)], mnemonic[nameof(Mnemonic.Int)]),
				(code[nameof(Code.Into)], mnemonic[nameof(Mnemonic.Into)]),
				(code[nameof(Code.Iretw)], mnemonic[nameof(Mnemonic.Iret)]),
				(code[nameof(Code.Iretd)], mnemonic[nameof(Mnemonic.Iret)]),
				(code[nameof(Code.Iretq)], mnemonic[nameof(Mnemonic.Iret)]),
				(code[nameof(Code.Rol_rm8_1)], mnemonic[nameof(Mnemonic.Rol)]),
				(code[nameof(Code.Ror_rm8_1)], mnemonic[nameof(Mnemonic.Ror)]),
				(code[nameof(Code.Rcl_rm8_1)], mnemonic[nameof(Mnemonic.Rcl)]),
				(code[nameof(Code.Rcr_rm8_1)], mnemonic[nameof(Mnemonic.Rcr)]),
				(code[nameof(Code.Shl_rm8_1)], mnemonic[nameof(Mnemonic.Shl)]),
				(code[nameof(Code.Shr_rm8_1)], mnemonic[nameof(Mnemonic.Shr)]),
				(code[nameof(Code.Sal_rm8_1)], mnemonic[nameof(Mnemonic.Sal)]),
				(code[nameof(Code.Sar_rm8_1)], mnemonic[nameof(Mnemonic.Sar)]),
				(code[nameof(Code.Rol_rm16_1)], mnemonic[nameof(Mnemonic.Rol)]),
				(code[nameof(Code.Rol_rm32_1)], mnemonic[nameof(Mnemonic.Rol)]),
				(code[nameof(Code.Rol_rm64_1)], mnemonic[nameof(Mnemonic.Rol)]),
				(code[nameof(Code.Ror_rm16_1)], mnemonic[nameof(Mnemonic.Ror)]),
				(code[nameof(Code.Ror_rm32_1)], mnemonic[nameof(Mnemonic.Ror)]),
				(code[nameof(Code.Ror_rm64_1)], mnemonic[nameof(Mnemonic.Ror)]),
				(code[nameof(Code.Rcl_rm16_1)], mnemonic[nameof(Mnemonic.Rcl)]),
				(code[nameof(Code.Rcl_rm32_1)], mnemonic[nameof(Mnemonic.Rcl)]),
				(code[nameof(Code.Rcl_rm64_1)], mnemonic[nameof(Mnemonic.Rcl)]),
				(code[nameof(Code.Rcr_rm16_1)], mnemonic[nameof(Mnemonic.Rcr)]),
				(code[nameof(Code.Rcr_rm32_1)], mnemonic[nameof(Mnemonic.Rcr)]),
				(code[nameof(Code.Rcr_rm64_1)], mnemonic[nameof(Mnemonic.Rcr)]),
				(code[nameof(Code.Shl_rm16_1)], mnemonic[nameof(Mnemonic.Shl)]),
				(code[nameof(Code.Shl_rm32_1)], mnemonic[nameof(Mnemonic.Shl)]),
				(code[nameof(Code.Shl_rm64_1)], mnemonic[nameof(Mnemonic.Shl)]),
				(code[nameof(Code.Shr_rm16_1)], mnemonic[nameof(Mnemonic.Shr)]),
				(code[nameof(Code.Shr_rm32_1)], mnemonic[nameof(Mnemonic.Shr)]),
				(code[nameof(Code.Shr_rm64_1)], mnemonic[nameof(Mnemonic.Shr)]),
				(code[nameof(Code.Sal_rm16_1)], mnemonic[nameof(Mnemonic.Sal)]),
				(code[nameof(Code.Sal_rm32_1)], mnemonic[nameof(Mnemonic.Sal)]),
				(code[nameof(Code.Sal_rm64_1)], mnemonic[nameof(Mnemonic.Sal)]),
				(code[nameof(Code.Sar_rm16_1)], mnemonic[nameof(Mnemonic.Sar)]),
				(code[nameof(Code.Sar_rm32_1)], mnemonic[nameof(Mnemonic.Sar)]),
				(code[nameof(Code.Sar_rm64_1)], mnemonic[nameof(Mnemonic.Sar)]),
				(code[nameof(Code.Rol_rm8_CL)], mnemonic[nameof(Mnemonic.Rol)]),
				(code[nameof(Code.Ror_rm8_CL)], mnemonic[nameof(Mnemonic.Ror)]),
				(code[nameof(Code.Rcl_rm8_CL)], mnemonic[nameof(Mnemonic.Rcl)]),
				(code[nameof(Code.Rcr_rm8_CL)], mnemonic[nameof(Mnemonic.Rcr)]),
				(code[nameof(Code.Shl_rm8_CL)], mnemonic[nameof(Mnemonic.Shl)]),
				(code[nameof(Code.Shr_rm8_CL)], mnemonic[nameof(Mnemonic.Shr)]),
				(code[nameof(Code.Sal_rm8_CL)], mnemonic[nameof(Mnemonic.Sal)]),
				(code[nameof(Code.Sar_rm8_CL)], mnemonic[nameof(Mnemonic.Sar)]),
				(code[nameof(Code.Rol_rm16_CL)], mnemonic[nameof(Mnemonic.Rol)]),
				(code[nameof(Code.Rol_rm32_CL)], mnemonic[nameof(Mnemonic.Rol)]),
				(code[nameof(Code.Rol_rm64_CL)], mnemonic[nameof(Mnemonic.Rol)]),
				(code[nameof(Code.Ror_rm16_CL)], mnemonic[nameof(Mnemonic.Ror)]),
				(code[nameof(Code.Ror_rm32_CL)], mnemonic[nameof(Mnemonic.Ror)]),
				(code[nameof(Code.Ror_rm64_CL)], mnemonic[nameof(Mnemonic.Ror)]),
				(code[nameof(Code.Rcl_rm16_CL)], mnemonic[nameof(Mnemonic.Rcl)]),
				(code[nameof(Code.Rcl_rm32_CL)], mnemonic[nameof(Mnemonic.Rcl)]),
				(code[nameof(Code.Rcl_rm64_CL)], mnemonic[nameof(Mnemonic.Rcl)]),
				(code[nameof(Code.Rcr_rm16_CL)], mnemonic[nameof(Mnemonic.Rcr)]),
				(code[nameof(Code.Rcr_rm32_CL)], mnemonic[nameof(Mnemonic.Rcr)]),
				(code[nameof(Code.Rcr_rm64_CL)], mnemonic[nameof(Mnemonic.Rcr)]),
				(code[nameof(Code.Shl_rm16_CL)], mnemonic[nameof(Mnemonic.Shl)]),
				(code[nameof(Code.Shl_rm32_CL)], mnemonic[nameof(Mnemonic.Shl)]),
				(code[nameof(Code.Shl_rm64_CL)], mnemonic[nameof(Mnemonic.Shl)]),
				(code[nameof(Code.Shr_rm16_CL)], mnemonic[nameof(Mnemonic.Shr)]),
				(code[nameof(Code.Shr_rm32_CL)], mnemonic[nameof(Mnemonic.Shr)]),
				(code[nameof(Code.Shr_rm64_CL)], mnemonic[nameof(Mnemonic.Shr)]),
				(code[nameof(Code.Sal_rm16_CL)], mnemonic[nameof(Mnemonic.Sal)]),
				(code[nameof(Code.Sal_rm32_CL)], mnemonic[nameof(Mnemonic.Sal)]),
				(code[nameof(Code.Sal_rm64_CL)], mnemonic[nameof(Mnemonic.Sal)]),
				(code[nameof(Code.Sar_rm16_CL)], mnemonic[nameof(Mnemonic.Sar)]),
				(code[nameof(Code.Sar_rm32_CL)], mnemonic[nameof(Mnemonic.Sar)]),
				(code[nameof(Code.Sar_rm64_CL)], mnemonic[nameof(Mnemonic.Sar)]),
				(code[nameof(Code.Aam_imm8)], mnemonic[nameof(Mnemonic.Aam)]),
				(code[nameof(Code.Aad_imm8)], mnemonic[nameof(Mnemonic.Aad)]),
				(code[nameof(Code.Salc)], mnemonic[nameof(Mnemonic.Salc)]),
				(code[nameof(Code.Xlat_m8)], mnemonic[nameof(Mnemonic.Xlatb)]),
				(code[nameof(Code.Fadd_m32fp)], mnemonic[nameof(Mnemonic.Fadd)]),
				(code[nameof(Code.Fmul_m32fp)], mnemonic[nameof(Mnemonic.Fmul)]),
				(code[nameof(Code.Fcom_m32fp)], mnemonic[nameof(Mnemonic.Fcom)]),
				(code[nameof(Code.Fcomp_m32fp)], mnemonic[nameof(Mnemonic.Fcomp)]),
				(code[nameof(Code.Fsub_m32fp)], mnemonic[nameof(Mnemonic.Fsub)]),
				(code[nameof(Code.Fsubr_m32fp)], mnemonic[nameof(Mnemonic.Fsubr)]),
				(code[nameof(Code.Fdiv_m32fp)], mnemonic[nameof(Mnemonic.Fdiv)]),
				(code[nameof(Code.Fdivr_m32fp)], mnemonic[nameof(Mnemonic.Fdivr)]),
				(code[nameof(Code.Fadd_st0_sti)], mnemonic[nameof(Mnemonic.Fadd)]),
				(code[nameof(Code.Fmul_st0_sti)], mnemonic[nameof(Mnemonic.Fmul)]),
				(code[nameof(Code.Fcom_st0_sti)], mnemonic[nameof(Mnemonic.Fcom)]),
				(code[nameof(Code.Fcomp_st0_sti)], mnemonic[nameof(Mnemonic.Fcomp)]),
				(code[nameof(Code.Fsub_st0_sti)], mnemonic[nameof(Mnemonic.Fsub)]),
				(code[nameof(Code.Fsubr_st0_sti)], mnemonic[nameof(Mnemonic.Fsubr)]),
				(code[nameof(Code.Fdiv_st0_sti)], mnemonic[nameof(Mnemonic.Fdiv)]),
				(code[nameof(Code.Fdivr_st0_sti)], mnemonic[nameof(Mnemonic.Fdivr)]),
				(code[nameof(Code.Fld_m32fp)], mnemonic[nameof(Mnemonic.Fld)]),
				(code[nameof(Code.Fst_m32fp)], mnemonic[nameof(Mnemonic.Fst)]),
				(code[nameof(Code.Fstp_m32fp)], mnemonic[nameof(Mnemonic.Fstp)]),
				(code[nameof(Code.Fldenv_m14byte)], mnemonic[nameof(Mnemonic.Fldenv)]),
				(code[nameof(Code.Fldenv_m28byte)], mnemonic[nameof(Mnemonic.Fldenv)]),
				(code[nameof(Code.Fldcw_m2byte)], mnemonic[nameof(Mnemonic.Fldcw)]),
				(code[nameof(Code.Fnstenv_m14byte)], mnemonic[nameof(Mnemonic.Fnstenv)]),
				(code[nameof(Code.Fstenv_m14byte)], mnemonic[nameof(Mnemonic.Fstenv)]),
				(code[nameof(Code.Fnstenv_m28byte)], mnemonic[nameof(Mnemonic.Fnstenv)]),
				(code[nameof(Code.Fstenv_m28byte)], mnemonic[nameof(Mnemonic.Fstenv)]),
				(code[nameof(Code.Fnstcw_m2byte)], mnemonic[nameof(Mnemonic.Fnstcw)]),
				(code[nameof(Code.Fstcw_m2byte)], mnemonic[nameof(Mnemonic.Fstcw)]),
				(code[nameof(Code.Fld_st0_sti)], mnemonic[nameof(Mnemonic.Fld)]),
				(code[nameof(Code.Fxch_st0_sti)], mnemonic[nameof(Mnemonic.Fxch)]),
				(code[nameof(Code.Fnop)], mnemonic[nameof(Mnemonic.Fnop)]),
				(code[nameof(Code.Fstpnce_sti)], mnemonic[nameof(Mnemonic.Fstpnce)]),
				(code[nameof(Code.Fchs)], mnemonic[nameof(Mnemonic.Fchs)]),
				(code[nameof(Code.Fabs)], mnemonic[nameof(Mnemonic.Fabs)]),
				(code[nameof(Code.Ftst)], mnemonic[nameof(Mnemonic.Ftst)]),
				(code[nameof(Code.Fxam)], mnemonic[nameof(Mnemonic.Fxam)]),
				(code[nameof(Code.Fld1)], mnemonic[nameof(Mnemonic.Fld1)]),
				(code[nameof(Code.Fldl2t)], mnemonic[nameof(Mnemonic.Fldl2t)]),
				(code[nameof(Code.Fldl2e)], mnemonic[nameof(Mnemonic.Fldl2e)]),
				(code[nameof(Code.Fldpi)], mnemonic[nameof(Mnemonic.Fldpi)]),
				(code[nameof(Code.Fldlg2)], mnemonic[nameof(Mnemonic.Fldlg2)]),
				(code[nameof(Code.Fldln2)], mnemonic[nameof(Mnemonic.Fldln2)]),
				(code[nameof(Code.Fldz)], mnemonic[nameof(Mnemonic.Fldz)]),
				(code[nameof(Code.F2xm1)], mnemonic[nameof(Mnemonic.F2xm1)]),
				(code[nameof(Code.Fyl2x)], mnemonic[nameof(Mnemonic.Fyl2x)]),
				(code[nameof(Code.Fptan)], mnemonic[nameof(Mnemonic.Fptan)]),
				(code[nameof(Code.Fpatan)], mnemonic[nameof(Mnemonic.Fpatan)]),
				(code[nameof(Code.Fxtract)], mnemonic[nameof(Mnemonic.Fxtract)]),
				(code[nameof(Code.Fprem1)], mnemonic[nameof(Mnemonic.Fprem1)]),
				(code[nameof(Code.Fdecstp)], mnemonic[nameof(Mnemonic.Fdecstp)]),
				(code[nameof(Code.Fincstp)], mnemonic[nameof(Mnemonic.Fincstp)]),
				(code[nameof(Code.Fprem)], mnemonic[nameof(Mnemonic.Fprem)]),
				(code[nameof(Code.Fyl2xp1)], mnemonic[nameof(Mnemonic.Fyl2xp1)]),
				(code[nameof(Code.Fsqrt)], mnemonic[nameof(Mnemonic.Fsqrt)]),
				(code[nameof(Code.Fsincos)], mnemonic[nameof(Mnemonic.Fsincos)]),
				(code[nameof(Code.Frndint)], mnemonic[nameof(Mnemonic.Frndint)]),
				(code[nameof(Code.Fscale)], mnemonic[nameof(Mnemonic.Fscale)]),
				(code[nameof(Code.Fsin)], mnemonic[nameof(Mnemonic.Fsin)]),
				(code[nameof(Code.Fcos)], mnemonic[nameof(Mnemonic.Fcos)]),
				(code[nameof(Code.Fiadd_m32int)], mnemonic[nameof(Mnemonic.Fiadd)]),
				(code[nameof(Code.Fimul_m32int)], mnemonic[nameof(Mnemonic.Fimul)]),
				(code[nameof(Code.Ficom_m32int)], mnemonic[nameof(Mnemonic.Ficom)]),
				(code[nameof(Code.Ficomp_m32int)], mnemonic[nameof(Mnemonic.Ficomp)]),
				(code[nameof(Code.Fisub_m32int)], mnemonic[nameof(Mnemonic.Fisub)]),
				(code[nameof(Code.Fisubr_m32int)], mnemonic[nameof(Mnemonic.Fisubr)]),
				(code[nameof(Code.Fidiv_m32int)], mnemonic[nameof(Mnemonic.Fidiv)]),
				(code[nameof(Code.Fidivr_m32int)], mnemonic[nameof(Mnemonic.Fidivr)]),
				(code[nameof(Code.Fcmovb_st0_sti)], mnemonic[nameof(Mnemonic.Fcmovb)]),
				(code[nameof(Code.Fcmove_st0_sti)], mnemonic[nameof(Mnemonic.Fcmove)]),
				(code[nameof(Code.Fcmovbe_st0_sti)], mnemonic[nameof(Mnemonic.Fcmovbe)]),
				(code[nameof(Code.Fcmovu_st0_sti)], mnemonic[nameof(Mnemonic.Fcmovu)]),
				(code[nameof(Code.Fucompp)], mnemonic[nameof(Mnemonic.Fucompp)]),
				(code[nameof(Code.Fild_m32int)], mnemonic[nameof(Mnemonic.Fild)]),
				(code[nameof(Code.Fisttp_m32int)], mnemonic[nameof(Mnemonic.Fisttp)]),
				(code[nameof(Code.Fist_m32int)], mnemonic[nameof(Mnemonic.Fist)]),
				(code[nameof(Code.Fistp_m32int)], mnemonic[nameof(Mnemonic.Fistp)]),
				(code[nameof(Code.Fld_m80fp)], mnemonic[nameof(Mnemonic.Fld)]),
				(code[nameof(Code.Fstp_m80fp)], mnemonic[nameof(Mnemonic.Fstp)]),
				(code[nameof(Code.Fcmovnb_st0_sti)], mnemonic[nameof(Mnemonic.Fcmovnb)]),
				(code[nameof(Code.Fcmovne_st0_sti)], mnemonic[nameof(Mnemonic.Fcmovne)]),
				(code[nameof(Code.Fcmovnbe_st0_sti)], mnemonic[nameof(Mnemonic.Fcmovnbe)]),
				(code[nameof(Code.Fcmovnu_st0_sti)], mnemonic[nameof(Mnemonic.Fcmovnu)]),
				(code[nameof(Code.Fneni)], mnemonic[nameof(Mnemonic.Fneni)]),
				(code[nameof(Code.Feni)], mnemonic[nameof(Mnemonic.Feni)]),
				(code[nameof(Code.Fndisi)], mnemonic[nameof(Mnemonic.Fndisi)]),
				(code[nameof(Code.Fdisi)], mnemonic[nameof(Mnemonic.Fdisi)]),
				(code[nameof(Code.Fnclex)], mnemonic[nameof(Mnemonic.Fnclex)]),
				(code[nameof(Code.Fclex)], mnemonic[nameof(Mnemonic.Fclex)]),
				(code[nameof(Code.Fninit)], mnemonic[nameof(Mnemonic.Fninit)]),
				(code[nameof(Code.Finit)], mnemonic[nameof(Mnemonic.Finit)]),
				(code[nameof(Code.Fnsetpm)], mnemonic[nameof(Mnemonic.Fnsetpm)]),
				(code[nameof(Code.Fsetpm)], mnemonic[nameof(Mnemonic.Fsetpm)]),
				(code[nameof(Code.Frstpm)], mnemonic[nameof(Mnemonic.Frstpm)]),
				(code[nameof(Code.Fucomi_st0_sti)], mnemonic[nameof(Mnemonic.Fucomi)]),
				(code[nameof(Code.Fcomi_st0_sti)], mnemonic[nameof(Mnemonic.Fcomi)]),
				(code[nameof(Code.Fadd_m64fp)], mnemonic[nameof(Mnemonic.Fadd)]),
				(code[nameof(Code.Fmul_m64fp)], mnemonic[nameof(Mnemonic.Fmul)]),
				(code[nameof(Code.Fcom_m64fp)], mnemonic[nameof(Mnemonic.Fcom)]),
				(code[nameof(Code.Fcomp_m64fp)], mnemonic[nameof(Mnemonic.Fcomp)]),
				(code[nameof(Code.Fsub_m64fp)], mnemonic[nameof(Mnemonic.Fsub)]),
				(code[nameof(Code.Fsubr_m64fp)], mnemonic[nameof(Mnemonic.Fsubr)]),
				(code[nameof(Code.Fdiv_m64fp)], mnemonic[nameof(Mnemonic.Fdiv)]),
				(code[nameof(Code.Fdivr_m64fp)], mnemonic[nameof(Mnemonic.Fdivr)]),
				(code[nameof(Code.Fadd_sti_st0)], mnemonic[nameof(Mnemonic.Fadd)]),
				(code[nameof(Code.Fmul_sti_st0)], mnemonic[nameof(Mnemonic.Fmul)]),
				(code[nameof(Code.Fcom_st0_sti_DCD0)], mnemonic[nameof(Mnemonic.Fcom)]),
				(code[nameof(Code.Fcomp_st0_sti_DCD8)], mnemonic[nameof(Mnemonic.Fcomp)]),
				(code[nameof(Code.Fsubr_sti_st0)], mnemonic[nameof(Mnemonic.Fsubr)]),
				(code[nameof(Code.Fsub_sti_st0)], mnemonic[nameof(Mnemonic.Fsub)]),
				(code[nameof(Code.Fdivr_sti_st0)], mnemonic[nameof(Mnemonic.Fdivr)]),
				(code[nameof(Code.Fdiv_sti_st0)], mnemonic[nameof(Mnemonic.Fdiv)]),
				(code[nameof(Code.Fld_m64fp)], mnemonic[nameof(Mnemonic.Fld)]),
				(code[nameof(Code.Fisttp_m64int)], mnemonic[nameof(Mnemonic.Fisttp)]),
				(code[nameof(Code.Fst_m64fp)], mnemonic[nameof(Mnemonic.Fst)]),
				(code[nameof(Code.Fstp_m64fp)], mnemonic[nameof(Mnemonic.Fstp)]),
				(code[nameof(Code.Frstor_m94byte)], mnemonic[nameof(Mnemonic.Frstor)]),
				(code[nameof(Code.Frstor_m108byte)], mnemonic[nameof(Mnemonic.Frstor)]),
				(code[nameof(Code.Fnsave_m94byte)], mnemonic[nameof(Mnemonic.Fnsave)]),
				(code[nameof(Code.Fsave_m94byte)], mnemonic[nameof(Mnemonic.Fsave)]),
				(code[nameof(Code.Fnsave_m108byte)], mnemonic[nameof(Mnemonic.Fnsave)]),
				(code[nameof(Code.Fsave_m108byte)], mnemonic[nameof(Mnemonic.Fsave)]),
				(code[nameof(Code.Fnstsw_m2byte)], mnemonic[nameof(Mnemonic.Fnstsw)]),
				(code[nameof(Code.Fstsw_m2byte)], mnemonic[nameof(Mnemonic.Fstsw)]),
				(code[nameof(Code.Ffree_sti)], mnemonic[nameof(Mnemonic.Ffree)]),
				(code[nameof(Code.Fxch_st0_sti_DDC8)], mnemonic[nameof(Mnemonic.Fxch)]),
				(code[nameof(Code.Fst_sti)], mnemonic[nameof(Mnemonic.Fst)]),
				(code[nameof(Code.Fstp_sti)], mnemonic[nameof(Mnemonic.Fstp)]),
				(code[nameof(Code.Fucom_st0_sti)], mnemonic[nameof(Mnemonic.Fucom)]),
				(code[nameof(Code.Fucomp_st0_sti)], mnemonic[nameof(Mnemonic.Fucomp)]),
				(code[nameof(Code.Fiadd_m16int)], mnemonic[nameof(Mnemonic.Fiadd)]),
				(code[nameof(Code.Fimul_m16int)], mnemonic[nameof(Mnemonic.Fimul)]),
				(code[nameof(Code.Ficom_m16int)], mnemonic[nameof(Mnemonic.Ficom)]),
				(code[nameof(Code.Ficomp_m16int)], mnemonic[nameof(Mnemonic.Ficomp)]),
				(code[nameof(Code.Fisub_m16int)], mnemonic[nameof(Mnemonic.Fisub)]),
				(code[nameof(Code.Fisubr_m16int)], mnemonic[nameof(Mnemonic.Fisubr)]),
				(code[nameof(Code.Fidiv_m16int)], mnemonic[nameof(Mnemonic.Fidiv)]),
				(code[nameof(Code.Fidivr_m16int)], mnemonic[nameof(Mnemonic.Fidivr)]),
				(code[nameof(Code.Faddp_sti_st0)], mnemonic[nameof(Mnemonic.Faddp)]),
				(code[nameof(Code.Fmulp_sti_st0)], mnemonic[nameof(Mnemonic.Fmulp)]),
				(code[nameof(Code.Fcomp_st0_sti_DED0)], mnemonic[nameof(Mnemonic.Fcomp)]),
				(code[nameof(Code.Fcompp)], mnemonic[nameof(Mnemonic.Fcompp)]),
				(code[nameof(Code.Fsubrp_sti_st0)], mnemonic[nameof(Mnemonic.Fsubrp)]),
				(code[nameof(Code.Fsubp_sti_st0)], mnemonic[nameof(Mnemonic.Fsubp)]),
				(code[nameof(Code.Fdivrp_sti_st0)], mnemonic[nameof(Mnemonic.Fdivrp)]),
				(code[nameof(Code.Fdivp_sti_st0)], mnemonic[nameof(Mnemonic.Fdivp)]),
				(code[nameof(Code.Fild_m16int)], mnemonic[nameof(Mnemonic.Fild)]),
				(code[nameof(Code.Fisttp_m16int)], mnemonic[nameof(Mnemonic.Fisttp)]),
				(code[nameof(Code.Fist_m16int)], mnemonic[nameof(Mnemonic.Fist)]),
				(code[nameof(Code.Fistp_m16int)], mnemonic[nameof(Mnemonic.Fistp)]),
				(code[nameof(Code.Fbld_m80bcd)], mnemonic[nameof(Mnemonic.Fbld)]),
				(code[nameof(Code.Fild_m64int)], mnemonic[nameof(Mnemonic.Fild)]),
				(code[nameof(Code.Fbstp_m80bcd)], mnemonic[nameof(Mnemonic.Fbstp)]),
				(code[nameof(Code.Fistp_m64int)], mnemonic[nameof(Mnemonic.Fistp)]),
				(code[nameof(Code.Ffreep_sti)], mnemonic[nameof(Mnemonic.Ffreep)]),
				(code[nameof(Code.Fxch_st0_sti_DFC8)], mnemonic[nameof(Mnemonic.Fxch)]),
				(code[nameof(Code.Fstp_sti_DFD0)], mnemonic[nameof(Mnemonic.Fstp)]),
				(code[nameof(Code.Fstp_sti_DFD8)], mnemonic[nameof(Mnemonic.Fstp)]),
				(code[nameof(Code.Fnstsw_AX)], mnemonic[nameof(Mnemonic.Fnstsw)]),
				(code[nameof(Code.Fstsw_AX)], mnemonic[nameof(Mnemonic.Fstsw)]),
				(code[nameof(Code.Fstdw_AX)], mnemonic[nameof(Mnemonic.Fstdw)]),
				(code[nameof(Code.Fstsg_AX)], mnemonic[nameof(Mnemonic.Fstsg)]),
				(code[nameof(Code.Fucomip_st0_sti)], mnemonic[nameof(Mnemonic.Fucomip)]),
				(code[nameof(Code.Fcomip_st0_sti)], mnemonic[nameof(Mnemonic.Fcomip)]),
				(code[nameof(Code.Loopne_rel8_16_CX)], mnemonic[nameof(Mnemonic.Loopne)]),
				(code[nameof(Code.Loopne_rel8_32_CX)], mnemonic[nameof(Mnemonic.Loopne)]),
				(code[nameof(Code.Loopne_rel8_16_ECX)], mnemonic[nameof(Mnemonic.Loopne)]),
				(code[nameof(Code.Loopne_rel8_32_ECX)], mnemonic[nameof(Mnemonic.Loopne)]),
				(code[nameof(Code.Loopne_rel8_64_ECX)], mnemonic[nameof(Mnemonic.Loopne)]),
				(code[nameof(Code.Loopne_rel8_16_RCX)], mnemonic[nameof(Mnemonic.Loopne)]),
				(code[nameof(Code.Loopne_rel8_64_RCX)], mnemonic[nameof(Mnemonic.Loopne)]),
				(code[nameof(Code.Loope_rel8_16_CX)], mnemonic[nameof(Mnemonic.Loope)]),
				(code[nameof(Code.Loope_rel8_32_CX)], mnemonic[nameof(Mnemonic.Loope)]),
				(code[nameof(Code.Loope_rel8_16_ECX)], mnemonic[nameof(Mnemonic.Loope)]),
				(code[nameof(Code.Loope_rel8_32_ECX)], mnemonic[nameof(Mnemonic.Loope)]),
				(code[nameof(Code.Loope_rel8_64_ECX)], mnemonic[nameof(Mnemonic.Loope)]),
				(code[nameof(Code.Loope_rel8_16_RCX)], mnemonic[nameof(Mnemonic.Loope)]),
				(code[nameof(Code.Loope_rel8_64_RCX)], mnemonic[nameof(Mnemonic.Loope)]),
				(code[nameof(Code.Loop_rel8_16_CX)], mnemonic[nameof(Mnemonic.Loop)]),
				(code[nameof(Code.Loop_rel8_32_CX)], mnemonic[nameof(Mnemonic.Loop)]),
				(code[nameof(Code.Loop_rel8_16_ECX)], mnemonic[nameof(Mnemonic.Loop)]),
				(code[nameof(Code.Loop_rel8_32_ECX)], mnemonic[nameof(Mnemonic.Loop)]),
				(code[nameof(Code.Loop_rel8_64_ECX)], mnemonic[nameof(Mnemonic.Loop)]),
				(code[nameof(Code.Loop_rel8_16_RCX)], mnemonic[nameof(Mnemonic.Loop)]),
				(code[nameof(Code.Loop_rel8_64_RCX)], mnemonic[nameof(Mnemonic.Loop)]),
				(code[nameof(Code.Jcxz_rel8_16)], mnemonic[nameof(Mnemonic.Jcxz)]),
				(code[nameof(Code.Jcxz_rel8_32)], mnemonic[nameof(Mnemonic.Jcxz)]),
				(code[nameof(Code.Jecxz_rel8_16)], mnemonic[nameof(Mnemonic.Jecxz)]),
				(code[nameof(Code.Jecxz_rel8_32)], mnemonic[nameof(Mnemonic.Jecxz)]),
				(code[nameof(Code.Jecxz_rel8_64)], mnemonic[nameof(Mnemonic.Jecxz)]),
				(code[nameof(Code.Jrcxz_rel8_16)], mnemonic[nameof(Mnemonic.Jrcxz)]),
				(code[nameof(Code.Jrcxz_rel8_64)], mnemonic[nameof(Mnemonic.Jrcxz)]),
				(code[nameof(Code.In_AL_imm8)], mnemonic[nameof(Mnemonic.In)]),
				(code[nameof(Code.In_AX_imm8)], mnemonic[nameof(Mnemonic.In)]),
				(code[nameof(Code.In_EAX_imm8)], mnemonic[nameof(Mnemonic.In)]),
				(code[nameof(Code.Out_imm8_AL)], mnemonic[nameof(Mnemonic.Out)]),
				(code[nameof(Code.Out_imm8_AX)], mnemonic[nameof(Mnemonic.Out)]),
				(code[nameof(Code.Out_imm8_EAX)], mnemonic[nameof(Mnemonic.Out)]),
				(code[nameof(Code.Call_rel16)], mnemonic[nameof(Mnemonic.Call)]),
				(code[nameof(Code.Call_rel32_32)], mnemonic[nameof(Mnemonic.Call)]),
				(code[nameof(Code.Call_rel32_64)], mnemonic[nameof(Mnemonic.Call)]),
				(code[nameof(Code.Jmp_rel16)], mnemonic[nameof(Mnemonic.Jmp)]),
				(code[nameof(Code.Jmp_rel32_32)], mnemonic[nameof(Mnemonic.Jmp)]),
				(code[nameof(Code.Jmp_rel32_64)], mnemonic[nameof(Mnemonic.Jmp)]),
				(code[nameof(Code.Jmp_ptr1616)], mnemonic[nameof(Mnemonic.Jmp)]),
				(code[nameof(Code.Jmp_ptr1632)], mnemonic[nameof(Mnemonic.Jmp)]),
				(code[nameof(Code.Jmp_rel8_16)], mnemonic[nameof(Mnemonic.Jmp)]),
				(code[nameof(Code.Jmp_rel8_32)], mnemonic[nameof(Mnemonic.Jmp)]),
				(code[nameof(Code.Jmp_rel8_64)], mnemonic[nameof(Mnemonic.Jmp)]),
				(code[nameof(Code.In_AL_DX)], mnemonic[nameof(Mnemonic.In)]),
				(code[nameof(Code.In_AX_DX)], mnemonic[nameof(Mnemonic.In)]),
				(code[nameof(Code.In_EAX_DX)], mnemonic[nameof(Mnemonic.In)]),
				(code[nameof(Code.Out_DX_AL)], mnemonic[nameof(Mnemonic.Out)]),
				(code[nameof(Code.Out_DX_AX)], mnemonic[nameof(Mnemonic.Out)]),
				(code[nameof(Code.Out_DX_EAX)], mnemonic[nameof(Mnemonic.Out)]),
				(code[nameof(Code.Int1)], mnemonic[nameof(Mnemonic.Int1)]),
				(code[nameof(Code.Hlt)], mnemonic[nameof(Mnemonic.Hlt)]),
				(code[nameof(Code.Cmc)], mnemonic[nameof(Mnemonic.Cmc)]),
				(code[nameof(Code.Test_rm8_imm8)], mnemonic[nameof(Mnemonic.Test)]),
				(code[nameof(Code.Test_rm8_imm8_F6r1)], mnemonic[nameof(Mnemonic.Test)]),
				(code[nameof(Code.Not_rm8)], mnemonic[nameof(Mnemonic.Not)]),
				(code[nameof(Code.Neg_rm8)], mnemonic[nameof(Mnemonic.Neg)]),
				(code[nameof(Code.Mul_rm8)], mnemonic[nameof(Mnemonic.Mul)]),
				(code[nameof(Code.Imul_rm8)], mnemonic[nameof(Mnemonic.Imul)]),
				(code[nameof(Code.Div_rm8)], mnemonic[nameof(Mnemonic.Div)]),
				(code[nameof(Code.Idiv_rm8)], mnemonic[nameof(Mnemonic.Idiv)]),
				(code[nameof(Code.Test_rm16_imm16)], mnemonic[nameof(Mnemonic.Test)]),
				(code[nameof(Code.Test_rm32_imm32)], mnemonic[nameof(Mnemonic.Test)]),
				(code[nameof(Code.Test_rm64_imm32)], mnemonic[nameof(Mnemonic.Test)]),
				(code[nameof(Code.Test_rm16_imm16_F7r1)], mnemonic[nameof(Mnemonic.Test)]),
				(code[nameof(Code.Test_rm32_imm32_F7r1)], mnemonic[nameof(Mnemonic.Test)]),
				(code[nameof(Code.Test_rm64_imm32_F7r1)], mnemonic[nameof(Mnemonic.Test)]),
				(code[nameof(Code.Not_rm16)], mnemonic[nameof(Mnemonic.Not)]),
				(code[nameof(Code.Not_rm32)], mnemonic[nameof(Mnemonic.Not)]),
				(code[nameof(Code.Not_rm64)], mnemonic[nameof(Mnemonic.Not)]),
				(code[nameof(Code.Neg_rm16)], mnemonic[nameof(Mnemonic.Neg)]),
				(code[nameof(Code.Neg_rm32)], mnemonic[nameof(Mnemonic.Neg)]),
				(code[nameof(Code.Neg_rm64)], mnemonic[nameof(Mnemonic.Neg)]),
				(code[nameof(Code.Mul_rm16)], mnemonic[nameof(Mnemonic.Mul)]),
				(code[nameof(Code.Mul_rm32)], mnemonic[nameof(Mnemonic.Mul)]),
				(code[nameof(Code.Mul_rm64)], mnemonic[nameof(Mnemonic.Mul)]),
				(code[nameof(Code.Imul_rm16)], mnemonic[nameof(Mnemonic.Imul)]),
				(code[nameof(Code.Imul_rm32)], mnemonic[nameof(Mnemonic.Imul)]),
				(code[nameof(Code.Imul_rm64)], mnemonic[nameof(Mnemonic.Imul)]),
				(code[nameof(Code.Div_rm16)], mnemonic[nameof(Mnemonic.Div)]),
				(code[nameof(Code.Div_rm32)], mnemonic[nameof(Mnemonic.Div)]),
				(code[nameof(Code.Div_rm64)], mnemonic[nameof(Mnemonic.Div)]),
				(code[nameof(Code.Idiv_rm16)], mnemonic[nameof(Mnemonic.Idiv)]),
				(code[nameof(Code.Idiv_rm32)], mnemonic[nameof(Mnemonic.Idiv)]),
				(code[nameof(Code.Idiv_rm64)], mnemonic[nameof(Mnemonic.Idiv)]),
				(code[nameof(Code.Clc)], mnemonic[nameof(Mnemonic.Clc)]),
				(code[nameof(Code.Stc)], mnemonic[nameof(Mnemonic.Stc)]),
				(code[nameof(Code.Cli)], mnemonic[nameof(Mnemonic.Cli)]),
				(code[nameof(Code.Sti)], mnemonic[nameof(Mnemonic.Sti)]),
				(code[nameof(Code.Cld)], mnemonic[nameof(Mnemonic.Cld)]),
				(code[nameof(Code.Std)], mnemonic[nameof(Mnemonic.Std)]),
				(code[nameof(Code.Inc_rm8)], mnemonic[nameof(Mnemonic.Inc)]),
				(code[nameof(Code.Dec_rm8)], mnemonic[nameof(Mnemonic.Dec)]),
				(code[nameof(Code.Inc_rm16)], mnemonic[nameof(Mnemonic.Inc)]),
				(code[nameof(Code.Inc_rm32)], mnemonic[nameof(Mnemonic.Inc)]),
				(code[nameof(Code.Inc_rm64)], mnemonic[nameof(Mnemonic.Inc)]),
				(code[nameof(Code.Dec_rm16)], mnemonic[nameof(Mnemonic.Dec)]),
				(code[nameof(Code.Dec_rm32)], mnemonic[nameof(Mnemonic.Dec)]),
				(code[nameof(Code.Dec_rm64)], mnemonic[nameof(Mnemonic.Dec)]),
				(code[nameof(Code.Call_rm16)], mnemonic[nameof(Mnemonic.Call)]),
				(code[nameof(Code.Call_rm32)], mnemonic[nameof(Mnemonic.Call)]),
				(code[nameof(Code.Call_rm64)], mnemonic[nameof(Mnemonic.Call)]),
				(code[nameof(Code.Call_m1616)], mnemonic[nameof(Mnemonic.Call)]),
				(code[nameof(Code.Call_m1632)], mnemonic[nameof(Mnemonic.Call)]),
				(code[nameof(Code.Call_m1664)], mnemonic[nameof(Mnemonic.Call)]),
				(code[nameof(Code.Jmp_rm16)], mnemonic[nameof(Mnemonic.Jmp)]),
				(code[nameof(Code.Jmp_rm32)], mnemonic[nameof(Mnemonic.Jmp)]),
				(code[nameof(Code.Jmp_rm64)], mnemonic[nameof(Mnemonic.Jmp)]),
				(code[nameof(Code.Jmp_m1616)], mnemonic[nameof(Mnemonic.Jmp)]),
				(code[nameof(Code.Jmp_m1632)], mnemonic[nameof(Mnemonic.Jmp)]),
				(code[nameof(Code.Jmp_m1664)], mnemonic[nameof(Mnemonic.Jmp)]),
				(code[nameof(Code.Push_rm16)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Push_rm32)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Push_rm64)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Sldt_rm16)], mnemonic[nameof(Mnemonic.Sldt)]),
				(code[nameof(Code.Sldt_r32m16)], mnemonic[nameof(Mnemonic.Sldt)]),
				(code[nameof(Code.Sldt_r64m16)], mnemonic[nameof(Mnemonic.Sldt)]),
				(code[nameof(Code.Str_rm16)], mnemonic[nameof(Mnemonic.Str)]),
				(code[nameof(Code.Str_r32m16)], mnemonic[nameof(Mnemonic.Str)]),
				(code[nameof(Code.Str_r64m16)], mnemonic[nameof(Mnemonic.Str)]),
				(code[nameof(Code.Lldt_rm16)], mnemonic[nameof(Mnemonic.Lldt)]),
				(code[nameof(Code.Lldt_r32m16)], mnemonic[nameof(Mnemonic.Lldt)]),
				(code[nameof(Code.Lldt_r64m16)], mnemonic[nameof(Mnemonic.Lldt)]),
				(code[nameof(Code.Ltr_rm16)], mnemonic[nameof(Mnemonic.Ltr)]),
				(code[nameof(Code.Ltr_r32m16)], mnemonic[nameof(Mnemonic.Ltr)]),
				(code[nameof(Code.Ltr_r64m16)], mnemonic[nameof(Mnemonic.Ltr)]),
				(code[nameof(Code.Verr_rm16)], mnemonic[nameof(Mnemonic.Verr)]),
				(code[nameof(Code.Verr_r32m16)], mnemonic[nameof(Mnemonic.Verr)]),
				(code[nameof(Code.Verr_r64m16)], mnemonic[nameof(Mnemonic.Verr)]),
				(code[nameof(Code.Verw_rm16)], mnemonic[nameof(Mnemonic.Verw)]),
				(code[nameof(Code.Verw_r32m16)], mnemonic[nameof(Mnemonic.Verw)]),
				(code[nameof(Code.Verw_r64m16)], mnemonic[nameof(Mnemonic.Verw)]),
				(code[nameof(Code.Jmpe_rm16)], mnemonic[nameof(Mnemonic.Jmpe)]),
				(code[nameof(Code.Jmpe_rm32)], mnemonic[nameof(Mnemonic.Jmpe)]),
				(code[nameof(Code.Sgdt_m1632_16)], mnemonic[nameof(Mnemonic.Sgdt)]),
				(code[nameof(Code.Sgdt_m1632)], mnemonic[nameof(Mnemonic.Sgdt)]),
				(code[nameof(Code.Sgdt_m1664)], mnemonic[nameof(Mnemonic.Sgdt)]),
				(code[nameof(Code.Sidt_m1632_16)], mnemonic[nameof(Mnemonic.Sidt)]),
				(code[nameof(Code.Sidt_m1632)], mnemonic[nameof(Mnemonic.Sidt)]),
				(code[nameof(Code.Sidt_m1664)], mnemonic[nameof(Mnemonic.Sidt)]),
				(code[nameof(Code.Lgdt_m1632_16)], mnemonic[nameof(Mnemonic.Lgdt)]),
				(code[nameof(Code.Lgdt_m1632)], mnemonic[nameof(Mnemonic.Lgdt)]),
				(code[nameof(Code.Lgdt_m1664)], mnemonic[nameof(Mnemonic.Lgdt)]),
				(code[nameof(Code.Lidt_m1632_16)], mnemonic[nameof(Mnemonic.Lidt)]),
				(code[nameof(Code.Lidt_m1632)], mnemonic[nameof(Mnemonic.Lidt)]),
				(code[nameof(Code.Lidt_m1664)], mnemonic[nameof(Mnemonic.Lidt)]),
				(code[nameof(Code.Smsw_rm16)], mnemonic[nameof(Mnemonic.Smsw)]),
				(code[nameof(Code.Smsw_r32m16)], mnemonic[nameof(Mnemonic.Smsw)]),
				(code[nameof(Code.Smsw_r64m16)], mnemonic[nameof(Mnemonic.Smsw)]),
				(code[nameof(Code.Rstorssp_m64)], mnemonic[nameof(Mnemonic.Rstorssp)]),
				(code[nameof(Code.Lmsw_rm16)], mnemonic[nameof(Mnemonic.Lmsw)]),
				(code[nameof(Code.Lmsw_r32m16)], mnemonic[nameof(Mnemonic.Lmsw)]),
				(code[nameof(Code.Lmsw_r64m16)], mnemonic[nameof(Mnemonic.Lmsw)]),
				(code[nameof(Code.Invlpg_m)], mnemonic[nameof(Mnemonic.Invlpg)]),
				(code[nameof(Code.Enclv)], mnemonic[nameof(Mnemonic.Enclv)]),
				(code[nameof(Code.Vmcall)], mnemonic[nameof(Mnemonic.Vmcall)]),
				(code[nameof(Code.Vmlaunch)], mnemonic[nameof(Mnemonic.Vmlaunch)]),
				(code[nameof(Code.Vmresume)], mnemonic[nameof(Mnemonic.Vmresume)]),
				(code[nameof(Code.Vmxoff)], mnemonic[nameof(Mnemonic.Vmxoff)]),
				(code[nameof(Code.Pconfig)], mnemonic[nameof(Mnemonic.Pconfig)]),
				(code[nameof(Code.Monitorw)], mnemonic[nameof(Mnemonic.Monitor)]),
				(code[nameof(Code.Monitord)], mnemonic[nameof(Mnemonic.Monitor)]),
				(code[nameof(Code.Monitorq)], mnemonic[nameof(Mnemonic.Monitor)]),
				(code[nameof(Code.Mwait)], mnemonic[nameof(Mnemonic.Mwait)]),
				(code[nameof(Code.Clac)], mnemonic[nameof(Mnemonic.Clac)]),
				(code[nameof(Code.Stac)], mnemonic[nameof(Mnemonic.Stac)]),
				(code[nameof(Code.Encls)], mnemonic[nameof(Mnemonic.Encls)]),
				(code[nameof(Code.Xgetbv)], mnemonic[nameof(Mnemonic.Xgetbv)]),
				(code[nameof(Code.Xsetbv)], mnemonic[nameof(Mnemonic.Xsetbv)]),
				(code[nameof(Code.Vmfunc)], mnemonic[nameof(Mnemonic.Vmfunc)]),
				(code[nameof(Code.Xend)], mnemonic[nameof(Mnemonic.Xend)]),
				(code[nameof(Code.Xtest)], mnemonic[nameof(Mnemonic.Xtest)]),
				(code[nameof(Code.Enclu)], mnemonic[nameof(Mnemonic.Enclu)]),
				(code[nameof(Code.Vmrunw)], mnemonic[nameof(Mnemonic.Vmrun)]),
				(code[nameof(Code.Vmrund)], mnemonic[nameof(Mnemonic.Vmrun)]),
				(code[nameof(Code.Vmrunq)], mnemonic[nameof(Mnemonic.Vmrun)]),
				(code[nameof(Code.Vmmcall)], mnemonic[nameof(Mnemonic.Vmmcall)]),
				(code[nameof(Code.Vmloadw)], mnemonic[nameof(Mnemonic.Vmload)]),
				(code[nameof(Code.Vmloadd)], mnemonic[nameof(Mnemonic.Vmload)]),
				(code[nameof(Code.Vmloadq)], mnemonic[nameof(Mnemonic.Vmload)]),
				(code[nameof(Code.Vmsavew)], mnemonic[nameof(Mnemonic.Vmsave)]),
				(code[nameof(Code.Vmsaved)], mnemonic[nameof(Mnemonic.Vmsave)]),
				(code[nameof(Code.Vmsaveq)], mnemonic[nameof(Mnemonic.Vmsave)]),
				(code[nameof(Code.Stgi)], mnemonic[nameof(Mnemonic.Stgi)]),
				(code[nameof(Code.Clgi)], mnemonic[nameof(Mnemonic.Clgi)]),
				(code[nameof(Code.Skinit)], mnemonic[nameof(Mnemonic.Skinit)]),
				(code[nameof(Code.Invlpgaw)], mnemonic[nameof(Mnemonic.Invlpga)]),
				(code[nameof(Code.Invlpgad)], mnemonic[nameof(Mnemonic.Invlpga)]),
				(code[nameof(Code.Invlpgaq)], mnemonic[nameof(Mnemonic.Invlpga)]),
				(code[nameof(Code.Setssbsy)], mnemonic[nameof(Mnemonic.Setssbsy)]),
				(code[nameof(Code.Saveprevssp)], mnemonic[nameof(Mnemonic.Saveprevssp)]),
				(code[nameof(Code.Rdpkru)], mnemonic[nameof(Mnemonic.Rdpkru)]),
				(code[nameof(Code.Wrpkru)], mnemonic[nameof(Mnemonic.Wrpkru)]),
				(code[nameof(Code.Swapgs)], mnemonic[nameof(Mnemonic.Swapgs)]),
				(code[nameof(Code.Rdtscp)], mnemonic[nameof(Mnemonic.Rdtscp)]),
				(code[nameof(Code.Monitorxw)], mnemonic[nameof(Mnemonic.Monitorx)]),
				(code[nameof(Code.Monitorxd)], mnemonic[nameof(Mnemonic.Monitorx)]),
				(code[nameof(Code.Monitorxq)], mnemonic[nameof(Mnemonic.Monitorx)]),
				(code[nameof(Code.Mcommit)], mnemonic[nameof(Mnemonic.Mcommit)]),
				(code[nameof(Code.Mwaitx)], mnemonic[nameof(Mnemonic.Mwaitx)]),
				(code[nameof(Code.Clzerow)], mnemonic[nameof(Mnemonic.Clzero)]),
				(code[nameof(Code.Clzerod)], mnemonic[nameof(Mnemonic.Clzero)]),
				(code[nameof(Code.Clzeroq)], mnemonic[nameof(Mnemonic.Clzero)]),
				(code[nameof(Code.Rdpru)], mnemonic[nameof(Mnemonic.Rdpru)]),
				(code[nameof(Code.Lar_r16_rm16)], mnemonic[nameof(Mnemonic.Lar)]),
				(code[nameof(Code.Lar_r32_r32m16)], mnemonic[nameof(Mnemonic.Lar)]),
				(code[nameof(Code.Lar_r64_r64m16)], mnemonic[nameof(Mnemonic.Lar)]),
				(code[nameof(Code.Lsl_r16_rm16)], mnemonic[nameof(Mnemonic.Lsl)]),
				(code[nameof(Code.Lsl_r32_r32m16)], mnemonic[nameof(Mnemonic.Lsl)]),
				(code[nameof(Code.Lsl_r64_r64m16)], mnemonic[nameof(Mnemonic.Lsl)]),
				(code[nameof(Code.Loadallreset286)], mnemonic[nameof(Mnemonic.Loadall)]),
				(code[nameof(Code.Loadall286)], mnemonic[nameof(Mnemonic.Loadall)]),
				(code[nameof(Code.Syscall)], mnemonic[nameof(Mnemonic.Syscall)]),
				(code[nameof(Code.Clts)], mnemonic[nameof(Mnemonic.Clts)]),
				(code[nameof(Code.Loadall386)], mnemonic[nameof(Mnemonic.Loadall)]),
				(code[nameof(Code.Sysretd)], mnemonic[nameof(Mnemonic.Sysret)]),
				(code[nameof(Code.Sysretq)], mnemonic[nameof(Mnemonic.Sysret)]),
				(code[nameof(Code.Invd)], mnemonic[nameof(Mnemonic.Invd)]),
				(code[nameof(Code.Wbinvd)], mnemonic[nameof(Mnemonic.Wbinvd)]),
				(code[nameof(Code.Wbnoinvd)], mnemonic[nameof(Mnemonic.Wbnoinvd)]),
				(code[nameof(Code.Cl1invmb)], mnemonic[nameof(Mnemonic.Cl1invmb)]),
				(code[nameof(Code.Ud2)], mnemonic[nameof(Mnemonic.Ud2)]),
				(code[nameof(Code.ReservedNop_rm16_r16_0F0D)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm32_r32_0F0D)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm64_r64_0F0D)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.Prefetch_m8)], mnemonic[nameof(Mnemonic.Prefetch)]),
				(code[nameof(Code.Prefetchw_m8)], mnemonic[nameof(Mnemonic.Prefetchw)]),
				(code[nameof(Code.Prefetchwt1_m8)], mnemonic[nameof(Mnemonic.Prefetchwt1)]),
				(code[nameof(Code.Femms)], mnemonic[nameof(Mnemonic.Femms)]),
				(code[nameof(Code.Umov_rm8_r8)], mnemonic[nameof(Mnemonic.Umov)]),
				(code[nameof(Code.Umov_rm16_r16)], mnemonic[nameof(Mnemonic.Umov)]),
				(code[nameof(Code.Umov_rm32_r32)], mnemonic[nameof(Mnemonic.Umov)]),
				(code[nameof(Code.Umov_r8_rm8)], mnemonic[nameof(Mnemonic.Umov)]),
				(code[nameof(Code.Umov_r16_rm16)], mnemonic[nameof(Mnemonic.Umov)]),
				(code[nameof(Code.Umov_r32_rm32)], mnemonic[nameof(Mnemonic.Umov)]),
				(code[nameof(Code.Movups_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Movups)]),
				(code[nameof(Code.VEX_Vmovups_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vmovups)]),
				(code[nameof(Code.VEX_Vmovups_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vmovups)]),
				(code[nameof(Code.EVEX_Vmovups_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vmovups)]),
				(code[nameof(Code.EVEX_Vmovups_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vmovups)]),
				(code[nameof(Code.EVEX_Vmovups_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vmovups)]),
				(code[nameof(Code.Movupd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Movupd)]),
				(code[nameof(Code.VEX_Vmovupd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vmovupd)]),
				(code[nameof(Code.VEX_Vmovupd_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vmovupd)]),
				(code[nameof(Code.EVEX_Vmovupd_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vmovupd)]),
				(code[nameof(Code.EVEX_Vmovupd_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vmovupd)]),
				(code[nameof(Code.EVEX_Vmovupd_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vmovupd)]),
				(code[nameof(Code.Movss_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Movss)]),
				(code[nameof(Code.VEX_Vmovss_xmm_xmm_xmm)], mnemonic[nameof(Mnemonic.Vmovss)]),
				(code[nameof(Code.VEX_Vmovss_xmm_m32)], mnemonic[nameof(Mnemonic.Vmovss)]),
				(code[nameof(Code.EVEX_Vmovss_xmm_k1z_xmm_xmm)], mnemonic[nameof(Mnemonic.Vmovss)]),
				(code[nameof(Code.EVEX_Vmovss_xmm_k1z_m32)], mnemonic[nameof(Mnemonic.Vmovss)]),
				(code[nameof(Code.Movsd_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Movsd)]),
				(code[nameof(Code.VEX_Vmovsd_xmm_xmm_xmm)], mnemonic[nameof(Mnemonic.Vmovsd)]),
				(code[nameof(Code.VEX_Vmovsd_xmm_m64)], mnemonic[nameof(Mnemonic.Vmovsd)]),
				(code[nameof(Code.EVEX_Vmovsd_xmm_k1z_xmm_xmm)], mnemonic[nameof(Mnemonic.Vmovsd)]),
				(code[nameof(Code.EVEX_Vmovsd_xmm_k1z_m64)], mnemonic[nameof(Mnemonic.Vmovsd)]),
				(code[nameof(Code.Movups_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Movups)]),
				(code[nameof(Code.VEX_Vmovups_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vmovups)]),
				(code[nameof(Code.VEX_Vmovups_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vmovups)]),
				(code[nameof(Code.EVEX_Vmovups_xmmm128_k1z_xmm)], mnemonic[nameof(Mnemonic.Vmovups)]),
				(code[nameof(Code.EVEX_Vmovups_ymmm256_k1z_ymm)], mnemonic[nameof(Mnemonic.Vmovups)]),
				(code[nameof(Code.EVEX_Vmovups_zmmm512_k1z_zmm)], mnemonic[nameof(Mnemonic.Vmovups)]),
				(code[nameof(Code.Movupd_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Movupd)]),
				(code[nameof(Code.VEX_Vmovupd_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vmovupd)]),
				(code[nameof(Code.VEX_Vmovupd_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vmovupd)]),
				(code[nameof(Code.EVEX_Vmovupd_xmmm128_k1z_xmm)], mnemonic[nameof(Mnemonic.Vmovupd)]),
				(code[nameof(Code.EVEX_Vmovupd_ymmm256_k1z_ymm)], mnemonic[nameof(Mnemonic.Vmovupd)]),
				(code[nameof(Code.EVEX_Vmovupd_zmmm512_k1z_zmm)], mnemonic[nameof(Mnemonic.Vmovupd)]),
				(code[nameof(Code.Movss_xmmm32_xmm)], mnemonic[nameof(Mnemonic.Movss)]),
				(code[nameof(Code.VEX_Vmovss_xmm_xmm_xmm_0F11)], mnemonic[nameof(Mnemonic.Vmovss)]),
				(code[nameof(Code.VEX_Vmovss_m32_xmm)], mnemonic[nameof(Mnemonic.Vmovss)]),
				(code[nameof(Code.EVEX_Vmovss_xmm_k1z_xmm_xmm_0F11)], mnemonic[nameof(Mnemonic.Vmovss)]),
				(code[nameof(Code.EVEX_Vmovss_m32_k1_xmm)], mnemonic[nameof(Mnemonic.Vmovss)]),
				(code[nameof(Code.Movsd_xmmm64_xmm)], mnemonic[nameof(Mnemonic.Movsd)]),
				(code[nameof(Code.VEX_Vmovsd_xmm_xmm_xmm_0F11)], mnemonic[nameof(Mnemonic.Vmovsd)]),
				(code[nameof(Code.VEX_Vmovsd_m64_xmm)], mnemonic[nameof(Mnemonic.Vmovsd)]),
				(code[nameof(Code.EVEX_Vmovsd_xmm_k1z_xmm_xmm_0F11)], mnemonic[nameof(Mnemonic.Vmovsd)]),
				(code[nameof(Code.EVEX_Vmovsd_m64_k1_xmm)], mnemonic[nameof(Mnemonic.Vmovsd)]),
				(code[nameof(Code.Movhlps_xmm_xmm)], mnemonic[nameof(Mnemonic.Movhlps)]),
				(code[nameof(Code.Movlps_xmm_m64)], mnemonic[nameof(Mnemonic.Movlps)]),
				(code[nameof(Code.VEX_Vmovhlps_xmm_xmm_xmm)], mnemonic[nameof(Mnemonic.Vmovhlps)]),
				(code[nameof(Code.VEX_Vmovlps_xmm_xmm_m64)], mnemonic[nameof(Mnemonic.Vmovlps)]),
				(code[nameof(Code.EVEX_Vmovhlps_xmm_xmm_xmm)], mnemonic[nameof(Mnemonic.Vmovhlps)]),
				(code[nameof(Code.EVEX_Vmovlps_xmm_xmm_m64)], mnemonic[nameof(Mnemonic.Vmovlps)]),
				(code[nameof(Code.Movlpd_xmm_m64)], mnemonic[nameof(Mnemonic.Movlpd)]),
				(code[nameof(Code.VEX_Vmovlpd_xmm_xmm_m64)], mnemonic[nameof(Mnemonic.Vmovlpd)]),
				(code[nameof(Code.EVEX_Vmovlpd_xmm_xmm_m64)], mnemonic[nameof(Mnemonic.Vmovlpd)]),
				(code[nameof(Code.Movsldup_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Movsldup)]),
				(code[nameof(Code.VEX_Vmovsldup_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vmovsldup)]),
				(code[nameof(Code.VEX_Vmovsldup_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vmovsldup)]),
				(code[nameof(Code.EVEX_Vmovsldup_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vmovsldup)]),
				(code[nameof(Code.EVEX_Vmovsldup_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vmovsldup)]),
				(code[nameof(Code.EVEX_Vmovsldup_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vmovsldup)]),
				(code[nameof(Code.Movddup_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Movddup)]),
				(code[nameof(Code.VEX_Vmovddup_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vmovddup)]),
				(code[nameof(Code.VEX_Vmovddup_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vmovddup)]),
				(code[nameof(Code.EVEX_Vmovddup_xmm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vmovddup)]),
				(code[nameof(Code.EVEX_Vmovddup_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vmovddup)]),
				(code[nameof(Code.EVEX_Vmovddup_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vmovddup)]),
				(code[nameof(Code.Movlps_m64_xmm)], mnemonic[nameof(Mnemonic.Movlps)]),
				(code[nameof(Code.VEX_Vmovlps_m64_xmm)], mnemonic[nameof(Mnemonic.Vmovlps)]),
				(code[nameof(Code.EVEX_Vmovlps_m64_xmm)], mnemonic[nameof(Mnemonic.Vmovlps)]),
				(code[nameof(Code.Movlpd_m64_xmm)], mnemonic[nameof(Mnemonic.Movlpd)]),
				(code[nameof(Code.VEX_Vmovlpd_m64_xmm)], mnemonic[nameof(Mnemonic.Vmovlpd)]),
				(code[nameof(Code.EVEX_Vmovlpd_m64_xmm)], mnemonic[nameof(Mnemonic.Vmovlpd)]),
				(code[nameof(Code.Unpcklps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Unpcklps)]),
				(code[nameof(Code.VEX_Vunpcklps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vunpcklps)]),
				(code[nameof(Code.VEX_Vunpcklps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vunpcklps)]),
				(code[nameof(Code.EVEX_Vunpcklps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vunpcklps)]),
				(code[nameof(Code.EVEX_Vunpcklps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vunpcklps)]),
				(code[nameof(Code.EVEX_Vunpcklps_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vunpcklps)]),
				(code[nameof(Code.Unpcklpd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Unpcklpd)]),
				(code[nameof(Code.VEX_Vunpcklpd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vunpcklpd)]),
				(code[nameof(Code.VEX_Vunpcklpd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vunpcklpd)]),
				(code[nameof(Code.EVEX_Vunpcklpd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vunpcklpd)]),
				(code[nameof(Code.EVEX_Vunpcklpd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vunpcklpd)]),
				(code[nameof(Code.EVEX_Vunpcklpd_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vunpcklpd)]),
				(code[nameof(Code.Unpckhps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Unpckhps)]),
				(code[nameof(Code.VEX_Vunpckhps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vunpckhps)]),
				(code[nameof(Code.VEX_Vunpckhps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vunpckhps)]),
				(code[nameof(Code.EVEX_Vunpckhps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vunpckhps)]),
				(code[nameof(Code.EVEX_Vunpckhps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vunpckhps)]),
				(code[nameof(Code.EVEX_Vunpckhps_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vunpckhps)]),
				(code[nameof(Code.Unpckhpd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Unpckhpd)]),
				(code[nameof(Code.VEX_Vunpckhpd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vunpckhpd)]),
				(code[nameof(Code.VEX_Vunpckhpd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vunpckhpd)]),
				(code[nameof(Code.EVEX_Vunpckhpd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vunpckhpd)]),
				(code[nameof(Code.EVEX_Vunpckhpd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vunpckhpd)]),
				(code[nameof(Code.EVEX_Vunpckhpd_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vunpckhpd)]),
				(code[nameof(Code.Movlhps_xmm_xmm)], mnemonic[nameof(Mnemonic.Movlhps)]),
				(code[nameof(Code.VEX_Vmovlhps_xmm_xmm_xmm)], mnemonic[nameof(Mnemonic.Vmovlhps)]),
				(code[nameof(Code.EVEX_Vmovlhps_xmm_xmm_xmm)], mnemonic[nameof(Mnemonic.Vmovlhps)]),
				(code[nameof(Code.Movhps_xmm_m64)], mnemonic[nameof(Mnemonic.Movhps)]),
				(code[nameof(Code.VEX_Vmovhps_xmm_xmm_m64)], mnemonic[nameof(Mnemonic.Vmovhps)]),
				(code[nameof(Code.EVEX_Vmovhps_xmm_xmm_m64)], mnemonic[nameof(Mnemonic.Vmovhps)]),
				(code[nameof(Code.Movhpd_xmm_m64)], mnemonic[nameof(Mnemonic.Movhpd)]),
				(code[nameof(Code.VEX_Vmovhpd_xmm_xmm_m64)], mnemonic[nameof(Mnemonic.Vmovhpd)]),
				(code[nameof(Code.EVEX_Vmovhpd_xmm_xmm_m64)], mnemonic[nameof(Mnemonic.Vmovhpd)]),
				(code[nameof(Code.Movshdup_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Movshdup)]),
				(code[nameof(Code.VEX_Vmovshdup_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vmovshdup)]),
				(code[nameof(Code.VEX_Vmovshdup_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vmovshdup)]),
				(code[nameof(Code.EVEX_Vmovshdup_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vmovshdup)]),
				(code[nameof(Code.EVEX_Vmovshdup_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vmovshdup)]),
				(code[nameof(Code.EVEX_Vmovshdup_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vmovshdup)]),
				(code[nameof(Code.Movhps_m64_xmm)], mnemonic[nameof(Mnemonic.Movhps)]),
				(code[nameof(Code.VEX_Vmovhps_m64_xmm)], mnemonic[nameof(Mnemonic.Vmovhps)]),
				(code[nameof(Code.EVEX_Vmovhps_m64_xmm)], mnemonic[nameof(Mnemonic.Vmovhps)]),
				(code[nameof(Code.Movhpd_m64_xmm)], mnemonic[nameof(Mnemonic.Movhpd)]),
				(code[nameof(Code.VEX_Vmovhpd_m64_xmm)], mnemonic[nameof(Mnemonic.Vmovhpd)]),
				(code[nameof(Code.EVEX_Vmovhpd_m64_xmm)], mnemonic[nameof(Mnemonic.Vmovhpd)]),
				(code[nameof(Code.ReservedNop_rm16_r16_0F18)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm32_r32_0F18)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm64_r64_0F18)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm16_r16_0F19)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm32_r32_0F19)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm64_r64_0F19)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm16_r16_0F1A)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm32_r32_0F1A)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm64_r64_0F1A)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm16_r16_0F1B)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm32_r32_0F1B)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm64_r64_0F1B)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm16_r16_0F1C)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm32_r32_0F1C)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm64_r64_0F1C)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm16_r16_0F1D)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm32_r32_0F1D)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm64_r64_0F1D)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm16_r16_0F1E)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm32_r32_0F1E)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm64_r64_0F1E)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm16_r16_0F1F)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm32_r32_0F1F)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.ReservedNop_rm64_r64_0F1F)], mnemonic[nameof(Mnemonic.ReservedNop)]),
				(code[nameof(Code.Prefetchnta_m8)], mnemonic[nameof(Mnemonic.Prefetchnta)]),
				(code[nameof(Code.Prefetcht0_m8)], mnemonic[nameof(Mnemonic.Prefetcht0)]),
				(code[nameof(Code.Prefetcht1_m8)], mnemonic[nameof(Mnemonic.Prefetcht1)]),
				(code[nameof(Code.Prefetcht2_m8)], mnemonic[nameof(Mnemonic.Prefetcht2)]),
				(code[nameof(Code.Bndldx_bnd_mib)], mnemonic[nameof(Mnemonic.Bndldx)]),
				(code[nameof(Code.Bndmov_bnd_bndm64)], mnemonic[nameof(Mnemonic.Bndmov)]),
				(code[nameof(Code.Bndmov_bnd_bndm128)], mnemonic[nameof(Mnemonic.Bndmov)]),
				(code[nameof(Code.Bndcl_bnd_rm32)], mnemonic[nameof(Mnemonic.Bndcl)]),
				(code[nameof(Code.Bndcl_bnd_rm64)], mnemonic[nameof(Mnemonic.Bndcl)]),
				(code[nameof(Code.Bndcu_bnd_rm32)], mnemonic[nameof(Mnemonic.Bndcu)]),
				(code[nameof(Code.Bndcu_bnd_rm64)], mnemonic[nameof(Mnemonic.Bndcu)]),
				(code[nameof(Code.Bndstx_mib_bnd)], mnemonic[nameof(Mnemonic.Bndstx)]),
				(code[nameof(Code.Bndmov_bndm64_bnd)], mnemonic[nameof(Mnemonic.Bndmov)]),
				(code[nameof(Code.Bndmov_bndm128_bnd)], mnemonic[nameof(Mnemonic.Bndmov)]),
				(code[nameof(Code.Bndmk_bnd_m32)], mnemonic[nameof(Mnemonic.Bndmk)]),
				(code[nameof(Code.Bndmk_bnd_m64)], mnemonic[nameof(Mnemonic.Bndmk)]),
				(code[nameof(Code.Bndcn_bnd_rm32)], mnemonic[nameof(Mnemonic.Bndcn)]),
				(code[nameof(Code.Bndcn_bnd_rm64)], mnemonic[nameof(Mnemonic.Bndcn)]),
				(code[nameof(Code.Cldemote_m8)], mnemonic[nameof(Mnemonic.Cldemote)]),
				(code[nameof(Code.Rdsspd_r32)], mnemonic[nameof(Mnemonic.Rdsspd)]),
				(code[nameof(Code.Rdsspq_r64)], mnemonic[nameof(Mnemonic.Rdsspq)]),
				(code[nameof(Code.Endbr64)], mnemonic[nameof(Mnemonic.Endbr64)]),
				(code[nameof(Code.Endbr32)], mnemonic[nameof(Mnemonic.Endbr32)]),
				(code[nameof(Code.Nop_rm16)], mnemonic[nameof(Mnemonic.Nop)]),
				(code[nameof(Code.Nop_rm32)], mnemonic[nameof(Mnemonic.Nop)]),
				(code[nameof(Code.Nop_rm64)], mnemonic[nameof(Mnemonic.Nop)]),
				(code[nameof(Code.Mov_r32_cr)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_r64_cr)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_r32_dr)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_r64_dr)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_cr_r32)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_cr_r64)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_dr_r32)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_dr_r64)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_r32_tr)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Mov_tr_r32)], mnemonic[nameof(Mnemonic.Mov)]),
				(code[nameof(Code.Movaps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Movaps)]),
				(code[nameof(Code.VEX_Vmovaps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vmovaps)]),
				(code[nameof(Code.VEX_Vmovaps_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vmovaps)]),
				(code[nameof(Code.EVEX_Vmovaps_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vmovaps)]),
				(code[nameof(Code.EVEX_Vmovaps_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vmovaps)]),
				(code[nameof(Code.EVEX_Vmovaps_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vmovaps)]),
				(code[nameof(Code.Movapd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Movapd)]),
				(code[nameof(Code.VEX_Vmovapd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vmovapd)]),
				(code[nameof(Code.VEX_Vmovapd_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vmovapd)]),
				(code[nameof(Code.EVEX_Vmovapd_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vmovapd)]),
				(code[nameof(Code.EVEX_Vmovapd_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vmovapd)]),
				(code[nameof(Code.EVEX_Vmovapd_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vmovapd)]),
				(code[nameof(Code.Movaps_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Movaps)]),
				(code[nameof(Code.VEX_Vmovaps_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vmovaps)]),
				(code[nameof(Code.VEX_Vmovaps_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vmovaps)]),
				(code[nameof(Code.EVEX_Vmovaps_xmmm128_k1z_xmm)], mnemonic[nameof(Mnemonic.Vmovaps)]),
				(code[nameof(Code.EVEX_Vmovaps_ymmm256_k1z_ymm)], mnemonic[nameof(Mnemonic.Vmovaps)]),
				(code[nameof(Code.EVEX_Vmovaps_zmmm512_k1z_zmm)], mnemonic[nameof(Mnemonic.Vmovaps)]),
				(code[nameof(Code.Movapd_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Movapd)]),
				(code[nameof(Code.VEX_Vmovapd_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vmovapd)]),
				(code[nameof(Code.VEX_Vmovapd_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vmovapd)]),
				(code[nameof(Code.EVEX_Vmovapd_xmmm128_k1z_xmm)], mnemonic[nameof(Mnemonic.Vmovapd)]),
				(code[nameof(Code.EVEX_Vmovapd_ymmm256_k1z_ymm)], mnemonic[nameof(Mnemonic.Vmovapd)]),
				(code[nameof(Code.EVEX_Vmovapd_zmmm512_k1z_zmm)], mnemonic[nameof(Mnemonic.Vmovapd)]),
				(code[nameof(Code.Cvtpi2ps_xmm_mmm64)], mnemonic[nameof(Mnemonic.Cvtpi2ps)]),
				(code[nameof(Code.Cvtpi2pd_xmm_mmm64)], mnemonic[nameof(Mnemonic.Cvtpi2pd)]),
				(code[nameof(Code.Cvtsi2ss_xmm_rm32)], mnemonic[nameof(Mnemonic.Cvtsi2ss)]),
				(code[nameof(Code.Cvtsi2ss_xmm_rm64)], mnemonic[nameof(Mnemonic.Cvtsi2ss)]),
				(code[nameof(Code.VEX_Vcvtsi2ss_xmm_xmm_rm32)], mnemonic[nameof(Mnemonic.Vcvtsi2ss)]),
				(code[nameof(Code.VEX_Vcvtsi2ss_xmm_xmm_rm64)], mnemonic[nameof(Mnemonic.Vcvtsi2ss)]),
				(code[nameof(Code.EVEX_Vcvtsi2ss_xmm_xmm_rm32_er)], mnemonic[nameof(Mnemonic.Vcvtsi2ss)]),
				(code[nameof(Code.EVEX_Vcvtsi2ss_xmm_xmm_rm64_er)], mnemonic[nameof(Mnemonic.Vcvtsi2ss)]),
				(code[nameof(Code.Cvtsi2sd_xmm_rm32)], mnemonic[nameof(Mnemonic.Cvtsi2sd)]),
				(code[nameof(Code.Cvtsi2sd_xmm_rm64)], mnemonic[nameof(Mnemonic.Cvtsi2sd)]),
				(code[nameof(Code.VEX_Vcvtsi2sd_xmm_xmm_rm32)], mnemonic[nameof(Mnemonic.Vcvtsi2sd)]),
				(code[nameof(Code.VEX_Vcvtsi2sd_xmm_xmm_rm64)], mnemonic[nameof(Mnemonic.Vcvtsi2sd)]),
				(code[nameof(Code.EVEX_Vcvtsi2sd_xmm_xmm_rm32_er)], mnemonic[nameof(Mnemonic.Vcvtsi2sd)]),
				(code[nameof(Code.EVEX_Vcvtsi2sd_xmm_xmm_rm64_er)], mnemonic[nameof(Mnemonic.Vcvtsi2sd)]),
				(code[nameof(Code.Movntps_m128_xmm)], mnemonic[nameof(Mnemonic.Movntps)]),
				(code[nameof(Code.VEX_Vmovntps_m128_xmm)], mnemonic[nameof(Mnemonic.Vmovntps)]),
				(code[nameof(Code.VEX_Vmovntps_m256_ymm)], mnemonic[nameof(Mnemonic.Vmovntps)]),
				(code[nameof(Code.EVEX_Vmovntps_m128_xmm)], mnemonic[nameof(Mnemonic.Vmovntps)]),
				(code[nameof(Code.EVEX_Vmovntps_m256_ymm)], mnemonic[nameof(Mnemonic.Vmovntps)]),
				(code[nameof(Code.EVEX_Vmovntps_m512_zmm)], mnemonic[nameof(Mnemonic.Vmovntps)]),
				(code[nameof(Code.Movntpd_m128_xmm)], mnemonic[nameof(Mnemonic.Movntpd)]),
				(code[nameof(Code.VEX_Vmovntpd_m128_xmm)], mnemonic[nameof(Mnemonic.Vmovntpd)]),
				(code[nameof(Code.VEX_Vmovntpd_m256_ymm)], mnemonic[nameof(Mnemonic.Vmovntpd)]),
				(code[nameof(Code.EVEX_Vmovntpd_m128_xmm)], mnemonic[nameof(Mnemonic.Vmovntpd)]),
				(code[nameof(Code.EVEX_Vmovntpd_m256_ymm)], mnemonic[nameof(Mnemonic.Vmovntpd)]),
				(code[nameof(Code.EVEX_Vmovntpd_m512_zmm)], mnemonic[nameof(Mnemonic.Vmovntpd)]),
				(code[nameof(Code.Movntss_m32_xmm)], mnemonic[nameof(Mnemonic.Movntss)]),
				(code[nameof(Code.Movntsd_m64_xmm)], mnemonic[nameof(Mnemonic.Movntsd)]),
				(code[nameof(Code.Cvttps2pi_mm_xmmm64)], mnemonic[nameof(Mnemonic.Cvttps2pi)]),
				(code[nameof(Code.Cvttpd2pi_mm_xmmm128)], mnemonic[nameof(Mnemonic.Cvttpd2pi)]),
				(code[nameof(Code.Cvttss2si_r32_xmmm32)], mnemonic[nameof(Mnemonic.Cvttss2si)]),
				(code[nameof(Code.Cvttss2si_r64_xmmm32)], mnemonic[nameof(Mnemonic.Cvttss2si)]),
				(code[nameof(Code.VEX_Vcvttss2si_r32_xmmm32)], mnemonic[nameof(Mnemonic.Vcvttss2si)]),
				(code[nameof(Code.VEX_Vcvttss2si_r64_xmmm32)], mnemonic[nameof(Mnemonic.Vcvttss2si)]),
				(code[nameof(Code.EVEX_Vcvttss2si_r32_xmmm32_sae)], mnemonic[nameof(Mnemonic.Vcvttss2si)]),
				(code[nameof(Code.EVEX_Vcvttss2si_r64_xmmm32_sae)], mnemonic[nameof(Mnemonic.Vcvttss2si)]),
				(code[nameof(Code.Cvttsd2si_r32_xmmm64)], mnemonic[nameof(Mnemonic.Cvttsd2si)]),
				(code[nameof(Code.Cvttsd2si_r64_xmmm64)], mnemonic[nameof(Mnemonic.Cvttsd2si)]),
				(code[nameof(Code.VEX_Vcvttsd2si_r32_xmmm64)], mnemonic[nameof(Mnemonic.Vcvttsd2si)]),
				(code[nameof(Code.VEX_Vcvttsd2si_r64_xmmm64)], mnemonic[nameof(Mnemonic.Vcvttsd2si)]),
				(code[nameof(Code.EVEX_Vcvttsd2si_r32_xmmm64_sae)], mnemonic[nameof(Mnemonic.Vcvttsd2si)]),
				(code[nameof(Code.EVEX_Vcvttsd2si_r64_xmmm64_sae)], mnemonic[nameof(Mnemonic.Vcvttsd2si)]),
				(code[nameof(Code.Cvtps2pi_mm_xmmm64)], mnemonic[nameof(Mnemonic.Cvtps2pi)]),
				(code[nameof(Code.Cvtpd2pi_mm_xmmm128)], mnemonic[nameof(Mnemonic.Cvtpd2pi)]),
				(code[nameof(Code.Cvtss2si_r32_xmmm32)], mnemonic[nameof(Mnemonic.Cvtss2si)]),
				(code[nameof(Code.Cvtss2si_r64_xmmm32)], mnemonic[nameof(Mnemonic.Cvtss2si)]),
				(code[nameof(Code.VEX_Vcvtss2si_r32_xmmm32)], mnemonic[nameof(Mnemonic.Vcvtss2si)]),
				(code[nameof(Code.VEX_Vcvtss2si_r64_xmmm32)], mnemonic[nameof(Mnemonic.Vcvtss2si)]),
				(code[nameof(Code.EVEX_Vcvtss2si_r32_xmmm32_er)], mnemonic[nameof(Mnemonic.Vcvtss2si)]),
				(code[nameof(Code.EVEX_Vcvtss2si_r64_xmmm32_er)], mnemonic[nameof(Mnemonic.Vcvtss2si)]),
				(code[nameof(Code.Cvtsd2si_r32_xmmm64)], mnemonic[nameof(Mnemonic.Cvtsd2si)]),
				(code[nameof(Code.Cvtsd2si_r64_xmmm64)], mnemonic[nameof(Mnemonic.Cvtsd2si)]),
				(code[nameof(Code.VEX_Vcvtsd2si_r32_xmmm64)], mnemonic[nameof(Mnemonic.Vcvtsd2si)]),
				(code[nameof(Code.VEX_Vcvtsd2si_r64_xmmm64)], mnemonic[nameof(Mnemonic.Vcvtsd2si)]),
				(code[nameof(Code.EVEX_Vcvtsd2si_r32_xmmm64_er)], mnemonic[nameof(Mnemonic.Vcvtsd2si)]),
				(code[nameof(Code.EVEX_Vcvtsd2si_r64_xmmm64_er)], mnemonic[nameof(Mnemonic.Vcvtsd2si)]),
				(code[nameof(Code.Ucomiss_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Ucomiss)]),
				(code[nameof(Code.VEX_Vucomiss_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vucomiss)]),
				(code[nameof(Code.EVEX_Vucomiss_xmm_xmmm32_sae)], mnemonic[nameof(Mnemonic.Vucomiss)]),
				(code[nameof(Code.Ucomisd_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Ucomisd)]),
				(code[nameof(Code.VEX_Vucomisd_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vucomisd)]),
				(code[nameof(Code.EVEX_Vucomisd_xmm_xmmm64_sae)], mnemonic[nameof(Mnemonic.Vucomisd)]),
				(code[nameof(Code.Comiss_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Comiss)]),
				(code[nameof(Code.Comisd_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Comisd)]),
				(code[nameof(Code.VEX_Vcomiss_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vcomiss)]),
				(code[nameof(Code.VEX_Vcomisd_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vcomisd)]),
				(code[nameof(Code.EVEX_Vcomiss_xmm_xmmm32_sae)], mnemonic[nameof(Mnemonic.Vcomiss)]),
				(code[nameof(Code.EVEX_Vcomisd_xmm_xmmm64_sae)], mnemonic[nameof(Mnemonic.Vcomisd)]),
				(code[nameof(Code.Wrmsr)], mnemonic[nameof(Mnemonic.Wrmsr)]),
				(code[nameof(Code.Rdtsc)], mnemonic[nameof(Mnemonic.Rdtsc)]),
				(code[nameof(Code.Rdmsr)], mnemonic[nameof(Mnemonic.Rdmsr)]),
				(code[nameof(Code.Rdpmc)], mnemonic[nameof(Mnemonic.Rdpmc)]),
				(code[nameof(Code.Sysenter)], mnemonic[nameof(Mnemonic.Sysenter)]),
				(code[nameof(Code.Sysexitd)], mnemonic[nameof(Mnemonic.Sysexit)]),
				(code[nameof(Code.Sysexitq)], mnemonic[nameof(Mnemonic.Sysexit)]),
				(code[nameof(Code.Getsec)], mnemonic[nameof(Mnemonic.Getsec)]),
				(code[nameof(Code.Cmovo_r16_rm16)], mnemonic[nameof(Mnemonic.Cmovo)]),
				(code[nameof(Code.Cmovo_r32_rm32)], mnemonic[nameof(Mnemonic.Cmovo)]),
				(code[nameof(Code.Cmovo_r64_rm64)], mnemonic[nameof(Mnemonic.Cmovo)]),
				(code[nameof(Code.Cmovno_r16_rm16)], mnemonic[nameof(Mnemonic.Cmovno)]),
				(code[nameof(Code.Cmovno_r32_rm32)], mnemonic[nameof(Mnemonic.Cmovno)]),
				(code[nameof(Code.Cmovno_r64_rm64)], mnemonic[nameof(Mnemonic.Cmovno)]),
				(code[nameof(Code.Cmovb_r16_rm16)], mnemonic[nameof(Mnemonic.Cmovb)]),
				(code[nameof(Code.Cmovb_r32_rm32)], mnemonic[nameof(Mnemonic.Cmovb)]),
				(code[nameof(Code.Cmovb_r64_rm64)], mnemonic[nameof(Mnemonic.Cmovb)]),
				(code[nameof(Code.Cmovae_r16_rm16)], mnemonic[nameof(Mnemonic.Cmovae)]),
				(code[nameof(Code.Cmovae_r32_rm32)], mnemonic[nameof(Mnemonic.Cmovae)]),
				(code[nameof(Code.Cmovae_r64_rm64)], mnemonic[nameof(Mnemonic.Cmovae)]),
				(code[nameof(Code.Cmove_r16_rm16)], mnemonic[nameof(Mnemonic.Cmove)]),
				(code[nameof(Code.Cmove_r32_rm32)], mnemonic[nameof(Mnemonic.Cmove)]),
				(code[nameof(Code.Cmove_r64_rm64)], mnemonic[nameof(Mnemonic.Cmove)]),
				(code[nameof(Code.Cmovne_r16_rm16)], mnemonic[nameof(Mnemonic.Cmovne)]),
				(code[nameof(Code.Cmovne_r32_rm32)], mnemonic[nameof(Mnemonic.Cmovne)]),
				(code[nameof(Code.Cmovne_r64_rm64)], mnemonic[nameof(Mnemonic.Cmovne)]),
				(code[nameof(Code.Cmovbe_r16_rm16)], mnemonic[nameof(Mnemonic.Cmovbe)]),
				(code[nameof(Code.Cmovbe_r32_rm32)], mnemonic[nameof(Mnemonic.Cmovbe)]),
				(code[nameof(Code.Cmovbe_r64_rm64)], mnemonic[nameof(Mnemonic.Cmovbe)]),
				(code[nameof(Code.Cmova_r16_rm16)], mnemonic[nameof(Mnemonic.Cmova)]),
				(code[nameof(Code.Cmova_r32_rm32)], mnemonic[nameof(Mnemonic.Cmova)]),
				(code[nameof(Code.Cmova_r64_rm64)], mnemonic[nameof(Mnemonic.Cmova)]),
				(code[nameof(Code.Cmovs_r16_rm16)], mnemonic[nameof(Mnemonic.Cmovs)]),
				(code[nameof(Code.Cmovs_r32_rm32)], mnemonic[nameof(Mnemonic.Cmovs)]),
				(code[nameof(Code.Cmovs_r64_rm64)], mnemonic[nameof(Mnemonic.Cmovs)]),
				(code[nameof(Code.Cmovns_r16_rm16)], mnemonic[nameof(Mnemonic.Cmovns)]),
				(code[nameof(Code.Cmovns_r32_rm32)], mnemonic[nameof(Mnemonic.Cmovns)]),
				(code[nameof(Code.Cmovns_r64_rm64)], mnemonic[nameof(Mnemonic.Cmovns)]),
				(code[nameof(Code.Cmovp_r16_rm16)], mnemonic[nameof(Mnemonic.Cmovp)]),
				(code[nameof(Code.Cmovp_r32_rm32)], mnemonic[nameof(Mnemonic.Cmovp)]),
				(code[nameof(Code.Cmovp_r64_rm64)], mnemonic[nameof(Mnemonic.Cmovp)]),
				(code[nameof(Code.Cmovnp_r16_rm16)], mnemonic[nameof(Mnemonic.Cmovnp)]),
				(code[nameof(Code.Cmovnp_r32_rm32)], mnemonic[nameof(Mnemonic.Cmovnp)]),
				(code[nameof(Code.Cmovnp_r64_rm64)], mnemonic[nameof(Mnemonic.Cmovnp)]),
				(code[nameof(Code.Cmovl_r16_rm16)], mnemonic[nameof(Mnemonic.Cmovl)]),
				(code[nameof(Code.Cmovl_r32_rm32)], mnemonic[nameof(Mnemonic.Cmovl)]),
				(code[nameof(Code.Cmovl_r64_rm64)], mnemonic[nameof(Mnemonic.Cmovl)]),
				(code[nameof(Code.Cmovge_r16_rm16)], mnemonic[nameof(Mnemonic.Cmovge)]),
				(code[nameof(Code.Cmovge_r32_rm32)], mnemonic[nameof(Mnemonic.Cmovge)]),
				(code[nameof(Code.Cmovge_r64_rm64)], mnemonic[nameof(Mnemonic.Cmovge)]),
				(code[nameof(Code.Cmovle_r16_rm16)], mnemonic[nameof(Mnemonic.Cmovle)]),
				(code[nameof(Code.Cmovle_r32_rm32)], mnemonic[nameof(Mnemonic.Cmovle)]),
				(code[nameof(Code.Cmovle_r64_rm64)], mnemonic[nameof(Mnemonic.Cmovle)]),
				(code[nameof(Code.Cmovg_r16_rm16)], mnemonic[nameof(Mnemonic.Cmovg)]),
				(code[nameof(Code.Cmovg_r32_rm32)], mnemonic[nameof(Mnemonic.Cmovg)]),
				(code[nameof(Code.Cmovg_r64_rm64)], mnemonic[nameof(Mnemonic.Cmovg)]),
				(code[nameof(Code.VEX_Kandw_k_k_k)], mnemonic[nameof(Mnemonic.Kandw)]),
				(code[nameof(Code.VEX_Kandq_k_k_k)], mnemonic[nameof(Mnemonic.Kandq)]),
				(code[nameof(Code.VEX_Kandb_k_k_k)], mnemonic[nameof(Mnemonic.Kandb)]),
				(code[nameof(Code.VEX_Kandd_k_k_k)], mnemonic[nameof(Mnemonic.Kandd)]),
				(code[nameof(Code.VEX_Kandnw_k_k_k)], mnemonic[nameof(Mnemonic.Kandnw)]),
				(code[nameof(Code.VEX_Kandnq_k_k_k)], mnemonic[nameof(Mnemonic.Kandnq)]),
				(code[nameof(Code.VEX_Kandnb_k_k_k)], mnemonic[nameof(Mnemonic.Kandnb)]),
				(code[nameof(Code.VEX_Kandnd_k_k_k)], mnemonic[nameof(Mnemonic.Kandnd)]),
				(code[nameof(Code.VEX_Knotw_k_k)], mnemonic[nameof(Mnemonic.Knotw)]),
				(code[nameof(Code.VEX_Knotq_k_k)], mnemonic[nameof(Mnemonic.Knotq)]),
				(code[nameof(Code.VEX_Knotb_k_k)], mnemonic[nameof(Mnemonic.Knotb)]),
				(code[nameof(Code.VEX_Knotd_k_k)], mnemonic[nameof(Mnemonic.Knotd)]),
				(code[nameof(Code.VEX_Korw_k_k_k)], mnemonic[nameof(Mnemonic.Korw)]),
				(code[nameof(Code.VEX_Korq_k_k_k)], mnemonic[nameof(Mnemonic.Korq)]),
				(code[nameof(Code.VEX_Korb_k_k_k)], mnemonic[nameof(Mnemonic.Korb)]),
				(code[nameof(Code.VEX_Kord_k_k_k)], mnemonic[nameof(Mnemonic.Kord)]),
				(code[nameof(Code.VEX_Kxnorw_k_k_k)], mnemonic[nameof(Mnemonic.Kxnorw)]),
				(code[nameof(Code.VEX_Kxnorq_k_k_k)], mnemonic[nameof(Mnemonic.Kxnorq)]),
				(code[nameof(Code.VEX_Kxnorb_k_k_k)], mnemonic[nameof(Mnemonic.Kxnorb)]),
				(code[nameof(Code.VEX_Kxnord_k_k_k)], mnemonic[nameof(Mnemonic.Kxnord)]),
				(code[nameof(Code.VEX_Kxorw_k_k_k)], mnemonic[nameof(Mnemonic.Kxorw)]),
				(code[nameof(Code.VEX_Kxorq_k_k_k)], mnemonic[nameof(Mnemonic.Kxorq)]),
				(code[nameof(Code.VEX_Kxorb_k_k_k)], mnemonic[nameof(Mnemonic.Kxorb)]),
				(code[nameof(Code.VEX_Kxord_k_k_k)], mnemonic[nameof(Mnemonic.Kxord)]),
				(code[nameof(Code.VEX_Kaddw_k_k_k)], mnemonic[nameof(Mnemonic.Kaddw)]),
				(code[nameof(Code.VEX_Kaddq_k_k_k)], mnemonic[nameof(Mnemonic.Kaddq)]),
				(code[nameof(Code.VEX_Kaddb_k_k_k)], mnemonic[nameof(Mnemonic.Kaddb)]),
				(code[nameof(Code.VEX_Kaddd_k_k_k)], mnemonic[nameof(Mnemonic.Kaddd)]),
				(code[nameof(Code.VEX_Kunpckwd_k_k_k)], mnemonic[nameof(Mnemonic.Kunpckwd)]),
				(code[nameof(Code.VEX_Kunpckdq_k_k_k)], mnemonic[nameof(Mnemonic.Kunpckdq)]),
				(code[nameof(Code.VEX_Kunpckbw_k_k_k)], mnemonic[nameof(Mnemonic.Kunpckbw)]),
				(code[nameof(Code.Movmskps_r32_xmm)], mnemonic[nameof(Mnemonic.Movmskps)]),
				(code[nameof(Code.Movmskps_r64_xmm)], mnemonic[nameof(Mnemonic.Movmskps)]),
				(code[nameof(Code.VEX_Vmovmskps_r32_xmm)], mnemonic[nameof(Mnemonic.Vmovmskps)]),
				(code[nameof(Code.VEX_Vmovmskps_r64_xmm)], mnemonic[nameof(Mnemonic.Vmovmskps)]),
				(code[nameof(Code.VEX_Vmovmskps_r32_ymm)], mnemonic[nameof(Mnemonic.Vmovmskps)]),
				(code[nameof(Code.VEX_Vmovmskps_r64_ymm)], mnemonic[nameof(Mnemonic.Vmovmskps)]),
				(code[nameof(Code.Movmskpd_r32_xmm)], mnemonic[nameof(Mnemonic.Movmskpd)]),
				(code[nameof(Code.Movmskpd_r64_xmm)], mnemonic[nameof(Mnemonic.Movmskpd)]),
				(code[nameof(Code.VEX_Vmovmskpd_r32_xmm)], mnemonic[nameof(Mnemonic.Vmovmskpd)]),
				(code[nameof(Code.VEX_Vmovmskpd_r64_xmm)], mnemonic[nameof(Mnemonic.Vmovmskpd)]),
				(code[nameof(Code.VEX_Vmovmskpd_r32_ymm)], mnemonic[nameof(Mnemonic.Vmovmskpd)]),
				(code[nameof(Code.VEX_Vmovmskpd_r64_ymm)], mnemonic[nameof(Mnemonic.Vmovmskpd)]),
				(code[nameof(Code.Sqrtps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Sqrtps)]),
				(code[nameof(Code.VEX_Vsqrtps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vsqrtps)]),
				(code[nameof(Code.VEX_Vsqrtps_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vsqrtps)]),
				(code[nameof(Code.EVEX_Vsqrtps_xmm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vsqrtps)]),
				(code[nameof(Code.EVEX_Vsqrtps_ymm_k1z_ymmm256b32)], mnemonic[nameof(Mnemonic.Vsqrtps)]),
				(code[nameof(Code.EVEX_Vsqrtps_zmm_k1z_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vsqrtps)]),
				(code[nameof(Code.Sqrtpd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Sqrtpd)]),
				(code[nameof(Code.VEX_Vsqrtpd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vsqrtpd)]),
				(code[nameof(Code.VEX_Vsqrtpd_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vsqrtpd)]),
				(code[nameof(Code.EVEX_Vsqrtpd_xmm_k1z_xmmm128b64)], mnemonic[nameof(Mnemonic.Vsqrtpd)]),
				(code[nameof(Code.EVEX_Vsqrtpd_ymm_k1z_ymmm256b64)], mnemonic[nameof(Mnemonic.Vsqrtpd)]),
				(code[nameof(Code.EVEX_Vsqrtpd_zmm_k1z_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vsqrtpd)]),
				(code[nameof(Code.Sqrtss_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Sqrtss)]),
				(code[nameof(Code.VEX_Vsqrtss_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vsqrtss)]),
				(code[nameof(Code.EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er)], mnemonic[nameof(Mnemonic.Vsqrtss)]),
				(code[nameof(Code.Sqrtsd_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Sqrtsd)]),
				(code[nameof(Code.VEX_Vsqrtsd_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vsqrtsd)]),
				(code[nameof(Code.EVEX_Vsqrtsd_xmm_k1z_xmm_xmmm64_er)], mnemonic[nameof(Mnemonic.Vsqrtsd)]),
				(code[nameof(Code.Rsqrtps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Rsqrtps)]),
				(code[nameof(Code.VEX_Vrsqrtps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vrsqrtps)]),
				(code[nameof(Code.VEX_Vrsqrtps_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vrsqrtps)]),
				(code[nameof(Code.Rsqrtss_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Rsqrtss)]),
				(code[nameof(Code.VEX_Vrsqrtss_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vrsqrtss)]),
				(code[nameof(Code.Rcpps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Rcpps)]),
				(code[nameof(Code.VEX_Vrcpps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vrcpps)]),
				(code[nameof(Code.VEX_Vrcpps_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vrcpps)]),
				(code[nameof(Code.Rcpss_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Rcpss)]),
				(code[nameof(Code.VEX_Vrcpss_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vrcpss)]),
				(code[nameof(Code.Andps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Andps)]),
				(code[nameof(Code.VEX_Vandps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vandps)]),
				(code[nameof(Code.VEX_Vandps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vandps)]),
				(code[nameof(Code.EVEX_Vandps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vandps)]),
				(code[nameof(Code.EVEX_Vandps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vandps)]),
				(code[nameof(Code.EVEX_Vandps_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vandps)]),
				(code[nameof(Code.Andpd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Andpd)]),
				(code[nameof(Code.VEX_Vandpd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vandpd)]),
				(code[nameof(Code.VEX_Vandpd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vandpd)]),
				(code[nameof(Code.EVEX_Vandpd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vandpd)]),
				(code[nameof(Code.EVEX_Vandpd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vandpd)]),
				(code[nameof(Code.EVEX_Vandpd_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vandpd)]),
				(code[nameof(Code.Andnps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Andnps)]),
				(code[nameof(Code.VEX_Vandnps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vandnps)]),
				(code[nameof(Code.VEX_Vandnps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vandnps)]),
				(code[nameof(Code.EVEX_Vandnps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vandnps)]),
				(code[nameof(Code.EVEX_Vandnps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vandnps)]),
				(code[nameof(Code.EVEX_Vandnps_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vandnps)]),
				(code[nameof(Code.Andnpd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Andnpd)]),
				(code[nameof(Code.VEX_Vandnpd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vandnpd)]),
				(code[nameof(Code.VEX_Vandnpd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vandnpd)]),
				(code[nameof(Code.EVEX_Vandnpd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vandnpd)]),
				(code[nameof(Code.EVEX_Vandnpd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vandnpd)]),
				(code[nameof(Code.EVEX_Vandnpd_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vandnpd)]),
				(code[nameof(Code.Orps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Orps)]),
				(code[nameof(Code.VEX_Vorps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vorps)]),
				(code[nameof(Code.VEX_Vorps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vorps)]),
				(code[nameof(Code.EVEX_Vorps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vorps)]),
				(code[nameof(Code.EVEX_Vorps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vorps)]),
				(code[nameof(Code.EVEX_Vorps_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vorps)]),
				(code[nameof(Code.Orpd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Orpd)]),
				(code[nameof(Code.VEX_Vorpd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vorpd)]),
				(code[nameof(Code.VEX_Vorpd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vorpd)]),
				(code[nameof(Code.EVEX_Vorpd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vorpd)]),
				(code[nameof(Code.EVEX_Vorpd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vorpd)]),
				(code[nameof(Code.EVEX_Vorpd_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vorpd)]),
				(code[nameof(Code.Xorps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Xorps)]),
				(code[nameof(Code.VEX_Vxorps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vxorps)]),
				(code[nameof(Code.VEX_Vxorps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vxorps)]),
				(code[nameof(Code.EVEX_Vxorps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vxorps)]),
				(code[nameof(Code.EVEX_Vxorps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vxorps)]),
				(code[nameof(Code.EVEX_Vxorps_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vxorps)]),
				(code[nameof(Code.Xorpd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Xorpd)]),
				(code[nameof(Code.VEX_Vxorpd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vxorpd)]),
				(code[nameof(Code.VEX_Vxorpd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vxorpd)]),
				(code[nameof(Code.EVEX_Vxorpd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vxorpd)]),
				(code[nameof(Code.EVEX_Vxorpd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vxorpd)]),
				(code[nameof(Code.EVEX_Vxorpd_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vxorpd)]),
				(code[nameof(Code.Addps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Addps)]),
				(code[nameof(Code.VEX_Vaddps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vaddps)]),
				(code[nameof(Code.VEX_Vaddps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vaddps)]),
				(code[nameof(Code.EVEX_Vaddps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vaddps)]),
				(code[nameof(Code.EVEX_Vaddps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vaddps)]),
				(code[nameof(Code.EVEX_Vaddps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vaddps)]),
				(code[nameof(Code.Addpd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Addpd)]),
				(code[nameof(Code.VEX_Vaddpd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vaddpd)]),
				(code[nameof(Code.VEX_Vaddpd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vaddpd)]),
				(code[nameof(Code.EVEX_Vaddpd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vaddpd)]),
				(code[nameof(Code.EVEX_Vaddpd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vaddpd)]),
				(code[nameof(Code.EVEX_Vaddpd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vaddpd)]),
				(code[nameof(Code.Addss_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Addss)]),
				(code[nameof(Code.VEX_Vaddss_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vaddss)]),
				(code[nameof(Code.EVEX_Vaddss_xmm_k1z_xmm_xmmm32_er)], mnemonic[nameof(Mnemonic.Vaddss)]),
				(code[nameof(Code.Addsd_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Addsd)]),
				(code[nameof(Code.VEX_Vaddsd_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vaddsd)]),
				(code[nameof(Code.EVEX_Vaddsd_xmm_k1z_xmm_xmmm64_er)], mnemonic[nameof(Mnemonic.Vaddsd)]),
				(code[nameof(Code.Mulps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Mulps)]),
				(code[nameof(Code.VEX_Vmulps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vmulps)]),
				(code[nameof(Code.VEX_Vmulps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vmulps)]),
				(code[nameof(Code.EVEX_Vmulps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vmulps)]),
				(code[nameof(Code.EVEX_Vmulps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vmulps)]),
				(code[nameof(Code.EVEX_Vmulps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vmulps)]),
				(code[nameof(Code.Mulpd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Mulpd)]),
				(code[nameof(Code.VEX_Vmulpd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vmulpd)]),
				(code[nameof(Code.VEX_Vmulpd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vmulpd)]),
				(code[nameof(Code.EVEX_Vmulpd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vmulpd)]),
				(code[nameof(Code.EVEX_Vmulpd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vmulpd)]),
				(code[nameof(Code.EVEX_Vmulpd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vmulpd)]),
				(code[nameof(Code.Mulss_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Mulss)]),
				(code[nameof(Code.VEX_Vmulss_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vmulss)]),
				(code[nameof(Code.EVEX_Vmulss_xmm_k1z_xmm_xmmm32_er)], mnemonic[nameof(Mnemonic.Vmulss)]),
				(code[nameof(Code.Mulsd_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Mulsd)]),
				(code[nameof(Code.VEX_Vmulsd_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vmulsd)]),
				(code[nameof(Code.EVEX_Vmulsd_xmm_k1z_xmm_xmmm64_er)], mnemonic[nameof(Mnemonic.Vmulsd)]),
				(code[nameof(Code.Cvtps2pd_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Cvtps2pd)]),
				(code[nameof(Code.VEX_Vcvtps2pd_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vcvtps2pd)]),
				(code[nameof(Code.VEX_Vcvtps2pd_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vcvtps2pd)]),
				(code[nameof(Code.EVEX_Vcvtps2pd_xmm_k1z_xmmm64b32)], mnemonic[nameof(Mnemonic.Vcvtps2pd)]),
				(code[nameof(Code.EVEX_Vcvtps2pd_ymm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vcvtps2pd)]),
				(code[nameof(Code.EVEX_Vcvtps2pd_zmm_k1z_ymmm256b32_sae)], mnemonic[nameof(Mnemonic.Vcvtps2pd)]),
				(code[nameof(Code.Cvtpd2ps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Cvtpd2ps)]),
				(code[nameof(Code.VEX_Vcvtpd2ps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vcvtpd2ps)]),
				(code[nameof(Code.VEX_Vcvtpd2ps_xmm_ymmm256)], mnemonic[nameof(Mnemonic.Vcvtpd2ps)]),
				(code[nameof(Code.EVEX_Vcvtpd2ps_xmm_k1z_xmmm128b64)], mnemonic[nameof(Mnemonic.Vcvtpd2ps)]),
				(code[nameof(Code.EVEX_Vcvtpd2ps_xmm_k1z_ymmm256b64)], mnemonic[nameof(Mnemonic.Vcvtpd2ps)]),
				(code[nameof(Code.EVEX_Vcvtpd2ps_ymm_k1z_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vcvtpd2ps)]),
				(code[nameof(Code.Cvtss2sd_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Cvtss2sd)]),
				(code[nameof(Code.VEX_Vcvtss2sd_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vcvtss2sd)]),
				(code[nameof(Code.EVEX_Vcvtss2sd_xmm_k1z_xmm_xmmm32_sae)], mnemonic[nameof(Mnemonic.Vcvtss2sd)]),
				(code[nameof(Code.Cvtsd2ss_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Cvtsd2ss)]),
				(code[nameof(Code.VEX_Vcvtsd2ss_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vcvtsd2ss)]),
				(code[nameof(Code.EVEX_Vcvtsd2ss_xmm_k1z_xmm_xmmm64_er)], mnemonic[nameof(Mnemonic.Vcvtsd2ss)]),
				(code[nameof(Code.Cvtdq2ps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Cvtdq2ps)]),
				(code[nameof(Code.VEX_Vcvtdq2ps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vcvtdq2ps)]),
				(code[nameof(Code.VEX_Vcvtdq2ps_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vcvtdq2ps)]),
				(code[nameof(Code.EVEX_Vcvtdq2ps_xmm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vcvtdq2ps)]),
				(code[nameof(Code.EVEX_Vcvtdq2ps_ymm_k1z_ymmm256b32)], mnemonic[nameof(Mnemonic.Vcvtdq2ps)]),
				(code[nameof(Code.EVEX_Vcvtdq2ps_zmm_k1z_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vcvtdq2ps)]),
				(code[nameof(Code.EVEX_Vcvtqq2ps_xmm_k1z_xmmm128b64)], mnemonic[nameof(Mnemonic.Vcvtqq2ps)]),
				(code[nameof(Code.EVEX_Vcvtqq2ps_xmm_k1z_ymmm256b64)], mnemonic[nameof(Mnemonic.Vcvtqq2ps)]),
				(code[nameof(Code.EVEX_Vcvtqq2ps_ymm_k1z_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vcvtqq2ps)]),
				(code[nameof(Code.Cvtps2dq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Cvtps2dq)]),
				(code[nameof(Code.VEX_Vcvtps2dq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vcvtps2dq)]),
				(code[nameof(Code.VEX_Vcvtps2dq_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vcvtps2dq)]),
				(code[nameof(Code.EVEX_Vcvtps2dq_xmm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vcvtps2dq)]),
				(code[nameof(Code.EVEX_Vcvtps2dq_ymm_k1z_ymmm256b32)], mnemonic[nameof(Mnemonic.Vcvtps2dq)]),
				(code[nameof(Code.EVEX_Vcvtps2dq_zmm_k1z_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vcvtps2dq)]),
				(code[nameof(Code.Cvttps2dq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Cvttps2dq)]),
				(code[nameof(Code.VEX_Vcvttps2dq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vcvttps2dq)]),
				(code[nameof(Code.VEX_Vcvttps2dq_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vcvttps2dq)]),
				(code[nameof(Code.EVEX_Vcvttps2dq_xmm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vcvttps2dq)]),
				(code[nameof(Code.EVEX_Vcvttps2dq_ymm_k1z_ymmm256b32)], mnemonic[nameof(Mnemonic.Vcvttps2dq)]),
				(code[nameof(Code.EVEX_Vcvttps2dq_zmm_k1z_zmmm512b32_sae)], mnemonic[nameof(Mnemonic.Vcvttps2dq)]),
				(code[nameof(Code.Subps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Subps)]),
				(code[nameof(Code.VEX_Vsubps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vsubps)]),
				(code[nameof(Code.VEX_Vsubps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vsubps)]),
				(code[nameof(Code.EVEX_Vsubps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vsubps)]),
				(code[nameof(Code.EVEX_Vsubps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vsubps)]),
				(code[nameof(Code.EVEX_Vsubps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vsubps)]),
				(code[nameof(Code.Subpd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Subpd)]),
				(code[nameof(Code.VEX_Vsubpd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vsubpd)]),
				(code[nameof(Code.VEX_Vsubpd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vsubpd)]),
				(code[nameof(Code.EVEX_Vsubpd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vsubpd)]),
				(code[nameof(Code.EVEX_Vsubpd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vsubpd)]),
				(code[nameof(Code.EVEX_Vsubpd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vsubpd)]),
				(code[nameof(Code.Subss_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Subss)]),
				(code[nameof(Code.VEX_Vsubss_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vsubss)]),
				(code[nameof(Code.EVEX_Vsubss_xmm_k1z_xmm_xmmm32_er)], mnemonic[nameof(Mnemonic.Vsubss)]),
				(code[nameof(Code.Subsd_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Subsd)]),
				(code[nameof(Code.VEX_Vsubsd_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vsubsd)]),
				(code[nameof(Code.EVEX_Vsubsd_xmm_k1z_xmm_xmmm64_er)], mnemonic[nameof(Mnemonic.Vsubsd)]),
				(code[nameof(Code.Minps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Minps)]),
				(code[nameof(Code.VEX_Vminps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vminps)]),
				(code[nameof(Code.VEX_Vminps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vminps)]),
				(code[nameof(Code.EVEX_Vminps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vminps)]),
				(code[nameof(Code.EVEX_Vminps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vminps)]),
				(code[nameof(Code.EVEX_Vminps_zmm_k1z_zmm_zmmm512b32_sae)], mnemonic[nameof(Mnemonic.Vminps)]),
				(code[nameof(Code.Minpd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Minpd)]),
				(code[nameof(Code.VEX_Vminpd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vminpd)]),
				(code[nameof(Code.VEX_Vminpd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vminpd)]),
				(code[nameof(Code.EVEX_Vminpd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vminpd)]),
				(code[nameof(Code.EVEX_Vminpd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vminpd)]),
				(code[nameof(Code.EVEX_Vminpd_zmm_k1z_zmm_zmmm512b64_sae)], mnemonic[nameof(Mnemonic.Vminpd)]),
				(code[nameof(Code.Minss_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Minss)]),
				(code[nameof(Code.VEX_Vminss_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vminss)]),
				(code[nameof(Code.EVEX_Vminss_xmm_k1z_xmm_xmmm32_sae)], mnemonic[nameof(Mnemonic.Vminss)]),
				(code[nameof(Code.Minsd_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Minsd)]),
				(code[nameof(Code.VEX_Vminsd_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vminsd)]),
				(code[nameof(Code.EVEX_Vminsd_xmm_k1z_xmm_xmmm64_sae)], mnemonic[nameof(Mnemonic.Vminsd)]),
				(code[nameof(Code.Divps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Divps)]),
				(code[nameof(Code.VEX_Vdivps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vdivps)]),
				(code[nameof(Code.VEX_Vdivps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vdivps)]),
				(code[nameof(Code.EVEX_Vdivps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vdivps)]),
				(code[nameof(Code.EVEX_Vdivps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vdivps)]),
				(code[nameof(Code.EVEX_Vdivps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vdivps)]),
				(code[nameof(Code.Divpd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Divpd)]),
				(code[nameof(Code.VEX_Vdivpd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vdivpd)]),
				(code[nameof(Code.VEX_Vdivpd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vdivpd)]),
				(code[nameof(Code.EVEX_Vdivpd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vdivpd)]),
				(code[nameof(Code.EVEX_Vdivpd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vdivpd)]),
				(code[nameof(Code.EVEX_Vdivpd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vdivpd)]),
				(code[nameof(Code.Divss_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Divss)]),
				(code[nameof(Code.VEX_Vdivss_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vdivss)]),
				(code[nameof(Code.EVEX_Vdivss_xmm_k1z_xmm_xmmm32_er)], mnemonic[nameof(Mnemonic.Vdivss)]),
				(code[nameof(Code.Divsd_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Divsd)]),
				(code[nameof(Code.VEX_Vdivsd_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vdivsd)]),
				(code[nameof(Code.EVEX_Vdivsd_xmm_k1z_xmm_xmmm64_er)], mnemonic[nameof(Mnemonic.Vdivsd)]),
				(code[nameof(Code.Maxps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Maxps)]),
				(code[nameof(Code.VEX_Vmaxps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vmaxps)]),
				(code[nameof(Code.VEX_Vmaxps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vmaxps)]),
				(code[nameof(Code.EVEX_Vmaxps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vmaxps)]),
				(code[nameof(Code.EVEX_Vmaxps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vmaxps)]),
				(code[nameof(Code.EVEX_Vmaxps_zmm_k1z_zmm_zmmm512b32_sae)], mnemonic[nameof(Mnemonic.Vmaxps)]),
				(code[nameof(Code.Maxpd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Maxpd)]),
				(code[nameof(Code.VEX_Vmaxpd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vmaxpd)]),
				(code[nameof(Code.VEX_Vmaxpd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vmaxpd)]),
				(code[nameof(Code.EVEX_Vmaxpd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vmaxpd)]),
				(code[nameof(Code.EVEX_Vmaxpd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vmaxpd)]),
				(code[nameof(Code.EVEX_Vmaxpd_zmm_k1z_zmm_zmmm512b64_sae)], mnemonic[nameof(Mnemonic.Vmaxpd)]),
				(code[nameof(Code.Maxss_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Maxss)]),
				(code[nameof(Code.VEX_Vmaxss_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vmaxss)]),
				(code[nameof(Code.EVEX_Vmaxss_xmm_k1z_xmm_xmmm32_sae)], mnemonic[nameof(Mnemonic.Vmaxss)]),
				(code[nameof(Code.Maxsd_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Maxsd)]),
				(code[nameof(Code.VEX_Vmaxsd_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vmaxsd)]),
				(code[nameof(Code.EVEX_Vmaxsd_xmm_k1z_xmm_xmmm64_sae)], mnemonic[nameof(Mnemonic.Vmaxsd)]),
				(code[nameof(Code.Punpcklbw_mm_mmm32)], mnemonic[nameof(Mnemonic.Punpcklbw)]),
				(code[nameof(Code.Punpcklbw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Punpcklbw)]),
				(code[nameof(Code.VEX_Vpunpcklbw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpunpcklbw)]),
				(code[nameof(Code.VEX_Vpunpcklbw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpunpcklbw)]),
				(code[nameof(Code.EVEX_Vpunpcklbw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpunpcklbw)]),
				(code[nameof(Code.EVEX_Vpunpcklbw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpunpcklbw)]),
				(code[nameof(Code.EVEX_Vpunpcklbw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpunpcklbw)]),
				(code[nameof(Code.Punpcklwd_mm_mmm32)], mnemonic[nameof(Mnemonic.Punpcklwd)]),
				(code[nameof(Code.Punpcklwd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Punpcklwd)]),
				(code[nameof(Code.VEX_Vpunpcklwd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpunpcklwd)]),
				(code[nameof(Code.VEX_Vpunpcklwd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpunpcklwd)]),
				(code[nameof(Code.EVEX_Vpunpcklwd_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpunpcklwd)]),
				(code[nameof(Code.EVEX_Vpunpcklwd_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpunpcklwd)]),
				(code[nameof(Code.EVEX_Vpunpcklwd_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpunpcklwd)]),
				(code[nameof(Code.Punpckldq_mm_mmm32)], mnemonic[nameof(Mnemonic.Punpckldq)]),
				(code[nameof(Code.Punpckldq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Punpckldq)]),
				(code[nameof(Code.VEX_Vpunpckldq_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpunpckldq)]),
				(code[nameof(Code.VEX_Vpunpckldq_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpunpckldq)]),
				(code[nameof(Code.EVEX_Vpunpckldq_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpunpckldq)]),
				(code[nameof(Code.EVEX_Vpunpckldq_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpunpckldq)]),
				(code[nameof(Code.EVEX_Vpunpckldq_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpunpckldq)]),
				(code[nameof(Code.Packsswb_mm_mmm64)], mnemonic[nameof(Mnemonic.Packsswb)]),
				(code[nameof(Code.Packsswb_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Packsswb)]),
				(code[nameof(Code.VEX_Vpacksswb_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpacksswb)]),
				(code[nameof(Code.VEX_Vpacksswb_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpacksswb)]),
				(code[nameof(Code.EVEX_Vpacksswb_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpacksswb)]),
				(code[nameof(Code.EVEX_Vpacksswb_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpacksswb)]),
				(code[nameof(Code.EVEX_Vpacksswb_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpacksswb)]),
				(code[nameof(Code.Pcmpgtb_mm_mmm64)], mnemonic[nameof(Mnemonic.Pcmpgtb)]),
				(code[nameof(Code.Pcmpgtb_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pcmpgtb)]),
				(code[nameof(Code.VEX_Vpcmpgtb_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpcmpgtb)]),
				(code[nameof(Code.VEX_Vpcmpgtb_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpcmpgtb)]),
				(code[nameof(Code.EVEX_Vpcmpgtb_k_k1_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpcmpgtb)]),
				(code[nameof(Code.EVEX_Vpcmpgtb_k_k1_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpcmpgtb)]),
				(code[nameof(Code.EVEX_Vpcmpgtb_k_k1_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpcmpgtb)]),
				(code[nameof(Code.Pcmpgtw_mm_mmm64)], mnemonic[nameof(Mnemonic.Pcmpgtw)]),
				(code[nameof(Code.Pcmpgtw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pcmpgtw)]),
				(code[nameof(Code.VEX_Vpcmpgtw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpcmpgtw)]),
				(code[nameof(Code.VEX_Vpcmpgtw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpcmpgtw)]),
				(code[nameof(Code.EVEX_Vpcmpgtw_k_k1_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpcmpgtw)]),
				(code[nameof(Code.EVEX_Vpcmpgtw_k_k1_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpcmpgtw)]),
				(code[nameof(Code.EVEX_Vpcmpgtw_k_k1_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpcmpgtw)]),
				(code[nameof(Code.Pcmpgtd_mm_mmm64)], mnemonic[nameof(Mnemonic.Pcmpgtd)]),
				(code[nameof(Code.Pcmpgtd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pcmpgtd)]),
				(code[nameof(Code.VEX_Vpcmpgtd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpcmpgtd)]),
				(code[nameof(Code.VEX_Vpcmpgtd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpcmpgtd)]),
				(code[nameof(Code.EVEX_Vpcmpgtd_k_k1_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpcmpgtd)]),
				(code[nameof(Code.EVEX_Vpcmpgtd_k_k1_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpcmpgtd)]),
				(code[nameof(Code.EVEX_Vpcmpgtd_k_k1_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpcmpgtd)]),
				(code[nameof(Code.Packuswb_mm_mmm64)], mnemonic[nameof(Mnemonic.Packuswb)]),
				(code[nameof(Code.Packuswb_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Packuswb)]),
				(code[nameof(Code.VEX_Vpackuswb_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpackuswb)]),
				(code[nameof(Code.VEX_Vpackuswb_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpackuswb)]),
				(code[nameof(Code.EVEX_Vpackuswb_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpackuswb)]),
				(code[nameof(Code.EVEX_Vpackuswb_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpackuswb)]),
				(code[nameof(Code.EVEX_Vpackuswb_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpackuswb)]),
				(code[nameof(Code.Punpckhbw_mm_mmm64)], mnemonic[nameof(Mnemonic.Punpckhbw)]),
				(code[nameof(Code.Punpckhbw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Punpckhbw)]),
				(code[nameof(Code.VEX_Vpunpckhbw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpunpckhbw)]),
				(code[nameof(Code.VEX_Vpunpckhbw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpunpckhbw)]),
				(code[nameof(Code.EVEX_Vpunpckhbw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpunpckhbw)]),
				(code[nameof(Code.EVEX_Vpunpckhbw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpunpckhbw)]),
				(code[nameof(Code.EVEX_Vpunpckhbw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpunpckhbw)]),
				(code[nameof(Code.Punpckhwd_mm_mmm64)], mnemonic[nameof(Mnemonic.Punpckhwd)]),
				(code[nameof(Code.Punpckhwd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Punpckhwd)]),
				(code[nameof(Code.VEX_Vpunpckhwd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpunpckhwd)]),
				(code[nameof(Code.VEX_Vpunpckhwd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpunpckhwd)]),
				(code[nameof(Code.EVEX_Vpunpckhwd_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpunpckhwd)]),
				(code[nameof(Code.EVEX_Vpunpckhwd_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpunpckhwd)]),
				(code[nameof(Code.EVEX_Vpunpckhwd_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpunpckhwd)]),
				(code[nameof(Code.Punpckhdq_mm_mmm64)], mnemonic[nameof(Mnemonic.Punpckhdq)]),
				(code[nameof(Code.Punpckhdq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Punpckhdq)]),
				(code[nameof(Code.VEX_Vpunpckhdq_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpunpckhdq)]),
				(code[nameof(Code.VEX_Vpunpckhdq_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpunpckhdq)]),
				(code[nameof(Code.EVEX_Vpunpckhdq_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpunpckhdq)]),
				(code[nameof(Code.EVEX_Vpunpckhdq_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpunpckhdq)]),
				(code[nameof(Code.EVEX_Vpunpckhdq_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpunpckhdq)]),
				(code[nameof(Code.Packssdw_mm_mmm64)], mnemonic[nameof(Mnemonic.Packssdw)]),
				(code[nameof(Code.Packssdw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Packssdw)]),
				(code[nameof(Code.VEX_Vpackssdw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpackssdw)]),
				(code[nameof(Code.VEX_Vpackssdw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpackssdw)]),
				(code[nameof(Code.EVEX_Vpackssdw_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpackssdw)]),
				(code[nameof(Code.EVEX_Vpackssdw_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpackssdw)]),
				(code[nameof(Code.EVEX_Vpackssdw_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpackssdw)]),
				(code[nameof(Code.Punpcklqdq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Punpcklqdq)]),
				(code[nameof(Code.VEX_Vpunpcklqdq_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpunpcklqdq)]),
				(code[nameof(Code.VEX_Vpunpcklqdq_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpunpcklqdq)]),
				(code[nameof(Code.EVEX_Vpunpcklqdq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpunpcklqdq)]),
				(code[nameof(Code.EVEX_Vpunpcklqdq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpunpcklqdq)]),
				(code[nameof(Code.EVEX_Vpunpcklqdq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpunpcklqdq)]),
				(code[nameof(Code.Punpckhqdq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Punpckhqdq)]),
				(code[nameof(Code.VEX_Vpunpckhqdq_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpunpckhqdq)]),
				(code[nameof(Code.VEX_Vpunpckhqdq_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpunpckhqdq)]),
				(code[nameof(Code.EVEX_Vpunpckhqdq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpunpckhqdq)]),
				(code[nameof(Code.EVEX_Vpunpckhqdq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpunpckhqdq)]),
				(code[nameof(Code.EVEX_Vpunpckhqdq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpunpckhqdq)]),
				(code[nameof(Code.Movd_mm_rm32)], mnemonic[nameof(Mnemonic.Movd)]),
				(code[nameof(Code.Movq_mm_rm64)], mnemonic[nameof(Mnemonic.Movq)]),
				(code[nameof(Code.Movd_xmm_rm32)], mnemonic[nameof(Mnemonic.Movd)]),
				(code[nameof(Code.Movq_xmm_rm64)], mnemonic[nameof(Mnemonic.Movq)]),
				(code[nameof(Code.VEX_Vmovd_xmm_rm32)], mnemonic[nameof(Mnemonic.Vmovd)]),
				(code[nameof(Code.VEX_Vmovq_xmm_rm64)], mnemonic[nameof(Mnemonic.Vmovq)]),
				(code[nameof(Code.EVEX_Vmovd_xmm_rm32)], mnemonic[nameof(Mnemonic.Vmovd)]),
				(code[nameof(Code.EVEX_Vmovq_xmm_rm64)], mnemonic[nameof(Mnemonic.Vmovq)]),
				(code[nameof(Code.Movq_mm_mmm64)], mnemonic[nameof(Mnemonic.Movq)]),
				(code[nameof(Code.Movdqa_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Movdqa)]),
				(code[nameof(Code.VEX_Vmovdqa_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vmovdqa)]),
				(code[nameof(Code.VEX_Vmovdqa_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vmovdqa)]),
				(code[nameof(Code.EVEX_Vmovdqa32_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vmovdqa32)]),
				(code[nameof(Code.EVEX_Vmovdqa32_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vmovdqa32)]),
				(code[nameof(Code.EVEX_Vmovdqa32_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vmovdqa32)]),
				(code[nameof(Code.EVEX_Vmovdqa64_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vmovdqa64)]),
				(code[nameof(Code.EVEX_Vmovdqa64_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vmovdqa64)]),
				(code[nameof(Code.EVEX_Vmovdqa64_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vmovdqa64)]),
				(code[nameof(Code.Movdqu_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Movdqu)]),
				(code[nameof(Code.VEX_Vmovdqu_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vmovdqu)]),
				(code[nameof(Code.VEX_Vmovdqu_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vmovdqu)]),
				(code[nameof(Code.EVEX_Vmovdqu32_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vmovdqu32)]),
				(code[nameof(Code.EVEX_Vmovdqu32_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vmovdqu32)]),
				(code[nameof(Code.EVEX_Vmovdqu32_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vmovdqu32)]),
				(code[nameof(Code.EVEX_Vmovdqu64_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vmovdqu64)]),
				(code[nameof(Code.EVEX_Vmovdqu64_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vmovdqu64)]),
				(code[nameof(Code.EVEX_Vmovdqu64_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vmovdqu64)]),
				(code[nameof(Code.EVEX_Vmovdqu8_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vmovdqu8)]),
				(code[nameof(Code.EVEX_Vmovdqu8_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vmovdqu8)]),
				(code[nameof(Code.EVEX_Vmovdqu8_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vmovdqu8)]),
				(code[nameof(Code.EVEX_Vmovdqu16_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vmovdqu16)]),
				(code[nameof(Code.EVEX_Vmovdqu16_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vmovdqu16)]),
				(code[nameof(Code.EVEX_Vmovdqu16_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vmovdqu16)]),
				(code[nameof(Code.Pshufw_mm_mmm64_imm8)], mnemonic[nameof(Mnemonic.Pshufw)]),
				(code[nameof(Code.Pshufd_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Pshufd)]),
				(code[nameof(Code.VEX_Vpshufd_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpshufd)]),
				(code[nameof(Code.VEX_Vpshufd_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpshufd)]),
				(code[nameof(Code.EVEX_Vpshufd_xmm_k1z_xmmm128b32_imm8)], mnemonic[nameof(Mnemonic.Vpshufd)]),
				(code[nameof(Code.EVEX_Vpshufd_ymm_k1z_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vpshufd)]),
				(code[nameof(Code.EVEX_Vpshufd_zmm_k1z_zmmm512b32_imm8)], mnemonic[nameof(Mnemonic.Vpshufd)]),
				(code[nameof(Code.Pshufhw_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Pshufhw)]),
				(code[nameof(Code.VEX_Vpshufhw_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpshufhw)]),
				(code[nameof(Code.VEX_Vpshufhw_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpshufhw)]),
				(code[nameof(Code.EVEX_Vpshufhw_xmm_k1z_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpshufhw)]),
				(code[nameof(Code.EVEX_Vpshufhw_ymm_k1z_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpshufhw)]),
				(code[nameof(Code.EVEX_Vpshufhw_zmm_k1z_zmmm512_imm8)], mnemonic[nameof(Mnemonic.Vpshufhw)]),
				(code[nameof(Code.Pshuflw_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Pshuflw)]),
				(code[nameof(Code.VEX_Vpshuflw_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpshuflw)]),
				(code[nameof(Code.VEX_Vpshuflw_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpshuflw)]),
				(code[nameof(Code.EVEX_Vpshuflw_xmm_k1z_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpshuflw)]),
				(code[nameof(Code.EVEX_Vpshuflw_ymm_k1z_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpshuflw)]),
				(code[nameof(Code.EVEX_Vpshuflw_zmm_k1z_zmmm512_imm8)], mnemonic[nameof(Mnemonic.Vpshuflw)]),
				(code[nameof(Code.Psrlw_mm_imm8)], mnemonic[nameof(Mnemonic.Psrlw)]),
				(code[nameof(Code.Psrlw_xmm_imm8)], mnemonic[nameof(Mnemonic.Psrlw)]),
				(code[nameof(Code.VEX_Vpsrlw_xmm_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpsrlw)]),
				(code[nameof(Code.VEX_Vpsrlw_ymm_ymm_imm8)], mnemonic[nameof(Mnemonic.Vpsrlw)]),
				(code[nameof(Code.EVEX_Vpsrlw_xmm_k1z_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpsrlw)]),
				(code[nameof(Code.EVEX_Vpsrlw_ymm_k1z_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpsrlw)]),
				(code[nameof(Code.EVEX_Vpsrlw_zmm_k1z_zmmm512_imm8)], mnemonic[nameof(Mnemonic.Vpsrlw)]),
				(code[nameof(Code.Psraw_mm_imm8)], mnemonic[nameof(Mnemonic.Psraw)]),
				(code[nameof(Code.Psraw_xmm_imm8)], mnemonic[nameof(Mnemonic.Psraw)]),
				(code[nameof(Code.VEX_Vpsraw_xmm_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpsraw)]),
				(code[nameof(Code.VEX_Vpsraw_ymm_ymm_imm8)], mnemonic[nameof(Mnemonic.Vpsraw)]),
				(code[nameof(Code.EVEX_Vpsraw_xmm_k1z_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpsraw)]),
				(code[nameof(Code.EVEX_Vpsraw_ymm_k1z_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpsraw)]),
				(code[nameof(Code.EVEX_Vpsraw_zmm_k1z_zmmm512_imm8)], mnemonic[nameof(Mnemonic.Vpsraw)]),
				(code[nameof(Code.Psllw_mm_imm8)], mnemonic[nameof(Mnemonic.Psllw)]),
				(code[nameof(Code.Psllw_xmm_imm8)], mnemonic[nameof(Mnemonic.Psllw)]),
				(code[nameof(Code.VEX_Vpsllw_xmm_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpsllw)]),
				(code[nameof(Code.VEX_Vpsllw_ymm_ymm_imm8)], mnemonic[nameof(Mnemonic.Vpsllw)]),
				(code[nameof(Code.EVEX_Vpsllw_xmm_k1z_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpsllw)]),
				(code[nameof(Code.EVEX_Vpsllw_ymm_k1z_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpsllw)]),
				(code[nameof(Code.EVEX_Vpsllw_zmm_k1z_zmmm512_imm8)], mnemonic[nameof(Mnemonic.Vpsllw)]),
				(code[nameof(Code.EVEX_Vprord_xmm_k1z_xmmm128b32_imm8)], mnemonic[nameof(Mnemonic.Vprord)]),
				(code[nameof(Code.EVEX_Vprord_ymm_k1z_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vprord)]),
				(code[nameof(Code.EVEX_Vprord_zmm_k1z_zmmm512b32_imm8)], mnemonic[nameof(Mnemonic.Vprord)]),
				(code[nameof(Code.EVEX_Vprorq_xmm_k1z_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Vprorq)]),
				(code[nameof(Code.EVEX_Vprorq_ymm_k1z_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vprorq)]),
				(code[nameof(Code.EVEX_Vprorq_zmm_k1z_zmmm512b64_imm8)], mnemonic[nameof(Mnemonic.Vprorq)]),
				(code[nameof(Code.EVEX_Vprold_xmm_k1z_xmmm128b32_imm8)], mnemonic[nameof(Mnemonic.Vprold)]),
				(code[nameof(Code.EVEX_Vprold_ymm_k1z_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vprold)]),
				(code[nameof(Code.EVEX_Vprold_zmm_k1z_zmmm512b32_imm8)], mnemonic[nameof(Mnemonic.Vprold)]),
				(code[nameof(Code.EVEX_Vprolq_xmm_k1z_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Vprolq)]),
				(code[nameof(Code.EVEX_Vprolq_ymm_k1z_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vprolq)]),
				(code[nameof(Code.EVEX_Vprolq_zmm_k1z_zmmm512b64_imm8)], mnemonic[nameof(Mnemonic.Vprolq)]),
				(code[nameof(Code.Psrld_mm_imm8)], mnemonic[nameof(Mnemonic.Psrld)]),
				(code[nameof(Code.Psrld_xmm_imm8)], mnemonic[nameof(Mnemonic.Psrld)]),
				(code[nameof(Code.VEX_Vpsrld_xmm_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpsrld)]),
				(code[nameof(Code.VEX_Vpsrld_ymm_ymm_imm8)], mnemonic[nameof(Mnemonic.Vpsrld)]),
				(code[nameof(Code.EVEX_Vpsrld_xmm_k1z_xmmm128b32_imm8)], mnemonic[nameof(Mnemonic.Vpsrld)]),
				(code[nameof(Code.EVEX_Vpsrld_ymm_k1z_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vpsrld)]),
				(code[nameof(Code.EVEX_Vpsrld_zmm_k1z_zmmm512b32_imm8)], mnemonic[nameof(Mnemonic.Vpsrld)]),
				(code[nameof(Code.Psrad_mm_imm8)], mnemonic[nameof(Mnemonic.Psrad)]),
				(code[nameof(Code.Psrad_xmm_imm8)], mnemonic[nameof(Mnemonic.Psrad)]),
				(code[nameof(Code.VEX_Vpsrad_xmm_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpsrad)]),
				(code[nameof(Code.VEX_Vpsrad_ymm_ymm_imm8)], mnemonic[nameof(Mnemonic.Vpsrad)]),
				(code[nameof(Code.EVEX_Vpsrad_xmm_k1z_xmmm128b32_imm8)], mnemonic[nameof(Mnemonic.Vpsrad)]),
				(code[nameof(Code.EVEX_Vpsrad_ymm_k1z_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vpsrad)]),
				(code[nameof(Code.EVEX_Vpsrad_zmm_k1z_zmmm512b32_imm8)], mnemonic[nameof(Mnemonic.Vpsrad)]),
				(code[nameof(Code.EVEX_Vpsraq_xmm_k1z_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Vpsraq)]),
				(code[nameof(Code.EVEX_Vpsraq_ymm_k1z_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vpsraq)]),
				(code[nameof(Code.EVEX_Vpsraq_zmm_k1z_zmmm512b64_imm8)], mnemonic[nameof(Mnemonic.Vpsraq)]),
				(code[nameof(Code.Pslld_mm_imm8)], mnemonic[nameof(Mnemonic.Pslld)]),
				(code[nameof(Code.Pslld_xmm_imm8)], mnemonic[nameof(Mnemonic.Pslld)]),
				(code[nameof(Code.VEX_Vpslld_xmm_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpslld)]),
				(code[nameof(Code.VEX_Vpslld_ymm_ymm_imm8)], mnemonic[nameof(Mnemonic.Vpslld)]),
				(code[nameof(Code.EVEX_Vpslld_xmm_k1z_xmmm128b32_imm8)], mnemonic[nameof(Mnemonic.Vpslld)]),
				(code[nameof(Code.EVEX_Vpslld_ymm_k1z_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vpslld)]),
				(code[nameof(Code.EVEX_Vpslld_zmm_k1z_zmmm512b32_imm8)], mnemonic[nameof(Mnemonic.Vpslld)]),
				(code[nameof(Code.Psrlq_mm_imm8)], mnemonic[nameof(Mnemonic.Psrlq)]),
				(code[nameof(Code.Psrlq_xmm_imm8)], mnemonic[nameof(Mnemonic.Psrlq)]),
				(code[nameof(Code.VEX_Vpsrlq_xmm_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpsrlq)]),
				(code[nameof(Code.VEX_Vpsrlq_ymm_ymm_imm8)], mnemonic[nameof(Mnemonic.Vpsrlq)]),
				(code[nameof(Code.EVEX_Vpsrlq_xmm_k1z_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Vpsrlq)]),
				(code[nameof(Code.EVEX_Vpsrlq_ymm_k1z_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vpsrlq)]),
				(code[nameof(Code.EVEX_Vpsrlq_zmm_k1z_zmmm512b64_imm8)], mnemonic[nameof(Mnemonic.Vpsrlq)]),
				(code[nameof(Code.Psrldq_xmm_imm8)], mnemonic[nameof(Mnemonic.Psrldq)]),
				(code[nameof(Code.VEX_Vpsrldq_xmm_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpsrldq)]),
				(code[nameof(Code.VEX_Vpsrldq_ymm_ymm_imm8)], mnemonic[nameof(Mnemonic.Vpsrldq)]),
				(code[nameof(Code.EVEX_Vpsrldq_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpsrldq)]),
				(code[nameof(Code.EVEX_Vpsrldq_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpsrldq)]),
				(code[nameof(Code.EVEX_Vpsrldq_zmm_zmmm512_imm8)], mnemonic[nameof(Mnemonic.Vpsrldq)]),
				(code[nameof(Code.Psllq_mm_imm8)], mnemonic[nameof(Mnemonic.Psllq)]),
				(code[nameof(Code.Psllq_xmm_imm8)], mnemonic[nameof(Mnemonic.Psllq)]),
				(code[nameof(Code.VEX_Vpsllq_xmm_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpsllq)]),
				(code[nameof(Code.VEX_Vpsllq_ymm_ymm_imm8)], mnemonic[nameof(Mnemonic.Vpsllq)]),
				(code[nameof(Code.EVEX_Vpsllq_xmm_k1z_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Vpsllq)]),
				(code[nameof(Code.EVEX_Vpsllq_ymm_k1z_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vpsllq)]),
				(code[nameof(Code.EVEX_Vpsllq_zmm_k1z_zmmm512b64_imm8)], mnemonic[nameof(Mnemonic.Vpsllq)]),
				(code[nameof(Code.Pslldq_xmm_imm8)], mnemonic[nameof(Mnemonic.Pslldq)]),
				(code[nameof(Code.VEX_Vpslldq_xmm_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpslldq)]),
				(code[nameof(Code.VEX_Vpslldq_ymm_ymm_imm8)], mnemonic[nameof(Mnemonic.Vpslldq)]),
				(code[nameof(Code.EVEX_Vpslldq_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpslldq)]),
				(code[nameof(Code.EVEX_Vpslldq_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpslldq)]),
				(code[nameof(Code.EVEX_Vpslldq_zmm_zmmm512_imm8)], mnemonic[nameof(Mnemonic.Vpslldq)]),
				(code[nameof(Code.Pcmpeqb_mm_mmm64)], mnemonic[nameof(Mnemonic.Pcmpeqb)]),
				(code[nameof(Code.Pcmpeqb_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pcmpeqb)]),
				(code[nameof(Code.VEX_Vpcmpeqb_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpcmpeqb)]),
				(code[nameof(Code.VEX_Vpcmpeqb_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpcmpeqb)]),
				(code[nameof(Code.EVEX_Vpcmpeqb_k_k1_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpcmpeqb)]),
				(code[nameof(Code.EVEX_Vpcmpeqb_k_k1_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpcmpeqb)]),
				(code[nameof(Code.EVEX_Vpcmpeqb_k_k1_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpcmpeqb)]),
				(code[nameof(Code.Pcmpeqw_mm_mmm64)], mnemonic[nameof(Mnemonic.Pcmpeqw)]),
				(code[nameof(Code.Pcmpeqw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pcmpeqw)]),
				(code[nameof(Code.VEX_Vpcmpeqw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpcmpeqw)]),
				(code[nameof(Code.VEX_Vpcmpeqw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpcmpeqw)]),
				(code[nameof(Code.EVEX_Vpcmpeqw_k_k1_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpcmpeqw)]),
				(code[nameof(Code.EVEX_Vpcmpeqw_k_k1_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpcmpeqw)]),
				(code[nameof(Code.EVEX_Vpcmpeqw_k_k1_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpcmpeqw)]),
				(code[nameof(Code.Pcmpeqd_mm_mmm64)], mnemonic[nameof(Mnemonic.Pcmpeqd)]),
				(code[nameof(Code.Pcmpeqd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pcmpeqd)]),
				(code[nameof(Code.VEX_Vpcmpeqd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpcmpeqd)]),
				(code[nameof(Code.VEX_Vpcmpeqd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpcmpeqd)]),
				(code[nameof(Code.EVEX_Vpcmpeqd_k_k1_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpcmpeqd)]),
				(code[nameof(Code.EVEX_Vpcmpeqd_k_k1_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpcmpeqd)]),
				(code[nameof(Code.EVEX_Vpcmpeqd_k_k1_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpcmpeqd)]),
				(code[nameof(Code.Emms)], mnemonic[nameof(Mnemonic.Emms)]),
				(code[nameof(Code.VEX_Vzeroupper)], mnemonic[nameof(Mnemonic.Vzeroupper)]),
				(code[nameof(Code.VEX_Vzeroall)], mnemonic[nameof(Mnemonic.Vzeroall)]),
				(code[nameof(Code.Vmread_rm32_r32)], mnemonic[nameof(Mnemonic.Vmread)]),
				(code[nameof(Code.Vmread_rm64_r64)], mnemonic[nameof(Mnemonic.Vmread)]),
				(code[nameof(Code.EVEX_Vcvttps2udq_xmm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vcvttps2udq)]),
				(code[nameof(Code.EVEX_Vcvttps2udq_ymm_k1z_ymmm256b32)], mnemonic[nameof(Mnemonic.Vcvttps2udq)]),
				(code[nameof(Code.EVEX_Vcvttps2udq_zmm_k1z_zmmm512b32_sae)], mnemonic[nameof(Mnemonic.Vcvttps2udq)]),
				(code[nameof(Code.EVEX_Vcvttpd2udq_xmm_k1z_xmmm128b64)], mnemonic[nameof(Mnemonic.Vcvttpd2udq)]),
				(code[nameof(Code.EVEX_Vcvttpd2udq_xmm_k1z_ymmm256b64)], mnemonic[nameof(Mnemonic.Vcvttpd2udq)]),
				(code[nameof(Code.EVEX_Vcvttpd2udq_ymm_k1z_zmmm512b64_sae)], mnemonic[nameof(Mnemonic.Vcvttpd2udq)]),
				(code[nameof(Code.Extrq_xmm_imm8_imm8)], mnemonic[nameof(Mnemonic.Extrq)]),
				(code[nameof(Code.EVEX_Vcvttps2uqq_xmm_k1z_xmmm64b32)], mnemonic[nameof(Mnemonic.Vcvttps2uqq)]),
				(code[nameof(Code.EVEX_Vcvttps2uqq_ymm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vcvttps2uqq)]),
				(code[nameof(Code.EVEX_Vcvttps2uqq_zmm_k1z_ymmm256b32_sae)], mnemonic[nameof(Mnemonic.Vcvttps2uqq)]),
				(code[nameof(Code.EVEX_Vcvttpd2uqq_xmm_k1z_xmmm128b64)], mnemonic[nameof(Mnemonic.Vcvttpd2uqq)]),
				(code[nameof(Code.EVEX_Vcvttpd2uqq_ymm_k1z_ymmm256b64)], mnemonic[nameof(Mnemonic.Vcvttpd2uqq)]),
				(code[nameof(Code.EVEX_Vcvttpd2uqq_zmm_k1z_zmmm512b64_sae)], mnemonic[nameof(Mnemonic.Vcvttpd2uqq)]),
				(code[nameof(Code.EVEX_Vcvttss2usi_r32_xmmm32_sae)], mnemonic[nameof(Mnemonic.Vcvttss2usi)]),
				(code[nameof(Code.EVEX_Vcvttss2usi_r64_xmmm32_sae)], mnemonic[nameof(Mnemonic.Vcvttss2usi)]),
				(code[nameof(Code.Insertq_xmm_xmm_imm8_imm8)], mnemonic[nameof(Mnemonic.Insertq)]),
				(code[nameof(Code.EVEX_Vcvttsd2usi_r32_xmmm64_sae)], mnemonic[nameof(Mnemonic.Vcvttsd2usi)]),
				(code[nameof(Code.EVEX_Vcvttsd2usi_r64_xmmm64_sae)], mnemonic[nameof(Mnemonic.Vcvttsd2usi)]),
				(code[nameof(Code.Vmwrite_r32_rm32)], mnemonic[nameof(Mnemonic.Vmwrite)]),
				(code[nameof(Code.Vmwrite_r64_rm64)], mnemonic[nameof(Mnemonic.Vmwrite)]),
				(code[nameof(Code.EVEX_Vcvtps2udq_xmm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vcvtps2udq)]),
				(code[nameof(Code.EVEX_Vcvtps2udq_ymm_k1z_ymmm256b32)], mnemonic[nameof(Mnemonic.Vcvtps2udq)]),
				(code[nameof(Code.EVEX_Vcvtps2udq_zmm_k1z_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vcvtps2udq)]),
				(code[nameof(Code.EVEX_Vcvtpd2udq_xmm_k1z_xmmm128b64)], mnemonic[nameof(Mnemonic.Vcvtpd2udq)]),
				(code[nameof(Code.EVEX_Vcvtpd2udq_xmm_k1z_ymmm256b64)], mnemonic[nameof(Mnemonic.Vcvtpd2udq)]),
				(code[nameof(Code.EVEX_Vcvtpd2udq_ymm_k1z_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vcvtpd2udq)]),
				(code[nameof(Code.Extrq_xmm_xmm)], mnemonic[nameof(Mnemonic.Extrq)]),
				(code[nameof(Code.EVEX_Vcvtps2uqq_xmm_k1z_xmmm64b32)], mnemonic[nameof(Mnemonic.Vcvtps2uqq)]),
				(code[nameof(Code.EVEX_Vcvtps2uqq_ymm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vcvtps2uqq)]),
				(code[nameof(Code.EVEX_Vcvtps2uqq_zmm_k1z_ymmm256b32_er)], mnemonic[nameof(Mnemonic.Vcvtps2uqq)]),
				(code[nameof(Code.EVEX_Vcvtpd2uqq_xmm_k1z_xmmm128b64)], mnemonic[nameof(Mnemonic.Vcvtpd2uqq)]),
				(code[nameof(Code.EVEX_Vcvtpd2uqq_ymm_k1z_ymmm256b64)], mnemonic[nameof(Mnemonic.Vcvtpd2uqq)]),
				(code[nameof(Code.EVEX_Vcvtpd2uqq_zmm_k1z_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vcvtpd2uqq)]),
				(code[nameof(Code.EVEX_Vcvtss2usi_r32_xmmm32_er)], mnemonic[nameof(Mnemonic.Vcvtss2usi)]),
				(code[nameof(Code.EVEX_Vcvtss2usi_r64_xmmm32_er)], mnemonic[nameof(Mnemonic.Vcvtss2usi)]),
				(code[nameof(Code.Insertq_xmm_xmm)], mnemonic[nameof(Mnemonic.Insertq)]),
				(code[nameof(Code.EVEX_Vcvtsd2usi_r32_xmmm64_er)], mnemonic[nameof(Mnemonic.Vcvtsd2usi)]),
				(code[nameof(Code.EVEX_Vcvtsd2usi_r64_xmmm64_er)], mnemonic[nameof(Mnemonic.Vcvtsd2usi)]),
				(code[nameof(Code.EVEX_Vcvttps2qq_xmm_k1z_xmmm64b32)], mnemonic[nameof(Mnemonic.Vcvttps2qq)]),
				(code[nameof(Code.EVEX_Vcvttps2qq_ymm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vcvttps2qq)]),
				(code[nameof(Code.EVEX_Vcvttps2qq_zmm_k1z_ymmm256b32_sae)], mnemonic[nameof(Mnemonic.Vcvttps2qq)]),
				(code[nameof(Code.EVEX_Vcvttpd2qq_xmm_k1z_xmmm128b64)], mnemonic[nameof(Mnemonic.Vcvttpd2qq)]),
				(code[nameof(Code.EVEX_Vcvttpd2qq_ymm_k1z_ymmm256b64)], mnemonic[nameof(Mnemonic.Vcvttpd2qq)]),
				(code[nameof(Code.EVEX_Vcvttpd2qq_zmm_k1z_zmmm512b64_sae)], mnemonic[nameof(Mnemonic.Vcvttpd2qq)]),
				(code[nameof(Code.EVEX_Vcvtudq2pd_xmm_k1z_xmmm64b32)], mnemonic[nameof(Mnemonic.Vcvtudq2pd)]),
				(code[nameof(Code.EVEX_Vcvtudq2pd_ymm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vcvtudq2pd)]),
				(code[nameof(Code.EVEX_Vcvtudq2pd_zmm_k1z_ymmm256b32)], mnemonic[nameof(Mnemonic.Vcvtudq2pd)]),
				(code[nameof(Code.EVEX_Vcvtuqq2pd_xmm_k1z_xmmm128b64)], mnemonic[nameof(Mnemonic.Vcvtuqq2pd)]),
				(code[nameof(Code.EVEX_Vcvtuqq2pd_ymm_k1z_ymmm256b64)], mnemonic[nameof(Mnemonic.Vcvtuqq2pd)]),
				(code[nameof(Code.EVEX_Vcvtuqq2pd_zmm_k1z_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vcvtuqq2pd)]),
				(code[nameof(Code.EVEX_Vcvtudq2ps_xmm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vcvtudq2ps)]),
				(code[nameof(Code.EVEX_Vcvtudq2ps_ymm_k1z_ymmm256b32)], mnemonic[nameof(Mnemonic.Vcvtudq2ps)]),
				(code[nameof(Code.EVEX_Vcvtudq2ps_zmm_k1z_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vcvtudq2ps)]),
				(code[nameof(Code.EVEX_Vcvtuqq2ps_xmm_k1z_xmmm128b64)], mnemonic[nameof(Mnemonic.Vcvtuqq2ps)]),
				(code[nameof(Code.EVEX_Vcvtuqq2ps_xmm_k1z_ymmm256b64)], mnemonic[nameof(Mnemonic.Vcvtuqq2ps)]),
				(code[nameof(Code.EVEX_Vcvtuqq2ps_ymm_k1z_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vcvtuqq2ps)]),
				(code[nameof(Code.EVEX_Vcvtps2qq_xmm_k1z_xmmm64b32)], mnemonic[nameof(Mnemonic.Vcvtps2qq)]),
				(code[nameof(Code.EVEX_Vcvtps2qq_ymm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vcvtps2qq)]),
				(code[nameof(Code.EVEX_Vcvtps2qq_zmm_k1z_ymmm256b32_er)], mnemonic[nameof(Mnemonic.Vcvtps2qq)]),
				(code[nameof(Code.EVEX_Vcvtpd2qq_xmm_k1z_xmmm128b64)], mnemonic[nameof(Mnemonic.Vcvtpd2qq)]),
				(code[nameof(Code.EVEX_Vcvtpd2qq_ymm_k1z_ymmm256b64)], mnemonic[nameof(Mnemonic.Vcvtpd2qq)]),
				(code[nameof(Code.EVEX_Vcvtpd2qq_zmm_k1z_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vcvtpd2qq)]),
				(code[nameof(Code.EVEX_Vcvtusi2ss_xmm_xmm_rm32_er)], mnemonic[nameof(Mnemonic.Vcvtusi2ss)]),
				(code[nameof(Code.EVEX_Vcvtusi2ss_xmm_xmm_rm64_er)], mnemonic[nameof(Mnemonic.Vcvtusi2ss)]),
				(code[nameof(Code.EVEX_Vcvtusi2sd_xmm_xmm_rm32_er)], mnemonic[nameof(Mnemonic.Vcvtusi2sd)]),
				(code[nameof(Code.EVEX_Vcvtusi2sd_xmm_xmm_rm64_er)], mnemonic[nameof(Mnemonic.Vcvtusi2sd)]),
				(code[nameof(Code.Haddpd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Haddpd)]),
				(code[nameof(Code.VEX_Vhaddpd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vhaddpd)]),
				(code[nameof(Code.VEX_Vhaddpd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vhaddpd)]),
				(code[nameof(Code.Haddps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Haddps)]),
				(code[nameof(Code.VEX_Vhaddps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vhaddps)]),
				(code[nameof(Code.VEX_Vhaddps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vhaddps)]),
				(code[nameof(Code.Hsubpd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Hsubpd)]),
				(code[nameof(Code.VEX_Vhsubpd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vhsubpd)]),
				(code[nameof(Code.VEX_Vhsubpd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vhsubpd)]),
				(code[nameof(Code.Hsubps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Hsubps)]),
				(code[nameof(Code.VEX_Vhsubps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vhsubps)]),
				(code[nameof(Code.VEX_Vhsubps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vhsubps)]),
				(code[nameof(Code.Movd_rm32_mm)], mnemonic[nameof(Mnemonic.Movd)]),
				(code[nameof(Code.Movq_rm64_mm)], mnemonic[nameof(Mnemonic.Movq)]),
				(code[nameof(Code.Movd_rm32_xmm)], mnemonic[nameof(Mnemonic.Movd)]),
				(code[nameof(Code.Movq_rm64_xmm)], mnemonic[nameof(Mnemonic.Movq)]),
				(code[nameof(Code.VEX_Vmovd_rm32_xmm)], mnemonic[nameof(Mnemonic.Vmovd)]),
				(code[nameof(Code.VEX_Vmovq_rm64_xmm)], mnemonic[nameof(Mnemonic.Vmovq)]),
				(code[nameof(Code.EVEX_Vmovd_rm32_xmm)], mnemonic[nameof(Mnemonic.Vmovd)]),
				(code[nameof(Code.EVEX_Vmovq_rm64_xmm)], mnemonic[nameof(Mnemonic.Vmovq)]),
				(code[nameof(Code.Movq_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Movq)]),
				(code[nameof(Code.VEX_Vmovq_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vmovq)]),
				(code[nameof(Code.EVEX_Vmovq_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vmovq)]),
				(code[nameof(Code.Movq_mmm64_mm)], mnemonic[nameof(Mnemonic.Movq)]),
				(code[nameof(Code.Movdqa_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Movdqa)]),
				(code[nameof(Code.VEX_Vmovdqa_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vmovdqa)]),
				(code[nameof(Code.VEX_Vmovdqa_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vmovdqa)]),
				(code[nameof(Code.EVEX_Vmovdqa32_xmmm128_k1z_xmm)], mnemonic[nameof(Mnemonic.Vmovdqa32)]),
				(code[nameof(Code.EVEX_Vmovdqa32_ymmm256_k1z_ymm)], mnemonic[nameof(Mnemonic.Vmovdqa32)]),
				(code[nameof(Code.EVEX_Vmovdqa32_zmmm512_k1z_zmm)], mnemonic[nameof(Mnemonic.Vmovdqa32)]),
				(code[nameof(Code.EVEX_Vmovdqa64_xmmm128_k1z_xmm)], mnemonic[nameof(Mnemonic.Vmovdqa64)]),
				(code[nameof(Code.EVEX_Vmovdqa64_ymmm256_k1z_ymm)], mnemonic[nameof(Mnemonic.Vmovdqa64)]),
				(code[nameof(Code.EVEX_Vmovdqa64_zmmm512_k1z_zmm)], mnemonic[nameof(Mnemonic.Vmovdqa64)]),
				(code[nameof(Code.Movdqu_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Movdqu)]),
				(code[nameof(Code.VEX_Vmovdqu_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vmovdqu)]),
				(code[nameof(Code.VEX_Vmovdqu_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vmovdqu)]),
				(code[nameof(Code.EVEX_Vmovdqu32_xmmm128_k1z_xmm)], mnemonic[nameof(Mnemonic.Vmovdqu32)]),
				(code[nameof(Code.EVEX_Vmovdqu32_ymmm256_k1z_ymm)], mnemonic[nameof(Mnemonic.Vmovdqu32)]),
				(code[nameof(Code.EVEX_Vmovdqu32_zmmm512_k1z_zmm)], mnemonic[nameof(Mnemonic.Vmovdqu32)]),
				(code[nameof(Code.EVEX_Vmovdqu64_xmmm128_k1z_xmm)], mnemonic[nameof(Mnemonic.Vmovdqu64)]),
				(code[nameof(Code.EVEX_Vmovdqu64_ymmm256_k1z_ymm)], mnemonic[nameof(Mnemonic.Vmovdqu64)]),
				(code[nameof(Code.EVEX_Vmovdqu64_zmmm512_k1z_zmm)], mnemonic[nameof(Mnemonic.Vmovdqu64)]),
				(code[nameof(Code.EVEX_Vmovdqu8_xmmm128_k1z_xmm)], mnemonic[nameof(Mnemonic.Vmovdqu8)]),
				(code[nameof(Code.EVEX_Vmovdqu8_ymmm256_k1z_ymm)], mnemonic[nameof(Mnemonic.Vmovdqu8)]),
				(code[nameof(Code.EVEX_Vmovdqu8_zmmm512_k1z_zmm)], mnemonic[nameof(Mnemonic.Vmovdqu8)]),
				(code[nameof(Code.EVEX_Vmovdqu16_xmmm128_k1z_xmm)], mnemonic[nameof(Mnemonic.Vmovdqu16)]),
				(code[nameof(Code.EVEX_Vmovdqu16_ymmm256_k1z_ymm)], mnemonic[nameof(Mnemonic.Vmovdqu16)]),
				(code[nameof(Code.EVEX_Vmovdqu16_zmmm512_k1z_zmm)], mnemonic[nameof(Mnemonic.Vmovdqu16)]),
				(code[nameof(Code.Jo_rel16)], mnemonic[nameof(Mnemonic.Jo)]),
				(code[nameof(Code.Jo_rel32_32)], mnemonic[nameof(Mnemonic.Jo)]),
				(code[nameof(Code.Jo_rel32_64)], mnemonic[nameof(Mnemonic.Jo)]),
				(code[nameof(Code.Jno_rel16)], mnemonic[nameof(Mnemonic.Jno)]),
				(code[nameof(Code.Jno_rel32_32)], mnemonic[nameof(Mnemonic.Jno)]),
				(code[nameof(Code.Jno_rel32_64)], mnemonic[nameof(Mnemonic.Jno)]),
				(code[nameof(Code.Jb_rel16)], mnemonic[nameof(Mnemonic.Jb)]),
				(code[nameof(Code.Jb_rel32_32)], mnemonic[nameof(Mnemonic.Jb)]),
				(code[nameof(Code.Jb_rel32_64)], mnemonic[nameof(Mnemonic.Jb)]),
				(code[nameof(Code.Jae_rel16)], mnemonic[nameof(Mnemonic.Jae)]),
				(code[nameof(Code.Jae_rel32_32)], mnemonic[nameof(Mnemonic.Jae)]),
				(code[nameof(Code.Jae_rel32_64)], mnemonic[nameof(Mnemonic.Jae)]),
				(code[nameof(Code.Je_rel16)], mnemonic[nameof(Mnemonic.Je)]),
				(code[nameof(Code.Je_rel32_32)], mnemonic[nameof(Mnemonic.Je)]),
				(code[nameof(Code.Je_rel32_64)], mnemonic[nameof(Mnemonic.Je)]),
				(code[nameof(Code.Jne_rel16)], mnemonic[nameof(Mnemonic.Jne)]),
				(code[nameof(Code.Jne_rel32_32)], mnemonic[nameof(Mnemonic.Jne)]),
				(code[nameof(Code.Jne_rel32_64)], mnemonic[nameof(Mnemonic.Jne)]),
				(code[nameof(Code.Jbe_rel16)], mnemonic[nameof(Mnemonic.Jbe)]),
				(code[nameof(Code.Jbe_rel32_32)], mnemonic[nameof(Mnemonic.Jbe)]),
				(code[nameof(Code.Jbe_rel32_64)], mnemonic[nameof(Mnemonic.Jbe)]),
				(code[nameof(Code.Ja_rel16)], mnemonic[nameof(Mnemonic.Ja)]),
				(code[nameof(Code.Ja_rel32_32)], mnemonic[nameof(Mnemonic.Ja)]),
				(code[nameof(Code.Ja_rel32_64)], mnemonic[nameof(Mnemonic.Ja)]),
				(code[nameof(Code.Js_rel16)], mnemonic[nameof(Mnemonic.Js)]),
				(code[nameof(Code.Js_rel32_32)], mnemonic[nameof(Mnemonic.Js)]),
				(code[nameof(Code.Js_rel32_64)], mnemonic[nameof(Mnemonic.Js)]),
				(code[nameof(Code.Jns_rel16)], mnemonic[nameof(Mnemonic.Jns)]),
				(code[nameof(Code.Jns_rel32_32)], mnemonic[nameof(Mnemonic.Jns)]),
				(code[nameof(Code.Jns_rel32_64)], mnemonic[nameof(Mnemonic.Jns)]),
				(code[nameof(Code.Jp_rel16)], mnemonic[nameof(Mnemonic.Jp)]),
				(code[nameof(Code.Jp_rel32_32)], mnemonic[nameof(Mnemonic.Jp)]),
				(code[nameof(Code.Jp_rel32_64)], mnemonic[nameof(Mnemonic.Jp)]),
				(code[nameof(Code.Jnp_rel16)], mnemonic[nameof(Mnemonic.Jnp)]),
				(code[nameof(Code.Jnp_rel32_32)], mnemonic[nameof(Mnemonic.Jnp)]),
				(code[nameof(Code.Jnp_rel32_64)], mnemonic[nameof(Mnemonic.Jnp)]),
				(code[nameof(Code.Jl_rel16)], mnemonic[nameof(Mnemonic.Jl)]),
				(code[nameof(Code.Jl_rel32_32)], mnemonic[nameof(Mnemonic.Jl)]),
				(code[nameof(Code.Jl_rel32_64)], mnemonic[nameof(Mnemonic.Jl)]),
				(code[nameof(Code.Jge_rel16)], mnemonic[nameof(Mnemonic.Jge)]),
				(code[nameof(Code.Jge_rel32_32)], mnemonic[nameof(Mnemonic.Jge)]),
				(code[nameof(Code.Jge_rel32_64)], mnemonic[nameof(Mnemonic.Jge)]),
				(code[nameof(Code.Jle_rel16)], mnemonic[nameof(Mnemonic.Jle)]),
				(code[nameof(Code.Jle_rel32_32)], mnemonic[nameof(Mnemonic.Jle)]),
				(code[nameof(Code.Jle_rel32_64)], mnemonic[nameof(Mnemonic.Jle)]),
				(code[nameof(Code.Jg_rel16)], mnemonic[nameof(Mnemonic.Jg)]),
				(code[nameof(Code.Jg_rel32_32)], mnemonic[nameof(Mnemonic.Jg)]),
				(code[nameof(Code.Jg_rel32_64)], mnemonic[nameof(Mnemonic.Jg)]),
				(code[nameof(Code.Seto_rm8)], mnemonic[nameof(Mnemonic.Seto)]),
				(code[nameof(Code.Setno_rm8)], mnemonic[nameof(Mnemonic.Setno)]),
				(code[nameof(Code.Setb_rm8)], mnemonic[nameof(Mnemonic.Setb)]),
				(code[nameof(Code.Setae_rm8)], mnemonic[nameof(Mnemonic.Setae)]),
				(code[nameof(Code.Sete_rm8)], mnemonic[nameof(Mnemonic.Sete)]),
				(code[nameof(Code.Setne_rm8)], mnemonic[nameof(Mnemonic.Setne)]),
				(code[nameof(Code.Setbe_rm8)], mnemonic[nameof(Mnemonic.Setbe)]),
				(code[nameof(Code.Seta_rm8)], mnemonic[nameof(Mnemonic.Seta)]),
				(code[nameof(Code.Sets_rm8)], mnemonic[nameof(Mnemonic.Sets)]),
				(code[nameof(Code.Setns_rm8)], mnemonic[nameof(Mnemonic.Setns)]),
				(code[nameof(Code.Setp_rm8)], mnemonic[nameof(Mnemonic.Setp)]),
				(code[nameof(Code.Setnp_rm8)], mnemonic[nameof(Mnemonic.Setnp)]),
				(code[nameof(Code.Setl_rm8)], mnemonic[nameof(Mnemonic.Setl)]),
				(code[nameof(Code.Setge_rm8)], mnemonic[nameof(Mnemonic.Setge)]),
				(code[nameof(Code.Setle_rm8)], mnemonic[nameof(Mnemonic.Setle)]),
				(code[nameof(Code.Setg_rm8)], mnemonic[nameof(Mnemonic.Setg)]),
				(code[nameof(Code.VEX_Kmovw_k_km16)], mnemonic[nameof(Mnemonic.Kmovw)]),
				(code[nameof(Code.VEX_Kmovq_k_km64)], mnemonic[nameof(Mnemonic.Kmovq)]),
				(code[nameof(Code.VEX_Kmovb_k_km8)], mnemonic[nameof(Mnemonic.Kmovb)]),
				(code[nameof(Code.VEX_Kmovd_k_km32)], mnemonic[nameof(Mnemonic.Kmovd)]),
				(code[nameof(Code.VEX_Kmovw_m16_k)], mnemonic[nameof(Mnemonic.Kmovw)]),
				(code[nameof(Code.VEX_Kmovq_m64_k)], mnemonic[nameof(Mnemonic.Kmovq)]),
				(code[nameof(Code.VEX_Kmovb_m8_k)], mnemonic[nameof(Mnemonic.Kmovb)]),
				(code[nameof(Code.VEX_Kmovd_m32_k)], mnemonic[nameof(Mnemonic.Kmovd)]),
				(code[nameof(Code.VEX_Kmovw_k_r32)], mnemonic[nameof(Mnemonic.Kmovw)]),
				(code[nameof(Code.VEX_Kmovb_k_r32)], mnemonic[nameof(Mnemonic.Kmovb)]),
				(code[nameof(Code.VEX_Kmovd_k_r32)], mnemonic[nameof(Mnemonic.Kmovd)]),
				(code[nameof(Code.VEX_Kmovq_k_r64)], mnemonic[nameof(Mnemonic.Kmovq)]),
				(code[nameof(Code.VEX_Kmovw_r32_k)], mnemonic[nameof(Mnemonic.Kmovw)]),
				(code[nameof(Code.VEX_Kmovb_r32_k)], mnemonic[nameof(Mnemonic.Kmovb)]),
				(code[nameof(Code.VEX_Kmovd_r32_k)], mnemonic[nameof(Mnemonic.Kmovd)]),
				(code[nameof(Code.VEX_Kmovq_r64_k)], mnemonic[nameof(Mnemonic.Kmovq)]),
				(code[nameof(Code.VEX_Kortestw_k_k)], mnemonic[nameof(Mnemonic.Kortestw)]),
				(code[nameof(Code.VEX_Kortestq_k_k)], mnemonic[nameof(Mnemonic.Kortestq)]),
				(code[nameof(Code.VEX_Kortestb_k_k)], mnemonic[nameof(Mnemonic.Kortestb)]),
				(code[nameof(Code.VEX_Kortestd_k_k)], mnemonic[nameof(Mnemonic.Kortestd)]),
				(code[nameof(Code.VEX_Ktestw_k_k)], mnemonic[nameof(Mnemonic.Ktestw)]),
				(code[nameof(Code.VEX_Ktestq_k_k)], mnemonic[nameof(Mnemonic.Ktestq)]),
				(code[nameof(Code.VEX_Ktestb_k_k)], mnemonic[nameof(Mnemonic.Ktestb)]),
				(code[nameof(Code.VEX_Ktestd_k_k)], mnemonic[nameof(Mnemonic.Ktestd)]),
				(code[nameof(Code.Pushw_FS)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Pushd_FS)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Pushq_FS)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Popw_FS)], mnemonic[nameof(Mnemonic.Pop)]),
				(code[nameof(Code.Popd_FS)], mnemonic[nameof(Mnemonic.Pop)]),
				(code[nameof(Code.Popq_FS)], mnemonic[nameof(Mnemonic.Pop)]),
				(code[nameof(Code.Cpuid)], mnemonic[nameof(Mnemonic.Cpuid)]),
				(code[nameof(Code.Bt_rm16_r16)], mnemonic[nameof(Mnemonic.Bt)]),
				(code[nameof(Code.Bt_rm32_r32)], mnemonic[nameof(Mnemonic.Bt)]),
				(code[nameof(Code.Bt_rm64_r64)], mnemonic[nameof(Mnemonic.Bt)]),
				(code[nameof(Code.Shld_rm16_r16_imm8)], mnemonic[nameof(Mnemonic.Shld)]),
				(code[nameof(Code.Shld_rm32_r32_imm8)], mnemonic[nameof(Mnemonic.Shld)]),
				(code[nameof(Code.Shld_rm64_r64_imm8)], mnemonic[nameof(Mnemonic.Shld)]),
				(code[nameof(Code.Shld_rm16_r16_CL)], mnemonic[nameof(Mnemonic.Shld)]),
				(code[nameof(Code.Shld_rm32_r32_CL)], mnemonic[nameof(Mnemonic.Shld)]),
				(code[nameof(Code.Shld_rm64_r64_CL)], mnemonic[nameof(Mnemonic.Shld)]),
				(code[nameof(Code.Montmul_16)], mnemonic[nameof(Mnemonic.Montmul)]),
				(code[nameof(Code.Montmul_32)], mnemonic[nameof(Mnemonic.Montmul)]),
				(code[nameof(Code.Montmul_64)], mnemonic[nameof(Mnemonic.Montmul)]),
				(code[nameof(Code.Xsha1_16)], mnemonic[nameof(Mnemonic.Xsha1)]),
				(code[nameof(Code.Xsha1_32)], mnemonic[nameof(Mnemonic.Xsha1)]),
				(code[nameof(Code.Xsha1_64)], mnemonic[nameof(Mnemonic.Xsha1)]),
				(code[nameof(Code.Xsha256_16)], mnemonic[nameof(Mnemonic.Xsha256)]),
				(code[nameof(Code.Xsha256_32)], mnemonic[nameof(Mnemonic.Xsha256)]),
				(code[nameof(Code.Xsha256_64)], mnemonic[nameof(Mnemonic.Xsha256)]),
				(code[nameof(Code.Xbts_r16_rm16)], mnemonic[nameof(Mnemonic.Xbts)]),
				(code[nameof(Code.Xbts_r32_rm32)], mnemonic[nameof(Mnemonic.Xbts)]),
				(code[nameof(Code.Xstore_16)], mnemonic[nameof(Mnemonic.Xstore)]),
				(code[nameof(Code.Xstore_32)], mnemonic[nameof(Mnemonic.Xstore)]),
				(code[nameof(Code.Xstore_64)], mnemonic[nameof(Mnemonic.Xstore)]),
				(code[nameof(Code.XcryptEcb_16)], mnemonic[nameof(Mnemonic.Xcryptecb)]),
				(code[nameof(Code.XcryptEcb_32)], mnemonic[nameof(Mnemonic.Xcryptecb)]),
				(code[nameof(Code.XcryptEcb_64)], mnemonic[nameof(Mnemonic.Xcryptecb)]),
				(code[nameof(Code.XcryptCbc_16)], mnemonic[nameof(Mnemonic.Xcryptcbc)]),
				(code[nameof(Code.XcryptCbc_32)], mnemonic[nameof(Mnemonic.Xcryptcbc)]),
				(code[nameof(Code.XcryptCbc_64)], mnemonic[nameof(Mnemonic.Xcryptcbc)]),
				(code[nameof(Code.XcryptCtr_16)], mnemonic[nameof(Mnemonic.Xcryptctr)]),
				(code[nameof(Code.XcryptCtr_32)], mnemonic[nameof(Mnemonic.Xcryptctr)]),
				(code[nameof(Code.XcryptCtr_64)], mnemonic[nameof(Mnemonic.Xcryptctr)]),
				(code[nameof(Code.XcryptCfb_16)], mnemonic[nameof(Mnemonic.Xcryptcfb)]),
				(code[nameof(Code.XcryptCfb_32)], mnemonic[nameof(Mnemonic.Xcryptcfb)]),
				(code[nameof(Code.XcryptCfb_64)], mnemonic[nameof(Mnemonic.Xcryptcfb)]),
				(code[nameof(Code.XcryptOfb_16)], mnemonic[nameof(Mnemonic.Xcryptofb)]),
				(code[nameof(Code.XcryptOfb_32)], mnemonic[nameof(Mnemonic.Xcryptofb)]),
				(code[nameof(Code.XcryptOfb_64)], mnemonic[nameof(Mnemonic.Xcryptofb)]),
				(code[nameof(Code.Ibts_rm16_r16)], mnemonic[nameof(Mnemonic.Ibts)]),
				(code[nameof(Code.Ibts_rm32_r32)], mnemonic[nameof(Mnemonic.Ibts)]),
				(code[nameof(Code.Cmpxchg486_rm8_r8)], mnemonic[nameof(Mnemonic.Cmpxchg)]),
				(code[nameof(Code.Cmpxchg486_rm16_r16)], mnemonic[nameof(Mnemonic.Cmpxchg)]),
				(code[nameof(Code.Cmpxchg486_rm32_r32)], mnemonic[nameof(Mnemonic.Cmpxchg)]),
				(code[nameof(Code.Pushw_GS)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Pushd_GS)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Pushq_GS)], mnemonic[nameof(Mnemonic.Push)]),
				(code[nameof(Code.Popw_GS)], mnemonic[nameof(Mnemonic.Pop)]),
				(code[nameof(Code.Popd_GS)], mnemonic[nameof(Mnemonic.Pop)]),
				(code[nameof(Code.Popq_GS)], mnemonic[nameof(Mnemonic.Pop)]),
				(code[nameof(Code.Rsm)], mnemonic[nameof(Mnemonic.Rsm)]),
				(code[nameof(Code.Bts_rm16_r16)], mnemonic[nameof(Mnemonic.Bts)]),
				(code[nameof(Code.Bts_rm32_r32)], mnemonic[nameof(Mnemonic.Bts)]),
				(code[nameof(Code.Bts_rm64_r64)], mnemonic[nameof(Mnemonic.Bts)]),
				(code[nameof(Code.Shrd_rm16_r16_imm8)], mnemonic[nameof(Mnemonic.Shrd)]),
				(code[nameof(Code.Shrd_rm32_r32_imm8)], mnemonic[nameof(Mnemonic.Shrd)]),
				(code[nameof(Code.Shrd_rm64_r64_imm8)], mnemonic[nameof(Mnemonic.Shrd)]),
				(code[nameof(Code.Shrd_rm16_r16_CL)], mnemonic[nameof(Mnemonic.Shrd)]),
				(code[nameof(Code.Shrd_rm32_r32_CL)], mnemonic[nameof(Mnemonic.Shrd)]),
				(code[nameof(Code.Shrd_rm64_r64_CL)], mnemonic[nameof(Mnemonic.Shrd)]),
				(code[nameof(Code.Fxsave_m512byte)], mnemonic[nameof(Mnemonic.Fxsave)]),
				(code[nameof(Code.Fxsave64_m512byte)], mnemonic[nameof(Mnemonic.Fxsave64)]),
				(code[nameof(Code.Rdfsbase_r32)], mnemonic[nameof(Mnemonic.Rdfsbase)]),
				(code[nameof(Code.Rdfsbase_r64)], mnemonic[nameof(Mnemonic.Rdfsbase)]),
				(code[nameof(Code.Fxrstor_m512byte)], mnemonic[nameof(Mnemonic.Fxrstor)]),
				(code[nameof(Code.Fxrstor64_m512byte)], mnemonic[nameof(Mnemonic.Fxrstor64)]),
				(code[nameof(Code.Rdgsbase_r32)], mnemonic[nameof(Mnemonic.Rdgsbase)]),
				(code[nameof(Code.Rdgsbase_r64)], mnemonic[nameof(Mnemonic.Rdgsbase)]),
				(code[nameof(Code.Ldmxcsr_m32)], mnemonic[nameof(Mnemonic.Ldmxcsr)]),
				(code[nameof(Code.Wrfsbase_r32)], mnemonic[nameof(Mnemonic.Wrfsbase)]),
				(code[nameof(Code.Wrfsbase_r64)], mnemonic[nameof(Mnemonic.Wrfsbase)]),
				(code[nameof(Code.VEX_Vldmxcsr_m32)], mnemonic[nameof(Mnemonic.Vldmxcsr)]),
				(code[nameof(Code.Stmxcsr_m32)], mnemonic[nameof(Mnemonic.Stmxcsr)]),
				(code[nameof(Code.Wrgsbase_r32)], mnemonic[nameof(Mnemonic.Wrgsbase)]),
				(code[nameof(Code.Wrgsbase_r64)], mnemonic[nameof(Mnemonic.Wrgsbase)]),
				(code[nameof(Code.VEX_Vstmxcsr_m32)], mnemonic[nameof(Mnemonic.Vstmxcsr)]),
				(code[nameof(Code.Xsave_mem)], mnemonic[nameof(Mnemonic.Xsave)]),
				(code[nameof(Code.Xsave64_mem)], mnemonic[nameof(Mnemonic.Xsave64)]),
				(code[nameof(Code.Ptwrite_rm32)], mnemonic[nameof(Mnemonic.Ptwrite)]),
				(code[nameof(Code.Ptwrite_rm64)], mnemonic[nameof(Mnemonic.Ptwrite)]),
				(code[nameof(Code.Xrstor_mem)], mnemonic[nameof(Mnemonic.Xrstor)]),
				(code[nameof(Code.Xrstor64_mem)], mnemonic[nameof(Mnemonic.Xrstor64)]),
				(code[nameof(Code.Incsspd_r32)], mnemonic[nameof(Mnemonic.Incsspd)]),
				(code[nameof(Code.Incsspq_r64)], mnemonic[nameof(Mnemonic.Incsspq)]),
				(code[nameof(Code.Xsaveopt_mem)], mnemonic[nameof(Mnemonic.Xsaveopt)]),
				(code[nameof(Code.Xsaveopt64_mem)], mnemonic[nameof(Mnemonic.Xsaveopt64)]),
				(code[nameof(Code.Clwb_m8)], mnemonic[nameof(Mnemonic.Clwb)]),
				(code[nameof(Code.Tpause_r32)], mnemonic[nameof(Mnemonic.Tpause)]),
				(code[nameof(Code.Tpause_r64)], mnemonic[nameof(Mnemonic.Tpause)]),
				(code[nameof(Code.Clrssbsy_m64)], mnemonic[nameof(Mnemonic.Clrssbsy)]),
				(code[nameof(Code.Umonitor_r16)], mnemonic[nameof(Mnemonic.Umonitor)]),
				(code[nameof(Code.Umonitor_r32)], mnemonic[nameof(Mnemonic.Umonitor)]),
				(code[nameof(Code.Umonitor_r64)], mnemonic[nameof(Mnemonic.Umonitor)]),
				(code[nameof(Code.Umwait_r32)], mnemonic[nameof(Mnemonic.Umwait)]),
				(code[nameof(Code.Umwait_r64)], mnemonic[nameof(Mnemonic.Umwait)]),
				(code[nameof(Code.Clflush_m8)], mnemonic[nameof(Mnemonic.Clflush)]),
				(code[nameof(Code.Clflushopt_m8)], mnemonic[nameof(Mnemonic.Clflushopt)]),
				(code[nameof(Code.Lfence)], mnemonic[nameof(Mnemonic.Lfence)]),
				(code[nameof(Code.Lfence_E9)], mnemonic[nameof(Mnemonic.Lfence)]),
				(code[nameof(Code.Lfence_EA)], mnemonic[nameof(Mnemonic.Lfence)]),
				(code[nameof(Code.Lfence_EB)], mnemonic[nameof(Mnemonic.Lfence)]),
				(code[nameof(Code.Lfence_EC)], mnemonic[nameof(Mnemonic.Lfence)]),
				(code[nameof(Code.Lfence_ED)], mnemonic[nameof(Mnemonic.Lfence)]),
				(code[nameof(Code.Lfence_EE)], mnemonic[nameof(Mnemonic.Lfence)]),
				(code[nameof(Code.Lfence_EF)], mnemonic[nameof(Mnemonic.Lfence)]),
				(code[nameof(Code.Mfence)], mnemonic[nameof(Mnemonic.Mfence)]),
				(code[nameof(Code.Mfence_F1)], mnemonic[nameof(Mnemonic.Mfence)]),
				(code[nameof(Code.Mfence_F2)], mnemonic[nameof(Mnemonic.Mfence)]),
				(code[nameof(Code.Mfence_F3)], mnemonic[nameof(Mnemonic.Mfence)]),
				(code[nameof(Code.Mfence_F4)], mnemonic[nameof(Mnemonic.Mfence)]),
				(code[nameof(Code.Mfence_F5)], mnemonic[nameof(Mnemonic.Mfence)]),
				(code[nameof(Code.Mfence_F6)], mnemonic[nameof(Mnemonic.Mfence)]),
				(code[nameof(Code.Mfence_F7)], mnemonic[nameof(Mnemonic.Mfence)]),
				(code[nameof(Code.Sfence)], mnemonic[nameof(Mnemonic.Sfence)]),
				(code[nameof(Code.Sfence_F9)], mnemonic[nameof(Mnemonic.Sfence)]),
				(code[nameof(Code.Sfence_FA)], mnemonic[nameof(Mnemonic.Sfence)]),
				(code[nameof(Code.Sfence_FB)], mnemonic[nameof(Mnemonic.Sfence)]),
				(code[nameof(Code.Sfence_FC)], mnemonic[nameof(Mnemonic.Sfence)]),
				(code[nameof(Code.Sfence_FD)], mnemonic[nameof(Mnemonic.Sfence)]),
				(code[nameof(Code.Sfence_FE)], mnemonic[nameof(Mnemonic.Sfence)]),
				(code[nameof(Code.Sfence_FF)], mnemonic[nameof(Mnemonic.Sfence)]),
				(code[nameof(Code.Pcommit)], mnemonic[nameof(Mnemonic.Pcommit)]),
				(code[nameof(Code.Imul_r16_rm16)], mnemonic[nameof(Mnemonic.Imul)]),
				(code[nameof(Code.Imul_r32_rm32)], mnemonic[nameof(Mnemonic.Imul)]),
				(code[nameof(Code.Imul_r64_rm64)], mnemonic[nameof(Mnemonic.Imul)]),
				(code[nameof(Code.Cmpxchg_rm8_r8)], mnemonic[nameof(Mnemonic.Cmpxchg)]),
				(code[nameof(Code.Cmpxchg_rm16_r16)], mnemonic[nameof(Mnemonic.Cmpxchg)]),
				(code[nameof(Code.Cmpxchg_rm32_r32)], mnemonic[nameof(Mnemonic.Cmpxchg)]),
				(code[nameof(Code.Cmpxchg_rm64_r64)], mnemonic[nameof(Mnemonic.Cmpxchg)]),
				(code[nameof(Code.Lss_r16_m1616)], mnemonic[nameof(Mnemonic.Lss)]),
				(code[nameof(Code.Lss_r32_m1632)], mnemonic[nameof(Mnemonic.Lss)]),
				(code[nameof(Code.Lss_r64_m1664)], mnemonic[nameof(Mnemonic.Lss)]),
				(code[nameof(Code.Btr_rm16_r16)], mnemonic[nameof(Mnemonic.Btr)]),
				(code[nameof(Code.Btr_rm32_r32)], mnemonic[nameof(Mnemonic.Btr)]),
				(code[nameof(Code.Btr_rm64_r64)], mnemonic[nameof(Mnemonic.Btr)]),
				(code[nameof(Code.Lfs_r16_m1616)], mnemonic[nameof(Mnemonic.Lfs)]),
				(code[nameof(Code.Lfs_r32_m1632)], mnemonic[nameof(Mnemonic.Lfs)]),
				(code[nameof(Code.Lfs_r64_m1664)], mnemonic[nameof(Mnemonic.Lfs)]),
				(code[nameof(Code.Lgs_r16_m1616)], mnemonic[nameof(Mnemonic.Lgs)]),
				(code[nameof(Code.Lgs_r32_m1632)], mnemonic[nameof(Mnemonic.Lgs)]),
				(code[nameof(Code.Lgs_r64_m1664)], mnemonic[nameof(Mnemonic.Lgs)]),
				(code[nameof(Code.Movzx_r16_rm8)], mnemonic[nameof(Mnemonic.Movzx)]),
				(code[nameof(Code.Movzx_r32_rm8)], mnemonic[nameof(Mnemonic.Movzx)]),
				(code[nameof(Code.Movzx_r64_rm8)], mnemonic[nameof(Mnemonic.Movzx)]),
				(code[nameof(Code.Movzx_r16_rm16)], mnemonic[nameof(Mnemonic.Movzx)]),
				(code[nameof(Code.Movzx_r32_rm16)], mnemonic[nameof(Mnemonic.Movzx)]),
				(code[nameof(Code.Movzx_r64_rm16)], mnemonic[nameof(Mnemonic.Movzx)]),
				(code[nameof(Code.Jmpe_disp16)], mnemonic[nameof(Mnemonic.Jmpe)]),
				(code[nameof(Code.Jmpe_disp32)], mnemonic[nameof(Mnemonic.Jmpe)]),
				(code[nameof(Code.Popcnt_r16_rm16)], mnemonic[nameof(Mnemonic.Popcnt)]),
				(code[nameof(Code.Popcnt_r32_rm32)], mnemonic[nameof(Mnemonic.Popcnt)]),
				(code[nameof(Code.Popcnt_r64_rm64)], mnemonic[nameof(Mnemonic.Popcnt)]),
				(code[nameof(Code.Ud1_r16_rm16)], mnemonic[nameof(Mnemonic.Ud1)]),
				(code[nameof(Code.Ud1_r32_rm32)], mnemonic[nameof(Mnemonic.Ud1)]),
				(code[nameof(Code.Ud1_r64_rm64)], mnemonic[nameof(Mnemonic.Ud1)]),
				(code[nameof(Code.Bt_rm16_imm8)], mnemonic[nameof(Mnemonic.Bt)]),
				(code[nameof(Code.Bt_rm32_imm8)], mnemonic[nameof(Mnemonic.Bt)]),
				(code[nameof(Code.Bt_rm64_imm8)], mnemonic[nameof(Mnemonic.Bt)]),
				(code[nameof(Code.Bts_rm16_imm8)], mnemonic[nameof(Mnemonic.Bts)]),
				(code[nameof(Code.Bts_rm32_imm8)], mnemonic[nameof(Mnemonic.Bts)]),
				(code[nameof(Code.Bts_rm64_imm8)], mnemonic[nameof(Mnemonic.Bts)]),
				(code[nameof(Code.Btr_rm16_imm8)], mnemonic[nameof(Mnemonic.Btr)]),
				(code[nameof(Code.Btr_rm32_imm8)], mnemonic[nameof(Mnemonic.Btr)]),
				(code[nameof(Code.Btr_rm64_imm8)], mnemonic[nameof(Mnemonic.Btr)]),
				(code[nameof(Code.Btc_rm16_imm8)], mnemonic[nameof(Mnemonic.Btc)]),
				(code[nameof(Code.Btc_rm32_imm8)], mnemonic[nameof(Mnemonic.Btc)]),
				(code[nameof(Code.Btc_rm64_imm8)], mnemonic[nameof(Mnemonic.Btc)]),
				(code[nameof(Code.Btc_rm16_r16)], mnemonic[nameof(Mnemonic.Btc)]),
				(code[nameof(Code.Btc_rm32_r32)], mnemonic[nameof(Mnemonic.Btc)]),
				(code[nameof(Code.Btc_rm64_r64)], mnemonic[nameof(Mnemonic.Btc)]),
				(code[nameof(Code.Bsf_r16_rm16)], mnemonic[nameof(Mnemonic.Bsf)]),
				(code[nameof(Code.Bsf_r32_rm32)], mnemonic[nameof(Mnemonic.Bsf)]),
				(code[nameof(Code.Bsf_r64_rm64)], mnemonic[nameof(Mnemonic.Bsf)]),
				(code[nameof(Code.Tzcnt_r16_rm16)], mnemonic[nameof(Mnemonic.Tzcnt)]),
				(code[nameof(Code.Tzcnt_r32_rm32)], mnemonic[nameof(Mnemonic.Tzcnt)]),
				(code[nameof(Code.Tzcnt_r64_rm64)], mnemonic[nameof(Mnemonic.Tzcnt)]),
				(code[nameof(Code.Bsr_r16_rm16)], mnemonic[nameof(Mnemonic.Bsr)]),
				(code[nameof(Code.Bsr_r32_rm32)], mnemonic[nameof(Mnemonic.Bsr)]),
				(code[nameof(Code.Bsr_r64_rm64)], mnemonic[nameof(Mnemonic.Bsr)]),
				(code[nameof(Code.Lzcnt_r16_rm16)], mnemonic[nameof(Mnemonic.Lzcnt)]),
				(code[nameof(Code.Lzcnt_r32_rm32)], mnemonic[nameof(Mnemonic.Lzcnt)]),
				(code[nameof(Code.Lzcnt_r64_rm64)], mnemonic[nameof(Mnemonic.Lzcnt)]),
				(code[nameof(Code.Movsx_r16_rm8)], mnemonic[nameof(Mnemonic.Movsx)]),
				(code[nameof(Code.Movsx_r32_rm8)], mnemonic[nameof(Mnemonic.Movsx)]),
				(code[nameof(Code.Movsx_r64_rm8)], mnemonic[nameof(Mnemonic.Movsx)]),
				(code[nameof(Code.Movsx_r16_rm16)], mnemonic[nameof(Mnemonic.Movsx)]),
				(code[nameof(Code.Movsx_r32_rm16)], mnemonic[nameof(Mnemonic.Movsx)]),
				(code[nameof(Code.Movsx_r64_rm16)], mnemonic[nameof(Mnemonic.Movsx)]),
				(code[nameof(Code.Xadd_rm8_r8)], mnemonic[nameof(Mnemonic.Xadd)]),
				(code[nameof(Code.Xadd_rm16_r16)], mnemonic[nameof(Mnemonic.Xadd)]),
				(code[nameof(Code.Xadd_rm32_r32)], mnemonic[nameof(Mnemonic.Xadd)]),
				(code[nameof(Code.Xadd_rm64_r64)], mnemonic[nameof(Mnemonic.Xadd)]),
				(code[nameof(Code.Cmpps_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Cmpps)]),
				(code[nameof(Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vcmpps)]),
				(code[nameof(Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vcmpps)]),
				(code[nameof(Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8)], mnemonic[nameof(Mnemonic.Vcmpps)]),
				(code[nameof(Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vcmpps)]),
				(code[nameof(Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae)], mnemonic[nameof(Mnemonic.Vcmpps)]),
				(code[nameof(Code.Cmppd_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Cmppd)]),
				(code[nameof(Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vcmppd)]),
				(code[nameof(Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vcmppd)]),
				(code[nameof(Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Vcmppd)]),
				(code[nameof(Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vcmppd)]),
				(code[nameof(Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae)], mnemonic[nameof(Mnemonic.Vcmppd)]),
				(code[nameof(Code.Cmpss_xmm_xmmm32_imm8)], mnemonic[nameof(Mnemonic.Cmpss)]),
				(code[nameof(Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8)], mnemonic[nameof(Mnemonic.Vcmpss)]),
				(code[nameof(Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae)], mnemonic[nameof(Mnemonic.Vcmpss)]),
				(code[nameof(Code.Cmpsd_xmm_xmmm64_imm8)], mnemonic[nameof(Mnemonic.Cmpsd)]),
				(code[nameof(Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8)], mnemonic[nameof(Mnemonic.Vcmpsd)]),
				(code[nameof(Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae)], mnemonic[nameof(Mnemonic.Vcmpsd)]),
				(code[nameof(Code.Movnti_m32_r32)], mnemonic[nameof(Mnemonic.Movnti)]),
				(code[nameof(Code.Movnti_m64_r64)], mnemonic[nameof(Mnemonic.Movnti)]),
				(code[nameof(Code.Pinsrw_mm_r32m16_imm8)], mnemonic[nameof(Mnemonic.Pinsrw)]),
				(code[nameof(Code.Pinsrw_mm_r64m16_imm8)], mnemonic[nameof(Mnemonic.Pinsrw)]),
				(code[nameof(Code.Pinsrw_xmm_r32m16_imm8)], mnemonic[nameof(Mnemonic.Pinsrw)]),
				(code[nameof(Code.Pinsrw_xmm_r64m16_imm8)], mnemonic[nameof(Mnemonic.Pinsrw)]),
				(code[nameof(Code.VEX_Vpinsrw_xmm_xmm_r32m16_imm8)], mnemonic[nameof(Mnemonic.Vpinsrw)]),
				(code[nameof(Code.VEX_Vpinsrw_xmm_xmm_r64m16_imm8)], mnemonic[nameof(Mnemonic.Vpinsrw)]),
				(code[nameof(Code.EVEX_Vpinsrw_xmm_xmm_r32m16_imm8)], mnemonic[nameof(Mnemonic.Vpinsrw)]),
				(code[nameof(Code.EVEX_Vpinsrw_xmm_xmm_r64m16_imm8)], mnemonic[nameof(Mnemonic.Vpinsrw)]),
				(code[nameof(Code.Pextrw_r32_mm_imm8)], mnemonic[nameof(Mnemonic.Pextrw)]),
				(code[nameof(Code.Pextrw_r64_mm_imm8)], mnemonic[nameof(Mnemonic.Pextrw)]),
				(code[nameof(Code.Pextrw_r32_xmm_imm8)], mnemonic[nameof(Mnemonic.Pextrw)]),
				(code[nameof(Code.Pextrw_r64_xmm_imm8)], mnemonic[nameof(Mnemonic.Pextrw)]),
				(code[nameof(Code.VEX_Vpextrw_r32_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpextrw)]),
				(code[nameof(Code.VEX_Vpextrw_r64_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpextrw)]),
				(code[nameof(Code.EVEX_Vpextrw_r32_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpextrw)]),
				(code[nameof(Code.EVEX_Vpextrw_r64_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpextrw)]),
				(code[nameof(Code.Shufps_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Shufps)]),
				(code[nameof(Code.VEX_Vshufps_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vshufps)]),
				(code[nameof(Code.VEX_Vshufps_ymm_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vshufps)]),
				(code[nameof(Code.EVEX_Vshufps_xmm_k1z_xmm_xmmm128b32_imm8)], mnemonic[nameof(Mnemonic.Vshufps)]),
				(code[nameof(Code.EVEX_Vshufps_ymm_k1z_ymm_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vshufps)]),
				(code[nameof(Code.EVEX_Vshufps_zmm_k1z_zmm_zmmm512b32_imm8)], mnemonic[nameof(Mnemonic.Vshufps)]),
				(code[nameof(Code.Shufpd_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Shufpd)]),
				(code[nameof(Code.VEX_Vshufpd_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vshufpd)]),
				(code[nameof(Code.VEX_Vshufpd_ymm_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vshufpd)]),
				(code[nameof(Code.EVEX_Vshufpd_xmm_k1z_xmm_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Vshufpd)]),
				(code[nameof(Code.EVEX_Vshufpd_ymm_k1z_ymm_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vshufpd)]),
				(code[nameof(Code.EVEX_Vshufpd_zmm_k1z_zmm_zmmm512b64_imm8)], mnemonic[nameof(Mnemonic.Vshufpd)]),
				(code[nameof(Code.Cmpxchg8b_m64)], mnemonic[nameof(Mnemonic.Cmpxchg8b)]),
				(code[nameof(Code.Cmpxchg16b_m128)], mnemonic[nameof(Mnemonic.Cmpxchg16b)]),
				(code[nameof(Code.Xrstors_mem)], mnemonic[nameof(Mnemonic.Xrstors)]),
				(code[nameof(Code.Xrstors64_mem)], mnemonic[nameof(Mnemonic.Xrstors64)]),
				(code[nameof(Code.Xsavec_mem)], mnemonic[nameof(Mnemonic.Xsavec)]),
				(code[nameof(Code.Xsavec64_mem)], mnemonic[nameof(Mnemonic.Xsavec64)]),
				(code[nameof(Code.Xsaves_mem)], mnemonic[nameof(Mnemonic.Xsaves)]),
				(code[nameof(Code.Xsaves64_mem)], mnemonic[nameof(Mnemonic.Xsaves64)]),
				(code[nameof(Code.Vmptrld_m64)], mnemonic[nameof(Mnemonic.Vmptrld)]),
				(code[nameof(Code.Vmclear_m64)], mnemonic[nameof(Mnemonic.Vmclear)]),
				(code[nameof(Code.Vmxon_m64)], mnemonic[nameof(Mnemonic.Vmxon)]),
				(code[nameof(Code.Rdrand_r16)], mnemonic[nameof(Mnemonic.Rdrand)]),
				(code[nameof(Code.Rdrand_r32)], mnemonic[nameof(Mnemonic.Rdrand)]),
				(code[nameof(Code.Rdrand_r64)], mnemonic[nameof(Mnemonic.Rdrand)]),
				(code[nameof(Code.Vmptrst_m64)], mnemonic[nameof(Mnemonic.Vmptrst)]),
				(code[nameof(Code.Rdseed_r16)], mnemonic[nameof(Mnemonic.Rdseed)]),
				(code[nameof(Code.Rdseed_r32)], mnemonic[nameof(Mnemonic.Rdseed)]),
				(code[nameof(Code.Rdseed_r64)], mnemonic[nameof(Mnemonic.Rdseed)]),
				(code[nameof(Code.Rdpid_r32)], mnemonic[nameof(Mnemonic.Rdpid)]),
				(code[nameof(Code.Rdpid_r64)], mnemonic[nameof(Mnemonic.Rdpid)]),
				(code[nameof(Code.Bswap_r16)], mnemonic[nameof(Mnemonic.Bswap)]),
				(code[nameof(Code.Bswap_r32)], mnemonic[nameof(Mnemonic.Bswap)]),
				(code[nameof(Code.Bswap_r64)], mnemonic[nameof(Mnemonic.Bswap)]),
				(code[nameof(Code.Addsubpd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Addsubpd)]),
				(code[nameof(Code.VEX_Vaddsubpd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vaddsubpd)]),
				(code[nameof(Code.VEX_Vaddsubpd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vaddsubpd)]),
				(code[nameof(Code.Addsubps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Addsubps)]),
				(code[nameof(Code.VEX_Vaddsubps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vaddsubps)]),
				(code[nameof(Code.VEX_Vaddsubps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vaddsubps)]),
				(code[nameof(Code.Psrlw_mm_mmm64)], mnemonic[nameof(Mnemonic.Psrlw)]),
				(code[nameof(Code.Psrlw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Psrlw)]),
				(code[nameof(Code.VEX_Vpsrlw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrlw)]),
				(code[nameof(Code.VEX_Vpsrlw_ymm_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrlw)]),
				(code[nameof(Code.EVEX_Vpsrlw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrlw)]),
				(code[nameof(Code.EVEX_Vpsrlw_ymm_k1z_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrlw)]),
				(code[nameof(Code.EVEX_Vpsrlw_zmm_k1z_zmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrlw)]),
				(code[nameof(Code.Psrld_mm_mmm64)], mnemonic[nameof(Mnemonic.Psrld)]),
				(code[nameof(Code.Psrld_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Psrld)]),
				(code[nameof(Code.VEX_Vpsrld_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrld)]),
				(code[nameof(Code.VEX_Vpsrld_ymm_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrld)]),
				(code[nameof(Code.EVEX_Vpsrld_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrld)]),
				(code[nameof(Code.EVEX_Vpsrld_ymm_k1z_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrld)]),
				(code[nameof(Code.EVEX_Vpsrld_zmm_k1z_zmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrld)]),
				(code[nameof(Code.Psrlq_mm_mmm64)], mnemonic[nameof(Mnemonic.Psrlq)]),
				(code[nameof(Code.Psrlq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Psrlq)]),
				(code[nameof(Code.VEX_Vpsrlq_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrlq)]),
				(code[nameof(Code.VEX_Vpsrlq_ymm_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrlq)]),
				(code[nameof(Code.EVEX_Vpsrlq_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrlq)]),
				(code[nameof(Code.EVEX_Vpsrlq_ymm_k1z_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrlq)]),
				(code[nameof(Code.EVEX_Vpsrlq_zmm_k1z_zmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrlq)]),
				(code[nameof(Code.Paddq_mm_mmm64)], mnemonic[nameof(Mnemonic.Paddq)]),
				(code[nameof(Code.Paddq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Paddq)]),
				(code[nameof(Code.VEX_Vpaddq_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpaddq)]),
				(code[nameof(Code.VEX_Vpaddq_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpaddq)]),
				(code[nameof(Code.EVEX_Vpaddq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpaddq)]),
				(code[nameof(Code.EVEX_Vpaddq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpaddq)]),
				(code[nameof(Code.EVEX_Vpaddq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpaddq)]),
				(code[nameof(Code.Pmullw_mm_mmm64)], mnemonic[nameof(Mnemonic.Pmullw)]),
				(code[nameof(Code.Pmullw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pmullw)]),
				(code[nameof(Code.VEX_Vpmullw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmullw)]),
				(code[nameof(Code.VEX_Vpmullw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmullw)]),
				(code[nameof(Code.EVEX_Vpmullw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmullw)]),
				(code[nameof(Code.EVEX_Vpmullw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmullw)]),
				(code[nameof(Code.EVEX_Vpmullw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpmullw)]),
				(code[nameof(Code.Movq_xmmm64_xmm)], mnemonic[nameof(Mnemonic.Movq)]),
				(code[nameof(Code.VEX_Vmovq_xmmm64_xmm)], mnemonic[nameof(Mnemonic.Vmovq)]),
				(code[nameof(Code.EVEX_Vmovq_xmmm64_xmm)], mnemonic[nameof(Mnemonic.Vmovq)]),
				(code[nameof(Code.Movq2dq_xmm_mm)], mnemonic[nameof(Mnemonic.Movq2dq)]),
				(code[nameof(Code.Movdq2q_mm_xmm)], mnemonic[nameof(Mnemonic.Movdq2q)]),
				(code[nameof(Code.Pmovmskb_r32_mm)], mnemonic[nameof(Mnemonic.Pmovmskb)]),
				(code[nameof(Code.Pmovmskb_r64_mm)], mnemonic[nameof(Mnemonic.Pmovmskb)]),
				(code[nameof(Code.Pmovmskb_r32_xmm)], mnemonic[nameof(Mnemonic.Pmovmskb)]),
				(code[nameof(Code.Pmovmskb_r64_xmm)], mnemonic[nameof(Mnemonic.Pmovmskb)]),
				(code[nameof(Code.VEX_Vpmovmskb_r32_xmm)], mnemonic[nameof(Mnemonic.Vpmovmskb)]),
				(code[nameof(Code.VEX_Vpmovmskb_r64_xmm)], mnemonic[nameof(Mnemonic.Vpmovmskb)]),
				(code[nameof(Code.VEX_Vpmovmskb_r32_ymm)], mnemonic[nameof(Mnemonic.Vpmovmskb)]),
				(code[nameof(Code.VEX_Vpmovmskb_r64_ymm)], mnemonic[nameof(Mnemonic.Vpmovmskb)]),
				(code[nameof(Code.Psubusb_mm_mmm64)], mnemonic[nameof(Mnemonic.Psubusb)]),
				(code[nameof(Code.Psubusb_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Psubusb)]),
				(code[nameof(Code.VEX_Vpsubusb_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsubusb)]),
				(code[nameof(Code.VEX_Vpsubusb_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsubusb)]),
				(code[nameof(Code.EVEX_Vpsubusb_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsubusb)]),
				(code[nameof(Code.EVEX_Vpsubusb_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsubusb)]),
				(code[nameof(Code.EVEX_Vpsubusb_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpsubusb)]),
				(code[nameof(Code.Psubusw_mm_mmm64)], mnemonic[nameof(Mnemonic.Psubusw)]),
				(code[nameof(Code.Psubusw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Psubusw)]),
				(code[nameof(Code.VEX_Vpsubusw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsubusw)]),
				(code[nameof(Code.VEX_Vpsubusw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsubusw)]),
				(code[nameof(Code.EVEX_Vpsubusw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsubusw)]),
				(code[nameof(Code.EVEX_Vpsubusw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsubusw)]),
				(code[nameof(Code.EVEX_Vpsubusw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpsubusw)]),
				(code[nameof(Code.Pminub_mm_mmm64)], mnemonic[nameof(Mnemonic.Pminub)]),
				(code[nameof(Code.Pminub_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pminub)]),
				(code[nameof(Code.VEX_Vpminub_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpminub)]),
				(code[nameof(Code.VEX_Vpminub_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpminub)]),
				(code[nameof(Code.EVEX_Vpminub_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpminub)]),
				(code[nameof(Code.EVEX_Vpminub_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpminub)]),
				(code[nameof(Code.EVEX_Vpminub_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpminub)]),
				(code[nameof(Code.Pand_mm_mmm64)], mnemonic[nameof(Mnemonic.Pand)]),
				(code[nameof(Code.Pand_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pand)]),
				(code[nameof(Code.VEX_Vpand_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpand)]),
				(code[nameof(Code.VEX_Vpand_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpand)]),
				(code[nameof(Code.EVEX_Vpandd_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpandd)]),
				(code[nameof(Code.EVEX_Vpandd_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpandd)]),
				(code[nameof(Code.EVEX_Vpandd_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpandd)]),
				(code[nameof(Code.EVEX_Vpandq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpandq)]),
				(code[nameof(Code.EVEX_Vpandq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpandq)]),
				(code[nameof(Code.EVEX_Vpandq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpandq)]),
				(code[nameof(Code.Paddusb_mm_mmm64)], mnemonic[nameof(Mnemonic.Paddusb)]),
				(code[nameof(Code.Paddusb_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Paddusb)]),
				(code[nameof(Code.VEX_Vpaddusb_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpaddusb)]),
				(code[nameof(Code.VEX_Vpaddusb_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpaddusb)]),
				(code[nameof(Code.EVEX_Vpaddusb_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpaddusb)]),
				(code[nameof(Code.EVEX_Vpaddusb_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpaddusb)]),
				(code[nameof(Code.EVEX_Vpaddusb_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpaddusb)]),
				(code[nameof(Code.Paddusw_mm_mmm64)], mnemonic[nameof(Mnemonic.Paddusw)]),
				(code[nameof(Code.Paddusw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Paddusw)]),
				(code[nameof(Code.VEX_Vpaddusw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpaddusw)]),
				(code[nameof(Code.VEX_Vpaddusw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpaddusw)]),
				(code[nameof(Code.EVEX_Vpaddusw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpaddusw)]),
				(code[nameof(Code.EVEX_Vpaddusw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpaddusw)]),
				(code[nameof(Code.EVEX_Vpaddusw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpaddusw)]),
				(code[nameof(Code.Pmaxub_mm_mmm64)], mnemonic[nameof(Mnemonic.Pmaxub)]),
				(code[nameof(Code.Pmaxub_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pmaxub)]),
				(code[nameof(Code.VEX_Vpmaxub_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmaxub)]),
				(code[nameof(Code.VEX_Vpmaxub_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmaxub)]),
				(code[nameof(Code.EVEX_Vpmaxub_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmaxub)]),
				(code[nameof(Code.EVEX_Vpmaxub_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmaxub)]),
				(code[nameof(Code.EVEX_Vpmaxub_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpmaxub)]),
				(code[nameof(Code.Pandn_mm_mmm64)], mnemonic[nameof(Mnemonic.Pandn)]),
				(code[nameof(Code.Pandn_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pandn)]),
				(code[nameof(Code.VEX_Vpandn_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpandn)]),
				(code[nameof(Code.VEX_Vpandn_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpandn)]),
				(code[nameof(Code.EVEX_Vpandnd_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpandnd)]),
				(code[nameof(Code.EVEX_Vpandnd_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpandnd)]),
				(code[nameof(Code.EVEX_Vpandnd_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpandnd)]),
				(code[nameof(Code.EVEX_Vpandnq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpandnq)]),
				(code[nameof(Code.EVEX_Vpandnq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpandnq)]),
				(code[nameof(Code.EVEX_Vpandnq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpandnq)]),
				(code[nameof(Code.Pavgb_mm_mmm64)], mnemonic[nameof(Mnemonic.Pavgb)]),
				(code[nameof(Code.Pavgb_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pavgb)]),
				(code[nameof(Code.VEX_Vpavgb_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpavgb)]),
				(code[nameof(Code.VEX_Vpavgb_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpavgb)]),
				(code[nameof(Code.EVEX_Vpavgb_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpavgb)]),
				(code[nameof(Code.EVEX_Vpavgb_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpavgb)]),
				(code[nameof(Code.EVEX_Vpavgb_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpavgb)]),
				(code[nameof(Code.Psraw_mm_mmm64)], mnemonic[nameof(Mnemonic.Psraw)]),
				(code[nameof(Code.Psraw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Psraw)]),
				(code[nameof(Code.VEX_Vpsraw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsraw)]),
				(code[nameof(Code.VEX_Vpsraw_ymm_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsraw)]),
				(code[nameof(Code.EVEX_Vpsraw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsraw)]),
				(code[nameof(Code.EVEX_Vpsraw_ymm_k1z_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsraw)]),
				(code[nameof(Code.EVEX_Vpsraw_zmm_k1z_zmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsraw)]),
				(code[nameof(Code.Psrad_mm_mmm64)], mnemonic[nameof(Mnemonic.Psrad)]),
				(code[nameof(Code.Psrad_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Psrad)]),
				(code[nameof(Code.VEX_Vpsrad_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrad)]),
				(code[nameof(Code.VEX_Vpsrad_ymm_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrad)]),
				(code[nameof(Code.EVEX_Vpsrad_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrad)]),
				(code[nameof(Code.EVEX_Vpsrad_ymm_k1z_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrad)]),
				(code[nameof(Code.EVEX_Vpsrad_zmm_k1z_zmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrad)]),
				(code[nameof(Code.EVEX_Vpsraq_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsraq)]),
				(code[nameof(Code.EVEX_Vpsraq_ymm_k1z_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsraq)]),
				(code[nameof(Code.EVEX_Vpsraq_zmm_k1z_zmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsraq)]),
				(code[nameof(Code.Pavgw_mm_mmm64)], mnemonic[nameof(Mnemonic.Pavgw)]),
				(code[nameof(Code.Pavgw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pavgw)]),
				(code[nameof(Code.VEX_Vpavgw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpavgw)]),
				(code[nameof(Code.VEX_Vpavgw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpavgw)]),
				(code[nameof(Code.EVEX_Vpavgw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpavgw)]),
				(code[nameof(Code.EVEX_Vpavgw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpavgw)]),
				(code[nameof(Code.EVEX_Vpavgw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpavgw)]),
				(code[nameof(Code.Pmulhuw_mm_mmm64)], mnemonic[nameof(Mnemonic.Pmulhuw)]),
				(code[nameof(Code.Pmulhuw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pmulhuw)]),
				(code[nameof(Code.VEX_Vpmulhuw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmulhuw)]),
				(code[nameof(Code.VEX_Vpmulhuw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmulhuw)]),
				(code[nameof(Code.EVEX_Vpmulhuw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmulhuw)]),
				(code[nameof(Code.EVEX_Vpmulhuw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmulhuw)]),
				(code[nameof(Code.EVEX_Vpmulhuw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpmulhuw)]),
				(code[nameof(Code.Pmulhw_mm_mmm64)], mnemonic[nameof(Mnemonic.Pmulhw)]),
				(code[nameof(Code.Pmulhw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pmulhw)]),
				(code[nameof(Code.VEX_Vpmulhw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmulhw)]),
				(code[nameof(Code.VEX_Vpmulhw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmulhw)]),
				(code[nameof(Code.EVEX_Vpmulhw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmulhw)]),
				(code[nameof(Code.EVEX_Vpmulhw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmulhw)]),
				(code[nameof(Code.EVEX_Vpmulhw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpmulhw)]),
				(code[nameof(Code.Cvttpd2dq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Cvttpd2dq)]),
				(code[nameof(Code.VEX_Vcvttpd2dq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vcvttpd2dq)]),
				(code[nameof(Code.VEX_Vcvttpd2dq_xmm_ymmm256)], mnemonic[nameof(Mnemonic.Vcvttpd2dq)]),
				(code[nameof(Code.EVEX_Vcvttpd2dq_xmm_k1z_xmmm128b64)], mnemonic[nameof(Mnemonic.Vcvttpd2dq)]),
				(code[nameof(Code.EVEX_Vcvttpd2dq_xmm_k1z_ymmm256b64)], mnemonic[nameof(Mnemonic.Vcvttpd2dq)]),
				(code[nameof(Code.EVEX_Vcvttpd2dq_ymm_k1z_zmmm512b64_sae)], mnemonic[nameof(Mnemonic.Vcvttpd2dq)]),
				(code[nameof(Code.Cvtdq2pd_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Cvtdq2pd)]),
				(code[nameof(Code.VEX_Vcvtdq2pd_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vcvtdq2pd)]),
				(code[nameof(Code.VEX_Vcvtdq2pd_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vcvtdq2pd)]),
				(code[nameof(Code.EVEX_Vcvtdq2pd_xmm_k1z_xmmm64b32)], mnemonic[nameof(Mnemonic.Vcvtdq2pd)]),
				(code[nameof(Code.EVEX_Vcvtdq2pd_ymm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vcvtdq2pd)]),
				(code[nameof(Code.EVEX_Vcvtdq2pd_zmm_k1z_ymmm256b32)], mnemonic[nameof(Mnemonic.Vcvtdq2pd)]),
				(code[nameof(Code.EVEX_Vcvtqq2pd_xmm_k1z_xmmm128b64)], mnemonic[nameof(Mnemonic.Vcvtqq2pd)]),
				(code[nameof(Code.EVEX_Vcvtqq2pd_ymm_k1z_ymmm256b64)], mnemonic[nameof(Mnemonic.Vcvtqq2pd)]),
				(code[nameof(Code.EVEX_Vcvtqq2pd_zmm_k1z_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vcvtqq2pd)]),
				(code[nameof(Code.Cvtpd2dq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Cvtpd2dq)]),
				(code[nameof(Code.VEX_Vcvtpd2dq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vcvtpd2dq)]),
				(code[nameof(Code.VEX_Vcvtpd2dq_xmm_ymmm256)], mnemonic[nameof(Mnemonic.Vcvtpd2dq)]),
				(code[nameof(Code.EVEX_Vcvtpd2dq_xmm_k1z_xmmm128b64)], mnemonic[nameof(Mnemonic.Vcvtpd2dq)]),
				(code[nameof(Code.EVEX_Vcvtpd2dq_xmm_k1z_ymmm256b64)], mnemonic[nameof(Mnemonic.Vcvtpd2dq)]),
				(code[nameof(Code.EVEX_Vcvtpd2dq_ymm_k1z_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vcvtpd2dq)]),
				(code[nameof(Code.Movntq_m64_mm)], mnemonic[nameof(Mnemonic.Movntq)]),
				(code[nameof(Code.Movntdq_m128_xmm)], mnemonic[nameof(Mnemonic.Movntdq)]),
				(code[nameof(Code.VEX_Vmovntdq_m128_xmm)], mnemonic[nameof(Mnemonic.Vmovntdq)]),
				(code[nameof(Code.VEX_Vmovntdq_m256_ymm)], mnemonic[nameof(Mnemonic.Vmovntdq)]),
				(code[nameof(Code.EVEX_Vmovntdq_m128_xmm)], mnemonic[nameof(Mnemonic.Vmovntdq)]),
				(code[nameof(Code.EVEX_Vmovntdq_m256_ymm)], mnemonic[nameof(Mnemonic.Vmovntdq)]),
				(code[nameof(Code.EVEX_Vmovntdq_m512_zmm)], mnemonic[nameof(Mnemonic.Vmovntdq)]),
				(code[nameof(Code.Psubsb_mm_mmm64)], mnemonic[nameof(Mnemonic.Psubsb)]),
				(code[nameof(Code.Psubsb_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Psubsb)]),
				(code[nameof(Code.VEX_Vpsubsb_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsubsb)]),
				(code[nameof(Code.VEX_Vpsubsb_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsubsb)]),
				(code[nameof(Code.EVEX_Vpsubsb_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsubsb)]),
				(code[nameof(Code.EVEX_Vpsubsb_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsubsb)]),
				(code[nameof(Code.EVEX_Vpsubsb_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpsubsb)]),
				(code[nameof(Code.Psubsw_mm_mmm64)], mnemonic[nameof(Mnemonic.Psubsw)]),
				(code[nameof(Code.Psubsw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Psubsw)]),
				(code[nameof(Code.VEX_Vpsubsw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsubsw)]),
				(code[nameof(Code.VEX_Vpsubsw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsubsw)]),
				(code[nameof(Code.EVEX_Vpsubsw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsubsw)]),
				(code[nameof(Code.EVEX_Vpsubsw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsubsw)]),
				(code[nameof(Code.EVEX_Vpsubsw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpsubsw)]),
				(code[nameof(Code.Pminsw_mm_mmm64)], mnemonic[nameof(Mnemonic.Pminsw)]),
				(code[nameof(Code.Pminsw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pminsw)]),
				(code[nameof(Code.VEX_Vpminsw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpminsw)]),
				(code[nameof(Code.VEX_Vpminsw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpminsw)]),
				(code[nameof(Code.EVEX_Vpminsw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpminsw)]),
				(code[nameof(Code.EVEX_Vpminsw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpminsw)]),
				(code[nameof(Code.EVEX_Vpminsw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpminsw)]),
				(code[nameof(Code.Por_mm_mmm64)], mnemonic[nameof(Mnemonic.Por)]),
				(code[nameof(Code.Por_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Por)]),
				(code[nameof(Code.VEX_Vpor_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpor)]),
				(code[nameof(Code.VEX_Vpor_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpor)]),
				(code[nameof(Code.EVEX_Vpord_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpord)]),
				(code[nameof(Code.EVEX_Vpord_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpord)]),
				(code[nameof(Code.EVEX_Vpord_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpord)]),
				(code[nameof(Code.EVEX_Vporq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vporq)]),
				(code[nameof(Code.EVEX_Vporq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vporq)]),
				(code[nameof(Code.EVEX_Vporq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vporq)]),
				(code[nameof(Code.Paddsb_mm_mmm64)], mnemonic[nameof(Mnemonic.Paddsb)]),
				(code[nameof(Code.Paddsb_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Paddsb)]),
				(code[nameof(Code.VEX_Vpaddsb_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpaddsb)]),
				(code[nameof(Code.VEX_Vpaddsb_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpaddsb)]),
				(code[nameof(Code.EVEX_Vpaddsb_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpaddsb)]),
				(code[nameof(Code.EVEX_Vpaddsb_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpaddsb)]),
				(code[nameof(Code.EVEX_Vpaddsb_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpaddsb)]),
				(code[nameof(Code.Paddsw_mm_mmm64)], mnemonic[nameof(Mnemonic.Paddsw)]),
				(code[nameof(Code.Paddsw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Paddsw)]),
				(code[nameof(Code.VEX_Vpaddsw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpaddsw)]),
				(code[nameof(Code.VEX_Vpaddsw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpaddsw)]),
				(code[nameof(Code.EVEX_Vpaddsw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpaddsw)]),
				(code[nameof(Code.EVEX_Vpaddsw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpaddsw)]),
				(code[nameof(Code.EVEX_Vpaddsw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpaddsw)]),
				(code[nameof(Code.Pmaxsw_mm_mmm64)], mnemonic[nameof(Mnemonic.Pmaxsw)]),
				(code[nameof(Code.Pmaxsw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pmaxsw)]),
				(code[nameof(Code.VEX_Vpmaxsw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmaxsw)]),
				(code[nameof(Code.VEX_Vpmaxsw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmaxsw)]),
				(code[nameof(Code.EVEX_Vpmaxsw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmaxsw)]),
				(code[nameof(Code.EVEX_Vpmaxsw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmaxsw)]),
				(code[nameof(Code.EVEX_Vpmaxsw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpmaxsw)]),
				(code[nameof(Code.Pxor_mm_mmm64)], mnemonic[nameof(Mnemonic.Pxor)]),
				(code[nameof(Code.Pxor_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pxor)]),
				(code[nameof(Code.VEX_Vpxor_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpxor)]),
				(code[nameof(Code.VEX_Vpxor_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpxor)]),
				(code[nameof(Code.EVEX_Vpxord_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpxord)]),
				(code[nameof(Code.EVEX_Vpxord_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpxord)]),
				(code[nameof(Code.EVEX_Vpxord_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpxord)]),
				(code[nameof(Code.EVEX_Vpxorq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpxorq)]),
				(code[nameof(Code.EVEX_Vpxorq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpxorq)]),
				(code[nameof(Code.EVEX_Vpxorq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpxorq)]),
				(code[nameof(Code.Lddqu_xmm_m128)], mnemonic[nameof(Mnemonic.Lddqu)]),
				(code[nameof(Code.VEX_Vlddqu_xmm_m128)], mnemonic[nameof(Mnemonic.Vlddqu)]),
				(code[nameof(Code.VEX_Vlddqu_ymm_m256)], mnemonic[nameof(Mnemonic.Vlddqu)]),
				(code[nameof(Code.Psllw_mm_mmm64)], mnemonic[nameof(Mnemonic.Psllw)]),
				(code[nameof(Code.Psllw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Psllw)]),
				(code[nameof(Code.VEX_Vpsllw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsllw)]),
				(code[nameof(Code.VEX_Vpsllw_ymm_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsllw)]),
				(code[nameof(Code.EVEX_Vpsllw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsllw)]),
				(code[nameof(Code.EVEX_Vpsllw_ymm_k1z_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsllw)]),
				(code[nameof(Code.EVEX_Vpsllw_zmm_k1z_zmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsllw)]),
				(code[nameof(Code.Pslld_mm_mmm64)], mnemonic[nameof(Mnemonic.Pslld)]),
				(code[nameof(Code.Pslld_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pslld)]),
				(code[nameof(Code.VEX_Vpslld_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpslld)]),
				(code[nameof(Code.VEX_Vpslld_ymm_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpslld)]),
				(code[nameof(Code.EVEX_Vpslld_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpslld)]),
				(code[nameof(Code.EVEX_Vpslld_ymm_k1z_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpslld)]),
				(code[nameof(Code.EVEX_Vpslld_zmm_k1z_zmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpslld)]),
				(code[nameof(Code.Psllq_mm_mmm64)], mnemonic[nameof(Mnemonic.Psllq)]),
				(code[nameof(Code.Psllq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Psllq)]),
				(code[nameof(Code.VEX_Vpsllq_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsllq)]),
				(code[nameof(Code.VEX_Vpsllq_ymm_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsllq)]),
				(code[nameof(Code.EVEX_Vpsllq_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsllq)]),
				(code[nameof(Code.EVEX_Vpsllq_ymm_k1z_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsllq)]),
				(code[nameof(Code.EVEX_Vpsllq_zmm_k1z_zmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsllq)]),
				(code[nameof(Code.Pmuludq_mm_mmm64)], mnemonic[nameof(Mnemonic.Pmuludq)]),
				(code[nameof(Code.Pmuludq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pmuludq)]),
				(code[nameof(Code.VEX_Vpmuludq_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmuludq)]),
				(code[nameof(Code.VEX_Vpmuludq_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmuludq)]),
				(code[nameof(Code.EVEX_Vpmuludq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpmuludq)]),
				(code[nameof(Code.EVEX_Vpmuludq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpmuludq)]),
				(code[nameof(Code.EVEX_Vpmuludq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpmuludq)]),
				(code[nameof(Code.Pmaddwd_mm_mmm64)], mnemonic[nameof(Mnemonic.Pmaddwd)]),
				(code[nameof(Code.Pmaddwd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pmaddwd)]),
				(code[nameof(Code.VEX_Vpmaddwd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmaddwd)]),
				(code[nameof(Code.VEX_Vpmaddwd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmaddwd)]),
				(code[nameof(Code.EVEX_Vpmaddwd_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmaddwd)]),
				(code[nameof(Code.EVEX_Vpmaddwd_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmaddwd)]),
				(code[nameof(Code.EVEX_Vpmaddwd_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpmaddwd)]),
				(code[nameof(Code.Psadbw_mm_mmm64)], mnemonic[nameof(Mnemonic.Psadbw)]),
				(code[nameof(Code.Psadbw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Psadbw)]),
				(code[nameof(Code.VEX_Vpsadbw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsadbw)]),
				(code[nameof(Code.VEX_Vpsadbw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsadbw)]),
				(code[nameof(Code.EVEX_Vpsadbw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsadbw)]),
				(code[nameof(Code.EVEX_Vpsadbw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsadbw)]),
				(code[nameof(Code.EVEX_Vpsadbw_zmm_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpsadbw)]),
				(code[nameof(Code.Maskmovq_rDI_mm_mm)], mnemonic[nameof(Mnemonic.Maskmovq)]),
				(code[nameof(Code.Maskmovdqu_rDI_xmm_xmm)], mnemonic[nameof(Mnemonic.Maskmovdqu)]),
				(code[nameof(Code.VEX_Vmaskmovdqu_rDI_xmm_xmm)], mnemonic[nameof(Mnemonic.Vmaskmovdqu)]),
				(code[nameof(Code.Psubb_mm_mmm64)], mnemonic[nameof(Mnemonic.Psubb)]),
				(code[nameof(Code.Psubb_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Psubb)]),
				(code[nameof(Code.VEX_Vpsubb_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsubb)]),
				(code[nameof(Code.VEX_Vpsubb_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsubb)]),
				(code[nameof(Code.EVEX_Vpsubb_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsubb)]),
				(code[nameof(Code.EVEX_Vpsubb_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsubb)]),
				(code[nameof(Code.EVEX_Vpsubb_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpsubb)]),
				(code[nameof(Code.Psubw_mm_mmm64)], mnemonic[nameof(Mnemonic.Psubw)]),
				(code[nameof(Code.Psubw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Psubw)]),
				(code[nameof(Code.VEX_Vpsubw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsubw)]),
				(code[nameof(Code.VEX_Vpsubw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsubw)]),
				(code[nameof(Code.EVEX_Vpsubw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsubw)]),
				(code[nameof(Code.EVEX_Vpsubw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsubw)]),
				(code[nameof(Code.EVEX_Vpsubw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpsubw)]),
				(code[nameof(Code.Psubd_mm_mmm64)], mnemonic[nameof(Mnemonic.Psubd)]),
				(code[nameof(Code.Psubd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Psubd)]),
				(code[nameof(Code.VEX_Vpsubd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsubd)]),
				(code[nameof(Code.VEX_Vpsubd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsubd)]),
				(code[nameof(Code.EVEX_Vpsubd_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpsubd)]),
				(code[nameof(Code.EVEX_Vpsubd_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpsubd)]),
				(code[nameof(Code.EVEX_Vpsubd_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpsubd)]),
				(code[nameof(Code.Psubq_mm_mmm64)], mnemonic[nameof(Mnemonic.Psubq)]),
				(code[nameof(Code.Psubq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Psubq)]),
				(code[nameof(Code.VEX_Vpsubq_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsubq)]),
				(code[nameof(Code.VEX_Vpsubq_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsubq)]),
				(code[nameof(Code.EVEX_Vpsubq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpsubq)]),
				(code[nameof(Code.EVEX_Vpsubq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpsubq)]),
				(code[nameof(Code.EVEX_Vpsubq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpsubq)]),
				(code[nameof(Code.Paddb_mm_mmm64)], mnemonic[nameof(Mnemonic.Paddb)]),
				(code[nameof(Code.Paddb_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Paddb)]),
				(code[nameof(Code.VEX_Vpaddb_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpaddb)]),
				(code[nameof(Code.VEX_Vpaddb_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpaddb)]),
				(code[nameof(Code.EVEX_Vpaddb_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpaddb)]),
				(code[nameof(Code.EVEX_Vpaddb_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpaddb)]),
				(code[nameof(Code.EVEX_Vpaddb_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpaddb)]),
				(code[nameof(Code.Paddw_mm_mmm64)], mnemonic[nameof(Mnemonic.Paddw)]),
				(code[nameof(Code.Paddw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Paddw)]),
				(code[nameof(Code.VEX_Vpaddw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpaddw)]),
				(code[nameof(Code.VEX_Vpaddw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpaddw)]),
				(code[nameof(Code.EVEX_Vpaddw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpaddw)]),
				(code[nameof(Code.EVEX_Vpaddw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpaddw)]),
				(code[nameof(Code.EVEX_Vpaddw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpaddw)]),
				(code[nameof(Code.Paddd_mm_mmm64)], mnemonic[nameof(Mnemonic.Paddd)]),
				(code[nameof(Code.Paddd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Paddd)]),
				(code[nameof(Code.VEX_Vpaddd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpaddd)]),
				(code[nameof(Code.VEX_Vpaddd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpaddd)]),
				(code[nameof(Code.EVEX_Vpaddd_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpaddd)]),
				(code[nameof(Code.EVEX_Vpaddd_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpaddd)]),
				(code[nameof(Code.EVEX_Vpaddd_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpaddd)]),
				(code[nameof(Code.Ud0_r16_rm16)], mnemonic[nameof(Mnemonic.Ud0)]),
				(code[nameof(Code.Ud0_r32_rm32)], mnemonic[nameof(Mnemonic.Ud0)]),
				(code[nameof(Code.Ud0_r64_rm64)], mnemonic[nameof(Mnemonic.Ud0)]),
				(code[nameof(Code.Pshufb_mm_mmm64)], mnemonic[nameof(Mnemonic.Pshufb)]),
				(code[nameof(Code.Pshufb_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pshufb)]),
				(code[nameof(Code.VEX_Vpshufb_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpshufb)]),
				(code[nameof(Code.VEX_Vpshufb_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpshufb)]),
				(code[nameof(Code.EVEX_Vpshufb_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpshufb)]),
				(code[nameof(Code.EVEX_Vpshufb_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpshufb)]),
				(code[nameof(Code.EVEX_Vpshufb_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpshufb)]),
				(code[nameof(Code.Phaddw_mm_mmm64)], mnemonic[nameof(Mnemonic.Phaddw)]),
				(code[nameof(Code.Phaddw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Phaddw)]),
				(code[nameof(Code.VEX_Vphaddw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphaddw)]),
				(code[nameof(Code.VEX_Vphaddw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vphaddw)]),
				(code[nameof(Code.Phaddd_mm_mmm64)], mnemonic[nameof(Mnemonic.Phaddd)]),
				(code[nameof(Code.Phaddd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Phaddd)]),
				(code[nameof(Code.VEX_Vphaddd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphaddd)]),
				(code[nameof(Code.VEX_Vphaddd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vphaddd)]),
				(code[nameof(Code.Phaddsw_mm_mmm64)], mnemonic[nameof(Mnemonic.Phaddsw)]),
				(code[nameof(Code.Phaddsw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Phaddsw)]),
				(code[nameof(Code.VEX_Vphaddsw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphaddsw)]),
				(code[nameof(Code.VEX_Vphaddsw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vphaddsw)]),
				(code[nameof(Code.Pmaddubsw_mm_mmm64)], mnemonic[nameof(Mnemonic.Pmaddubsw)]),
				(code[nameof(Code.Pmaddubsw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pmaddubsw)]),
				(code[nameof(Code.VEX_Vpmaddubsw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmaddubsw)]),
				(code[nameof(Code.VEX_Vpmaddubsw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmaddubsw)]),
				(code[nameof(Code.EVEX_Vpmaddubsw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmaddubsw)]),
				(code[nameof(Code.EVEX_Vpmaddubsw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmaddubsw)]),
				(code[nameof(Code.EVEX_Vpmaddubsw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpmaddubsw)]),
				(code[nameof(Code.Phsubw_mm_mmm64)], mnemonic[nameof(Mnemonic.Phsubw)]),
				(code[nameof(Code.Phsubw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Phsubw)]),
				(code[nameof(Code.VEX_Vphsubw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphsubw)]),
				(code[nameof(Code.VEX_Vphsubw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vphsubw)]),
				(code[nameof(Code.Phsubd_mm_mmm64)], mnemonic[nameof(Mnemonic.Phsubd)]),
				(code[nameof(Code.Phsubd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Phsubd)]),
				(code[nameof(Code.VEX_Vphsubd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphsubd)]),
				(code[nameof(Code.VEX_Vphsubd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vphsubd)]),
				(code[nameof(Code.Phsubsw_mm_mmm64)], mnemonic[nameof(Mnemonic.Phsubsw)]),
				(code[nameof(Code.Phsubsw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Phsubsw)]),
				(code[nameof(Code.VEX_Vphsubsw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphsubsw)]),
				(code[nameof(Code.VEX_Vphsubsw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vphsubsw)]),
				(code[nameof(Code.Psignb_mm_mmm64)], mnemonic[nameof(Mnemonic.Psignb)]),
				(code[nameof(Code.Psignb_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Psignb)]),
				(code[nameof(Code.VEX_Vpsignb_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsignb)]),
				(code[nameof(Code.VEX_Vpsignb_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsignb)]),
				(code[nameof(Code.Psignw_mm_mmm64)], mnemonic[nameof(Mnemonic.Psignw)]),
				(code[nameof(Code.Psignw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Psignw)]),
				(code[nameof(Code.VEX_Vpsignw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsignw)]),
				(code[nameof(Code.VEX_Vpsignw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsignw)]),
				(code[nameof(Code.Psignd_mm_mmm64)], mnemonic[nameof(Mnemonic.Psignd)]),
				(code[nameof(Code.Psignd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Psignd)]),
				(code[nameof(Code.VEX_Vpsignd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsignd)]),
				(code[nameof(Code.VEX_Vpsignd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsignd)]),
				(code[nameof(Code.Pmulhrsw_mm_mmm64)], mnemonic[nameof(Mnemonic.Pmulhrsw)]),
				(code[nameof(Code.Pmulhrsw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pmulhrsw)]),
				(code[nameof(Code.VEX_Vpmulhrsw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmulhrsw)]),
				(code[nameof(Code.VEX_Vpmulhrsw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmulhrsw)]),
				(code[nameof(Code.EVEX_Vpmulhrsw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmulhrsw)]),
				(code[nameof(Code.EVEX_Vpmulhrsw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmulhrsw)]),
				(code[nameof(Code.EVEX_Vpmulhrsw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpmulhrsw)]),
				(code[nameof(Code.VEX_Vpermilps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpermilps)]),
				(code[nameof(Code.VEX_Vpermilps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpermilps)]),
				(code[nameof(Code.EVEX_Vpermilps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpermilps)]),
				(code[nameof(Code.EVEX_Vpermilps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpermilps)]),
				(code[nameof(Code.EVEX_Vpermilps_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpermilps)]),
				(code[nameof(Code.VEX_Vpermilpd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpermilpd)]),
				(code[nameof(Code.VEX_Vpermilpd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpermilpd)]),
				(code[nameof(Code.EVEX_Vpermilpd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpermilpd)]),
				(code[nameof(Code.EVEX_Vpermilpd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpermilpd)]),
				(code[nameof(Code.EVEX_Vpermilpd_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpermilpd)]),
				(code[nameof(Code.VEX_Vtestps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vtestps)]),
				(code[nameof(Code.VEX_Vtestps_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vtestps)]),
				(code[nameof(Code.VEX_Vtestpd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vtestpd)]),
				(code[nameof(Code.VEX_Vtestpd_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vtestpd)]),
				(code[nameof(Code.Pblendvb_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pblendvb)]),
				(code[nameof(Code.EVEX_Vpsrlvw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrlvw)]),
				(code[nameof(Code.EVEX_Vpsrlvw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsrlvw)]),
				(code[nameof(Code.EVEX_Vpsrlvw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpsrlvw)]),
				(code[nameof(Code.EVEX_Vpmovuswb_xmmm64_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpmovuswb)]),
				(code[nameof(Code.EVEX_Vpmovuswb_xmmm128_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpmovuswb)]),
				(code[nameof(Code.EVEX_Vpmovuswb_ymmm256_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpmovuswb)]),
				(code[nameof(Code.EVEX_Vpsravw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsravw)]),
				(code[nameof(Code.EVEX_Vpsravw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsravw)]),
				(code[nameof(Code.EVEX_Vpsravw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpsravw)]),
				(code[nameof(Code.EVEX_Vpmovusdb_xmmm32_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpmovusdb)]),
				(code[nameof(Code.EVEX_Vpmovusdb_xmmm64_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpmovusdb)]),
				(code[nameof(Code.EVEX_Vpmovusdb_xmmm128_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpmovusdb)]),
				(code[nameof(Code.EVEX_Vpsllvw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsllvw)]),
				(code[nameof(Code.EVEX_Vpsllvw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsllvw)]),
				(code[nameof(Code.EVEX_Vpsllvw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpsllvw)]),
				(code[nameof(Code.EVEX_Vpmovusqb_xmmm16_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpmovusqb)]),
				(code[nameof(Code.EVEX_Vpmovusqb_xmmm32_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpmovusqb)]),
				(code[nameof(Code.EVEX_Vpmovusqb_xmmm64_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpmovusqb)]),
				(code[nameof(Code.VEX_Vcvtph2ps_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vcvtph2ps)]),
				(code[nameof(Code.VEX_Vcvtph2ps_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vcvtph2ps)]),
				(code[nameof(Code.EVEX_Vcvtph2ps_xmm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vcvtph2ps)]),
				(code[nameof(Code.EVEX_Vcvtph2ps_ymm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vcvtph2ps)]),
				(code[nameof(Code.EVEX_Vcvtph2ps_zmm_k1z_ymmm256_sae)], mnemonic[nameof(Mnemonic.Vcvtph2ps)]),
				(code[nameof(Code.EVEX_Vpmovusdw_xmmm64_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpmovusdw)]),
				(code[nameof(Code.EVEX_Vpmovusdw_xmmm128_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpmovusdw)]),
				(code[nameof(Code.EVEX_Vpmovusdw_ymmm256_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpmovusdw)]),
				(code[nameof(Code.Blendvps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Blendvps)]),
				(code[nameof(Code.EVEX_Vprorvd_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vprorvd)]),
				(code[nameof(Code.EVEX_Vprorvd_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vprorvd)]),
				(code[nameof(Code.EVEX_Vprorvd_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vprorvd)]),
				(code[nameof(Code.EVEX_Vprorvq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vprorvq)]),
				(code[nameof(Code.EVEX_Vprorvq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vprorvq)]),
				(code[nameof(Code.EVEX_Vprorvq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vprorvq)]),
				(code[nameof(Code.EVEX_Vpmovusqw_xmmm32_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpmovusqw)]),
				(code[nameof(Code.EVEX_Vpmovusqw_xmmm64_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpmovusqw)]),
				(code[nameof(Code.EVEX_Vpmovusqw_xmmm128_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpmovusqw)]),
				(code[nameof(Code.Blendvpd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Blendvpd)]),
				(code[nameof(Code.EVEX_Vprolvd_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vprolvd)]),
				(code[nameof(Code.EVEX_Vprolvd_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vprolvd)]),
				(code[nameof(Code.EVEX_Vprolvd_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vprolvd)]),
				(code[nameof(Code.EVEX_Vprolvq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vprolvq)]),
				(code[nameof(Code.EVEX_Vprolvq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vprolvq)]),
				(code[nameof(Code.EVEX_Vprolvq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vprolvq)]),
				(code[nameof(Code.EVEX_Vpmovusqd_xmmm64_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpmovusqd)]),
				(code[nameof(Code.EVEX_Vpmovusqd_xmmm128_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpmovusqd)]),
				(code[nameof(Code.EVEX_Vpmovusqd_ymmm256_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpmovusqd)]),
				(code[nameof(Code.VEX_Vpermps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpermps)]),
				(code[nameof(Code.EVEX_Vpermps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpermps)]),
				(code[nameof(Code.EVEX_Vpermps_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpermps)]),
				(code[nameof(Code.EVEX_Vpermpd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpermpd)]),
				(code[nameof(Code.EVEX_Vpermpd_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpermpd)]),
				(code[nameof(Code.Ptest_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Ptest)]),
				(code[nameof(Code.VEX_Vptest_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vptest)]),
				(code[nameof(Code.VEX_Vptest_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vptest)]),
				(code[nameof(Code.VEX_Vbroadcastss_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vbroadcastss)]),
				(code[nameof(Code.VEX_Vbroadcastss_ymm_xmmm32)], mnemonic[nameof(Mnemonic.Vbroadcastss)]),
				(code[nameof(Code.EVEX_Vbroadcastss_xmm_k1z_xmmm32)], mnemonic[nameof(Mnemonic.Vbroadcastss)]),
				(code[nameof(Code.EVEX_Vbroadcastss_ymm_k1z_xmmm32)], mnemonic[nameof(Mnemonic.Vbroadcastss)]),
				(code[nameof(Code.EVEX_Vbroadcastss_zmm_k1z_xmmm32)], mnemonic[nameof(Mnemonic.Vbroadcastss)]),
				(code[nameof(Code.VEX_Vbroadcastsd_ymm_xmmm64)], mnemonic[nameof(Mnemonic.Vbroadcastsd)]),
				(code[nameof(Code.EVEX_Vbroadcastf32x2_ymm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vbroadcastf32x2)]),
				(code[nameof(Code.EVEX_Vbroadcastf32x2_zmm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vbroadcastf32x2)]),
				(code[nameof(Code.EVEX_Vbroadcastsd_ymm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vbroadcastsd)]),
				(code[nameof(Code.EVEX_Vbroadcastsd_zmm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vbroadcastsd)]),
				(code[nameof(Code.VEX_Vbroadcastf128_ymm_m128)], mnemonic[nameof(Mnemonic.Vbroadcastf128)]),
				(code[nameof(Code.EVEX_Vbroadcastf32x4_ymm_k1z_m128)], mnemonic[nameof(Mnemonic.Vbroadcastf32x4)]),
				(code[nameof(Code.EVEX_Vbroadcastf32x4_zmm_k1z_m128)], mnemonic[nameof(Mnemonic.Vbroadcastf32x4)]),
				(code[nameof(Code.EVEX_Vbroadcastf64x2_ymm_k1z_m128)], mnemonic[nameof(Mnemonic.Vbroadcastf64x2)]),
				(code[nameof(Code.EVEX_Vbroadcastf64x2_zmm_k1z_m128)], mnemonic[nameof(Mnemonic.Vbroadcastf64x2)]),
				(code[nameof(Code.EVEX_Vbroadcastf32x8_zmm_k1z_m256)], mnemonic[nameof(Mnemonic.Vbroadcastf32x8)]),
				(code[nameof(Code.EVEX_Vbroadcastf64x4_zmm_k1z_m256)], mnemonic[nameof(Mnemonic.Vbroadcastf64x4)]),
				(code[nameof(Code.Pabsb_mm_mmm64)], mnemonic[nameof(Mnemonic.Pabsb)]),
				(code[nameof(Code.Pabsb_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pabsb)]),
				(code[nameof(Code.VEX_Vpabsb_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpabsb)]),
				(code[nameof(Code.VEX_Vpabsb_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpabsb)]),
				(code[nameof(Code.EVEX_Vpabsb_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vpabsb)]),
				(code[nameof(Code.EVEX_Vpabsb_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vpabsb)]),
				(code[nameof(Code.EVEX_Vpabsb_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vpabsb)]),
				(code[nameof(Code.Pabsw_mm_mmm64)], mnemonic[nameof(Mnemonic.Pabsw)]),
				(code[nameof(Code.Pabsw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pabsw)]),
				(code[nameof(Code.VEX_Vpabsw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpabsw)]),
				(code[nameof(Code.VEX_Vpabsw_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpabsw)]),
				(code[nameof(Code.EVEX_Vpabsw_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vpabsw)]),
				(code[nameof(Code.EVEX_Vpabsw_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vpabsw)]),
				(code[nameof(Code.EVEX_Vpabsw_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vpabsw)]),
				(code[nameof(Code.Pabsd_mm_mmm64)], mnemonic[nameof(Mnemonic.Pabsd)]),
				(code[nameof(Code.Pabsd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pabsd)]),
				(code[nameof(Code.VEX_Vpabsd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpabsd)]),
				(code[nameof(Code.VEX_Vpabsd_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpabsd)]),
				(code[nameof(Code.EVEX_Vpabsd_xmm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpabsd)]),
				(code[nameof(Code.EVEX_Vpabsd_ymm_k1z_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpabsd)]),
				(code[nameof(Code.EVEX_Vpabsd_zmm_k1z_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpabsd)]),
				(code[nameof(Code.EVEX_Vpabsq_xmm_k1z_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpabsq)]),
				(code[nameof(Code.EVEX_Vpabsq_ymm_k1z_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpabsq)]),
				(code[nameof(Code.EVEX_Vpabsq_zmm_k1z_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpabsq)]),
				(code[nameof(Code.Pmovsxbw_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Pmovsxbw)]),
				(code[nameof(Code.VEX_Vpmovsxbw_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovsxbw)]),
				(code[nameof(Code.VEX_Vpmovsxbw_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmovsxbw)]),
				(code[nameof(Code.EVEX_Vpmovsxbw_xmm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovsxbw)]),
				(code[nameof(Code.EVEX_Vpmovsxbw_ymm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vpmovsxbw)]),
				(code[nameof(Code.EVEX_Vpmovsxbw_zmm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vpmovsxbw)]),
				(code[nameof(Code.EVEX_Vpmovswb_xmmm64_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpmovswb)]),
				(code[nameof(Code.EVEX_Vpmovswb_xmmm128_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpmovswb)]),
				(code[nameof(Code.EVEX_Vpmovswb_ymmm256_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpmovswb)]),
				(code[nameof(Code.Pmovsxbd_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Pmovsxbd)]),
				(code[nameof(Code.VEX_Vpmovsxbd_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vpmovsxbd)]),
				(code[nameof(Code.VEX_Vpmovsxbd_ymm_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovsxbd)]),
				(code[nameof(Code.EVEX_Vpmovsxbd_xmm_k1z_xmmm32)], mnemonic[nameof(Mnemonic.Vpmovsxbd)]),
				(code[nameof(Code.EVEX_Vpmovsxbd_ymm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovsxbd)]),
				(code[nameof(Code.EVEX_Vpmovsxbd_zmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vpmovsxbd)]),
				(code[nameof(Code.EVEX_Vpmovsdb_xmmm32_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpmovsdb)]),
				(code[nameof(Code.EVEX_Vpmovsdb_xmmm64_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpmovsdb)]),
				(code[nameof(Code.EVEX_Vpmovsdb_xmmm128_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpmovsdb)]),
				(code[nameof(Code.Pmovsxbq_xmm_xmmm16)], mnemonic[nameof(Mnemonic.Pmovsxbq)]),
				(code[nameof(Code.VEX_Vpmovsxbq_xmm_xmmm16)], mnemonic[nameof(Mnemonic.Vpmovsxbq)]),
				(code[nameof(Code.VEX_Vpmovsxbq_ymm_xmmm32)], mnemonic[nameof(Mnemonic.Vpmovsxbq)]),
				(code[nameof(Code.EVEX_Vpmovsxbq_xmm_k1z_xmmm16)], mnemonic[nameof(Mnemonic.Vpmovsxbq)]),
				(code[nameof(Code.EVEX_Vpmovsxbq_ymm_k1z_xmmm32)], mnemonic[nameof(Mnemonic.Vpmovsxbq)]),
				(code[nameof(Code.EVEX_Vpmovsxbq_zmm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovsxbq)]),
				(code[nameof(Code.EVEX_Vpmovsqb_xmmm16_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpmovsqb)]),
				(code[nameof(Code.EVEX_Vpmovsqb_xmmm32_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpmovsqb)]),
				(code[nameof(Code.EVEX_Vpmovsqb_xmmm64_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpmovsqb)]),
				(code[nameof(Code.Pmovsxwd_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Pmovsxwd)]),
				(code[nameof(Code.VEX_Vpmovsxwd_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovsxwd)]),
				(code[nameof(Code.VEX_Vpmovsxwd_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmovsxwd)]),
				(code[nameof(Code.EVEX_Vpmovsxwd_xmm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovsxwd)]),
				(code[nameof(Code.EVEX_Vpmovsxwd_ymm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vpmovsxwd)]),
				(code[nameof(Code.EVEX_Vpmovsxwd_zmm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vpmovsxwd)]),
				(code[nameof(Code.EVEX_Vpmovsdw_xmmm64_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpmovsdw)]),
				(code[nameof(Code.EVEX_Vpmovsdw_xmmm128_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpmovsdw)]),
				(code[nameof(Code.EVEX_Vpmovsdw_ymmm256_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpmovsdw)]),
				(code[nameof(Code.Pmovsxwq_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Pmovsxwq)]),
				(code[nameof(Code.VEX_Vpmovsxwq_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vpmovsxwq)]),
				(code[nameof(Code.VEX_Vpmovsxwq_ymm_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovsxwq)]),
				(code[nameof(Code.EVEX_Vpmovsxwq_xmm_k1z_xmmm32)], mnemonic[nameof(Mnemonic.Vpmovsxwq)]),
				(code[nameof(Code.EVEX_Vpmovsxwq_ymm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovsxwq)]),
				(code[nameof(Code.EVEX_Vpmovsxwq_zmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vpmovsxwq)]),
				(code[nameof(Code.EVEX_Vpmovsqw_xmmm32_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpmovsqw)]),
				(code[nameof(Code.EVEX_Vpmovsqw_xmmm64_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpmovsqw)]),
				(code[nameof(Code.EVEX_Vpmovsqw_xmmm128_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpmovsqw)]),
				(code[nameof(Code.Pmovsxdq_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Pmovsxdq)]),
				(code[nameof(Code.VEX_Vpmovsxdq_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovsxdq)]),
				(code[nameof(Code.VEX_Vpmovsxdq_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmovsxdq)]),
				(code[nameof(Code.EVEX_Vpmovsxdq_xmm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovsxdq)]),
				(code[nameof(Code.EVEX_Vpmovsxdq_ymm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vpmovsxdq)]),
				(code[nameof(Code.EVEX_Vpmovsxdq_zmm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vpmovsxdq)]),
				(code[nameof(Code.EVEX_Vpmovsqd_xmmm64_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpmovsqd)]),
				(code[nameof(Code.EVEX_Vpmovsqd_xmmm128_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpmovsqd)]),
				(code[nameof(Code.EVEX_Vpmovsqd_ymmm256_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpmovsqd)]),
				(code[nameof(Code.EVEX_Vptestmb_k_k1_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vptestmb)]),
				(code[nameof(Code.EVEX_Vptestmb_k_k1_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vptestmb)]),
				(code[nameof(Code.EVEX_Vptestmb_k_k1_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vptestmb)]),
				(code[nameof(Code.EVEX_Vptestmw_k_k1_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vptestmw)]),
				(code[nameof(Code.EVEX_Vptestmw_k_k1_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vptestmw)]),
				(code[nameof(Code.EVEX_Vptestmw_k_k1_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vptestmw)]),
				(code[nameof(Code.EVEX_Vptestnmb_k_k1_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vptestnmb)]),
				(code[nameof(Code.EVEX_Vptestnmb_k_k1_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vptestnmb)]),
				(code[nameof(Code.EVEX_Vptestnmb_k_k1_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vptestnmb)]),
				(code[nameof(Code.EVEX_Vptestnmw_k_k1_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vptestnmw)]),
				(code[nameof(Code.EVEX_Vptestnmw_k_k1_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vptestnmw)]),
				(code[nameof(Code.EVEX_Vptestnmw_k_k1_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vptestnmw)]),
				(code[nameof(Code.EVEX_Vptestmd_k_k1_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vptestmd)]),
				(code[nameof(Code.EVEX_Vptestmd_k_k1_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vptestmd)]),
				(code[nameof(Code.EVEX_Vptestmd_k_k1_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vptestmd)]),
				(code[nameof(Code.EVEX_Vptestmq_k_k1_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vptestmq)]),
				(code[nameof(Code.EVEX_Vptestmq_k_k1_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vptestmq)]),
				(code[nameof(Code.EVEX_Vptestmq_k_k1_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vptestmq)]),
				(code[nameof(Code.EVEX_Vptestnmd_k_k1_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vptestnmd)]),
				(code[nameof(Code.EVEX_Vptestnmd_k_k1_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vptestnmd)]),
				(code[nameof(Code.EVEX_Vptestnmd_k_k1_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vptestnmd)]),
				(code[nameof(Code.EVEX_Vptestnmq_k_k1_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vptestnmq)]),
				(code[nameof(Code.EVEX_Vptestnmq_k_k1_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vptestnmq)]),
				(code[nameof(Code.EVEX_Vptestnmq_k_k1_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vptestnmq)]),
				(code[nameof(Code.Pmuldq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pmuldq)]),
				(code[nameof(Code.VEX_Vpmuldq_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmuldq)]),
				(code[nameof(Code.VEX_Vpmuldq_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmuldq)]),
				(code[nameof(Code.EVEX_Vpmuldq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpmuldq)]),
				(code[nameof(Code.EVEX_Vpmuldq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpmuldq)]),
				(code[nameof(Code.EVEX_Vpmuldq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpmuldq)]),
				(code[nameof(Code.EVEX_Vpmovm2b_xmm_k)], mnemonic[nameof(Mnemonic.Vpmovm2b)]),
				(code[nameof(Code.EVEX_Vpmovm2b_ymm_k)], mnemonic[nameof(Mnemonic.Vpmovm2b)]),
				(code[nameof(Code.EVEX_Vpmovm2b_zmm_k)], mnemonic[nameof(Mnemonic.Vpmovm2b)]),
				(code[nameof(Code.EVEX_Vpmovm2w_xmm_k)], mnemonic[nameof(Mnemonic.Vpmovm2w)]),
				(code[nameof(Code.EVEX_Vpmovm2w_ymm_k)], mnemonic[nameof(Mnemonic.Vpmovm2w)]),
				(code[nameof(Code.EVEX_Vpmovm2w_zmm_k)], mnemonic[nameof(Mnemonic.Vpmovm2w)]),
				(code[nameof(Code.Pcmpeqq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pcmpeqq)]),
				(code[nameof(Code.VEX_Vpcmpeqq_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpcmpeqq)]),
				(code[nameof(Code.VEX_Vpcmpeqq_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpcmpeqq)]),
				(code[nameof(Code.EVEX_Vpcmpeqq_k_k1_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpcmpeqq)]),
				(code[nameof(Code.EVEX_Vpcmpeqq_k_k1_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpcmpeqq)]),
				(code[nameof(Code.EVEX_Vpcmpeqq_k_k1_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpcmpeqq)]),
				(code[nameof(Code.EVEX_Vpmovb2m_k_xmm)], mnemonic[nameof(Mnemonic.Vpmovb2m)]),
				(code[nameof(Code.EVEX_Vpmovb2m_k_ymm)], mnemonic[nameof(Mnemonic.Vpmovb2m)]),
				(code[nameof(Code.EVEX_Vpmovb2m_k_zmm)], mnemonic[nameof(Mnemonic.Vpmovb2m)]),
				(code[nameof(Code.EVEX_Vpmovw2m_k_xmm)], mnemonic[nameof(Mnemonic.Vpmovw2m)]),
				(code[nameof(Code.EVEX_Vpmovw2m_k_ymm)], mnemonic[nameof(Mnemonic.Vpmovw2m)]),
				(code[nameof(Code.EVEX_Vpmovw2m_k_zmm)], mnemonic[nameof(Mnemonic.Vpmovw2m)]),
				(code[nameof(Code.Movntdqa_xmm_m128)], mnemonic[nameof(Mnemonic.Movntdqa)]),
				(code[nameof(Code.VEX_Vmovntdqa_xmm_m128)], mnemonic[nameof(Mnemonic.Vmovntdqa)]),
				(code[nameof(Code.VEX_Vmovntdqa_ymm_m256)], mnemonic[nameof(Mnemonic.Vmovntdqa)]),
				(code[nameof(Code.EVEX_Vmovntdqa_xmm_m128)], mnemonic[nameof(Mnemonic.Vmovntdqa)]),
				(code[nameof(Code.EVEX_Vmovntdqa_ymm_m256)], mnemonic[nameof(Mnemonic.Vmovntdqa)]),
				(code[nameof(Code.EVEX_Vmovntdqa_zmm_m512)], mnemonic[nameof(Mnemonic.Vmovntdqa)]),
				(code[nameof(Code.EVEX_Vpbroadcastmb2q_xmm_k)], mnemonic[nameof(Mnemonic.Vpbroadcastmb2q)]),
				(code[nameof(Code.EVEX_Vpbroadcastmb2q_ymm_k)], mnemonic[nameof(Mnemonic.Vpbroadcastmb2q)]),
				(code[nameof(Code.EVEX_Vpbroadcastmb2q_zmm_k)], mnemonic[nameof(Mnemonic.Vpbroadcastmb2q)]),
				(code[nameof(Code.Packusdw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Packusdw)]),
				(code[nameof(Code.VEX_Vpackusdw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpackusdw)]),
				(code[nameof(Code.VEX_Vpackusdw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpackusdw)]),
				(code[nameof(Code.EVEX_Vpackusdw_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpackusdw)]),
				(code[nameof(Code.EVEX_Vpackusdw_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpackusdw)]),
				(code[nameof(Code.EVEX_Vpackusdw_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpackusdw)]),
				(code[nameof(Code.VEX_Vmaskmovps_xmm_xmm_m128)], mnemonic[nameof(Mnemonic.Vmaskmovps)]),
				(code[nameof(Code.VEX_Vmaskmovps_ymm_ymm_m256)], mnemonic[nameof(Mnemonic.Vmaskmovps)]),
				(code[nameof(Code.EVEX_Vscalefps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vscalefps)]),
				(code[nameof(Code.EVEX_Vscalefps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vscalefps)]),
				(code[nameof(Code.EVEX_Vscalefps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vscalefps)]),
				(code[nameof(Code.EVEX_Vscalefpd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vscalefpd)]),
				(code[nameof(Code.EVEX_Vscalefpd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vscalefpd)]),
				(code[nameof(Code.EVEX_Vscalefpd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vscalefpd)]),
				(code[nameof(Code.VEX_Vmaskmovpd_xmm_xmm_m128)], mnemonic[nameof(Mnemonic.Vmaskmovpd)]),
				(code[nameof(Code.VEX_Vmaskmovpd_ymm_ymm_m256)], mnemonic[nameof(Mnemonic.Vmaskmovpd)]),
				(code[nameof(Code.EVEX_Vscalefss_xmm_k1z_xmm_xmmm32_er)], mnemonic[nameof(Mnemonic.Vscalefss)]),
				(code[nameof(Code.EVEX_Vscalefsd_xmm_k1z_xmm_xmmm64_er)], mnemonic[nameof(Mnemonic.Vscalefsd)]),
				(code[nameof(Code.VEX_Vmaskmovps_m128_xmm_xmm)], mnemonic[nameof(Mnemonic.Vmaskmovps)]),
				(code[nameof(Code.VEX_Vmaskmovps_m256_ymm_ymm)], mnemonic[nameof(Mnemonic.Vmaskmovps)]),
				(code[nameof(Code.VEX_Vmaskmovpd_m128_xmm_xmm)], mnemonic[nameof(Mnemonic.Vmaskmovpd)]),
				(code[nameof(Code.VEX_Vmaskmovpd_m256_ymm_ymm)], mnemonic[nameof(Mnemonic.Vmaskmovpd)]),
				(code[nameof(Code.Pmovzxbw_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Pmovzxbw)]),
				(code[nameof(Code.VEX_Vpmovzxbw_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovzxbw)]),
				(code[nameof(Code.VEX_Vpmovzxbw_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmovzxbw)]),
				(code[nameof(Code.EVEX_Vpmovzxbw_xmm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovzxbw)]),
				(code[nameof(Code.EVEX_Vpmovzxbw_ymm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vpmovzxbw)]),
				(code[nameof(Code.EVEX_Vpmovzxbw_zmm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vpmovzxbw)]),
				(code[nameof(Code.EVEX_Vpmovwb_xmmm64_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpmovwb)]),
				(code[nameof(Code.EVEX_Vpmovwb_xmmm128_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpmovwb)]),
				(code[nameof(Code.EVEX_Vpmovwb_ymmm256_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpmovwb)]),
				(code[nameof(Code.Pmovzxbd_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Pmovzxbd)]),
				(code[nameof(Code.VEX_Vpmovzxbd_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vpmovzxbd)]),
				(code[nameof(Code.VEX_Vpmovzxbd_ymm_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovzxbd)]),
				(code[nameof(Code.EVEX_Vpmovzxbd_xmm_k1z_xmmm32)], mnemonic[nameof(Mnemonic.Vpmovzxbd)]),
				(code[nameof(Code.EVEX_Vpmovzxbd_ymm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovzxbd)]),
				(code[nameof(Code.EVEX_Vpmovzxbd_zmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vpmovzxbd)]),
				(code[nameof(Code.EVEX_Vpmovdb_xmmm32_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpmovdb)]),
				(code[nameof(Code.EVEX_Vpmovdb_xmmm64_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpmovdb)]),
				(code[nameof(Code.EVEX_Vpmovdb_xmmm128_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpmovdb)]),
				(code[nameof(Code.Pmovzxbq_xmm_xmmm16)], mnemonic[nameof(Mnemonic.Pmovzxbq)]),
				(code[nameof(Code.VEX_Vpmovzxbq_xmm_xmmm16)], mnemonic[nameof(Mnemonic.Vpmovzxbq)]),
				(code[nameof(Code.VEX_Vpmovzxbq_ymm_xmmm32)], mnemonic[nameof(Mnemonic.Vpmovzxbq)]),
				(code[nameof(Code.EVEX_Vpmovzxbq_xmm_k1z_xmmm16)], mnemonic[nameof(Mnemonic.Vpmovzxbq)]),
				(code[nameof(Code.EVEX_Vpmovzxbq_ymm_k1z_xmmm32)], mnemonic[nameof(Mnemonic.Vpmovzxbq)]),
				(code[nameof(Code.EVEX_Vpmovzxbq_zmm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovzxbq)]),
				(code[nameof(Code.EVEX_Vpmovqb_xmmm16_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpmovqb)]),
				(code[nameof(Code.EVEX_Vpmovqb_xmmm32_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpmovqb)]),
				(code[nameof(Code.EVEX_Vpmovqb_xmmm64_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpmovqb)]),
				(code[nameof(Code.Pmovzxwd_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Pmovzxwd)]),
				(code[nameof(Code.VEX_Vpmovzxwd_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovzxwd)]),
				(code[nameof(Code.VEX_Vpmovzxwd_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmovzxwd)]),
				(code[nameof(Code.EVEX_Vpmovzxwd_xmm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovzxwd)]),
				(code[nameof(Code.EVEX_Vpmovzxwd_ymm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vpmovzxwd)]),
				(code[nameof(Code.EVEX_Vpmovzxwd_zmm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vpmovzxwd)]),
				(code[nameof(Code.EVEX_Vpmovdw_xmmm64_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpmovdw)]),
				(code[nameof(Code.EVEX_Vpmovdw_xmmm128_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpmovdw)]),
				(code[nameof(Code.EVEX_Vpmovdw_ymmm256_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpmovdw)]),
				(code[nameof(Code.Pmovzxwq_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Pmovzxwq)]),
				(code[nameof(Code.VEX_Vpmovzxwq_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vpmovzxwq)]),
				(code[nameof(Code.VEX_Vpmovzxwq_ymm_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovzxwq)]),
				(code[nameof(Code.EVEX_Vpmovzxwq_xmm_k1z_xmmm32)], mnemonic[nameof(Mnemonic.Vpmovzxwq)]),
				(code[nameof(Code.EVEX_Vpmovzxwq_ymm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovzxwq)]),
				(code[nameof(Code.EVEX_Vpmovzxwq_zmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vpmovzxwq)]),
				(code[nameof(Code.EVEX_Vpmovqw_xmmm32_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpmovqw)]),
				(code[nameof(Code.EVEX_Vpmovqw_xmmm64_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpmovqw)]),
				(code[nameof(Code.EVEX_Vpmovqw_xmmm128_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpmovqw)]),
				(code[nameof(Code.Pmovzxdq_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Pmovzxdq)]),
				(code[nameof(Code.VEX_Vpmovzxdq_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovzxdq)]),
				(code[nameof(Code.VEX_Vpmovzxdq_ymm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmovzxdq)]),
				(code[nameof(Code.EVEX_Vpmovzxdq_xmm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vpmovzxdq)]),
				(code[nameof(Code.EVEX_Vpmovzxdq_ymm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vpmovzxdq)]),
				(code[nameof(Code.EVEX_Vpmovzxdq_zmm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vpmovzxdq)]),
				(code[nameof(Code.EVEX_Vpmovqd_xmmm64_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpmovqd)]),
				(code[nameof(Code.EVEX_Vpmovqd_xmmm128_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpmovqd)]),
				(code[nameof(Code.EVEX_Vpmovqd_ymmm256_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpmovqd)]),
				(code[nameof(Code.VEX_Vpermd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpermd)]),
				(code[nameof(Code.EVEX_Vpermd_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpermd)]),
				(code[nameof(Code.EVEX_Vpermd_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpermd)]),
				(code[nameof(Code.EVEX_Vpermq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpermq)]),
				(code[nameof(Code.EVEX_Vpermq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpermq)]),
				(code[nameof(Code.Pcmpgtq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pcmpgtq)]),
				(code[nameof(Code.VEX_Vpcmpgtq_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpcmpgtq)]),
				(code[nameof(Code.VEX_Vpcmpgtq_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpcmpgtq)]),
				(code[nameof(Code.EVEX_Vpcmpgtq_k_k1_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpcmpgtq)]),
				(code[nameof(Code.EVEX_Vpcmpgtq_k_k1_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpcmpgtq)]),
				(code[nameof(Code.EVEX_Vpcmpgtq_k_k1_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpcmpgtq)]),
				(code[nameof(Code.Pminsb_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pminsb)]),
				(code[nameof(Code.VEX_Vpminsb_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpminsb)]),
				(code[nameof(Code.VEX_Vpminsb_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpminsb)]),
				(code[nameof(Code.EVEX_Vpminsb_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpminsb)]),
				(code[nameof(Code.EVEX_Vpminsb_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpminsb)]),
				(code[nameof(Code.EVEX_Vpminsb_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpminsb)]),
				(code[nameof(Code.EVEX_Vpmovm2d_xmm_k)], mnemonic[nameof(Mnemonic.Vpmovm2d)]),
				(code[nameof(Code.EVEX_Vpmovm2d_ymm_k)], mnemonic[nameof(Mnemonic.Vpmovm2d)]),
				(code[nameof(Code.EVEX_Vpmovm2d_zmm_k)], mnemonic[nameof(Mnemonic.Vpmovm2d)]),
				(code[nameof(Code.EVEX_Vpmovm2q_xmm_k)], mnemonic[nameof(Mnemonic.Vpmovm2q)]),
				(code[nameof(Code.EVEX_Vpmovm2q_ymm_k)], mnemonic[nameof(Mnemonic.Vpmovm2q)]),
				(code[nameof(Code.EVEX_Vpmovm2q_zmm_k)], mnemonic[nameof(Mnemonic.Vpmovm2q)]),
				(code[nameof(Code.Pminsd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pminsd)]),
				(code[nameof(Code.VEX_Vpminsd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpminsd)]),
				(code[nameof(Code.VEX_Vpminsd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpminsd)]),
				(code[nameof(Code.EVEX_Vpminsd_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpminsd)]),
				(code[nameof(Code.EVEX_Vpminsd_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpminsd)]),
				(code[nameof(Code.EVEX_Vpminsd_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpminsd)]),
				(code[nameof(Code.EVEX_Vpminsq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpminsq)]),
				(code[nameof(Code.EVEX_Vpminsq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpminsq)]),
				(code[nameof(Code.EVEX_Vpminsq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpminsq)]),
				(code[nameof(Code.EVEX_Vpmovd2m_k_xmm)], mnemonic[nameof(Mnemonic.Vpmovd2m)]),
				(code[nameof(Code.EVEX_Vpmovd2m_k_ymm)], mnemonic[nameof(Mnemonic.Vpmovd2m)]),
				(code[nameof(Code.EVEX_Vpmovd2m_k_zmm)], mnemonic[nameof(Mnemonic.Vpmovd2m)]),
				(code[nameof(Code.EVEX_Vpmovq2m_k_xmm)], mnemonic[nameof(Mnemonic.Vpmovq2m)]),
				(code[nameof(Code.EVEX_Vpmovq2m_k_ymm)], mnemonic[nameof(Mnemonic.Vpmovq2m)]),
				(code[nameof(Code.EVEX_Vpmovq2m_k_zmm)], mnemonic[nameof(Mnemonic.Vpmovq2m)]),
				(code[nameof(Code.Pminuw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pminuw)]),
				(code[nameof(Code.VEX_Vpminuw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpminuw)]),
				(code[nameof(Code.VEX_Vpminuw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpminuw)]),
				(code[nameof(Code.EVEX_Vpminuw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpminuw)]),
				(code[nameof(Code.EVEX_Vpminuw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpminuw)]),
				(code[nameof(Code.EVEX_Vpminuw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpminuw)]),
				(code[nameof(Code.EVEX_Vpbroadcastmw2d_xmm_k)], mnemonic[nameof(Mnemonic.Vpbroadcastmw2d)]),
				(code[nameof(Code.EVEX_Vpbroadcastmw2d_ymm_k)], mnemonic[nameof(Mnemonic.Vpbroadcastmw2d)]),
				(code[nameof(Code.EVEX_Vpbroadcastmw2d_zmm_k)], mnemonic[nameof(Mnemonic.Vpbroadcastmw2d)]),
				(code[nameof(Code.Pminud_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pminud)]),
				(code[nameof(Code.VEX_Vpminud_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpminud)]),
				(code[nameof(Code.VEX_Vpminud_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpminud)]),
				(code[nameof(Code.EVEX_Vpminud_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpminud)]),
				(code[nameof(Code.EVEX_Vpminud_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpminud)]),
				(code[nameof(Code.EVEX_Vpminud_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpminud)]),
				(code[nameof(Code.EVEX_Vpminuq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpminuq)]),
				(code[nameof(Code.EVEX_Vpminuq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpminuq)]),
				(code[nameof(Code.EVEX_Vpminuq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpminuq)]),
				(code[nameof(Code.Pmaxsb_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pmaxsb)]),
				(code[nameof(Code.VEX_Vpmaxsb_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmaxsb)]),
				(code[nameof(Code.VEX_Vpmaxsb_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmaxsb)]),
				(code[nameof(Code.EVEX_Vpmaxsb_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmaxsb)]),
				(code[nameof(Code.EVEX_Vpmaxsb_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmaxsb)]),
				(code[nameof(Code.EVEX_Vpmaxsb_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpmaxsb)]),
				(code[nameof(Code.Pmaxsd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pmaxsd)]),
				(code[nameof(Code.VEX_Vpmaxsd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmaxsd)]),
				(code[nameof(Code.VEX_Vpmaxsd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmaxsd)]),
				(code[nameof(Code.EVEX_Vpmaxsd_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpmaxsd)]),
				(code[nameof(Code.EVEX_Vpmaxsd_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpmaxsd)]),
				(code[nameof(Code.EVEX_Vpmaxsd_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpmaxsd)]),
				(code[nameof(Code.EVEX_Vpmaxsq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpmaxsq)]),
				(code[nameof(Code.EVEX_Vpmaxsq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpmaxsq)]),
				(code[nameof(Code.EVEX_Vpmaxsq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpmaxsq)]),
				(code[nameof(Code.Pmaxuw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pmaxuw)]),
				(code[nameof(Code.VEX_Vpmaxuw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmaxuw)]),
				(code[nameof(Code.VEX_Vpmaxuw_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmaxuw)]),
				(code[nameof(Code.EVEX_Vpmaxuw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmaxuw)]),
				(code[nameof(Code.EVEX_Vpmaxuw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmaxuw)]),
				(code[nameof(Code.EVEX_Vpmaxuw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpmaxuw)]),
				(code[nameof(Code.Pmaxud_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pmaxud)]),
				(code[nameof(Code.VEX_Vpmaxud_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmaxud)]),
				(code[nameof(Code.VEX_Vpmaxud_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmaxud)]),
				(code[nameof(Code.EVEX_Vpmaxud_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpmaxud)]),
				(code[nameof(Code.EVEX_Vpmaxud_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpmaxud)]),
				(code[nameof(Code.EVEX_Vpmaxud_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpmaxud)]),
				(code[nameof(Code.EVEX_Vpmaxuq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpmaxuq)]),
				(code[nameof(Code.EVEX_Vpmaxuq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpmaxuq)]),
				(code[nameof(Code.EVEX_Vpmaxuq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpmaxuq)]),
				(code[nameof(Code.Pmulld_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Pmulld)]),
				(code[nameof(Code.VEX_Vpmulld_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpmulld)]),
				(code[nameof(Code.VEX_Vpmulld_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpmulld)]),
				(code[nameof(Code.EVEX_Vpmulld_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpmulld)]),
				(code[nameof(Code.EVEX_Vpmulld_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpmulld)]),
				(code[nameof(Code.EVEX_Vpmulld_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpmulld)]),
				(code[nameof(Code.EVEX_Vpmullq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpmullq)]),
				(code[nameof(Code.EVEX_Vpmullq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpmullq)]),
				(code[nameof(Code.EVEX_Vpmullq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpmullq)]),
				(code[nameof(Code.Phminposuw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Phminposuw)]),
				(code[nameof(Code.VEX_Vphminposuw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphminposuw)]),
				(code[nameof(Code.EVEX_Vgetexpps_xmm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vgetexpps)]),
				(code[nameof(Code.EVEX_Vgetexpps_ymm_k1z_ymmm256b32)], mnemonic[nameof(Mnemonic.Vgetexpps)]),
				(code[nameof(Code.EVEX_Vgetexpps_zmm_k1z_zmmm512b32_sae)], mnemonic[nameof(Mnemonic.Vgetexpps)]),
				(code[nameof(Code.EVEX_Vgetexppd_xmm_k1z_xmmm128b64)], mnemonic[nameof(Mnemonic.Vgetexppd)]),
				(code[nameof(Code.EVEX_Vgetexppd_ymm_k1z_ymmm256b64)], mnemonic[nameof(Mnemonic.Vgetexppd)]),
				(code[nameof(Code.EVEX_Vgetexppd_zmm_k1z_zmmm512b64_sae)], mnemonic[nameof(Mnemonic.Vgetexppd)]),
				(code[nameof(Code.EVEX_Vgetexpss_xmm_k1z_xmm_xmmm32_sae)], mnemonic[nameof(Mnemonic.Vgetexpss)]),
				(code[nameof(Code.EVEX_Vgetexpsd_xmm_k1z_xmm_xmmm64_sae)], mnemonic[nameof(Mnemonic.Vgetexpsd)]),
				(code[nameof(Code.EVEX_Vplzcntd_xmm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vplzcntd)]),
				(code[nameof(Code.EVEX_Vplzcntd_ymm_k1z_ymmm256b32)], mnemonic[nameof(Mnemonic.Vplzcntd)]),
				(code[nameof(Code.EVEX_Vplzcntd_zmm_k1z_zmmm512b32)], mnemonic[nameof(Mnemonic.Vplzcntd)]),
				(code[nameof(Code.EVEX_Vplzcntq_xmm_k1z_xmmm128b64)], mnemonic[nameof(Mnemonic.Vplzcntq)]),
				(code[nameof(Code.EVEX_Vplzcntq_ymm_k1z_ymmm256b64)], mnemonic[nameof(Mnemonic.Vplzcntq)]),
				(code[nameof(Code.EVEX_Vplzcntq_zmm_k1z_zmmm512b64)], mnemonic[nameof(Mnemonic.Vplzcntq)]),
				(code[nameof(Code.VEX_Vpsrlvd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrlvd)]),
				(code[nameof(Code.VEX_Vpsrlvd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsrlvd)]),
				(code[nameof(Code.VEX_Vpsrlvq_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsrlvq)]),
				(code[nameof(Code.VEX_Vpsrlvq_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsrlvq)]),
				(code[nameof(Code.EVEX_Vpsrlvd_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpsrlvd)]),
				(code[nameof(Code.EVEX_Vpsrlvd_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpsrlvd)]),
				(code[nameof(Code.EVEX_Vpsrlvd_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpsrlvd)]),
				(code[nameof(Code.EVEX_Vpsrlvq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpsrlvq)]),
				(code[nameof(Code.EVEX_Vpsrlvq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpsrlvq)]),
				(code[nameof(Code.EVEX_Vpsrlvq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpsrlvq)]),
				(code[nameof(Code.VEX_Vpsravd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsravd)]),
				(code[nameof(Code.VEX_Vpsravd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsravd)]),
				(code[nameof(Code.EVEX_Vpsravd_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpsravd)]),
				(code[nameof(Code.EVEX_Vpsravd_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpsravd)]),
				(code[nameof(Code.EVEX_Vpsravd_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpsravd)]),
				(code[nameof(Code.EVEX_Vpsravq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpsravq)]),
				(code[nameof(Code.EVEX_Vpsravq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpsravq)]),
				(code[nameof(Code.EVEX_Vpsravq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpsravq)]),
				(code[nameof(Code.VEX_Vpsllvd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsllvd)]),
				(code[nameof(Code.VEX_Vpsllvd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsllvd)]),
				(code[nameof(Code.VEX_Vpsllvq_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpsllvq)]),
				(code[nameof(Code.VEX_Vpsllvq_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpsllvq)]),
				(code[nameof(Code.EVEX_Vpsllvd_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpsllvd)]),
				(code[nameof(Code.EVEX_Vpsllvd_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpsllvd)]),
				(code[nameof(Code.EVEX_Vpsllvd_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpsllvd)]),
				(code[nameof(Code.EVEX_Vpsllvq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpsllvq)]),
				(code[nameof(Code.EVEX_Vpsllvq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpsllvq)]),
				(code[nameof(Code.EVEX_Vpsllvq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpsllvq)]),
				(code[nameof(Code.EVEX_Vrcp14ps_xmm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vrcp14ps)]),
				(code[nameof(Code.EVEX_Vrcp14ps_ymm_k1z_ymmm256b32)], mnemonic[nameof(Mnemonic.Vrcp14ps)]),
				(code[nameof(Code.EVEX_Vrcp14ps_zmm_k1z_zmmm512b32)], mnemonic[nameof(Mnemonic.Vrcp14ps)]),
				(code[nameof(Code.EVEX_Vrcp14pd_xmm_k1z_xmmm128b64)], mnemonic[nameof(Mnemonic.Vrcp14pd)]),
				(code[nameof(Code.EVEX_Vrcp14pd_ymm_k1z_ymmm256b64)], mnemonic[nameof(Mnemonic.Vrcp14pd)]),
				(code[nameof(Code.EVEX_Vrcp14pd_zmm_k1z_zmmm512b64)], mnemonic[nameof(Mnemonic.Vrcp14pd)]),
				(code[nameof(Code.EVEX_Vrcp14ss_xmm_k1z_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vrcp14ss)]),
				(code[nameof(Code.EVEX_Vrcp14sd_xmm_k1z_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vrcp14sd)]),
				(code[nameof(Code.EVEX_Vrsqrt14ps_xmm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vrsqrt14ps)]),
				(code[nameof(Code.EVEX_Vrsqrt14ps_ymm_k1z_ymmm256b32)], mnemonic[nameof(Mnemonic.Vrsqrt14ps)]),
				(code[nameof(Code.EVEX_Vrsqrt14ps_zmm_k1z_zmmm512b32)], mnemonic[nameof(Mnemonic.Vrsqrt14ps)]),
				(code[nameof(Code.EVEX_Vrsqrt14pd_xmm_k1z_xmmm128b64)], mnemonic[nameof(Mnemonic.Vrsqrt14pd)]),
				(code[nameof(Code.EVEX_Vrsqrt14pd_ymm_k1z_ymmm256b64)], mnemonic[nameof(Mnemonic.Vrsqrt14pd)]),
				(code[nameof(Code.EVEX_Vrsqrt14pd_zmm_k1z_zmmm512b64)], mnemonic[nameof(Mnemonic.Vrsqrt14pd)]),
				(code[nameof(Code.EVEX_Vrsqrt14ss_xmm_k1z_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vrsqrt14ss)]),
				(code[nameof(Code.EVEX_Vrsqrt14sd_xmm_k1z_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vrsqrt14sd)]),
				(code[nameof(Code.EVEX_Vpdpbusd_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpdpbusd)]),
				(code[nameof(Code.EVEX_Vpdpbusd_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpdpbusd)]),
				(code[nameof(Code.EVEX_Vpdpbusd_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpdpbusd)]),
				(code[nameof(Code.EVEX_Vpdpbusds_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpdpbusds)]),
				(code[nameof(Code.EVEX_Vpdpbusds_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpdpbusds)]),
				(code[nameof(Code.EVEX_Vpdpbusds_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpdpbusds)]),
				(code[nameof(Code.EVEX_Vpdpwssd_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpdpwssd)]),
				(code[nameof(Code.EVEX_Vpdpwssd_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpdpwssd)]),
				(code[nameof(Code.EVEX_Vpdpwssd_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpdpwssd)]),
				(code[nameof(Code.EVEX_Vdpbf16ps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vdpbf16ps)]),
				(code[nameof(Code.EVEX_Vdpbf16ps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vdpbf16ps)]),
				(code[nameof(Code.EVEX_Vdpbf16ps_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vdpbf16ps)]),
				(code[nameof(Code.EVEX_Vp4dpwssd_zmm_k1z_zmmp3_m128)], mnemonic[nameof(Mnemonic.Vp4dpwssd)]),
				(code[nameof(Code.EVEX_Vpdpwssds_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpdpwssds)]),
				(code[nameof(Code.EVEX_Vpdpwssds_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpdpwssds)]),
				(code[nameof(Code.EVEX_Vpdpwssds_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpdpwssds)]),
				(code[nameof(Code.EVEX_Vp4dpwssds_zmm_k1z_zmmp3_m128)], mnemonic[nameof(Mnemonic.Vp4dpwssds)]),
				(code[nameof(Code.EVEX_Vpopcntb_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vpopcntb)]),
				(code[nameof(Code.EVEX_Vpopcntb_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vpopcntb)]),
				(code[nameof(Code.EVEX_Vpopcntb_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vpopcntb)]),
				(code[nameof(Code.EVEX_Vpopcntw_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vpopcntw)]),
				(code[nameof(Code.EVEX_Vpopcntw_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vpopcntw)]),
				(code[nameof(Code.EVEX_Vpopcntw_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vpopcntw)]),
				(code[nameof(Code.EVEX_Vpopcntd_xmm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpopcntd)]),
				(code[nameof(Code.EVEX_Vpopcntd_ymm_k1z_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpopcntd)]),
				(code[nameof(Code.EVEX_Vpopcntd_zmm_k1z_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpopcntd)]),
				(code[nameof(Code.EVEX_Vpopcntq_xmm_k1z_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpopcntq)]),
				(code[nameof(Code.EVEX_Vpopcntq_ymm_k1z_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpopcntq)]),
				(code[nameof(Code.EVEX_Vpopcntq_zmm_k1z_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpopcntq)]),
				(code[nameof(Code.VEX_Vpbroadcastd_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vpbroadcastd)]),
				(code[nameof(Code.VEX_Vpbroadcastd_ymm_xmmm32)], mnemonic[nameof(Mnemonic.Vpbroadcastd)]),
				(code[nameof(Code.EVEX_Vpbroadcastd_xmm_k1z_xmmm32)], mnemonic[nameof(Mnemonic.Vpbroadcastd)]),
				(code[nameof(Code.EVEX_Vpbroadcastd_ymm_k1z_xmmm32)], mnemonic[nameof(Mnemonic.Vpbroadcastd)]),
				(code[nameof(Code.EVEX_Vpbroadcastd_zmm_k1z_xmmm32)], mnemonic[nameof(Mnemonic.Vpbroadcastd)]),
				(code[nameof(Code.VEX_Vpbroadcastq_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vpbroadcastq)]),
				(code[nameof(Code.VEX_Vpbroadcastq_ymm_xmmm64)], mnemonic[nameof(Mnemonic.Vpbroadcastq)]),
				(code[nameof(Code.EVEX_Vbroadcasti32x2_xmm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vbroadcasti32x2)]),
				(code[nameof(Code.EVEX_Vbroadcasti32x2_ymm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vbroadcasti32x2)]),
				(code[nameof(Code.EVEX_Vbroadcasti32x2_zmm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vbroadcasti32x2)]),
				(code[nameof(Code.EVEX_Vpbroadcastq_xmm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vpbroadcastq)]),
				(code[nameof(Code.EVEX_Vpbroadcastq_ymm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vpbroadcastq)]),
				(code[nameof(Code.EVEX_Vpbroadcastq_zmm_k1z_xmmm64)], mnemonic[nameof(Mnemonic.Vpbroadcastq)]),
				(code[nameof(Code.VEX_Vbroadcasti128_ymm_m128)], mnemonic[nameof(Mnemonic.Vbroadcasti128)]),
				(code[nameof(Code.EVEX_Vbroadcasti32x4_ymm_k1z_m128)], mnemonic[nameof(Mnemonic.Vbroadcasti32x4)]),
				(code[nameof(Code.EVEX_Vbroadcasti32x4_zmm_k1z_m128)], mnemonic[nameof(Mnemonic.Vbroadcasti32x4)]),
				(code[nameof(Code.EVEX_Vbroadcasti64x2_ymm_k1z_m128)], mnemonic[nameof(Mnemonic.Vbroadcasti64x2)]),
				(code[nameof(Code.EVEX_Vbroadcasti64x2_zmm_k1z_m128)], mnemonic[nameof(Mnemonic.Vbroadcasti64x2)]),
				(code[nameof(Code.EVEX_Vbroadcasti32x8_zmm_k1z_m256)], mnemonic[nameof(Mnemonic.Vbroadcasti32x8)]),
				(code[nameof(Code.EVEX_Vbroadcasti64x4_zmm_k1z_m256)], mnemonic[nameof(Mnemonic.Vbroadcasti64x4)]),
				(code[nameof(Code.EVEX_Vpexpandb_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vpexpandb)]),
				(code[nameof(Code.EVEX_Vpexpandb_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vpexpandb)]),
				(code[nameof(Code.EVEX_Vpexpandb_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vpexpandb)]),
				(code[nameof(Code.EVEX_Vpexpandw_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vpexpandw)]),
				(code[nameof(Code.EVEX_Vpexpandw_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vpexpandw)]),
				(code[nameof(Code.EVEX_Vpexpandw_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vpexpandw)]),
				(code[nameof(Code.EVEX_Vpcompressb_xmmm128_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpcompressb)]),
				(code[nameof(Code.EVEX_Vpcompressb_ymmm256_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpcompressb)]),
				(code[nameof(Code.EVEX_Vpcompressb_zmmm512_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpcompressb)]),
				(code[nameof(Code.EVEX_Vpcompressw_xmmm128_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpcompressw)]),
				(code[nameof(Code.EVEX_Vpcompressw_ymmm256_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpcompressw)]),
				(code[nameof(Code.EVEX_Vpcompressw_zmmm512_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpcompressw)]),
				(code[nameof(Code.EVEX_Vpblendmd_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpblendmd)]),
				(code[nameof(Code.EVEX_Vpblendmd_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpblendmd)]),
				(code[nameof(Code.EVEX_Vpblendmd_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpblendmd)]),
				(code[nameof(Code.EVEX_Vpblendmq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpblendmq)]),
				(code[nameof(Code.EVEX_Vpblendmq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpblendmq)]),
				(code[nameof(Code.EVEX_Vpblendmq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpblendmq)]),
				(code[nameof(Code.EVEX_Vblendmps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vblendmps)]),
				(code[nameof(Code.EVEX_Vblendmps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vblendmps)]),
				(code[nameof(Code.EVEX_Vblendmps_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vblendmps)]),
				(code[nameof(Code.EVEX_Vblendmpd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vblendmpd)]),
				(code[nameof(Code.EVEX_Vblendmpd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vblendmpd)]),
				(code[nameof(Code.EVEX_Vblendmpd_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vblendmpd)]),
				(code[nameof(Code.EVEX_Vpblendmb_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpblendmb)]),
				(code[nameof(Code.EVEX_Vpblendmb_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpblendmb)]),
				(code[nameof(Code.EVEX_Vpblendmb_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpblendmb)]),
				(code[nameof(Code.EVEX_Vpblendmw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpblendmw)]),
				(code[nameof(Code.EVEX_Vpblendmw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpblendmw)]),
				(code[nameof(Code.EVEX_Vpblendmw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpblendmw)]),
				(code[nameof(Code.EVEX_Vp2intersectd_kp1_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vp2intersectd)]),
				(code[nameof(Code.EVEX_Vp2intersectd_kp1_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vp2intersectd)]),
				(code[nameof(Code.EVEX_Vp2intersectd_kp1_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vp2intersectd)]),
				(code[nameof(Code.EVEX_Vp2intersectq_kp1_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vp2intersectq)]),
				(code[nameof(Code.EVEX_Vp2intersectq_kp1_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vp2intersectq)]),
				(code[nameof(Code.EVEX_Vp2intersectq_kp1_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vp2intersectq)]),
				(code[nameof(Code.EVEX_Vpshldvw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpshldvw)]),
				(code[nameof(Code.EVEX_Vpshldvw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpshldvw)]),
				(code[nameof(Code.EVEX_Vpshldvw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpshldvw)]),
				(code[nameof(Code.EVEX_Vpshldvd_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpshldvd)]),
				(code[nameof(Code.EVEX_Vpshldvd_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpshldvd)]),
				(code[nameof(Code.EVEX_Vpshldvd_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpshldvd)]),
				(code[nameof(Code.EVEX_Vpshldvq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpshldvq)]),
				(code[nameof(Code.EVEX_Vpshldvq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpshldvq)]),
				(code[nameof(Code.EVEX_Vpshldvq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpshldvq)]),
				(code[nameof(Code.EVEX_Vpshrdvw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpshrdvw)]),
				(code[nameof(Code.EVEX_Vpshrdvw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpshrdvw)]),
				(code[nameof(Code.EVEX_Vpshrdvw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpshrdvw)]),
				(code[nameof(Code.EVEX_Vcvtneps2bf16_xmm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vcvtneps2bf16)]),
				(code[nameof(Code.EVEX_Vcvtneps2bf16_xmm_k1z_ymmm256b32)], mnemonic[nameof(Mnemonic.Vcvtneps2bf16)]),
				(code[nameof(Code.EVEX_Vcvtneps2bf16_ymm_k1z_zmmm512b32)], mnemonic[nameof(Mnemonic.Vcvtneps2bf16)]),
				(code[nameof(Code.EVEX_Vcvtne2ps2bf16_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vcvtne2ps2bf16)]),
				(code[nameof(Code.EVEX_Vcvtne2ps2bf16_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vcvtne2ps2bf16)]),
				(code[nameof(Code.EVEX_Vcvtne2ps2bf16_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vcvtne2ps2bf16)]),
				(code[nameof(Code.EVEX_Vpshrdvd_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpshrdvd)]),
				(code[nameof(Code.EVEX_Vpshrdvd_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpshrdvd)]),
				(code[nameof(Code.EVEX_Vpshrdvd_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpshrdvd)]),
				(code[nameof(Code.EVEX_Vpshrdvq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpshrdvq)]),
				(code[nameof(Code.EVEX_Vpshrdvq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpshrdvq)]),
				(code[nameof(Code.EVEX_Vpshrdvq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpshrdvq)]),
				(code[nameof(Code.EVEX_Vpermi2b_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpermi2b)]),
				(code[nameof(Code.EVEX_Vpermi2b_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpermi2b)]),
				(code[nameof(Code.EVEX_Vpermi2b_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpermi2b)]),
				(code[nameof(Code.EVEX_Vpermi2w_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpermi2w)]),
				(code[nameof(Code.EVEX_Vpermi2w_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpermi2w)]),
				(code[nameof(Code.EVEX_Vpermi2w_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpermi2w)]),
				(code[nameof(Code.EVEX_Vpermi2d_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpermi2d)]),
				(code[nameof(Code.EVEX_Vpermi2d_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpermi2d)]),
				(code[nameof(Code.EVEX_Vpermi2d_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpermi2d)]),
				(code[nameof(Code.EVEX_Vpermi2q_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpermi2q)]),
				(code[nameof(Code.EVEX_Vpermi2q_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpermi2q)]),
				(code[nameof(Code.EVEX_Vpermi2q_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpermi2q)]),
				(code[nameof(Code.EVEX_Vpermi2ps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpermi2ps)]),
				(code[nameof(Code.EVEX_Vpermi2ps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpermi2ps)]),
				(code[nameof(Code.EVEX_Vpermi2ps_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpermi2ps)]),
				(code[nameof(Code.EVEX_Vpermi2pd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpermi2pd)]),
				(code[nameof(Code.EVEX_Vpermi2pd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpermi2pd)]),
				(code[nameof(Code.EVEX_Vpermi2pd_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpermi2pd)]),
				(code[nameof(Code.VEX_Vpbroadcastb_xmm_xmmm8)], mnemonic[nameof(Mnemonic.Vpbroadcastb)]),
				(code[nameof(Code.VEX_Vpbroadcastb_ymm_xmmm8)], mnemonic[nameof(Mnemonic.Vpbroadcastb)]),
				(code[nameof(Code.EVEX_Vpbroadcastb_xmm_k1z_xmmm8)], mnemonic[nameof(Mnemonic.Vpbroadcastb)]),
				(code[nameof(Code.EVEX_Vpbroadcastb_ymm_k1z_xmmm8)], mnemonic[nameof(Mnemonic.Vpbroadcastb)]),
				(code[nameof(Code.EVEX_Vpbroadcastb_zmm_k1z_xmmm8)], mnemonic[nameof(Mnemonic.Vpbroadcastb)]),
				(code[nameof(Code.VEX_Vpbroadcastw_xmm_xmmm16)], mnemonic[nameof(Mnemonic.Vpbroadcastw)]),
				(code[nameof(Code.VEX_Vpbroadcastw_ymm_xmmm16)], mnemonic[nameof(Mnemonic.Vpbroadcastw)]),
				(code[nameof(Code.EVEX_Vpbroadcastw_xmm_k1z_xmmm16)], mnemonic[nameof(Mnemonic.Vpbroadcastw)]),
				(code[nameof(Code.EVEX_Vpbroadcastw_ymm_k1z_xmmm16)], mnemonic[nameof(Mnemonic.Vpbroadcastw)]),
				(code[nameof(Code.EVEX_Vpbroadcastw_zmm_k1z_xmmm16)], mnemonic[nameof(Mnemonic.Vpbroadcastw)]),
				(code[nameof(Code.EVEX_Vpbroadcastb_xmm_k1z_r32)], mnemonic[nameof(Mnemonic.Vpbroadcastb)]),
				(code[nameof(Code.EVEX_Vpbroadcastb_ymm_k1z_r32)], mnemonic[nameof(Mnemonic.Vpbroadcastb)]),
				(code[nameof(Code.EVEX_Vpbroadcastb_zmm_k1z_r32)], mnemonic[nameof(Mnemonic.Vpbroadcastb)]),
				(code[nameof(Code.EVEX_Vpbroadcastw_xmm_k1z_r32)], mnemonic[nameof(Mnemonic.Vpbroadcastw)]),
				(code[nameof(Code.EVEX_Vpbroadcastw_ymm_k1z_r32)], mnemonic[nameof(Mnemonic.Vpbroadcastw)]),
				(code[nameof(Code.EVEX_Vpbroadcastw_zmm_k1z_r32)], mnemonic[nameof(Mnemonic.Vpbroadcastw)]),
				(code[nameof(Code.EVEX_Vpbroadcastd_xmm_k1z_r32)], mnemonic[nameof(Mnemonic.Vpbroadcastd)]),
				(code[nameof(Code.EVEX_Vpbroadcastd_ymm_k1z_r32)], mnemonic[nameof(Mnemonic.Vpbroadcastd)]),
				(code[nameof(Code.EVEX_Vpbroadcastd_zmm_k1z_r32)], mnemonic[nameof(Mnemonic.Vpbroadcastd)]),
				(code[nameof(Code.EVEX_Vpbroadcastq_xmm_k1z_r64)], mnemonic[nameof(Mnemonic.Vpbroadcastq)]),
				(code[nameof(Code.EVEX_Vpbroadcastq_ymm_k1z_r64)], mnemonic[nameof(Mnemonic.Vpbroadcastq)]),
				(code[nameof(Code.EVEX_Vpbroadcastq_zmm_k1z_r64)], mnemonic[nameof(Mnemonic.Vpbroadcastq)]),
				(code[nameof(Code.EVEX_Vpermt2b_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpermt2b)]),
				(code[nameof(Code.EVEX_Vpermt2b_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpermt2b)]),
				(code[nameof(Code.EVEX_Vpermt2b_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpermt2b)]),
				(code[nameof(Code.EVEX_Vpermt2w_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpermt2w)]),
				(code[nameof(Code.EVEX_Vpermt2w_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpermt2w)]),
				(code[nameof(Code.EVEX_Vpermt2w_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpermt2w)]),
				(code[nameof(Code.EVEX_Vpermt2d_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpermt2d)]),
				(code[nameof(Code.EVEX_Vpermt2d_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpermt2d)]),
				(code[nameof(Code.EVEX_Vpermt2d_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpermt2d)]),
				(code[nameof(Code.EVEX_Vpermt2q_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpermt2q)]),
				(code[nameof(Code.EVEX_Vpermt2q_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpermt2q)]),
				(code[nameof(Code.EVEX_Vpermt2q_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpermt2q)]),
				(code[nameof(Code.EVEX_Vpermt2ps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpermt2ps)]),
				(code[nameof(Code.EVEX_Vpermt2ps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpermt2ps)]),
				(code[nameof(Code.EVEX_Vpermt2ps_zmm_k1z_zmm_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpermt2ps)]),
				(code[nameof(Code.EVEX_Vpermt2pd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpermt2pd)]),
				(code[nameof(Code.EVEX_Vpermt2pd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpermt2pd)]),
				(code[nameof(Code.EVEX_Vpermt2pd_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpermt2pd)]),
				(code[nameof(Code.Invept_r32_m128)], mnemonic[nameof(Mnemonic.Invept)]),
				(code[nameof(Code.Invept_r64_m128)], mnemonic[nameof(Mnemonic.Invept)]),
				(code[nameof(Code.Invvpid_r32_m128)], mnemonic[nameof(Mnemonic.Invvpid)]),
				(code[nameof(Code.Invvpid_r64_m128)], mnemonic[nameof(Mnemonic.Invvpid)]),
				(code[nameof(Code.Invpcid_r32_m128)], mnemonic[nameof(Mnemonic.Invpcid)]),
				(code[nameof(Code.Invpcid_r64_m128)], mnemonic[nameof(Mnemonic.Invpcid)]),
				(code[nameof(Code.EVEX_Vpmultishiftqb_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpmultishiftqb)]),
				(code[nameof(Code.EVEX_Vpmultishiftqb_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpmultishiftqb)]),
				(code[nameof(Code.EVEX_Vpmultishiftqb_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpmultishiftqb)]),
				(code[nameof(Code.EVEX_Vexpandps_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vexpandps)]),
				(code[nameof(Code.EVEX_Vexpandps_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vexpandps)]),
				(code[nameof(Code.EVEX_Vexpandps_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vexpandps)]),
				(code[nameof(Code.EVEX_Vexpandpd_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vexpandpd)]),
				(code[nameof(Code.EVEX_Vexpandpd_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vexpandpd)]),
				(code[nameof(Code.EVEX_Vexpandpd_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vexpandpd)]),
				(code[nameof(Code.EVEX_Vpexpandd_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vpexpandd)]),
				(code[nameof(Code.EVEX_Vpexpandd_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vpexpandd)]),
				(code[nameof(Code.EVEX_Vpexpandd_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vpexpandd)]),
				(code[nameof(Code.EVEX_Vpexpandq_xmm_k1z_xmmm128)], mnemonic[nameof(Mnemonic.Vpexpandq)]),
				(code[nameof(Code.EVEX_Vpexpandq_ymm_k1z_ymmm256)], mnemonic[nameof(Mnemonic.Vpexpandq)]),
				(code[nameof(Code.EVEX_Vpexpandq_zmm_k1z_zmmm512)], mnemonic[nameof(Mnemonic.Vpexpandq)]),
				(code[nameof(Code.EVEX_Vcompressps_xmmm128_k1z_xmm)], mnemonic[nameof(Mnemonic.Vcompressps)]),
				(code[nameof(Code.EVEX_Vcompressps_ymmm256_k1z_ymm)], mnemonic[nameof(Mnemonic.Vcompressps)]),
				(code[nameof(Code.EVEX_Vcompressps_zmmm512_k1z_zmm)], mnemonic[nameof(Mnemonic.Vcompressps)]),
				(code[nameof(Code.EVEX_Vcompresspd_xmmm128_k1z_xmm)], mnemonic[nameof(Mnemonic.Vcompresspd)]),
				(code[nameof(Code.EVEX_Vcompresspd_ymmm256_k1z_ymm)], mnemonic[nameof(Mnemonic.Vcompresspd)]),
				(code[nameof(Code.EVEX_Vcompresspd_zmmm512_k1z_zmm)], mnemonic[nameof(Mnemonic.Vcompresspd)]),
				(code[nameof(Code.EVEX_Vpcompressd_xmmm128_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpcompressd)]),
				(code[nameof(Code.EVEX_Vpcompressd_ymmm256_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpcompressd)]),
				(code[nameof(Code.EVEX_Vpcompressd_zmmm512_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpcompressd)]),
				(code[nameof(Code.EVEX_Vpcompressq_xmmm128_k1z_xmm)], mnemonic[nameof(Mnemonic.Vpcompressq)]),
				(code[nameof(Code.EVEX_Vpcompressq_ymmm256_k1z_ymm)], mnemonic[nameof(Mnemonic.Vpcompressq)]),
				(code[nameof(Code.EVEX_Vpcompressq_zmmm512_k1z_zmm)], mnemonic[nameof(Mnemonic.Vpcompressq)]),
				(code[nameof(Code.VEX_Vpmaskmovd_xmm_xmm_m128)], mnemonic[nameof(Mnemonic.Vpmaskmovd)]),
				(code[nameof(Code.VEX_Vpmaskmovd_ymm_ymm_m256)], mnemonic[nameof(Mnemonic.Vpmaskmovd)]),
				(code[nameof(Code.VEX_Vpmaskmovq_xmm_xmm_m128)], mnemonic[nameof(Mnemonic.Vpmaskmovq)]),
				(code[nameof(Code.VEX_Vpmaskmovq_ymm_ymm_m256)], mnemonic[nameof(Mnemonic.Vpmaskmovq)]),
				(code[nameof(Code.EVEX_Vpermb_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpermb)]),
				(code[nameof(Code.EVEX_Vpermb_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpermb)]),
				(code[nameof(Code.EVEX_Vpermb_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpermb)]),
				(code[nameof(Code.EVEX_Vpermw_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpermw)]),
				(code[nameof(Code.EVEX_Vpermw_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpermw)]),
				(code[nameof(Code.EVEX_Vpermw_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpermw)]),
				(code[nameof(Code.VEX_Vpmaskmovd_m128_xmm_xmm)], mnemonic[nameof(Mnemonic.Vpmaskmovd)]),
				(code[nameof(Code.VEX_Vpmaskmovd_m256_ymm_ymm)], mnemonic[nameof(Mnemonic.Vpmaskmovd)]),
				(code[nameof(Code.VEX_Vpmaskmovq_m128_xmm_xmm)], mnemonic[nameof(Mnemonic.Vpmaskmovq)]),
				(code[nameof(Code.VEX_Vpmaskmovq_m256_ymm_ymm)], mnemonic[nameof(Mnemonic.Vpmaskmovq)]),
				(code[nameof(Code.EVEX_Vpshufbitqmb_k_k1_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpshufbitqmb)]),
				(code[nameof(Code.EVEX_Vpshufbitqmb_k_k1_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpshufbitqmb)]),
				(code[nameof(Code.EVEX_Vpshufbitqmb_k_k1_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vpshufbitqmb)]),
				(code[nameof(Code.VEX_Vpgatherdd_xmm_vm32x_xmm)], mnemonic[nameof(Mnemonic.Vpgatherdd)]),
				(code[nameof(Code.VEX_Vpgatherdd_ymm_vm32y_ymm)], mnemonic[nameof(Mnemonic.Vpgatherdd)]),
				(code[nameof(Code.VEX_Vpgatherdq_xmm_vm32x_xmm)], mnemonic[nameof(Mnemonic.Vpgatherdq)]),
				(code[nameof(Code.VEX_Vpgatherdq_ymm_vm32x_ymm)], mnemonic[nameof(Mnemonic.Vpgatherdq)]),
				(code[nameof(Code.EVEX_Vpgatherdd_xmm_k1_vm32x)], mnemonic[nameof(Mnemonic.Vpgatherdd)]),
				(code[nameof(Code.EVEX_Vpgatherdd_ymm_k1_vm32y)], mnemonic[nameof(Mnemonic.Vpgatherdd)]),
				(code[nameof(Code.EVEX_Vpgatherdd_zmm_k1_vm32z)], mnemonic[nameof(Mnemonic.Vpgatherdd)]),
				(code[nameof(Code.EVEX_Vpgatherdq_xmm_k1_vm32x)], mnemonic[nameof(Mnemonic.Vpgatherdq)]),
				(code[nameof(Code.EVEX_Vpgatherdq_ymm_k1_vm32x)], mnemonic[nameof(Mnemonic.Vpgatherdq)]),
				(code[nameof(Code.EVEX_Vpgatherdq_zmm_k1_vm32y)], mnemonic[nameof(Mnemonic.Vpgatherdq)]),
				(code[nameof(Code.VEX_Vpgatherqd_xmm_vm64x_xmm)], mnemonic[nameof(Mnemonic.Vpgatherqd)]),
				(code[nameof(Code.VEX_Vpgatherqd_xmm_vm64y_xmm)], mnemonic[nameof(Mnemonic.Vpgatherqd)]),
				(code[nameof(Code.VEX_Vpgatherqq_xmm_vm64x_xmm)], mnemonic[nameof(Mnemonic.Vpgatherqq)]),
				(code[nameof(Code.VEX_Vpgatherqq_ymm_vm64y_ymm)], mnemonic[nameof(Mnemonic.Vpgatherqq)]),
				(code[nameof(Code.EVEX_Vpgatherqd_xmm_k1_vm64x)], mnemonic[nameof(Mnemonic.Vpgatherqd)]),
				(code[nameof(Code.EVEX_Vpgatherqd_xmm_k1_vm64y)], mnemonic[nameof(Mnemonic.Vpgatherqd)]),
				(code[nameof(Code.EVEX_Vpgatherqd_ymm_k1_vm64z)], mnemonic[nameof(Mnemonic.Vpgatherqd)]),
				(code[nameof(Code.EVEX_Vpgatherqq_xmm_k1_vm64x)], mnemonic[nameof(Mnemonic.Vpgatherqq)]),
				(code[nameof(Code.EVEX_Vpgatherqq_ymm_k1_vm64y)], mnemonic[nameof(Mnemonic.Vpgatherqq)]),
				(code[nameof(Code.EVEX_Vpgatherqq_zmm_k1_vm64z)], mnemonic[nameof(Mnemonic.Vpgatherqq)]),
				(code[nameof(Code.VEX_Vgatherdps_xmm_vm32x_xmm)], mnemonic[nameof(Mnemonic.Vgatherdps)]),
				(code[nameof(Code.VEX_Vgatherdps_ymm_vm32y_ymm)], mnemonic[nameof(Mnemonic.Vgatherdps)]),
				(code[nameof(Code.VEX_Vgatherdpd_xmm_vm32x_xmm)], mnemonic[nameof(Mnemonic.Vgatherdpd)]),
				(code[nameof(Code.VEX_Vgatherdpd_ymm_vm32x_ymm)], mnemonic[nameof(Mnemonic.Vgatherdpd)]),
				(code[nameof(Code.EVEX_Vgatherdps_xmm_k1_vm32x)], mnemonic[nameof(Mnemonic.Vgatherdps)]),
				(code[nameof(Code.EVEX_Vgatherdps_ymm_k1_vm32y)], mnemonic[nameof(Mnemonic.Vgatherdps)]),
				(code[nameof(Code.EVEX_Vgatherdps_zmm_k1_vm32z)], mnemonic[nameof(Mnemonic.Vgatherdps)]),
				(code[nameof(Code.EVEX_Vgatherdpd_xmm_k1_vm32x)], mnemonic[nameof(Mnemonic.Vgatherdpd)]),
				(code[nameof(Code.EVEX_Vgatherdpd_ymm_k1_vm32x)], mnemonic[nameof(Mnemonic.Vgatherdpd)]),
				(code[nameof(Code.EVEX_Vgatherdpd_zmm_k1_vm32y)], mnemonic[nameof(Mnemonic.Vgatherdpd)]),
				(code[nameof(Code.VEX_Vgatherqps_xmm_vm64x_xmm)], mnemonic[nameof(Mnemonic.Vgatherqps)]),
				(code[nameof(Code.VEX_Vgatherqps_xmm_vm64y_xmm)], mnemonic[nameof(Mnemonic.Vgatherqps)]),
				(code[nameof(Code.VEX_Vgatherqpd_xmm_vm64x_xmm)], mnemonic[nameof(Mnemonic.Vgatherqpd)]),
				(code[nameof(Code.VEX_Vgatherqpd_ymm_vm64y_ymm)], mnemonic[nameof(Mnemonic.Vgatherqpd)]),
				(code[nameof(Code.EVEX_Vgatherqps_xmm_k1_vm64x)], mnemonic[nameof(Mnemonic.Vgatherqps)]),
				(code[nameof(Code.EVEX_Vgatherqps_xmm_k1_vm64y)], mnemonic[nameof(Mnemonic.Vgatherqps)]),
				(code[nameof(Code.EVEX_Vgatherqps_ymm_k1_vm64z)], mnemonic[nameof(Mnemonic.Vgatherqps)]),
				(code[nameof(Code.EVEX_Vgatherqpd_xmm_k1_vm64x)], mnemonic[nameof(Mnemonic.Vgatherqpd)]),
				(code[nameof(Code.EVEX_Vgatherqpd_ymm_k1_vm64y)], mnemonic[nameof(Mnemonic.Vgatherqpd)]),
				(code[nameof(Code.EVEX_Vgatherqpd_zmm_k1_vm64z)], mnemonic[nameof(Mnemonic.Vgatherqpd)]),
				(code[nameof(Code.VEX_Vfmaddsub132ps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmaddsub132ps)]),
				(code[nameof(Code.VEX_Vfmaddsub132ps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmaddsub132ps)]),
				(code[nameof(Code.VEX_Vfmaddsub132pd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmaddsub132pd)]),
				(code[nameof(Code.VEX_Vfmaddsub132pd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmaddsub132pd)]),
				(code[nameof(Code.EVEX_Vfmaddsub132ps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vfmaddsub132ps)]),
				(code[nameof(Code.EVEX_Vfmaddsub132ps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vfmaddsub132ps)]),
				(code[nameof(Code.EVEX_Vfmaddsub132ps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vfmaddsub132ps)]),
				(code[nameof(Code.EVEX_Vfmaddsub132pd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vfmaddsub132pd)]),
				(code[nameof(Code.EVEX_Vfmaddsub132pd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vfmaddsub132pd)]),
				(code[nameof(Code.EVEX_Vfmaddsub132pd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vfmaddsub132pd)]),
				(code[nameof(Code.VEX_Vfmsubadd132ps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmsubadd132ps)]),
				(code[nameof(Code.VEX_Vfmsubadd132ps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmsubadd132ps)]),
				(code[nameof(Code.VEX_Vfmsubadd132pd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmsubadd132pd)]),
				(code[nameof(Code.VEX_Vfmsubadd132pd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmsubadd132pd)]),
				(code[nameof(Code.EVEX_Vfmsubadd132ps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vfmsubadd132ps)]),
				(code[nameof(Code.EVEX_Vfmsubadd132ps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vfmsubadd132ps)]),
				(code[nameof(Code.EVEX_Vfmsubadd132ps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vfmsubadd132ps)]),
				(code[nameof(Code.EVEX_Vfmsubadd132pd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vfmsubadd132pd)]),
				(code[nameof(Code.EVEX_Vfmsubadd132pd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vfmsubadd132pd)]),
				(code[nameof(Code.EVEX_Vfmsubadd132pd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vfmsubadd132pd)]),
				(code[nameof(Code.VEX_Vfmadd132ps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmadd132ps)]),
				(code[nameof(Code.VEX_Vfmadd132ps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmadd132ps)]),
				(code[nameof(Code.VEX_Vfmadd132pd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmadd132pd)]),
				(code[nameof(Code.VEX_Vfmadd132pd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmadd132pd)]),
				(code[nameof(Code.EVEX_Vfmadd132ps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vfmadd132ps)]),
				(code[nameof(Code.EVEX_Vfmadd132ps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vfmadd132ps)]),
				(code[nameof(Code.EVEX_Vfmadd132ps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vfmadd132ps)]),
				(code[nameof(Code.EVEX_Vfmadd132pd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vfmadd132pd)]),
				(code[nameof(Code.EVEX_Vfmadd132pd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vfmadd132pd)]),
				(code[nameof(Code.EVEX_Vfmadd132pd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vfmadd132pd)]),
				(code[nameof(Code.VEX_Vfmadd132ss_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vfmadd132ss)]),
				(code[nameof(Code.VEX_Vfmadd132sd_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vfmadd132sd)]),
				(code[nameof(Code.EVEX_Vfmadd132ss_xmm_k1z_xmm_xmmm32_er)], mnemonic[nameof(Mnemonic.Vfmadd132ss)]),
				(code[nameof(Code.EVEX_Vfmadd132sd_xmm_k1z_xmm_xmmm64_er)], mnemonic[nameof(Mnemonic.Vfmadd132sd)]),
				(code[nameof(Code.VEX_Vfmsub132ps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmsub132ps)]),
				(code[nameof(Code.VEX_Vfmsub132ps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmsub132ps)]),
				(code[nameof(Code.VEX_Vfmsub132pd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmsub132pd)]),
				(code[nameof(Code.VEX_Vfmsub132pd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmsub132pd)]),
				(code[nameof(Code.EVEX_Vfmsub132ps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vfmsub132ps)]),
				(code[nameof(Code.EVEX_Vfmsub132ps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vfmsub132ps)]),
				(code[nameof(Code.EVEX_Vfmsub132ps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vfmsub132ps)]),
				(code[nameof(Code.EVEX_Vfmsub132pd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vfmsub132pd)]),
				(code[nameof(Code.EVEX_Vfmsub132pd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vfmsub132pd)]),
				(code[nameof(Code.EVEX_Vfmsub132pd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vfmsub132pd)]),
				(code[nameof(Code.EVEX_V4fmaddps_zmm_k1z_zmmp3_m128)], mnemonic[nameof(Mnemonic.V4fmaddps)]),
				(code[nameof(Code.VEX_Vfmsub132ss_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vfmsub132ss)]),
				(code[nameof(Code.VEX_Vfmsub132sd_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vfmsub132sd)]),
				(code[nameof(Code.EVEX_Vfmsub132ss_xmm_k1z_xmm_xmmm32_er)], mnemonic[nameof(Mnemonic.Vfmsub132ss)]),
				(code[nameof(Code.EVEX_Vfmsub132sd_xmm_k1z_xmm_xmmm64_er)], mnemonic[nameof(Mnemonic.Vfmsub132sd)]),
				(code[nameof(Code.EVEX_V4fmaddss_xmm_k1z_xmmp3_m128)], mnemonic[nameof(Mnemonic.V4fmaddss)]),
				(code[nameof(Code.VEX_Vfnmadd132ps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfnmadd132ps)]),
				(code[nameof(Code.VEX_Vfnmadd132ps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfnmadd132ps)]),
				(code[nameof(Code.VEX_Vfnmadd132pd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfnmadd132pd)]),
				(code[nameof(Code.VEX_Vfnmadd132pd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfnmadd132pd)]),
				(code[nameof(Code.EVEX_Vfnmadd132ps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vfnmadd132ps)]),
				(code[nameof(Code.EVEX_Vfnmadd132ps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vfnmadd132ps)]),
				(code[nameof(Code.EVEX_Vfnmadd132ps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vfnmadd132ps)]),
				(code[nameof(Code.EVEX_Vfnmadd132pd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vfnmadd132pd)]),
				(code[nameof(Code.EVEX_Vfnmadd132pd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vfnmadd132pd)]),
				(code[nameof(Code.EVEX_Vfnmadd132pd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vfnmadd132pd)]),
				(code[nameof(Code.VEX_Vfnmadd132ss_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vfnmadd132ss)]),
				(code[nameof(Code.VEX_Vfnmadd132sd_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vfnmadd132sd)]),
				(code[nameof(Code.EVEX_Vfnmadd132ss_xmm_k1z_xmm_xmmm32_er)], mnemonic[nameof(Mnemonic.Vfnmadd132ss)]),
				(code[nameof(Code.EVEX_Vfnmadd132sd_xmm_k1z_xmm_xmmm64_er)], mnemonic[nameof(Mnemonic.Vfnmadd132sd)]),
				(code[nameof(Code.VEX_Vfnmsub132ps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfnmsub132ps)]),
				(code[nameof(Code.VEX_Vfnmsub132ps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfnmsub132ps)]),
				(code[nameof(Code.VEX_Vfnmsub132pd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfnmsub132pd)]),
				(code[nameof(Code.VEX_Vfnmsub132pd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfnmsub132pd)]),
				(code[nameof(Code.EVEX_Vfnmsub132ps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vfnmsub132ps)]),
				(code[nameof(Code.EVEX_Vfnmsub132ps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vfnmsub132ps)]),
				(code[nameof(Code.EVEX_Vfnmsub132ps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vfnmsub132ps)]),
				(code[nameof(Code.EVEX_Vfnmsub132pd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vfnmsub132pd)]),
				(code[nameof(Code.EVEX_Vfnmsub132pd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vfnmsub132pd)]),
				(code[nameof(Code.EVEX_Vfnmsub132pd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vfnmsub132pd)]),
				(code[nameof(Code.VEX_Vfnmsub132ss_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vfnmsub132ss)]),
				(code[nameof(Code.VEX_Vfnmsub132sd_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vfnmsub132sd)]),
				(code[nameof(Code.EVEX_Vfnmsub132ss_xmm_k1z_xmm_xmmm32_er)], mnemonic[nameof(Mnemonic.Vfnmsub132ss)]),
				(code[nameof(Code.EVEX_Vfnmsub132sd_xmm_k1z_xmm_xmmm64_er)], mnemonic[nameof(Mnemonic.Vfnmsub132sd)]),
				(code[nameof(Code.EVEX_Vpscatterdd_vm32x_k1_xmm)], mnemonic[nameof(Mnemonic.Vpscatterdd)]),
				(code[nameof(Code.EVEX_Vpscatterdd_vm32y_k1_ymm)], mnemonic[nameof(Mnemonic.Vpscatterdd)]),
				(code[nameof(Code.EVEX_Vpscatterdd_vm32z_k1_zmm)], mnemonic[nameof(Mnemonic.Vpscatterdd)]),
				(code[nameof(Code.EVEX_Vpscatterdq_vm32x_k1_xmm)], mnemonic[nameof(Mnemonic.Vpscatterdq)]),
				(code[nameof(Code.EVEX_Vpscatterdq_vm32x_k1_ymm)], mnemonic[nameof(Mnemonic.Vpscatterdq)]),
				(code[nameof(Code.EVEX_Vpscatterdq_vm32y_k1_zmm)], mnemonic[nameof(Mnemonic.Vpscatterdq)]),
				(code[nameof(Code.EVEX_Vpscatterqd_vm64x_k1_xmm)], mnemonic[nameof(Mnemonic.Vpscatterqd)]),
				(code[nameof(Code.EVEX_Vpscatterqd_vm64y_k1_xmm)], mnemonic[nameof(Mnemonic.Vpscatterqd)]),
				(code[nameof(Code.EVEX_Vpscatterqd_vm64z_k1_ymm)], mnemonic[nameof(Mnemonic.Vpscatterqd)]),
				(code[nameof(Code.EVEX_Vpscatterqq_vm64x_k1_xmm)], mnemonic[nameof(Mnemonic.Vpscatterqq)]),
				(code[nameof(Code.EVEX_Vpscatterqq_vm64y_k1_ymm)], mnemonic[nameof(Mnemonic.Vpscatterqq)]),
				(code[nameof(Code.EVEX_Vpscatterqq_vm64z_k1_zmm)], mnemonic[nameof(Mnemonic.Vpscatterqq)]),
				(code[nameof(Code.EVEX_Vscatterdps_vm32x_k1_xmm)], mnemonic[nameof(Mnemonic.Vscatterdps)]),
				(code[nameof(Code.EVEX_Vscatterdps_vm32y_k1_ymm)], mnemonic[nameof(Mnemonic.Vscatterdps)]),
				(code[nameof(Code.EVEX_Vscatterdps_vm32z_k1_zmm)], mnemonic[nameof(Mnemonic.Vscatterdps)]),
				(code[nameof(Code.EVEX_Vscatterdpd_vm32x_k1_xmm)], mnemonic[nameof(Mnemonic.Vscatterdpd)]),
				(code[nameof(Code.EVEX_Vscatterdpd_vm32x_k1_ymm)], mnemonic[nameof(Mnemonic.Vscatterdpd)]),
				(code[nameof(Code.EVEX_Vscatterdpd_vm32y_k1_zmm)], mnemonic[nameof(Mnemonic.Vscatterdpd)]),
				(code[nameof(Code.EVEX_Vscatterqps_vm64x_k1_xmm)], mnemonic[nameof(Mnemonic.Vscatterqps)]),
				(code[nameof(Code.EVEX_Vscatterqps_vm64y_k1_xmm)], mnemonic[nameof(Mnemonic.Vscatterqps)]),
				(code[nameof(Code.EVEX_Vscatterqps_vm64z_k1_ymm)], mnemonic[nameof(Mnemonic.Vscatterqps)]),
				(code[nameof(Code.EVEX_Vscatterqpd_vm64x_k1_xmm)], mnemonic[nameof(Mnemonic.Vscatterqpd)]),
				(code[nameof(Code.EVEX_Vscatterqpd_vm64y_k1_ymm)], mnemonic[nameof(Mnemonic.Vscatterqpd)]),
				(code[nameof(Code.EVEX_Vscatterqpd_vm64z_k1_zmm)], mnemonic[nameof(Mnemonic.Vscatterqpd)]),
				(code[nameof(Code.VEX_Vfmaddsub213ps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmaddsub213ps)]),
				(code[nameof(Code.VEX_Vfmaddsub213ps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmaddsub213ps)]),
				(code[nameof(Code.VEX_Vfmaddsub213pd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmaddsub213pd)]),
				(code[nameof(Code.VEX_Vfmaddsub213pd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmaddsub213pd)]),
				(code[nameof(Code.EVEX_Vfmaddsub213ps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vfmaddsub213ps)]),
				(code[nameof(Code.EVEX_Vfmaddsub213ps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vfmaddsub213ps)]),
				(code[nameof(Code.EVEX_Vfmaddsub213ps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vfmaddsub213ps)]),
				(code[nameof(Code.EVEX_Vfmaddsub213pd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vfmaddsub213pd)]),
				(code[nameof(Code.EVEX_Vfmaddsub213pd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vfmaddsub213pd)]),
				(code[nameof(Code.EVEX_Vfmaddsub213pd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vfmaddsub213pd)]),
				(code[nameof(Code.VEX_Vfmsubadd213ps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmsubadd213ps)]),
				(code[nameof(Code.VEX_Vfmsubadd213ps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmsubadd213ps)]),
				(code[nameof(Code.VEX_Vfmsubadd213pd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmsubadd213pd)]),
				(code[nameof(Code.VEX_Vfmsubadd213pd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmsubadd213pd)]),
				(code[nameof(Code.EVEX_Vfmsubadd213ps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vfmsubadd213ps)]),
				(code[nameof(Code.EVEX_Vfmsubadd213ps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vfmsubadd213ps)]),
				(code[nameof(Code.EVEX_Vfmsubadd213ps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vfmsubadd213ps)]),
				(code[nameof(Code.EVEX_Vfmsubadd213pd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vfmsubadd213pd)]),
				(code[nameof(Code.EVEX_Vfmsubadd213pd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vfmsubadd213pd)]),
				(code[nameof(Code.EVEX_Vfmsubadd213pd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vfmsubadd213pd)]),
				(code[nameof(Code.VEX_Vfmadd213ps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmadd213ps)]),
				(code[nameof(Code.VEX_Vfmadd213ps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmadd213ps)]),
				(code[nameof(Code.VEX_Vfmadd213pd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmadd213pd)]),
				(code[nameof(Code.VEX_Vfmadd213pd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmadd213pd)]),
				(code[nameof(Code.EVEX_Vfmadd213ps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vfmadd213ps)]),
				(code[nameof(Code.EVEX_Vfmadd213ps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vfmadd213ps)]),
				(code[nameof(Code.EVEX_Vfmadd213ps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vfmadd213ps)]),
				(code[nameof(Code.EVEX_Vfmadd213pd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vfmadd213pd)]),
				(code[nameof(Code.EVEX_Vfmadd213pd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vfmadd213pd)]),
				(code[nameof(Code.EVEX_Vfmadd213pd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vfmadd213pd)]),
				(code[nameof(Code.VEX_Vfmadd213ss_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vfmadd213ss)]),
				(code[nameof(Code.VEX_Vfmadd213sd_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vfmadd213sd)]),
				(code[nameof(Code.EVEX_Vfmadd213ss_xmm_k1z_xmm_xmmm32_er)], mnemonic[nameof(Mnemonic.Vfmadd213ss)]),
				(code[nameof(Code.EVEX_Vfmadd213sd_xmm_k1z_xmm_xmmm64_er)], mnemonic[nameof(Mnemonic.Vfmadd213sd)]),
				(code[nameof(Code.VEX_Vfmsub213ps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmsub213ps)]),
				(code[nameof(Code.VEX_Vfmsub213ps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmsub213ps)]),
				(code[nameof(Code.VEX_Vfmsub213pd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmsub213pd)]),
				(code[nameof(Code.VEX_Vfmsub213pd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmsub213pd)]),
				(code[nameof(Code.EVEX_Vfmsub213ps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vfmsub213ps)]),
				(code[nameof(Code.EVEX_Vfmsub213ps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vfmsub213ps)]),
				(code[nameof(Code.EVEX_Vfmsub213ps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vfmsub213ps)]),
				(code[nameof(Code.EVEX_Vfmsub213pd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vfmsub213pd)]),
				(code[nameof(Code.EVEX_Vfmsub213pd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vfmsub213pd)]),
				(code[nameof(Code.EVEX_Vfmsub213pd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vfmsub213pd)]),
				(code[nameof(Code.EVEX_V4fnmaddps_zmm_k1z_zmmp3_m128)], mnemonic[nameof(Mnemonic.V4fnmaddps)]),
				(code[nameof(Code.VEX_Vfmsub213ss_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vfmsub213ss)]),
				(code[nameof(Code.VEX_Vfmsub213sd_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vfmsub213sd)]),
				(code[nameof(Code.EVEX_Vfmsub213ss_xmm_k1z_xmm_xmmm32_er)], mnemonic[nameof(Mnemonic.Vfmsub213ss)]),
				(code[nameof(Code.EVEX_Vfmsub213sd_xmm_k1z_xmm_xmmm64_er)], mnemonic[nameof(Mnemonic.Vfmsub213sd)]),
				(code[nameof(Code.EVEX_V4fnmaddss_xmm_k1z_xmmp3_m128)], mnemonic[nameof(Mnemonic.V4fnmaddss)]),
				(code[nameof(Code.VEX_Vfnmadd213ps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfnmadd213ps)]),
				(code[nameof(Code.VEX_Vfnmadd213ps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfnmadd213ps)]),
				(code[nameof(Code.VEX_Vfnmadd213pd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfnmadd213pd)]),
				(code[nameof(Code.VEX_Vfnmadd213pd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfnmadd213pd)]),
				(code[nameof(Code.EVEX_Vfnmadd213ps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vfnmadd213ps)]),
				(code[nameof(Code.EVEX_Vfnmadd213ps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vfnmadd213ps)]),
				(code[nameof(Code.EVEX_Vfnmadd213ps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vfnmadd213ps)]),
				(code[nameof(Code.EVEX_Vfnmadd213pd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vfnmadd213pd)]),
				(code[nameof(Code.EVEX_Vfnmadd213pd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vfnmadd213pd)]),
				(code[nameof(Code.EVEX_Vfnmadd213pd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vfnmadd213pd)]),
				(code[nameof(Code.VEX_Vfnmadd213ss_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vfnmadd213ss)]),
				(code[nameof(Code.VEX_Vfnmadd213sd_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vfnmadd213sd)]),
				(code[nameof(Code.EVEX_Vfnmadd213ss_xmm_k1z_xmm_xmmm32_er)], mnemonic[nameof(Mnemonic.Vfnmadd213ss)]),
				(code[nameof(Code.EVEX_Vfnmadd213sd_xmm_k1z_xmm_xmmm64_er)], mnemonic[nameof(Mnemonic.Vfnmadd213sd)]),
				(code[nameof(Code.VEX_Vfnmsub213ps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfnmsub213ps)]),
				(code[nameof(Code.VEX_Vfnmsub213ps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfnmsub213ps)]),
				(code[nameof(Code.VEX_Vfnmsub213pd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfnmsub213pd)]),
				(code[nameof(Code.VEX_Vfnmsub213pd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfnmsub213pd)]),
				(code[nameof(Code.EVEX_Vfnmsub213ps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vfnmsub213ps)]),
				(code[nameof(Code.EVEX_Vfnmsub213ps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vfnmsub213ps)]),
				(code[nameof(Code.EVEX_Vfnmsub213ps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vfnmsub213ps)]),
				(code[nameof(Code.EVEX_Vfnmsub213pd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vfnmsub213pd)]),
				(code[nameof(Code.EVEX_Vfnmsub213pd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vfnmsub213pd)]),
				(code[nameof(Code.EVEX_Vfnmsub213pd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vfnmsub213pd)]),
				(code[nameof(Code.VEX_Vfnmsub213ss_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vfnmsub213ss)]),
				(code[nameof(Code.VEX_Vfnmsub213sd_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vfnmsub213sd)]),
				(code[nameof(Code.EVEX_Vfnmsub213ss_xmm_k1z_xmm_xmmm32_er)], mnemonic[nameof(Mnemonic.Vfnmsub213ss)]),
				(code[nameof(Code.EVEX_Vfnmsub213sd_xmm_k1z_xmm_xmmm64_er)], mnemonic[nameof(Mnemonic.Vfnmsub213sd)]),
				(code[nameof(Code.EVEX_Vpmadd52luq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpmadd52luq)]),
				(code[nameof(Code.EVEX_Vpmadd52luq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpmadd52luq)]),
				(code[nameof(Code.EVEX_Vpmadd52luq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpmadd52luq)]),
				(code[nameof(Code.EVEX_Vpmadd52huq_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpmadd52huq)]),
				(code[nameof(Code.EVEX_Vpmadd52huq_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpmadd52huq)]),
				(code[nameof(Code.EVEX_Vpmadd52huq_zmm_k1z_zmm_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpmadd52huq)]),
				(code[nameof(Code.VEX_Vfmaddsub231ps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmaddsub231ps)]),
				(code[nameof(Code.VEX_Vfmaddsub231ps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmaddsub231ps)]),
				(code[nameof(Code.VEX_Vfmaddsub231pd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmaddsub231pd)]),
				(code[nameof(Code.VEX_Vfmaddsub231pd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmaddsub231pd)]),
				(code[nameof(Code.EVEX_Vfmaddsub231ps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vfmaddsub231ps)]),
				(code[nameof(Code.EVEX_Vfmaddsub231ps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vfmaddsub231ps)]),
				(code[nameof(Code.EVEX_Vfmaddsub231ps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vfmaddsub231ps)]),
				(code[nameof(Code.EVEX_Vfmaddsub231pd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vfmaddsub231pd)]),
				(code[nameof(Code.EVEX_Vfmaddsub231pd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vfmaddsub231pd)]),
				(code[nameof(Code.EVEX_Vfmaddsub231pd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vfmaddsub231pd)]),
				(code[nameof(Code.VEX_Vfmsubadd231ps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmsubadd231ps)]),
				(code[nameof(Code.VEX_Vfmsubadd231ps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmsubadd231ps)]),
				(code[nameof(Code.VEX_Vfmsubadd231pd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmsubadd231pd)]),
				(code[nameof(Code.VEX_Vfmsubadd231pd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmsubadd231pd)]),
				(code[nameof(Code.EVEX_Vfmsubadd231ps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vfmsubadd231ps)]),
				(code[nameof(Code.EVEX_Vfmsubadd231ps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vfmsubadd231ps)]),
				(code[nameof(Code.EVEX_Vfmsubadd231ps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vfmsubadd231ps)]),
				(code[nameof(Code.EVEX_Vfmsubadd231pd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vfmsubadd231pd)]),
				(code[nameof(Code.EVEX_Vfmsubadd231pd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vfmsubadd231pd)]),
				(code[nameof(Code.EVEX_Vfmsubadd231pd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vfmsubadd231pd)]),
				(code[nameof(Code.VEX_Vfmadd231ps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmadd231ps)]),
				(code[nameof(Code.VEX_Vfmadd231ps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmadd231ps)]),
				(code[nameof(Code.VEX_Vfmadd231pd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmadd231pd)]),
				(code[nameof(Code.VEX_Vfmadd231pd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmadd231pd)]),
				(code[nameof(Code.EVEX_Vfmadd231ps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vfmadd231ps)]),
				(code[nameof(Code.EVEX_Vfmadd231ps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vfmadd231ps)]),
				(code[nameof(Code.EVEX_Vfmadd231ps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vfmadd231ps)]),
				(code[nameof(Code.EVEX_Vfmadd231pd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vfmadd231pd)]),
				(code[nameof(Code.EVEX_Vfmadd231pd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vfmadd231pd)]),
				(code[nameof(Code.EVEX_Vfmadd231pd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vfmadd231pd)]),
				(code[nameof(Code.VEX_Vfmadd231ss_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vfmadd231ss)]),
				(code[nameof(Code.VEX_Vfmadd231sd_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vfmadd231sd)]),
				(code[nameof(Code.EVEX_Vfmadd231ss_xmm_k1z_xmm_xmmm32_er)], mnemonic[nameof(Mnemonic.Vfmadd231ss)]),
				(code[nameof(Code.EVEX_Vfmadd231sd_xmm_k1z_xmm_xmmm64_er)], mnemonic[nameof(Mnemonic.Vfmadd231sd)]),
				(code[nameof(Code.VEX_Vfmsub231ps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmsub231ps)]),
				(code[nameof(Code.VEX_Vfmsub231ps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmsub231ps)]),
				(code[nameof(Code.VEX_Vfmsub231pd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmsub231pd)]),
				(code[nameof(Code.VEX_Vfmsub231pd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmsub231pd)]),
				(code[nameof(Code.EVEX_Vfmsub231ps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vfmsub231ps)]),
				(code[nameof(Code.EVEX_Vfmsub231ps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vfmsub231ps)]),
				(code[nameof(Code.EVEX_Vfmsub231ps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vfmsub231ps)]),
				(code[nameof(Code.EVEX_Vfmsub231pd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vfmsub231pd)]),
				(code[nameof(Code.EVEX_Vfmsub231pd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vfmsub231pd)]),
				(code[nameof(Code.EVEX_Vfmsub231pd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vfmsub231pd)]),
				(code[nameof(Code.VEX_Vfmsub231ss_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vfmsub231ss)]),
				(code[nameof(Code.VEX_Vfmsub231sd_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vfmsub231sd)]),
				(code[nameof(Code.EVEX_Vfmsub231ss_xmm_k1z_xmm_xmmm32_er)], mnemonic[nameof(Mnemonic.Vfmsub231ss)]),
				(code[nameof(Code.EVEX_Vfmsub231sd_xmm_k1z_xmm_xmmm64_er)], mnemonic[nameof(Mnemonic.Vfmsub231sd)]),
				(code[nameof(Code.VEX_Vfnmadd231ps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfnmadd231ps)]),
				(code[nameof(Code.VEX_Vfnmadd231ps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfnmadd231ps)]),
				(code[nameof(Code.VEX_Vfnmadd231pd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfnmadd231pd)]),
				(code[nameof(Code.VEX_Vfnmadd231pd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfnmadd231pd)]),
				(code[nameof(Code.EVEX_Vfnmadd231ps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vfnmadd231ps)]),
				(code[nameof(Code.EVEX_Vfnmadd231ps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vfnmadd231ps)]),
				(code[nameof(Code.EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vfnmadd231ps)]),
				(code[nameof(Code.EVEX_Vfnmadd231pd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vfnmadd231pd)]),
				(code[nameof(Code.EVEX_Vfnmadd231pd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vfnmadd231pd)]),
				(code[nameof(Code.EVEX_Vfnmadd231pd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vfnmadd231pd)]),
				(code[nameof(Code.VEX_Vfnmadd231ss_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vfnmadd231ss)]),
				(code[nameof(Code.VEX_Vfnmadd231sd_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vfnmadd231sd)]),
				(code[nameof(Code.EVEX_Vfnmadd231ss_xmm_k1z_xmm_xmmm32_er)], mnemonic[nameof(Mnemonic.Vfnmadd231ss)]),
				(code[nameof(Code.EVEX_Vfnmadd231sd_xmm_k1z_xmm_xmmm64_er)], mnemonic[nameof(Mnemonic.Vfnmadd231sd)]),
				(code[nameof(Code.VEX_Vfnmsub231ps_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfnmsub231ps)]),
				(code[nameof(Code.VEX_Vfnmsub231ps_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfnmsub231ps)]),
				(code[nameof(Code.VEX_Vfnmsub231pd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfnmsub231pd)]),
				(code[nameof(Code.VEX_Vfnmsub231pd_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfnmsub231pd)]),
				(code[nameof(Code.EVEX_Vfnmsub231ps_xmm_k1z_xmm_xmmm128b32)], mnemonic[nameof(Mnemonic.Vfnmsub231ps)]),
				(code[nameof(Code.EVEX_Vfnmsub231ps_ymm_k1z_ymm_ymmm256b32)], mnemonic[nameof(Mnemonic.Vfnmsub231ps)]),
				(code[nameof(Code.EVEX_Vfnmsub231ps_zmm_k1z_zmm_zmmm512b32_er)], mnemonic[nameof(Mnemonic.Vfnmsub231ps)]),
				(code[nameof(Code.EVEX_Vfnmsub231pd_xmm_k1z_xmm_xmmm128b64)], mnemonic[nameof(Mnemonic.Vfnmsub231pd)]),
				(code[nameof(Code.EVEX_Vfnmsub231pd_ymm_k1z_ymm_ymmm256b64)], mnemonic[nameof(Mnemonic.Vfnmsub231pd)]),
				(code[nameof(Code.EVEX_Vfnmsub231pd_zmm_k1z_zmm_zmmm512b64_er)], mnemonic[nameof(Mnemonic.Vfnmsub231pd)]),
				(code[nameof(Code.VEX_Vfnmsub231ss_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vfnmsub231ss)]),
				(code[nameof(Code.VEX_Vfnmsub231sd_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vfnmsub231sd)]),
				(code[nameof(Code.EVEX_Vfnmsub231ss_xmm_k1z_xmm_xmmm32_er)], mnemonic[nameof(Mnemonic.Vfnmsub231ss)]),
				(code[nameof(Code.EVEX_Vfnmsub231sd_xmm_k1z_xmm_xmmm64_er)], mnemonic[nameof(Mnemonic.Vfnmsub231sd)]),
				(code[nameof(Code.EVEX_Vpconflictd_xmm_k1z_xmmm128b32)], mnemonic[nameof(Mnemonic.Vpconflictd)]),
				(code[nameof(Code.EVEX_Vpconflictd_ymm_k1z_ymmm256b32)], mnemonic[nameof(Mnemonic.Vpconflictd)]),
				(code[nameof(Code.EVEX_Vpconflictd_zmm_k1z_zmmm512b32)], mnemonic[nameof(Mnemonic.Vpconflictd)]),
				(code[nameof(Code.EVEX_Vpconflictq_xmm_k1z_xmmm128b64)], mnemonic[nameof(Mnemonic.Vpconflictq)]),
				(code[nameof(Code.EVEX_Vpconflictq_ymm_k1z_ymmm256b64)], mnemonic[nameof(Mnemonic.Vpconflictq)]),
				(code[nameof(Code.EVEX_Vpconflictq_zmm_k1z_zmmm512b64)], mnemonic[nameof(Mnemonic.Vpconflictq)]),
				(code[nameof(Code.EVEX_Vgatherpf0dps_vm32z_k1)], mnemonic[nameof(Mnemonic.Vgatherpf0dps)]),
				(code[nameof(Code.EVEX_Vgatherpf0dpd_vm32y_k1)], mnemonic[nameof(Mnemonic.Vgatherpf0dpd)]),
				(code[nameof(Code.EVEX_Vgatherpf1dps_vm32z_k1)], mnemonic[nameof(Mnemonic.Vgatherpf1dps)]),
				(code[nameof(Code.EVEX_Vgatherpf1dpd_vm32y_k1)], mnemonic[nameof(Mnemonic.Vgatherpf1dpd)]),
				(code[nameof(Code.EVEX_Vscatterpf0dps_vm32z_k1)], mnemonic[nameof(Mnemonic.Vscatterpf0dps)]),
				(code[nameof(Code.EVEX_Vscatterpf0dpd_vm32y_k1)], mnemonic[nameof(Mnemonic.Vscatterpf0dpd)]),
				(code[nameof(Code.EVEX_Vscatterpf1dps_vm32z_k1)], mnemonic[nameof(Mnemonic.Vscatterpf1dps)]),
				(code[nameof(Code.EVEX_Vscatterpf1dpd_vm32y_k1)], mnemonic[nameof(Mnemonic.Vscatterpf1dpd)]),
				(code[nameof(Code.EVEX_Vgatherpf0qps_vm64z_k1)], mnemonic[nameof(Mnemonic.Vgatherpf0qps)]),
				(code[nameof(Code.EVEX_Vgatherpf0qpd_vm64z_k1)], mnemonic[nameof(Mnemonic.Vgatherpf0qpd)]),
				(code[nameof(Code.EVEX_Vgatherpf1qps_vm64z_k1)], mnemonic[nameof(Mnemonic.Vgatherpf1qps)]),
				(code[nameof(Code.EVEX_Vgatherpf1qpd_vm64z_k1)], mnemonic[nameof(Mnemonic.Vgatherpf1qpd)]),
				(code[nameof(Code.EVEX_Vscatterpf0qps_vm64z_k1)], mnemonic[nameof(Mnemonic.Vscatterpf0qps)]),
				(code[nameof(Code.EVEX_Vscatterpf0qpd_vm64z_k1)], mnemonic[nameof(Mnemonic.Vscatterpf0qpd)]),
				(code[nameof(Code.EVEX_Vscatterpf1qps_vm64z_k1)], mnemonic[nameof(Mnemonic.Vscatterpf1qps)]),
				(code[nameof(Code.EVEX_Vscatterpf1qpd_vm64z_k1)], mnemonic[nameof(Mnemonic.Vscatterpf1qpd)]),
				(code[nameof(Code.Sha1nexte_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Sha1nexte)]),
				(code[nameof(Code.EVEX_Vexp2ps_zmm_k1z_zmmm512b32_sae)], mnemonic[nameof(Mnemonic.Vexp2ps)]),
				(code[nameof(Code.EVEX_Vexp2pd_zmm_k1z_zmmm512b64_sae)], mnemonic[nameof(Mnemonic.Vexp2pd)]),
				(code[nameof(Code.Sha1msg1_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Sha1msg1)]),
				(code[nameof(Code.Sha1msg2_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Sha1msg2)]),
				(code[nameof(Code.EVEX_Vrcp28ps_zmm_k1z_zmmm512b32_sae)], mnemonic[nameof(Mnemonic.Vrcp28ps)]),
				(code[nameof(Code.EVEX_Vrcp28pd_zmm_k1z_zmmm512b64_sae)], mnemonic[nameof(Mnemonic.Vrcp28pd)]),
				(code[nameof(Code.Sha256rnds2_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Sha256rnds2)]),
				(code[nameof(Code.EVEX_Vrcp28ss_xmm_k1z_xmm_xmmm32_sae)], mnemonic[nameof(Mnemonic.Vrcp28ss)]),
				(code[nameof(Code.EVEX_Vrcp28sd_xmm_k1z_xmm_xmmm64_sae)], mnemonic[nameof(Mnemonic.Vrcp28sd)]),
				(code[nameof(Code.Sha256msg1_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Sha256msg1)]),
				(code[nameof(Code.EVEX_Vrsqrt28ps_zmm_k1z_zmmm512b32_sae)], mnemonic[nameof(Mnemonic.Vrsqrt28ps)]),
				(code[nameof(Code.EVEX_Vrsqrt28pd_zmm_k1z_zmmm512b64_sae)], mnemonic[nameof(Mnemonic.Vrsqrt28pd)]),
				(code[nameof(Code.Sha256msg2_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Sha256msg2)]),
				(code[nameof(Code.EVEX_Vrsqrt28ss_xmm_k1z_xmm_xmmm32_sae)], mnemonic[nameof(Mnemonic.Vrsqrt28ss)]),
				(code[nameof(Code.EVEX_Vrsqrt28sd_xmm_k1z_xmm_xmmm64_sae)], mnemonic[nameof(Mnemonic.Vrsqrt28sd)]),
				(code[nameof(Code.Gf2p8mulb_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Gf2p8mulb)]),
				(code[nameof(Code.VEX_Vgf2p8mulb_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vgf2p8mulb)]),
				(code[nameof(Code.VEX_Vgf2p8mulb_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vgf2p8mulb)]),
				(code[nameof(Code.EVEX_Vgf2p8mulb_xmm_k1z_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vgf2p8mulb)]),
				(code[nameof(Code.EVEX_Vgf2p8mulb_ymm_k1z_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vgf2p8mulb)]),
				(code[nameof(Code.EVEX_Vgf2p8mulb_zmm_k1z_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vgf2p8mulb)]),
				(code[nameof(Code.Aesimc_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Aesimc)]),
				(code[nameof(Code.VEX_Vaesimc_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vaesimc)]),
				(code[nameof(Code.Aesenc_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Aesenc)]),
				(code[nameof(Code.VEX_Vaesenc_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vaesenc)]),
				(code[nameof(Code.VEX_Vaesenc_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vaesenc)]),
				(code[nameof(Code.EVEX_Vaesenc_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vaesenc)]),
				(code[nameof(Code.EVEX_Vaesenc_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vaesenc)]),
				(code[nameof(Code.EVEX_Vaesenc_zmm_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vaesenc)]),
				(code[nameof(Code.Aesenclast_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Aesenclast)]),
				(code[nameof(Code.VEX_Vaesenclast_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vaesenclast)]),
				(code[nameof(Code.VEX_Vaesenclast_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vaesenclast)]),
				(code[nameof(Code.EVEX_Vaesenclast_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vaesenclast)]),
				(code[nameof(Code.EVEX_Vaesenclast_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vaesenclast)]),
				(code[nameof(Code.EVEX_Vaesenclast_zmm_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vaesenclast)]),
				(code[nameof(Code.Aesdec_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Aesdec)]),
				(code[nameof(Code.VEX_Vaesdec_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vaesdec)]),
				(code[nameof(Code.VEX_Vaesdec_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vaesdec)]),
				(code[nameof(Code.EVEX_Vaesdec_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vaesdec)]),
				(code[nameof(Code.EVEX_Vaesdec_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vaesdec)]),
				(code[nameof(Code.EVEX_Vaesdec_zmm_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vaesdec)]),
				(code[nameof(Code.Aesdeclast_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Aesdeclast)]),
				(code[nameof(Code.VEX_Vaesdeclast_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vaesdeclast)]),
				(code[nameof(Code.VEX_Vaesdeclast_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vaesdeclast)]),
				(code[nameof(Code.EVEX_Vaesdeclast_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vaesdeclast)]),
				(code[nameof(Code.EVEX_Vaesdeclast_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vaesdeclast)]),
				(code[nameof(Code.EVEX_Vaesdeclast_zmm_zmm_zmmm512)], mnemonic[nameof(Mnemonic.Vaesdeclast)]),
				(code[nameof(Code.Movbe_r16_m16)], mnemonic[nameof(Mnemonic.Movbe)]),
				(code[nameof(Code.Movbe_r32_m32)], mnemonic[nameof(Mnemonic.Movbe)]),
				(code[nameof(Code.Movbe_r64_m64)], mnemonic[nameof(Mnemonic.Movbe)]),
				(code[nameof(Code.Crc32_r32_rm8)], mnemonic[nameof(Mnemonic.Crc32)]),
				(code[nameof(Code.Crc32_r64_rm8)], mnemonic[nameof(Mnemonic.Crc32)]),
				(code[nameof(Code.Movbe_m16_r16)], mnemonic[nameof(Mnemonic.Movbe)]),
				(code[nameof(Code.Movbe_m32_r32)], mnemonic[nameof(Mnemonic.Movbe)]),
				(code[nameof(Code.Movbe_m64_r64)], mnemonic[nameof(Mnemonic.Movbe)]),
				(code[nameof(Code.Crc32_r32_rm16)], mnemonic[nameof(Mnemonic.Crc32)]),
				(code[nameof(Code.Crc32_r32_rm32)], mnemonic[nameof(Mnemonic.Crc32)]),
				(code[nameof(Code.Crc32_r64_rm64)], mnemonic[nameof(Mnemonic.Crc32)]),
				(code[nameof(Code.VEX_Andn_r32_r32_rm32)], mnemonic[nameof(Mnemonic.Andn)]),
				(code[nameof(Code.VEX_Andn_r64_r64_rm64)], mnemonic[nameof(Mnemonic.Andn)]),
				(code[nameof(Code.VEX_Blsr_r32_rm32)], mnemonic[nameof(Mnemonic.Blsr)]),
				(code[nameof(Code.VEX_Blsr_r64_rm64)], mnemonic[nameof(Mnemonic.Blsr)]),
				(code[nameof(Code.VEX_Blsmsk_r32_rm32)], mnemonic[nameof(Mnemonic.Blsmsk)]),
				(code[nameof(Code.VEX_Blsmsk_r64_rm64)], mnemonic[nameof(Mnemonic.Blsmsk)]),
				(code[nameof(Code.VEX_Blsi_r32_rm32)], mnemonic[nameof(Mnemonic.Blsi)]),
				(code[nameof(Code.VEX_Blsi_r64_rm64)], mnemonic[nameof(Mnemonic.Blsi)]),
				(code[nameof(Code.VEX_Bzhi_r32_rm32_r32)], mnemonic[nameof(Mnemonic.Bzhi)]),
				(code[nameof(Code.VEX_Bzhi_r64_rm64_r64)], mnemonic[nameof(Mnemonic.Bzhi)]),
				(code[nameof(Code.Wrussd_m32_r32)], mnemonic[nameof(Mnemonic.Wrussd)]),
				(code[nameof(Code.Wrussq_m64_r64)], mnemonic[nameof(Mnemonic.Wrussq)]),
				(code[nameof(Code.VEX_Pext_r32_r32_rm32)], mnemonic[nameof(Mnemonic.Pext)]),
				(code[nameof(Code.VEX_Pext_r64_r64_rm64)], mnemonic[nameof(Mnemonic.Pext)]),
				(code[nameof(Code.VEX_Pdep_r32_r32_rm32)], mnemonic[nameof(Mnemonic.Pdep)]),
				(code[nameof(Code.VEX_Pdep_r64_r64_rm64)], mnemonic[nameof(Mnemonic.Pdep)]),
				(code[nameof(Code.Wrssd_m32_r32)], mnemonic[nameof(Mnemonic.Wrssd)]),
				(code[nameof(Code.Wrssq_m64_r64)], mnemonic[nameof(Mnemonic.Wrssq)]),
				(code[nameof(Code.Adcx_r32_rm32)], mnemonic[nameof(Mnemonic.Adcx)]),
				(code[nameof(Code.Adcx_r64_rm64)], mnemonic[nameof(Mnemonic.Adcx)]),
				(code[nameof(Code.Adox_r32_rm32)], mnemonic[nameof(Mnemonic.Adox)]),
				(code[nameof(Code.Adox_r64_rm64)], mnemonic[nameof(Mnemonic.Adox)]),
				(code[nameof(Code.VEX_Mulx_r32_r32_rm32)], mnemonic[nameof(Mnemonic.Mulx)]),
				(code[nameof(Code.VEX_Mulx_r64_r64_rm64)], mnemonic[nameof(Mnemonic.Mulx)]),
				(code[nameof(Code.VEX_Bextr_r32_rm32_r32)], mnemonic[nameof(Mnemonic.Bextr)]),
				(code[nameof(Code.VEX_Bextr_r64_rm64_r64)], mnemonic[nameof(Mnemonic.Bextr)]),
				(code[nameof(Code.VEX_Shlx_r32_rm32_r32)], mnemonic[nameof(Mnemonic.Shlx)]),
				(code[nameof(Code.VEX_Shlx_r64_rm64_r64)], mnemonic[nameof(Mnemonic.Shlx)]),
				(code[nameof(Code.VEX_Sarx_r32_rm32_r32)], mnemonic[nameof(Mnemonic.Sarx)]),
				(code[nameof(Code.VEX_Sarx_r64_rm64_r64)], mnemonic[nameof(Mnemonic.Sarx)]),
				(code[nameof(Code.VEX_Shrx_r32_rm32_r32)], mnemonic[nameof(Mnemonic.Shrx)]),
				(code[nameof(Code.VEX_Shrx_r64_rm64_r64)], mnemonic[nameof(Mnemonic.Shrx)]),
				(code[nameof(Code.Movdir64b_r16_m512)], mnemonic[nameof(Mnemonic.Movdir64b)]),
				(code[nameof(Code.Movdir64b_r32_m512)], mnemonic[nameof(Mnemonic.Movdir64b)]),
				(code[nameof(Code.Movdir64b_r64_m512)], mnemonic[nameof(Mnemonic.Movdir64b)]),
				(code[nameof(Code.Enqcmds_r16_m512)], mnemonic[nameof(Mnemonic.Enqcmds)]),
				(code[nameof(Code.Enqcmds_r32_m512)], mnemonic[nameof(Mnemonic.Enqcmds)]),
				(code[nameof(Code.Enqcmds_r64_m512)], mnemonic[nameof(Mnemonic.Enqcmds)]),
				(code[nameof(Code.Enqcmd_r16_m512)], mnemonic[nameof(Mnemonic.Enqcmd)]),
				(code[nameof(Code.Enqcmd_r32_m512)], mnemonic[nameof(Mnemonic.Enqcmd)]),
				(code[nameof(Code.Enqcmd_r64_m512)], mnemonic[nameof(Mnemonic.Enqcmd)]),
				(code[nameof(Code.Movdiri_m32_r32)], mnemonic[nameof(Mnemonic.Movdiri)]),
				(code[nameof(Code.Movdiri_m64_r64)], mnemonic[nameof(Mnemonic.Movdiri)]),
				(code[nameof(Code.VEX_Vpermq_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpermq)]),
				(code[nameof(Code.EVEX_Vpermq_ymm_k1z_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vpermq)]),
				(code[nameof(Code.EVEX_Vpermq_zmm_k1z_zmmm512b64_imm8)], mnemonic[nameof(Mnemonic.Vpermq)]),
				(code[nameof(Code.VEX_Vpermpd_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpermpd)]),
				(code[nameof(Code.EVEX_Vpermpd_ymm_k1z_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vpermpd)]),
				(code[nameof(Code.EVEX_Vpermpd_zmm_k1z_zmmm512b64_imm8)], mnemonic[nameof(Mnemonic.Vpermpd)]),
				(code[nameof(Code.VEX_Vpblendd_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpblendd)]),
				(code[nameof(Code.VEX_Vpblendd_ymm_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpblendd)]),
				(code[nameof(Code.EVEX_Valignd_xmm_k1z_xmm_xmmm128b32_imm8)], mnemonic[nameof(Mnemonic.Valignd)]),
				(code[nameof(Code.EVEX_Valignd_ymm_k1z_ymm_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Valignd)]),
				(code[nameof(Code.EVEX_Valignd_zmm_k1z_zmm_zmmm512b32_imm8)], mnemonic[nameof(Mnemonic.Valignd)]),
				(code[nameof(Code.EVEX_Valignq_xmm_k1z_xmm_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Valignq)]),
				(code[nameof(Code.EVEX_Valignq_ymm_k1z_ymm_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Valignq)]),
				(code[nameof(Code.EVEX_Valignq_zmm_k1z_zmm_zmmm512b64_imm8)], mnemonic[nameof(Mnemonic.Valignq)]),
				(code[nameof(Code.VEX_Vpermilps_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpermilps)]),
				(code[nameof(Code.VEX_Vpermilps_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpermilps)]),
				(code[nameof(Code.EVEX_Vpermilps_xmm_k1z_xmmm128b32_imm8)], mnemonic[nameof(Mnemonic.Vpermilps)]),
				(code[nameof(Code.EVEX_Vpermilps_ymm_k1z_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vpermilps)]),
				(code[nameof(Code.EVEX_Vpermilps_zmm_k1z_zmmm512b32_imm8)], mnemonic[nameof(Mnemonic.Vpermilps)]),
				(code[nameof(Code.VEX_Vpermilpd_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpermilpd)]),
				(code[nameof(Code.VEX_Vpermilpd_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpermilpd)]),
				(code[nameof(Code.EVEX_Vpermilpd_xmm_k1z_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Vpermilpd)]),
				(code[nameof(Code.EVEX_Vpermilpd_ymm_k1z_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vpermilpd)]),
				(code[nameof(Code.EVEX_Vpermilpd_zmm_k1z_zmmm512b64_imm8)], mnemonic[nameof(Mnemonic.Vpermilpd)]),
				(code[nameof(Code.VEX_Vperm2f128_ymm_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vperm2f128)]),
				(code[nameof(Code.Roundps_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Roundps)]),
				(code[nameof(Code.VEX_Vroundps_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vroundps)]),
				(code[nameof(Code.VEX_Vroundps_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vroundps)]),
				(code[nameof(Code.EVEX_Vrndscaleps_xmm_k1z_xmmm128b32_imm8)], mnemonic[nameof(Mnemonic.Vrndscaleps)]),
				(code[nameof(Code.EVEX_Vrndscaleps_ymm_k1z_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vrndscaleps)]),
				(code[nameof(Code.EVEX_Vrndscaleps_zmm_k1z_zmmm512b32_imm8_sae)], mnemonic[nameof(Mnemonic.Vrndscaleps)]),
				(code[nameof(Code.Roundpd_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Roundpd)]),
				(code[nameof(Code.VEX_Vroundpd_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vroundpd)]),
				(code[nameof(Code.VEX_Vroundpd_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vroundpd)]),
				(code[nameof(Code.EVEX_Vrndscalepd_xmm_k1z_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Vrndscalepd)]),
				(code[nameof(Code.EVEX_Vrndscalepd_ymm_k1z_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vrndscalepd)]),
				(code[nameof(Code.EVEX_Vrndscalepd_zmm_k1z_zmmm512b64_imm8_sae)], mnemonic[nameof(Mnemonic.Vrndscalepd)]),
				(code[nameof(Code.Roundss_xmm_xmmm32_imm8)], mnemonic[nameof(Mnemonic.Roundss)]),
				(code[nameof(Code.VEX_Vroundss_xmm_xmm_xmmm32_imm8)], mnemonic[nameof(Mnemonic.Vroundss)]),
				(code[nameof(Code.EVEX_Vrndscaless_xmm_k1z_xmm_xmmm32_imm8_sae)], mnemonic[nameof(Mnemonic.Vrndscaless)]),
				(code[nameof(Code.Roundsd_xmm_xmmm64_imm8)], mnemonic[nameof(Mnemonic.Roundsd)]),
				(code[nameof(Code.VEX_Vroundsd_xmm_xmm_xmmm64_imm8)], mnemonic[nameof(Mnemonic.Vroundsd)]),
				(code[nameof(Code.EVEX_Vrndscalesd_xmm_k1z_xmm_xmmm64_imm8_sae)], mnemonic[nameof(Mnemonic.Vrndscalesd)]),
				(code[nameof(Code.Blendps_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Blendps)]),
				(code[nameof(Code.VEX_Vblendps_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vblendps)]),
				(code[nameof(Code.VEX_Vblendps_ymm_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vblendps)]),
				(code[nameof(Code.Blendpd_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Blendpd)]),
				(code[nameof(Code.VEX_Vblendpd_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vblendpd)]),
				(code[nameof(Code.VEX_Vblendpd_ymm_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vblendpd)]),
				(code[nameof(Code.Pblendw_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Pblendw)]),
				(code[nameof(Code.VEX_Vpblendw_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpblendw)]),
				(code[nameof(Code.VEX_Vpblendw_ymm_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpblendw)]),
				(code[nameof(Code.Palignr_mm_mmm64_imm8)], mnemonic[nameof(Mnemonic.Palignr)]),
				(code[nameof(Code.Palignr_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Palignr)]),
				(code[nameof(Code.VEX_Vpalignr_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpalignr)]),
				(code[nameof(Code.VEX_Vpalignr_ymm_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpalignr)]),
				(code[nameof(Code.EVEX_Vpalignr_xmm_k1z_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpalignr)]),
				(code[nameof(Code.EVEX_Vpalignr_ymm_k1z_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpalignr)]),
				(code[nameof(Code.EVEX_Vpalignr_zmm_k1z_zmm_zmmm512_imm8)], mnemonic[nameof(Mnemonic.Vpalignr)]),
				(code[nameof(Code.Pextrb_r32m8_xmm_imm8)], mnemonic[nameof(Mnemonic.Pextrb)]),
				(code[nameof(Code.Pextrb_r64m8_xmm_imm8)], mnemonic[nameof(Mnemonic.Pextrb)]),
				(code[nameof(Code.VEX_Vpextrb_r32m8_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpextrb)]),
				(code[nameof(Code.VEX_Vpextrb_r64m8_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpextrb)]),
				(code[nameof(Code.EVEX_Vpextrb_r32m8_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpextrb)]),
				(code[nameof(Code.EVEX_Vpextrb_r64m8_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpextrb)]),
				(code[nameof(Code.Pextrw_r32m16_xmm_imm8)], mnemonic[nameof(Mnemonic.Pextrw)]),
				(code[nameof(Code.Pextrw_r64m16_xmm_imm8)], mnemonic[nameof(Mnemonic.Pextrw)]),
				(code[nameof(Code.VEX_Vpextrw_r32m16_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpextrw)]),
				(code[nameof(Code.VEX_Vpextrw_r64m16_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpextrw)]),
				(code[nameof(Code.EVEX_Vpextrw_r32m16_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpextrw)]),
				(code[nameof(Code.EVEX_Vpextrw_r64m16_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpextrw)]),
				(code[nameof(Code.Pextrd_rm32_xmm_imm8)], mnemonic[nameof(Mnemonic.Pextrd)]),
				(code[nameof(Code.Pextrq_rm64_xmm_imm8)], mnemonic[nameof(Mnemonic.Pextrq)]),
				(code[nameof(Code.VEX_Vpextrd_rm32_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpextrd)]),
				(code[nameof(Code.VEX_Vpextrq_rm64_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpextrq)]),
				(code[nameof(Code.EVEX_Vpextrd_rm32_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpextrd)]),
				(code[nameof(Code.EVEX_Vpextrq_rm64_xmm_imm8)], mnemonic[nameof(Mnemonic.Vpextrq)]),
				(code[nameof(Code.Extractps_rm32_xmm_imm8)], mnemonic[nameof(Mnemonic.Extractps)]),
				(code[nameof(Code.Extractps_r64m32_xmm_imm8)], mnemonic[nameof(Mnemonic.Extractps)]),
				(code[nameof(Code.VEX_Vextractps_rm32_xmm_imm8)], mnemonic[nameof(Mnemonic.Vextractps)]),
				(code[nameof(Code.VEX_Vextractps_r64m32_xmm_imm8)], mnemonic[nameof(Mnemonic.Vextractps)]),
				(code[nameof(Code.EVEX_Vextractps_rm32_xmm_imm8)], mnemonic[nameof(Mnemonic.Vextractps)]),
				(code[nameof(Code.EVEX_Vextractps_r64m32_xmm_imm8)], mnemonic[nameof(Mnemonic.Vextractps)]),
				(code[nameof(Code.VEX_Vinsertf128_ymm_ymm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vinsertf128)]),
				(code[nameof(Code.EVEX_Vinsertf32x4_ymm_k1z_ymm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vinsertf32x4)]),
				(code[nameof(Code.EVEX_Vinsertf32x4_zmm_k1z_zmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vinsertf32x4)]),
				(code[nameof(Code.EVEX_Vinsertf64x2_ymm_k1z_ymm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vinsertf64x2)]),
				(code[nameof(Code.EVEX_Vinsertf64x2_zmm_k1z_zmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vinsertf64x2)]),
				(code[nameof(Code.VEX_Vextractf128_xmmm128_ymm_imm8)], mnemonic[nameof(Mnemonic.Vextractf128)]),
				(code[nameof(Code.EVEX_Vextractf32x4_xmmm128_k1z_ymm_imm8)], mnemonic[nameof(Mnemonic.Vextractf32x4)]),
				(code[nameof(Code.EVEX_Vextractf32x4_xmmm128_k1z_zmm_imm8)], mnemonic[nameof(Mnemonic.Vextractf32x4)]),
				(code[nameof(Code.EVEX_Vextractf64x2_xmmm128_k1z_ymm_imm8)], mnemonic[nameof(Mnemonic.Vextractf64x2)]),
				(code[nameof(Code.EVEX_Vextractf64x2_xmmm128_k1z_zmm_imm8)], mnemonic[nameof(Mnemonic.Vextractf64x2)]),
				(code[nameof(Code.EVEX_Vinsertf32x8_zmm_k1z_zmm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vinsertf32x8)]),
				(code[nameof(Code.EVEX_Vinsertf64x4_zmm_k1z_zmm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vinsertf64x4)]),
				(code[nameof(Code.EVEX_Vextractf32x8_ymmm256_k1z_zmm_imm8)], mnemonic[nameof(Mnemonic.Vextractf32x8)]),
				(code[nameof(Code.EVEX_Vextractf64x4_ymmm256_k1z_zmm_imm8)], mnemonic[nameof(Mnemonic.Vextractf64x4)]),
				(code[nameof(Code.VEX_Vcvtps2ph_xmmm64_xmm_imm8)], mnemonic[nameof(Mnemonic.Vcvtps2ph)]),
				(code[nameof(Code.VEX_Vcvtps2ph_xmmm128_ymm_imm8)], mnemonic[nameof(Mnemonic.Vcvtps2ph)]),
				(code[nameof(Code.EVEX_Vcvtps2ph_xmmm64_k1z_xmm_imm8)], mnemonic[nameof(Mnemonic.Vcvtps2ph)]),
				(code[nameof(Code.EVEX_Vcvtps2ph_xmmm128_k1z_ymm_imm8)], mnemonic[nameof(Mnemonic.Vcvtps2ph)]),
				(code[nameof(Code.EVEX_Vcvtps2ph_ymmm256_k1z_zmm_imm8_sae)], mnemonic[nameof(Mnemonic.Vcvtps2ph)]),
				(code[nameof(Code.EVEX_Vpcmpud_k_k1_xmm_xmmm128b32_imm8)], mnemonic[nameof(Mnemonic.Vpcmpud)]),
				(code[nameof(Code.EVEX_Vpcmpud_k_k1_ymm_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vpcmpud)]),
				(code[nameof(Code.EVEX_Vpcmpud_k_k1_zmm_zmmm512b32_imm8)], mnemonic[nameof(Mnemonic.Vpcmpud)]),
				(code[nameof(Code.EVEX_Vpcmpuq_k_k1_xmm_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Vpcmpuq)]),
				(code[nameof(Code.EVEX_Vpcmpuq_k_k1_ymm_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vpcmpuq)]),
				(code[nameof(Code.EVEX_Vpcmpuq_k_k1_zmm_zmmm512b64_imm8)], mnemonic[nameof(Mnemonic.Vpcmpuq)]),
				(code[nameof(Code.EVEX_Vpcmpd_k_k1_xmm_xmmm128b32_imm8)], mnemonic[nameof(Mnemonic.Vpcmpd)]),
				(code[nameof(Code.EVEX_Vpcmpd_k_k1_ymm_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vpcmpd)]),
				(code[nameof(Code.EVEX_Vpcmpd_k_k1_zmm_zmmm512b32_imm8)], mnemonic[nameof(Mnemonic.Vpcmpd)]),
				(code[nameof(Code.EVEX_Vpcmpq_k_k1_xmm_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Vpcmpq)]),
				(code[nameof(Code.EVEX_Vpcmpq_k_k1_ymm_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vpcmpq)]),
				(code[nameof(Code.EVEX_Vpcmpq_k_k1_zmm_zmmm512b64_imm8)], mnemonic[nameof(Mnemonic.Vpcmpq)]),
				(code[nameof(Code.Pinsrb_xmm_r32m8_imm8)], mnemonic[nameof(Mnemonic.Pinsrb)]),
				(code[nameof(Code.Pinsrb_xmm_r64m8_imm8)], mnemonic[nameof(Mnemonic.Pinsrb)]),
				(code[nameof(Code.VEX_Vpinsrb_xmm_xmm_r32m8_imm8)], mnemonic[nameof(Mnemonic.Vpinsrb)]),
				(code[nameof(Code.VEX_Vpinsrb_xmm_xmm_r64m8_imm8)], mnemonic[nameof(Mnemonic.Vpinsrb)]),
				(code[nameof(Code.EVEX_Vpinsrb_xmm_xmm_r32m8_imm8)], mnemonic[nameof(Mnemonic.Vpinsrb)]),
				(code[nameof(Code.EVEX_Vpinsrb_xmm_xmm_r64m8_imm8)], mnemonic[nameof(Mnemonic.Vpinsrb)]),
				(code[nameof(Code.Insertps_xmm_xmmm32_imm8)], mnemonic[nameof(Mnemonic.Insertps)]),
				(code[nameof(Code.VEX_Vinsertps_xmm_xmm_xmmm32_imm8)], mnemonic[nameof(Mnemonic.Vinsertps)]),
				(code[nameof(Code.EVEX_Vinsertps_xmm_xmm_xmmm32_imm8)], mnemonic[nameof(Mnemonic.Vinsertps)]),
				(code[nameof(Code.Pinsrd_xmm_rm32_imm8)], mnemonic[nameof(Mnemonic.Pinsrd)]),
				(code[nameof(Code.Pinsrq_xmm_rm64_imm8)], mnemonic[nameof(Mnemonic.Pinsrq)]),
				(code[nameof(Code.VEX_Vpinsrd_xmm_xmm_rm32_imm8)], mnemonic[nameof(Mnemonic.Vpinsrd)]),
				(code[nameof(Code.VEX_Vpinsrq_xmm_xmm_rm64_imm8)], mnemonic[nameof(Mnemonic.Vpinsrq)]),
				(code[nameof(Code.EVEX_Vpinsrd_xmm_xmm_rm32_imm8)], mnemonic[nameof(Mnemonic.Vpinsrd)]),
				(code[nameof(Code.EVEX_Vpinsrq_xmm_xmm_rm64_imm8)], mnemonic[nameof(Mnemonic.Vpinsrq)]),
				(code[nameof(Code.EVEX_Vshuff32x4_ymm_k1z_ymm_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vshuff32x4)]),
				(code[nameof(Code.EVEX_Vshuff32x4_zmm_k1z_zmm_zmmm512b32_imm8)], mnemonic[nameof(Mnemonic.Vshuff32x4)]),
				(code[nameof(Code.EVEX_Vshuff64x2_ymm_k1z_ymm_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vshuff64x2)]),
				(code[nameof(Code.EVEX_Vshuff64x2_zmm_k1z_zmm_zmmm512b64_imm8)], mnemonic[nameof(Mnemonic.Vshuff64x2)]),
				(code[nameof(Code.EVEX_Vpternlogd_xmm_k1z_xmm_xmmm128b32_imm8)], mnemonic[nameof(Mnemonic.Vpternlogd)]),
				(code[nameof(Code.EVEX_Vpternlogd_ymm_k1z_ymm_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vpternlogd)]),
				(code[nameof(Code.EVEX_Vpternlogd_zmm_k1z_zmm_zmmm512b32_imm8)], mnemonic[nameof(Mnemonic.Vpternlogd)]),
				(code[nameof(Code.EVEX_Vpternlogq_xmm_k1z_xmm_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Vpternlogq)]),
				(code[nameof(Code.EVEX_Vpternlogq_ymm_k1z_ymm_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vpternlogq)]),
				(code[nameof(Code.EVEX_Vpternlogq_zmm_k1z_zmm_zmmm512b64_imm8)], mnemonic[nameof(Mnemonic.Vpternlogq)]),
				(code[nameof(Code.EVEX_Vgetmantps_xmm_k1z_xmmm128b32_imm8)], mnemonic[nameof(Mnemonic.Vgetmantps)]),
				(code[nameof(Code.EVEX_Vgetmantps_ymm_k1z_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vgetmantps)]),
				(code[nameof(Code.EVEX_Vgetmantps_zmm_k1z_zmmm512b32_imm8_sae)], mnemonic[nameof(Mnemonic.Vgetmantps)]),
				(code[nameof(Code.EVEX_Vgetmantpd_xmm_k1z_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Vgetmantpd)]),
				(code[nameof(Code.EVEX_Vgetmantpd_ymm_k1z_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vgetmantpd)]),
				(code[nameof(Code.EVEX_Vgetmantpd_zmm_k1z_zmmm512b64_imm8_sae)], mnemonic[nameof(Mnemonic.Vgetmantpd)]),
				(code[nameof(Code.EVEX_Vgetmantss_xmm_k1z_xmm_xmmm32_imm8_sae)], mnemonic[nameof(Mnemonic.Vgetmantss)]),
				(code[nameof(Code.EVEX_Vgetmantsd_xmm_k1z_xmm_xmmm64_imm8_sae)], mnemonic[nameof(Mnemonic.Vgetmantsd)]),
				(code[nameof(Code.VEX_Kshiftrb_k_k_imm8)], mnemonic[nameof(Mnemonic.Kshiftrb)]),
				(code[nameof(Code.VEX_Kshiftrw_k_k_imm8)], mnemonic[nameof(Mnemonic.Kshiftrw)]),
				(code[nameof(Code.VEX_Kshiftrd_k_k_imm8)], mnemonic[nameof(Mnemonic.Kshiftrd)]),
				(code[nameof(Code.VEX_Kshiftrq_k_k_imm8)], mnemonic[nameof(Mnemonic.Kshiftrq)]),
				(code[nameof(Code.VEX_Kshiftlb_k_k_imm8)], mnemonic[nameof(Mnemonic.Kshiftlb)]),
				(code[nameof(Code.VEX_Kshiftlw_k_k_imm8)], mnemonic[nameof(Mnemonic.Kshiftlw)]),
				(code[nameof(Code.VEX_Kshiftld_k_k_imm8)], mnemonic[nameof(Mnemonic.Kshiftld)]),
				(code[nameof(Code.VEX_Kshiftlq_k_k_imm8)], mnemonic[nameof(Mnemonic.Kshiftlq)]),
				(code[nameof(Code.VEX_Vinserti128_ymm_ymm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vinserti128)]),
				(code[nameof(Code.EVEX_Vinserti32x4_ymm_k1z_ymm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vinserti32x4)]),
				(code[nameof(Code.EVEX_Vinserti32x4_zmm_k1z_zmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vinserti32x4)]),
				(code[nameof(Code.EVEX_Vinserti64x2_ymm_k1z_ymm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vinserti64x2)]),
				(code[nameof(Code.EVEX_Vinserti64x2_zmm_k1z_zmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vinserti64x2)]),
				(code[nameof(Code.VEX_Vextracti128_xmmm128_ymm_imm8)], mnemonic[nameof(Mnemonic.Vextracti128)]),
				(code[nameof(Code.EVEX_Vextracti32x4_xmmm128_k1z_ymm_imm8)], mnemonic[nameof(Mnemonic.Vextracti32x4)]),
				(code[nameof(Code.EVEX_Vextracti32x4_xmmm128_k1z_zmm_imm8)], mnemonic[nameof(Mnemonic.Vextracti32x4)]),
				(code[nameof(Code.EVEX_Vextracti64x2_xmmm128_k1z_ymm_imm8)], mnemonic[nameof(Mnemonic.Vextracti64x2)]),
				(code[nameof(Code.EVEX_Vextracti64x2_xmmm128_k1z_zmm_imm8)], mnemonic[nameof(Mnemonic.Vextracti64x2)]),
				(code[nameof(Code.EVEX_Vinserti32x8_zmm_k1z_zmm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vinserti32x8)]),
				(code[nameof(Code.EVEX_Vinserti64x4_zmm_k1z_zmm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vinserti64x4)]),
				(code[nameof(Code.EVEX_Vextracti32x8_ymmm256_k1z_zmm_imm8)], mnemonic[nameof(Mnemonic.Vextracti32x8)]),
				(code[nameof(Code.EVEX_Vextracti64x4_ymmm256_k1z_zmm_imm8)], mnemonic[nameof(Mnemonic.Vextracti64x4)]),
				(code[nameof(Code.EVEX_Vpcmpub_k_k1_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpcmpub)]),
				(code[nameof(Code.EVEX_Vpcmpub_k_k1_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpcmpub)]),
				(code[nameof(Code.EVEX_Vpcmpub_k_k1_zmm_zmmm512_imm8)], mnemonic[nameof(Mnemonic.Vpcmpub)]),
				(code[nameof(Code.EVEX_Vpcmpuw_k_k1_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpcmpuw)]),
				(code[nameof(Code.EVEX_Vpcmpuw_k_k1_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpcmpuw)]),
				(code[nameof(Code.EVEX_Vpcmpuw_k_k1_zmm_zmmm512_imm8)], mnemonic[nameof(Mnemonic.Vpcmpuw)]),
				(code[nameof(Code.EVEX_Vpcmpb_k_k1_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpcmpb)]),
				(code[nameof(Code.EVEX_Vpcmpb_k_k1_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpcmpb)]),
				(code[nameof(Code.EVEX_Vpcmpb_k_k1_zmm_zmmm512_imm8)], mnemonic[nameof(Mnemonic.Vpcmpb)]),
				(code[nameof(Code.EVEX_Vpcmpw_k_k1_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpcmpw)]),
				(code[nameof(Code.EVEX_Vpcmpw_k_k1_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpcmpw)]),
				(code[nameof(Code.EVEX_Vpcmpw_k_k1_zmm_zmmm512_imm8)], mnemonic[nameof(Mnemonic.Vpcmpw)]),
				(code[nameof(Code.Dpps_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Dpps)]),
				(code[nameof(Code.VEX_Vdpps_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vdpps)]),
				(code[nameof(Code.VEX_Vdpps_ymm_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vdpps)]),
				(code[nameof(Code.Dppd_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Dppd)]),
				(code[nameof(Code.VEX_Vdppd_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vdppd)]),
				(code[nameof(Code.Mpsadbw_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Mpsadbw)]),
				(code[nameof(Code.VEX_Vmpsadbw_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vmpsadbw)]),
				(code[nameof(Code.VEX_Vmpsadbw_ymm_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vmpsadbw)]),
				(code[nameof(Code.EVEX_Vdbpsadbw_xmm_k1z_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vdbpsadbw)]),
				(code[nameof(Code.EVEX_Vdbpsadbw_ymm_k1z_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vdbpsadbw)]),
				(code[nameof(Code.EVEX_Vdbpsadbw_zmm_k1z_zmm_zmmm512_imm8)], mnemonic[nameof(Mnemonic.Vdbpsadbw)]),
				(code[nameof(Code.EVEX_Vshufi32x4_ymm_k1z_ymm_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vshufi32x4)]),
				(code[nameof(Code.EVEX_Vshufi32x4_zmm_k1z_zmm_zmmm512b32_imm8)], mnemonic[nameof(Mnemonic.Vshufi32x4)]),
				(code[nameof(Code.EVEX_Vshufi64x2_ymm_k1z_ymm_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vshufi64x2)]),
				(code[nameof(Code.EVEX_Vshufi64x2_zmm_k1z_zmm_zmmm512b64_imm8)], mnemonic[nameof(Mnemonic.Vshufi64x2)]),
				(code[nameof(Code.Pclmulqdq_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Pclmulqdq)]),
				(code[nameof(Code.VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpclmulqdq)]),
				(code[nameof(Code.VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpclmulqdq)]),
				(code[nameof(Code.EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpclmulqdq)]),
				(code[nameof(Code.EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpclmulqdq)]),
				(code[nameof(Code.EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8)], mnemonic[nameof(Mnemonic.Vpclmulqdq)]),
				(code[nameof(Code.VEX_Vperm2i128_ymm_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vperm2i128)]),
				(code[nameof(Code.VEX_Vpermil2ps_xmm_xmm_xmmm128_xmm_imm2)], mnemonic[nameof(Mnemonic.Vpermil2ps)]),
				(code[nameof(Code.VEX_Vpermil2ps_ymm_ymm_ymmm256_ymm_imm2)], mnemonic[nameof(Mnemonic.Vpermil2ps)]),
				(code[nameof(Code.VEX_Vpermil2ps_xmm_xmm_xmm_xmmm128_imm2)], mnemonic[nameof(Mnemonic.Vpermil2ps)]),
				(code[nameof(Code.VEX_Vpermil2ps_ymm_ymm_ymm_ymmm256_imm2)], mnemonic[nameof(Mnemonic.Vpermil2ps)]),
				(code[nameof(Code.VEX_Vpermil2pd_xmm_xmm_xmmm128_xmm_imm2)], mnemonic[nameof(Mnemonic.Vpermil2pd)]),
				(code[nameof(Code.VEX_Vpermil2pd_ymm_ymm_ymmm256_ymm_imm2)], mnemonic[nameof(Mnemonic.Vpermil2pd)]),
				(code[nameof(Code.VEX_Vpermil2pd_xmm_xmm_xmm_xmmm128_imm2)], mnemonic[nameof(Mnemonic.Vpermil2pd)]),
				(code[nameof(Code.VEX_Vpermil2pd_ymm_ymm_ymm_ymmm256_imm2)], mnemonic[nameof(Mnemonic.Vpermil2pd)]),
				(code[nameof(Code.VEX_Vblendvps_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vblendvps)]),
				(code[nameof(Code.VEX_Vblendvps_ymm_ymm_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vblendvps)]),
				(code[nameof(Code.VEX_Vblendvpd_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vblendvpd)]),
				(code[nameof(Code.VEX_Vblendvpd_ymm_ymm_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vblendvpd)]),
				(code[nameof(Code.VEX_Vpblendvb_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpblendvb)]),
				(code[nameof(Code.VEX_Vpblendvb_ymm_ymm_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vpblendvb)]),
				(code[nameof(Code.EVEX_Vrangeps_xmm_k1z_xmm_xmmm128b32_imm8)], mnemonic[nameof(Mnemonic.Vrangeps)]),
				(code[nameof(Code.EVEX_Vrangeps_ymm_k1z_ymm_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vrangeps)]),
				(code[nameof(Code.EVEX_Vrangeps_zmm_k1z_zmm_zmmm512b32_imm8_sae)], mnemonic[nameof(Mnemonic.Vrangeps)]),
				(code[nameof(Code.EVEX_Vrangepd_xmm_k1z_xmm_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Vrangepd)]),
				(code[nameof(Code.EVEX_Vrangepd_ymm_k1z_ymm_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vrangepd)]),
				(code[nameof(Code.EVEX_Vrangepd_zmm_k1z_zmm_zmmm512b64_imm8_sae)], mnemonic[nameof(Mnemonic.Vrangepd)]),
				(code[nameof(Code.EVEX_Vrangess_xmm_k1z_xmm_xmmm32_imm8_sae)], mnemonic[nameof(Mnemonic.Vrangess)]),
				(code[nameof(Code.EVEX_Vrangesd_xmm_k1z_xmm_xmmm64_imm8_sae)], mnemonic[nameof(Mnemonic.Vrangesd)]),
				(code[nameof(Code.EVEX_Vfixupimmps_xmm_k1z_xmm_xmmm128b32_imm8)], mnemonic[nameof(Mnemonic.Vfixupimmps)]),
				(code[nameof(Code.EVEX_Vfixupimmps_ymm_k1z_ymm_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vfixupimmps)]),
				(code[nameof(Code.EVEX_Vfixupimmps_zmm_k1z_zmm_zmmm512b32_imm8_sae)], mnemonic[nameof(Mnemonic.Vfixupimmps)]),
				(code[nameof(Code.EVEX_Vfixupimmpd_xmm_k1z_xmm_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Vfixupimmpd)]),
				(code[nameof(Code.EVEX_Vfixupimmpd_ymm_k1z_ymm_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vfixupimmpd)]),
				(code[nameof(Code.EVEX_Vfixupimmpd_zmm_k1z_zmm_zmmm512b64_imm8_sae)], mnemonic[nameof(Mnemonic.Vfixupimmpd)]),
				(code[nameof(Code.EVEX_Vfixupimmss_xmm_k1z_xmm_xmmm32_imm8_sae)], mnemonic[nameof(Mnemonic.Vfixupimmss)]),
				(code[nameof(Code.EVEX_Vfixupimmsd_xmm_k1z_xmm_xmmm64_imm8_sae)], mnemonic[nameof(Mnemonic.Vfixupimmsd)]),
				(code[nameof(Code.EVEX_Vreduceps_xmm_k1z_xmmm128b32_imm8)], mnemonic[nameof(Mnemonic.Vreduceps)]),
				(code[nameof(Code.EVEX_Vreduceps_ymm_k1z_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vreduceps)]),
				(code[nameof(Code.EVEX_Vreduceps_zmm_k1z_zmmm512b32_imm8_sae)], mnemonic[nameof(Mnemonic.Vreduceps)]),
				(code[nameof(Code.EVEX_Vreducepd_xmm_k1z_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Vreducepd)]),
				(code[nameof(Code.EVEX_Vreducepd_ymm_k1z_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vreducepd)]),
				(code[nameof(Code.EVEX_Vreducepd_zmm_k1z_zmmm512b64_imm8_sae)], mnemonic[nameof(Mnemonic.Vreducepd)]),
				(code[nameof(Code.EVEX_Vreducess_xmm_k1z_xmm_xmmm32_imm8_sae)], mnemonic[nameof(Mnemonic.Vreducess)]),
				(code[nameof(Code.EVEX_Vreducesd_xmm_k1z_xmm_xmmm64_imm8_sae)], mnemonic[nameof(Mnemonic.Vreducesd)]),
				(code[nameof(Code.VEX_Vfmaddsubps_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vfmaddsubps)]),
				(code[nameof(Code.VEX_Vfmaddsubps_ymm_ymm_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vfmaddsubps)]),
				(code[nameof(Code.VEX_Vfmaddsubps_xmm_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmaddsubps)]),
				(code[nameof(Code.VEX_Vfmaddsubps_ymm_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmaddsubps)]),
				(code[nameof(Code.VEX_Vfmaddsubpd_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vfmaddsubpd)]),
				(code[nameof(Code.VEX_Vfmaddsubpd_ymm_ymm_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vfmaddsubpd)]),
				(code[nameof(Code.VEX_Vfmaddsubpd_xmm_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmaddsubpd)]),
				(code[nameof(Code.VEX_Vfmaddsubpd_ymm_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmaddsubpd)]),
				(code[nameof(Code.VEX_Vfmsubaddps_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vfmsubaddps)]),
				(code[nameof(Code.VEX_Vfmsubaddps_ymm_ymm_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vfmsubaddps)]),
				(code[nameof(Code.VEX_Vfmsubaddps_xmm_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmsubaddps)]),
				(code[nameof(Code.VEX_Vfmsubaddps_ymm_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmsubaddps)]),
				(code[nameof(Code.VEX_Vfmsubaddpd_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vfmsubaddpd)]),
				(code[nameof(Code.VEX_Vfmsubaddpd_ymm_ymm_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vfmsubaddpd)]),
				(code[nameof(Code.VEX_Vfmsubaddpd_xmm_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmsubaddpd)]),
				(code[nameof(Code.VEX_Vfmsubaddpd_ymm_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmsubaddpd)]),
				(code[nameof(Code.Pcmpestrm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Pcmpestrm)]),
				(code[nameof(Code.Pcmpestrm64_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Pcmpestrm64)]),
				(code[nameof(Code.VEX_Vpcmpestrm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpcmpestrm)]),
				(code[nameof(Code.VEX_Vpcmpestrm64_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpcmpestrm64)]),
				(code[nameof(Code.Pcmpestri_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Pcmpestri)]),
				(code[nameof(Code.Pcmpestri64_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Pcmpestri64)]),
				(code[nameof(Code.VEX_Vpcmpestri_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpcmpestri)]),
				(code[nameof(Code.VEX_Vpcmpestri64_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpcmpestri64)]),
				(code[nameof(Code.Pcmpistrm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Pcmpistrm)]),
				(code[nameof(Code.VEX_Vpcmpistrm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpcmpistrm)]),
				(code[nameof(Code.Pcmpistri_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Pcmpistri)]),
				(code[nameof(Code.VEX_Vpcmpistri_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpcmpistri)]),
				(code[nameof(Code.EVEX_Vfpclassps_k_k1_xmmm128b32_imm8)], mnemonic[nameof(Mnemonic.Vfpclassps)]),
				(code[nameof(Code.EVEX_Vfpclassps_k_k1_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vfpclassps)]),
				(code[nameof(Code.EVEX_Vfpclassps_k_k1_zmmm512b32_imm8)], mnemonic[nameof(Mnemonic.Vfpclassps)]),
				(code[nameof(Code.EVEX_Vfpclasspd_k_k1_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Vfpclasspd)]),
				(code[nameof(Code.EVEX_Vfpclasspd_k_k1_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vfpclasspd)]),
				(code[nameof(Code.EVEX_Vfpclasspd_k_k1_zmmm512b64_imm8)], mnemonic[nameof(Mnemonic.Vfpclasspd)]),
				(code[nameof(Code.EVEX_Vfpclassss_k_k1_xmmm32_imm8)], mnemonic[nameof(Mnemonic.Vfpclassss)]),
				(code[nameof(Code.EVEX_Vfpclasssd_k_k1_xmmm64_imm8)], mnemonic[nameof(Mnemonic.Vfpclasssd)]),
				(code[nameof(Code.VEX_Vfmaddps_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vfmaddps)]),
				(code[nameof(Code.VEX_Vfmaddps_ymm_ymm_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vfmaddps)]),
				(code[nameof(Code.VEX_Vfmaddps_xmm_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmaddps)]),
				(code[nameof(Code.VEX_Vfmaddps_ymm_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmaddps)]),
				(code[nameof(Code.VEX_Vfmaddpd_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vfmaddpd)]),
				(code[nameof(Code.VEX_Vfmaddpd_ymm_ymm_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vfmaddpd)]),
				(code[nameof(Code.VEX_Vfmaddpd_xmm_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmaddpd)]),
				(code[nameof(Code.VEX_Vfmaddpd_ymm_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmaddpd)]),
				(code[nameof(Code.VEX_Vfmaddss_xmm_xmm_xmmm32_xmm)], mnemonic[nameof(Mnemonic.Vfmaddss)]),
				(code[nameof(Code.VEX_Vfmaddss_xmm_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vfmaddss)]),
				(code[nameof(Code.VEX_Vfmaddsd_xmm_xmm_xmmm64_xmm)], mnemonic[nameof(Mnemonic.Vfmaddsd)]),
				(code[nameof(Code.VEX_Vfmaddsd_xmm_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vfmaddsd)]),
				(code[nameof(Code.VEX_Vfmsubps_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vfmsubps)]),
				(code[nameof(Code.VEX_Vfmsubps_ymm_ymm_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vfmsubps)]),
				(code[nameof(Code.VEX_Vfmsubps_xmm_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmsubps)]),
				(code[nameof(Code.VEX_Vfmsubps_ymm_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmsubps)]),
				(code[nameof(Code.VEX_Vfmsubpd_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vfmsubpd)]),
				(code[nameof(Code.VEX_Vfmsubpd_ymm_ymm_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vfmsubpd)]),
				(code[nameof(Code.VEX_Vfmsubpd_xmm_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfmsubpd)]),
				(code[nameof(Code.VEX_Vfmsubpd_ymm_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfmsubpd)]),
				(code[nameof(Code.VEX_Vfmsubss_xmm_xmm_xmmm32_xmm)], mnemonic[nameof(Mnemonic.Vfmsubss)]),
				(code[nameof(Code.VEX_Vfmsubss_xmm_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vfmsubss)]),
				(code[nameof(Code.VEX_Vfmsubsd_xmm_xmm_xmmm64_xmm)], mnemonic[nameof(Mnemonic.Vfmsubsd)]),
				(code[nameof(Code.VEX_Vfmsubsd_xmm_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vfmsubsd)]),
				(code[nameof(Code.EVEX_Vpshldw_xmm_k1z_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpshldw)]),
				(code[nameof(Code.EVEX_Vpshldw_ymm_k1z_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpshldw)]),
				(code[nameof(Code.EVEX_Vpshldw_zmm_k1z_zmm_zmmm512_imm8)], mnemonic[nameof(Mnemonic.Vpshldw)]),
				(code[nameof(Code.EVEX_Vpshldd_xmm_k1z_xmm_xmmm128b32_imm8)], mnemonic[nameof(Mnemonic.Vpshldd)]),
				(code[nameof(Code.EVEX_Vpshldd_ymm_k1z_ymm_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vpshldd)]),
				(code[nameof(Code.EVEX_Vpshldd_zmm_k1z_zmm_zmmm512b32_imm8)], mnemonic[nameof(Mnemonic.Vpshldd)]),
				(code[nameof(Code.EVEX_Vpshldq_xmm_k1z_xmm_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Vpshldq)]),
				(code[nameof(Code.EVEX_Vpshldq_ymm_k1z_ymm_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vpshldq)]),
				(code[nameof(Code.EVEX_Vpshldq_zmm_k1z_zmm_zmmm512b64_imm8)], mnemonic[nameof(Mnemonic.Vpshldq)]),
				(code[nameof(Code.EVEX_Vpshrdw_xmm_k1z_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpshrdw)]),
				(code[nameof(Code.EVEX_Vpshrdw_ymm_k1z_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vpshrdw)]),
				(code[nameof(Code.EVEX_Vpshrdw_zmm_k1z_zmm_zmmm512_imm8)], mnemonic[nameof(Mnemonic.Vpshrdw)]),
				(code[nameof(Code.EVEX_Vpshrdd_xmm_k1z_xmm_xmmm128b32_imm8)], mnemonic[nameof(Mnemonic.Vpshrdd)]),
				(code[nameof(Code.EVEX_Vpshrdd_ymm_k1z_ymm_ymmm256b32_imm8)], mnemonic[nameof(Mnemonic.Vpshrdd)]),
				(code[nameof(Code.EVEX_Vpshrdd_zmm_k1z_zmm_zmmm512b32_imm8)], mnemonic[nameof(Mnemonic.Vpshrdd)]),
				(code[nameof(Code.EVEX_Vpshrdq_xmm_k1z_xmm_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Vpshrdq)]),
				(code[nameof(Code.EVEX_Vpshrdq_ymm_k1z_ymm_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vpshrdq)]),
				(code[nameof(Code.EVEX_Vpshrdq_zmm_k1z_zmm_zmmm512b64_imm8)], mnemonic[nameof(Mnemonic.Vpshrdq)]),
				(code[nameof(Code.VEX_Vfnmaddps_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vfnmaddps)]),
				(code[nameof(Code.VEX_Vfnmaddps_ymm_ymm_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vfnmaddps)]),
				(code[nameof(Code.VEX_Vfnmaddps_xmm_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfnmaddps)]),
				(code[nameof(Code.VEX_Vfnmaddps_ymm_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfnmaddps)]),
				(code[nameof(Code.VEX_Vfnmaddpd_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vfnmaddpd)]),
				(code[nameof(Code.VEX_Vfnmaddpd_ymm_ymm_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vfnmaddpd)]),
				(code[nameof(Code.VEX_Vfnmaddpd_xmm_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfnmaddpd)]),
				(code[nameof(Code.VEX_Vfnmaddpd_ymm_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfnmaddpd)]),
				(code[nameof(Code.VEX_Vfnmaddss_xmm_xmm_xmmm32_xmm)], mnemonic[nameof(Mnemonic.Vfnmaddss)]),
				(code[nameof(Code.VEX_Vfnmaddss_xmm_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vfnmaddss)]),
				(code[nameof(Code.VEX_Vfnmaddsd_xmm_xmm_xmmm64_xmm)], mnemonic[nameof(Mnemonic.Vfnmaddsd)]),
				(code[nameof(Code.VEX_Vfnmaddsd_xmm_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vfnmaddsd)]),
				(code[nameof(Code.VEX_Vfnmsubps_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vfnmsubps)]),
				(code[nameof(Code.VEX_Vfnmsubps_ymm_ymm_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vfnmsubps)]),
				(code[nameof(Code.VEX_Vfnmsubps_xmm_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfnmsubps)]),
				(code[nameof(Code.VEX_Vfnmsubps_ymm_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfnmsubps)]),
				(code[nameof(Code.VEX_Vfnmsubpd_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vfnmsubpd)]),
				(code[nameof(Code.VEX_Vfnmsubpd_ymm_ymm_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vfnmsubpd)]),
				(code[nameof(Code.VEX_Vfnmsubpd_xmm_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfnmsubpd)]),
				(code[nameof(Code.VEX_Vfnmsubpd_ymm_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfnmsubpd)]),
				(code[nameof(Code.VEX_Vfnmsubss_xmm_xmm_xmmm32_xmm)], mnemonic[nameof(Mnemonic.Vfnmsubss)]),
				(code[nameof(Code.VEX_Vfnmsubss_xmm_xmm_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vfnmsubss)]),
				(code[nameof(Code.VEX_Vfnmsubsd_xmm_xmm_xmmm64_xmm)], mnemonic[nameof(Mnemonic.Vfnmsubsd)]),
				(code[nameof(Code.VEX_Vfnmsubsd_xmm_xmm_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vfnmsubsd)]),
				(code[nameof(Code.Sha1rnds4_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Sha1rnds4)]),
				(code[nameof(Code.Gf2p8affineqb_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Gf2p8affineqb)]),
				(code[nameof(Code.VEX_Vgf2p8affineqb_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vgf2p8affineqb)]),
				(code[nameof(Code.VEX_Vgf2p8affineqb_ymm_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vgf2p8affineqb)]),
				(code[nameof(Code.EVEX_Vgf2p8affineqb_xmm_k1z_xmm_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Vgf2p8affineqb)]),
				(code[nameof(Code.EVEX_Vgf2p8affineqb_ymm_k1z_ymm_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vgf2p8affineqb)]),
				(code[nameof(Code.EVEX_Vgf2p8affineqb_zmm_k1z_zmm_zmmm512b64_imm8)], mnemonic[nameof(Mnemonic.Vgf2p8affineqb)]),
				(code[nameof(Code.Gf2p8affineinvqb_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Gf2p8affineinvqb)]),
				(code[nameof(Code.VEX_Vgf2p8affineinvqb_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vgf2p8affineinvqb)]),
				(code[nameof(Code.VEX_Vgf2p8affineinvqb_ymm_ymm_ymmm256_imm8)], mnemonic[nameof(Mnemonic.Vgf2p8affineinvqb)]),
				(code[nameof(Code.EVEX_Vgf2p8affineinvqb_xmm_k1z_xmm_xmmm128b64_imm8)], mnemonic[nameof(Mnemonic.Vgf2p8affineinvqb)]),
				(code[nameof(Code.EVEX_Vgf2p8affineinvqb_ymm_k1z_ymm_ymmm256b64_imm8)], mnemonic[nameof(Mnemonic.Vgf2p8affineinvqb)]),
				(code[nameof(Code.EVEX_Vgf2p8affineinvqb_zmm_k1z_zmm_zmmm512b64_imm8)], mnemonic[nameof(Mnemonic.Vgf2p8affineinvqb)]),
				(code[nameof(Code.Aeskeygenassist_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Aeskeygenassist)]),
				(code[nameof(Code.VEX_Vaeskeygenassist_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vaeskeygenassist)]),
				(code[nameof(Code.VEX_Rorx_r32_rm32_imm8)], mnemonic[nameof(Mnemonic.Rorx)]),
				(code[nameof(Code.VEX_Rorx_r64_rm64_imm8)], mnemonic[nameof(Mnemonic.Rorx)]),
				(code[nameof(Code.XOP_Vpmacssww_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpmacssww)]),
				(code[nameof(Code.XOP_Vpmacsswd_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpmacsswd)]),
				(code[nameof(Code.XOP_Vpmacssdql_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpmacssdql)]),
				(code[nameof(Code.XOP_Vpmacssdd_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpmacssdd)]),
				(code[nameof(Code.XOP_Vpmacssdqh_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpmacssdqh)]),
				(code[nameof(Code.XOP_Vpmacsww_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpmacsww)]),
				(code[nameof(Code.XOP_Vpmacswd_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpmacswd)]),
				(code[nameof(Code.XOP_Vpmacsdql_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpmacsdql)]),
				(code[nameof(Code.XOP_Vpmacsdd_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpmacsdd)]),
				(code[nameof(Code.XOP_Vpmacsdqh_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpmacsdqh)]),
				(code[nameof(Code.XOP_Vpcmov_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpcmov)]),
				(code[nameof(Code.XOP_Vpcmov_ymm_ymm_ymmm256_ymm)], mnemonic[nameof(Mnemonic.Vpcmov)]),
				(code[nameof(Code.XOP_Vpcmov_xmm_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpcmov)]),
				(code[nameof(Code.XOP_Vpcmov_ymm_ymm_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vpcmov)]),
				(code[nameof(Code.XOP_Vpperm_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpperm)]),
				(code[nameof(Code.XOP_Vpperm_xmm_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpperm)]),
				(code[nameof(Code.XOP_Vpmadcsswd_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpmadcsswd)]),
				(code[nameof(Code.XOP_Vpmadcswd_xmm_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpmadcswd)]),
				(code[nameof(Code.XOP_Vprotb_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vprotb)]),
				(code[nameof(Code.XOP_Vprotw_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vprotw)]),
				(code[nameof(Code.XOP_Vprotd_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vprotd)]),
				(code[nameof(Code.XOP_Vprotq_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vprotq)]),
				(code[nameof(Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpcomb)]),
				(code[nameof(Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpcomw)]),
				(code[nameof(Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpcomd)]),
				(code[nameof(Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpcomq)]),
				(code[nameof(Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpcomub)]),
				(code[nameof(Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpcomuw)]),
				(code[nameof(Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpcomud)]),
				(code[nameof(Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8)], mnemonic[nameof(Mnemonic.Vpcomuq)]),
				(code[nameof(Code.XOP_Blcfill_r32_rm32)], mnemonic[nameof(Mnemonic.Blcfill)]),
				(code[nameof(Code.XOP_Blcfill_r64_rm64)], mnemonic[nameof(Mnemonic.Blcfill)]),
				(code[nameof(Code.XOP_Blsfill_r32_rm32)], mnemonic[nameof(Mnemonic.Blsfill)]),
				(code[nameof(Code.XOP_Blsfill_r64_rm64)], mnemonic[nameof(Mnemonic.Blsfill)]),
				(code[nameof(Code.XOP_Blcs_r32_rm32)], mnemonic[nameof(Mnemonic.Blcs)]),
				(code[nameof(Code.XOP_Blcs_r64_rm64)], mnemonic[nameof(Mnemonic.Blcs)]),
				(code[nameof(Code.XOP_Tzmsk_r32_rm32)], mnemonic[nameof(Mnemonic.Tzmsk)]),
				(code[nameof(Code.XOP_Tzmsk_r64_rm64)], mnemonic[nameof(Mnemonic.Tzmsk)]),
				(code[nameof(Code.XOP_Blcic_r32_rm32)], mnemonic[nameof(Mnemonic.Blcic)]),
				(code[nameof(Code.XOP_Blcic_r64_rm64)], mnemonic[nameof(Mnemonic.Blcic)]),
				(code[nameof(Code.XOP_Blsic_r32_rm32)], mnemonic[nameof(Mnemonic.Blsic)]),
				(code[nameof(Code.XOP_Blsic_r64_rm64)], mnemonic[nameof(Mnemonic.Blsic)]),
				(code[nameof(Code.XOP_T1mskc_r32_rm32)], mnemonic[nameof(Mnemonic.T1mskc)]),
				(code[nameof(Code.XOP_T1mskc_r64_rm64)], mnemonic[nameof(Mnemonic.T1mskc)]),
				(code[nameof(Code.XOP_Blcmsk_r32_rm32)], mnemonic[nameof(Mnemonic.Blcmsk)]),
				(code[nameof(Code.XOP_Blcmsk_r64_rm64)], mnemonic[nameof(Mnemonic.Blcmsk)]),
				(code[nameof(Code.XOP_Blci_r32_rm32)], mnemonic[nameof(Mnemonic.Blci)]),
				(code[nameof(Code.XOP_Blci_r64_rm64)], mnemonic[nameof(Mnemonic.Blci)]),
				(code[nameof(Code.XOP_Llwpcb_r32)], mnemonic[nameof(Mnemonic.Llwpcb)]),
				(code[nameof(Code.XOP_Llwpcb_r64)], mnemonic[nameof(Mnemonic.Llwpcb)]),
				(code[nameof(Code.XOP_Slwpcb_r32)], mnemonic[nameof(Mnemonic.Slwpcb)]),
				(code[nameof(Code.XOP_Slwpcb_r64)], mnemonic[nameof(Mnemonic.Slwpcb)]),
				(code[nameof(Code.XOP_Vfrczps_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfrczps)]),
				(code[nameof(Code.XOP_Vfrczps_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfrczps)]),
				(code[nameof(Code.XOP_Vfrczpd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vfrczpd)]),
				(code[nameof(Code.XOP_Vfrczpd_ymm_ymmm256)], mnemonic[nameof(Mnemonic.Vfrczpd)]),
				(code[nameof(Code.XOP_Vfrczss_xmm_xmmm32)], mnemonic[nameof(Mnemonic.Vfrczss)]),
				(code[nameof(Code.XOP_Vfrczsd_xmm_xmmm64)], mnemonic[nameof(Mnemonic.Vfrczsd)]),
				(code[nameof(Code.XOP_Vprotb_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vprotb)]),
				(code[nameof(Code.XOP_Vprotb_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vprotb)]),
				(code[nameof(Code.XOP_Vprotw_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vprotw)]),
				(code[nameof(Code.XOP_Vprotw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vprotw)]),
				(code[nameof(Code.XOP_Vprotd_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vprotd)]),
				(code[nameof(Code.XOP_Vprotd_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vprotd)]),
				(code[nameof(Code.XOP_Vprotq_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vprotq)]),
				(code[nameof(Code.XOP_Vprotq_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vprotq)]),
				(code[nameof(Code.XOP_Vpshlb_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpshlb)]),
				(code[nameof(Code.XOP_Vpshlb_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpshlb)]),
				(code[nameof(Code.XOP_Vpshlw_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpshlw)]),
				(code[nameof(Code.XOP_Vpshlw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpshlw)]),
				(code[nameof(Code.XOP_Vpshld_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpshld)]),
				(code[nameof(Code.XOP_Vpshld_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpshld)]),
				(code[nameof(Code.XOP_Vpshlq_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpshlq)]),
				(code[nameof(Code.XOP_Vpshlq_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpshlq)]),
				(code[nameof(Code.XOP_Vpshab_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpshab)]),
				(code[nameof(Code.XOP_Vpshab_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpshab)]),
				(code[nameof(Code.XOP_Vpshaw_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpshaw)]),
				(code[nameof(Code.XOP_Vpshaw_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpshaw)]),
				(code[nameof(Code.XOP_Vpshad_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpshad)]),
				(code[nameof(Code.XOP_Vpshad_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpshad)]),
				(code[nameof(Code.XOP_Vpshaq_xmm_xmmm128_xmm)], mnemonic[nameof(Mnemonic.Vpshaq)]),
				(code[nameof(Code.XOP_Vpshaq_xmm_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vpshaq)]),
				(code[nameof(Code.XOP_Vphaddbw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphaddbw)]),
				(code[nameof(Code.XOP_Vphaddbd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphaddbd)]),
				(code[nameof(Code.XOP_Vphaddbq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphaddbq)]),
				(code[nameof(Code.XOP_Vphaddwd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphaddwd)]),
				(code[nameof(Code.XOP_Vphaddwq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphaddwq)]),
				(code[nameof(Code.XOP_Vphadddq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphadddq)]),
				(code[nameof(Code.XOP_Vphaddubw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphaddubw)]),
				(code[nameof(Code.XOP_Vphaddubd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphaddubd)]),
				(code[nameof(Code.XOP_Vphaddubq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphaddubq)]),
				(code[nameof(Code.XOP_Vphadduwd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphadduwd)]),
				(code[nameof(Code.XOP_Vphadduwq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphadduwq)]),
				(code[nameof(Code.XOP_Vphaddudq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphaddudq)]),
				(code[nameof(Code.XOP_Vphsubbw_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphsubbw)]),
				(code[nameof(Code.XOP_Vphsubwd_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphsubwd)]),
				(code[nameof(Code.XOP_Vphsubdq_xmm_xmmm128)], mnemonic[nameof(Mnemonic.Vphsubdq)]),
				(code[nameof(Code.XOP_Bextr_r32_rm32_imm32)], mnemonic[nameof(Mnemonic.Bextr)]),
				(code[nameof(Code.XOP_Bextr_r64_rm64_imm32)], mnemonic[nameof(Mnemonic.Bextr)]),
				(code[nameof(Code.XOP_Lwpins_r32_rm32_imm32)], mnemonic[nameof(Mnemonic.Lwpins)]),
				(code[nameof(Code.XOP_Lwpins_r64_rm32_imm32)], mnemonic[nameof(Mnemonic.Lwpins)]),
				(code[nameof(Code.XOP_Lwpval_r32_rm32_imm32)], mnemonic[nameof(Mnemonic.Lwpval)]),
				(code[nameof(Code.XOP_Lwpval_r64_rm32_imm32)], mnemonic[nameof(Mnemonic.Lwpval)]),
				(code[nameof(Code.D3NOW_Pi2fw_mm_mmm64)], mnemonic[nameof(Mnemonic.Pi2fw)]),
				(code[nameof(Code.D3NOW_Pi2fd_mm_mmm64)], mnemonic[nameof(Mnemonic.Pi2fd)]),
				(code[nameof(Code.D3NOW_Pf2iw_mm_mmm64)], mnemonic[nameof(Mnemonic.Pf2iw)]),
				(code[nameof(Code.D3NOW_Pf2id_mm_mmm64)], mnemonic[nameof(Mnemonic.Pf2id)]),
				(code[nameof(Code.D3NOW_Pfrcpv_mm_mmm64)], mnemonic[nameof(Mnemonic.Pfrcpv)]),
				(code[nameof(Code.D3NOW_Pfrsqrtv_mm_mmm64)], mnemonic[nameof(Mnemonic.Pfrsqrtv)]),
				(code[nameof(Code.D3NOW_Pfnacc_mm_mmm64)], mnemonic[nameof(Mnemonic.Pfnacc)]),
				(code[nameof(Code.D3NOW_Pfpnacc_mm_mmm64)], mnemonic[nameof(Mnemonic.Pfpnacc)]),
				(code[nameof(Code.D3NOW_Pfcmpge_mm_mmm64)], mnemonic[nameof(Mnemonic.Pfcmpge)]),
				(code[nameof(Code.D3NOW_Pfmin_mm_mmm64)], mnemonic[nameof(Mnemonic.Pfmin)]),
				(code[nameof(Code.D3NOW_Pfrcp_mm_mmm64)], mnemonic[nameof(Mnemonic.Pfrcp)]),
				(code[nameof(Code.D3NOW_Pfrsqrt_mm_mmm64)], mnemonic[nameof(Mnemonic.Pfrsqrt)]),
				(code[nameof(Code.D3NOW_Pfsub_mm_mmm64)], mnemonic[nameof(Mnemonic.Pfsub)]),
				(code[nameof(Code.D3NOW_Pfadd_mm_mmm64)], mnemonic[nameof(Mnemonic.Pfadd)]),
				(code[nameof(Code.D3NOW_Pfcmpgt_mm_mmm64)], mnemonic[nameof(Mnemonic.Pfcmpgt)]),
				(code[nameof(Code.D3NOW_Pfmax_mm_mmm64)], mnemonic[nameof(Mnemonic.Pfmax)]),
				(code[nameof(Code.D3NOW_Pfrcpit1_mm_mmm64)], mnemonic[nameof(Mnemonic.Pfrcpit1)]),
				(code[nameof(Code.D3NOW_Pfrsqit1_mm_mmm64)], mnemonic[nameof(Mnemonic.Pfrsqit1)]),
				(code[nameof(Code.D3NOW_Pfsubr_mm_mmm64)], mnemonic[nameof(Mnemonic.Pfsubr)]),
				(code[nameof(Code.D3NOW_Pfacc_mm_mmm64)], mnemonic[nameof(Mnemonic.Pfacc)]),
				(code[nameof(Code.D3NOW_Pfcmpeq_mm_mmm64)], mnemonic[nameof(Mnemonic.Pfcmpeq)]),
				(code[nameof(Code.D3NOW_Pfmul_mm_mmm64)], mnemonic[nameof(Mnemonic.Pfmul)]),
				(code[nameof(Code.D3NOW_Pfrcpit2_mm_mmm64)], mnemonic[nameof(Mnemonic.Pfrcpit2)]),
				(code[nameof(Code.D3NOW_Pmulhrw_mm_mmm64)], mnemonic[nameof(Mnemonic.Pmulhrw)]),
				(code[nameof(Code.D3NOW_Pswapd_mm_mmm64)], mnemonic[nameof(Mnemonic.Pswapd)]),
				(code[nameof(Code.D3NOW_Pavgusb_mm_mmm64)], mnemonic[nameof(Mnemonic.Pavgusb)]),
				(code[nameof(Code.Rmpadjust)], mnemonic[nameof(Mnemonic.Rmpadjust)]),
				(code[nameof(Code.Rmpupdate)], mnemonic[nameof(Mnemonic.Rmpupdate)]),
				(code[nameof(Code.Psmash)], mnemonic[nameof(Mnemonic.Psmash)]),
				(code[nameof(Code.Pvalidatew)], mnemonic[nameof(Mnemonic.Pvalidate)]),
				(code[nameof(Code.Pvalidated)], mnemonic[nameof(Mnemonic.Pvalidate)]),
				(code[nameof(Code.Pvalidateq)], mnemonic[nameof(Mnemonic.Pvalidate)]),
				(code[nameof(Code.Serialize)], mnemonic[nameof(Mnemonic.Serialize)]),
				(code[nameof(Code.Xsusldtrk)], mnemonic[nameof(Mnemonic.Xsusldtrk)]),
				(code[nameof(Code.Xresldtrk)], mnemonic[nameof(Mnemonic.Xresldtrk)]),
			};

			if (result.Length != code.Values.Length)
				throw new InvalidOperationException();
			if (result.Select(a => a.codeEnum).ToHashSet<EnumValue>().Count != code.Values.Length)
				throw new InvalidOperationException();
			Array.Sort(result, (a, b) => a.codeEnum.Value.CompareTo(b.codeEnum.Value));
			return result;
		}
	}
}
